#include "utils.h"
#include "intlist.h"
#include "progress.h"

const unsigned NULL_COMP = UINT_MAX;

static void EnumerateComp(const IntVecVec &AdjMx, BoolVecVec &SignsMx,
  IntList &NodeIndexQueue, BoolList &SignQueue, IntVec &NodeIndexToCompIndex,
  BoolVec &NodeSigns, unsigned CompIndex)
	{
	assert(NodeIndexQueue.size() > 0);
	assert(NodeIndexQueue.size() == SignQueue.size());

	const unsigned NodeCount = (unsigned) AdjMx.size();

	unsigned NodeIndex = NodeIndexQueue.front();
	bool CurrentSign = SignQueue.front();
	NodeIndexQueue.pop_front();
	SignQueue.pop_front();

	asserta(NodeIndex < NodeCount);
	const IntVec &ToNodeIndexes = AdjMx[NodeIndex];
	const BoolVec &EdgeSigns = SignsMx[NodeIndex];
	const unsigned NeighborCount = SIZE(ToNodeIndexes);
	asserta(EdgeSigns.size() == NeighborCount);

	for (unsigned i = 0; i < NeighborCount; ++i)
		{
		unsigned ToNodeIndex = ToNodeIndexes[i];
		asserta(ToNodeIndex < NodeCount);

		bool EdgeSign = EdgeSigns[i];
		bool Sign = (EdgeSign == CurrentSign);

		unsigned ToCompIndex = NodeIndexToCompIndex[ToNodeIndex];
		if (ToCompIndex == NULL_COMP)
			{
			NodeIndexToCompIndex[ToNodeIndex] = CompIndex;
			NodeSigns[ToNodeIndex] = Sign;

			NodeIndexQueue.push_back(ToNodeIndex);
			SignQueue.push_back(Sign);
			}
		else
			{
			assert(ToCompIndex == CompIndex);
			}
		}
	}

void GetConnectedComponents(const IntVecVec &AdjMx_, BoolVecVec &SignsMx_,
  IntVecVec &CCs, BoolVec &NodeSigns, bool ShowProgress)
	{
	const unsigned NodeCount = SIZE(AdjMx_);
	assert(NodeCount == SIZE(SignsMx_));

	NodeSigns.resize(NodeCount);

// Construct undirected adjacency matrix
	IntVecVec AdjMx;
	BoolVecVec SignsMx;
	IntVec EmptyIntVec;
	BoolVec EmptyBoolVec;
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		AdjMx.push_back(EmptyIntVec);
		SignsMx.push_back(EmptyBoolVec);
		}

	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		const IntVec &Neighbors = AdjMx_[NodeIndex];
		const BoolVec &NodeSigns = SignsMx_[NodeIndex];

		unsigned NeighborCount = SIZE(Neighbors);
		asserta(SIZE(NodeSigns) == NeighborCount);
		for (unsigned i = 0; i < NeighborCount; ++i)
			{
			unsigned ToNodeIndex = Neighbors[i];
			bool EdgeSign = NodeSigns[i];

			AdjMx[NodeIndex].push_back(ToNodeIndex);
			SignsMx[NodeIndex].push_back(EdgeSign);

			AdjMx[ToNodeIndex].push_back(NodeIndex);
			SignsMx[ToNodeIndex].push_back(EdgeSign);
			}
		}

// NodeIndexToCompIndex[i] is connected component index of node i.
// Also functions as a "visited?" flag.
// NodeSigns[i] is sign of node i.
	IntVec NodeIndexToCompIndex;

	NodeIndexToCompIndex.resize(NodeCount, NULL_COMP);
	NodeSigns.resize(NodeCount);

	unsigned CompIndex = 0;
	if (ShowProgress)
		ProgressStart("Get CCs, %u nodes", NodeCount);
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		if (ShowProgress && NodeIndex%1000 == 0)
				ProgressStep(NodeIndex, NodeCount);

		const unsigned ThisCompIndex = NodeIndexToCompIndex[NodeIndex];

	// Already visited this node?
		if (ThisCompIndex != UINT_MAX)
			continue;

	// Root node for traversal of this component
		NodeIndexToCompIndex[NodeIndex] = CompIndex;

	// Root is +, other nodes are oriented relative to root.
		NodeSigns[NodeIndex] = true;

		IntList NodeIndexQueue;
		BoolList SignQueue;

		NodeIndexQueue.push_back(NodeIndex);
		SignQueue.push_back(true);

		while (NodeIndexQueue.size() > 0)
			EnumerateComp(AdjMx, SignsMx, NodeIndexQueue, SignQueue,
			  NodeIndexToCompIndex, NodeSigns, CompIndex);

	// Visited everything reachable from current node,
	// go to next component.
		++CompIndex;
		}
	if (ShowProgress)
		ProgressDone();

#ifndef NDEBUG
	{
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		asserta(NodeIndexToCompIndex[NodeIndex] != NULL_COMP);
	}
#endif

	const unsigned CompCount = CompIndex;
	CCs.clear();
	CCs.resize(CompCount);

	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		unsigned CompIndex = NodeIndexToCompIndex[NodeIndex];
		CCs[CompIndex].push_back(NodeIndex);
		}
	}
