#include "stack.h"

#define	TRACE			0
#define	TRACE_RESULT	0
#define VALIDATE		0
#define TEST			0

void LogBWS(const BaseWithSign &BWS)
	{
	Log("%u,%u:%u[%c]%c",
	  BWS.Base.GenIndex,
	  BWS.Base.ChrId,
	  (unsigned) BWS.Base.Pos,
	  BWS.GetLetter(),
	  BWS.Plus ? '+' : '-');
	}

void LogCol(const std::vector<BaseWithSign> &Col)
	{
	const unsigned BaseCount = SIZE(Col);
	for (unsigned BaseIndex = 0; BaseIndex < BaseCount; ++BaseIndex)
		{
		Log("[%u]=", BaseIndex);
		LogBWS(Col[BaseIndex]);
		}
	}

static bool ColsEq(const std::vector<BaseWithSign> &ColA,
  const std::vector<BaseWithSign> &ColB)
	{
	const unsigned BaseCountA = SIZE(ColA);
	const unsigned BaseCountB = SIZE(ColB);
	if (BaseCountA != BaseCountB)
		return false;
	for (unsigned Index = 0; Index < BaseCountA; ++Index)
		{
		const BaseWithSign &BWSA = ColA[Index];
		const BaseWithSign &BWSB = ColB[Index];
		if (BWSA != BWSB)
			return false;
		}
	return true;
	}

void AssertColSorted(const std::vector<BaseWithSign> &Col)
	{
	const unsigned BaseCount = SIZE(Col);
	for (unsigned BaseIndex = 1; BaseIndex < BaseCount; ++BaseIndex)
		{
		const BaseWithSign &Prev = Col[BaseIndex - 1];
		const BaseWithSign &Curr = Col[BaseIndex];
		if (!(Prev <= Curr))
			{
			LogCol(Col);
			Log("Prev[%u]=", BaseIndex-1);
			LogBWS(Prev);
			Log("\n");
			Log("Curr[%u]=", BaseIndex);
			LogBWS(Curr);
			Log("\n");
			Log("Prev < Curr=%c, Curr < Prev=%c\n",
			  Prev < Curr ? 'T' : 'F',
			  Curr < Prev ? 'T' : 'F');
			asserta(false);
			}
		}
	}

static unsigned Closest(const BaseInfo &Base, const std::vector<BaseWithSign> &Col)
	{
	const unsigned BaseCount = SIZE(Col);

	unsigned ClosestIndex = UINT_MAX;
	coord_t ClosestDist = COORD_T_MAX;

	for (unsigned i = 0; i < BaseCount; ++i)
		{
		const BaseWithSign &BWS2 = Col[i];
		const BaseInfo &Base2 = BWS2.Base;
		if (Base.GenIndex != Base2.GenIndex || Base.ChrId != Base2.ChrId)
			continue;
		coord_t Dist = (Base.Pos > Base2.Pos ? Base.Pos - Base2.Pos : Base2.Pos - Base.Pos);
		if (Dist < ClosestDist)
			{
			ClosestDist = Dist;
			ClosestIndex = i;
			}
		}
	return ClosestIndex;
	}

static unsigned g_SortGenIndex = UINT_MAX;

void SetSortGenIndex(unsigned GenIndex)
	{
	g_SortGenIndex = GenIndex;
	}

static coord_t GetLowestCoord(const std::vector<BaseWithSign> &Col, unsigned GenIndex,
  bool &Plus)
	{
	Plus = true;
	const unsigned BaseCount = SIZE(Col);
	coord_t LowestCoord = COORD_T_MAX;
	for (unsigned Index = 0; Index < BaseCount; ++Index)
		{
		const BaseWithSign &BWS = Col[Index];
		if (BWS.Base.GenIndex == GenIndex && BWS.Base.Pos < LowestCoord)
			{
			Plus = BWS.Plus;
			LowestCoord = BWS.Base.Pos;
			}
		}
	return LowestCoord;
	}

bool ColLTGen(const std::vector<BaseWithSign> *ColA,
  const std::vector<BaseWithSign> *ColB)
	{
	for (unsigned GenIndex = 0; GenIndex <= g_SortGenIndex; ++GenIndex)
		{
		bool PlusA;
		bool PlusB;
		coord_t LowestA = GetLowestCoord(*ColA, g_SortGenIndex, PlusA);
		coord_t LowestB = GetLowestCoord(*ColB, g_SortGenIndex, PlusB);
		if (LowestA != COORD_T_MAX && LowestB != COORD_T_MAX)
			{
			if (PlusA && PlusB)
				return LowestA < LowestB;
			return LowestB < LowestA;
			}
		}
	return false;
	}

bool ColLT(const std::vector<BaseWithSign> &ColA, const std::vector<BaseWithSign> &ColB)
	{
#if	VALIDATE
	AssertColSorted(ColA);
	AssertColSorted(ColB);
#endif

	const unsigned BaseCountA = SIZE(ColA);
	const unsigned BaseCountB = SIZE(ColB);

#if	TRACE
	Log("ColLT ");
	Log(" &A=%x", &ColA);
	Log(" &B=%x", &ColB);
	Log(" IndexA=%u", ColA[0].OutputColIndex);
	Log(" IndexB=%u", ColB[0].OutputColIndex);
	Log(" BaseCountA=%u", BaseCountA);
	Log(" BaseCountB=%u", BaseCountB);
	Log("\n");
#endif

	if (BaseCountA == 0 || BaseCountA > SANE_BASE_COUNT)
		{
		Log("\n");
		Log("&A=%x\n", &ColA);
		Log("&B=%x\n", &ColB);
		Log("BaseCountA=%u\n", BaseCountA);
		Log("BaseCountB=%u\n", BaseCountB);
		Die("BaseCountA");
		}
	if (BaseCountB == 0 || BaseCountB > SANE_BASE_COUNT)
		{
		Log("\n");
		Log("&A=%x\n", &ColA);
		Log("&B=%x\n", &ColB);
		Log("BaseCountA=%u\n", BaseCountA);
		Log("BaseCountB=%u\n", BaseCountB);
		Die("BaseCountB");
		}

	const BaseWithSign &BWSA0 = ColA[0];
	const BaseWithSign &BWSB0 = ColB[0];
	if (BWSA0 == BWSB0)
		{
#if	VALIDATE
		asserta(ColsEq(ColA, ColB));
#endif
		return false;
		}

	IntVec IndexAToClosestInB;
	IntVec IndexBToClosestInA;

	IndexAToClosestInB.resize(BaseCountA, UINT_MAX);
	IndexBToClosestInA.resize(BaseCountB, UINT_MAX);

	for (unsigned IndexA = 0; IndexA < BaseCountA; ++IndexA)
		{
		const BaseWithSign &BWS = ColA[IndexA];
		unsigned IndexB = Closest(BWS.Base, ColB);
		IndexAToClosestInB[IndexA] = IndexB;
		}

	for (unsigned IndexB = 0; IndexB < BaseCountB; ++IndexB)
		{
		const BaseWithSign &BWS = ColB[IndexB];
		unsigned IndexA = Closest(BWS.Base, ColA);
		IndexBToClosestInA[IndexB] = IndexA;
		}

	for (unsigned IndexA = 0; IndexA < BaseCountA; ++IndexA)
		{
		unsigned IndexB = IndexAToClosestInB[IndexA];
		if (IndexB == UINT_MAX)
			continue;
		if (IndexBToClosestInA[IndexB] != IndexA)
			continue;
		const BaseWithSign &BWSA = ColA[IndexA];
		const BaseWithSign &BWSB = ColB[IndexB];
		asserta(BWSA != BWSB);
		bool LT = BWSA < BWSB;
#if	TRACE_RESULT
		Log("ColLT A=");
		LogCol(ColA);
		Log(" %s ", LT ? "<" : ">=");
		Log(" B=");
		LogCol(ColB);
		Log("\n");
		Log("  BWSA=");
		LogBWS(BWSA);
		Log(" %s ", LT ? "<" : ">=");
		Log(" BWSB=");
		LogBWS(BWSB);
		Log("\n");
#endif
		return LT;
		}

	return BWSA0 < BWSB0;
	}

bool ColPtrLT(const std::vector<BaseWithSign> *ColA,
  const std::vector<BaseWithSign> *ColB)
	{
	return ColLT(*ColA, *ColB);
	}

#ifdef _MSC_VER
int _cdecl ColQsortCmp(const void *pvColA, const void *pvColB)
#else
int ColQsortCmp(const void *pvColA, const void *pvColB)
#endif
	{
	const std::vector<BaseWithSign> *ColA = *((const std::vector<BaseWithSign> **) pvColA);
	const std::vector<BaseWithSign> *ColB = *((const std::vector<BaseWithSign> **) pvColB);
	bool LT = ColLT(*ColA, *ColB);
	if (LT)
		return -1;
	if (ColsEq(*ColA, *ColB))
		return 0;
	assert(ColLT(*ColB, *ColA));
	return 1;
	}

#if	TEST
void Test2(const std::vector<BaseWithSign> &ColA,
  const std::vector<BaseWithSign> &ColB)
	{
	if (ColLT(ColA, ColB))
		asserta(!ColLT(ColB, ColA));
	if (ColLT(ColB, ColA))
		asserta(!ColLT(ColA, ColB));
	asserta(!ColLT(ColA, ColA));
	asserta(!ColLT(ColB, ColB));

	if (ColPtrLT(&ColA, &ColB))
		asserta(!ColPtrLT(&ColB, &ColA));
	if (ColPtrLT(&ColB, &ColA))
		asserta(!ColPtrLT(&ColA, &ColB));
	asserta(!ColPtrLT(&ColA, &ColA));
	asserta(!ColPtrLT(&ColB, &ColB));

	std::list<std::vector<BaseWithSign> > OutputCols;
	std::vector<const std::vector<BaseWithSign> *> OutputColPtrs;

	OutputColPtrs.push_back(&ColA);
	OutputColPtrs.push_back(&ColB);
	std::sort(OutputColPtrs.begin(), OutputColPtrs.end(), ColPtrLT);

	OutputColPtrs.clear();
	OutputColPtrs.push_back(&ColB);
	OutputColPtrs.push_back(&ColA);
	std::sort(OutputColPtrs.begin(), OutputColPtrs.end(), ColPtrLT);
	}

static void Add(std::vector<BaseWithSign> &Col, unsigned GenIndex, unsigned ChrId,
  coord_t Pos, char Letter, double ConfScore, bool Sign)
	{
	BaseWithSign BWS(GenIndex, ChrId, Pos, Letter, ConfScore, Sign);
	Col.push_back(BWS);
	}

void Test()
	{
	std::vector<BaseWithSign> ColA;
	std::vector<BaseWithSign> ColB;

	Add(ColA, 0, 0, 1637483, 'T', 0, true);
	Add(ColA, 1, 0, 1234444, 'A', 0, false);
	Add(ColA, 0, 0, 945065, 'T', 0, true);
	Add(ColA, 1, 0, 629251, 'A', 0, false);

	Add(ColB, 0, 0, 945013, 'G', 0, true);
	Add(ColB, 1, 0, 1234495, 'C', 0, false);
	Add(ColB, 1, 13, 583486, 'C', 0, false);

	std::sort(ColA.begin(), ColA.end());
	std::sort(ColB.begin(), ColB.end());

	AssertColSorted(ColA);
	AssertColSorted(ColB);

	Test2(ColA, ColB);
	}
#endif	// TEST
