#include "evo.h"

void ComputeMicroDist(const char *Msg, const vector<pair<gpos_t, double> > &Rates,
  vector<double> &MicroRates, double &IntegratedMicroRate,
  vector<unsigned> &MicroBins)
	{
// Special cases
	const unsigned N = SIZE(Rates);
	if (N == 0)
		{
		MicroRates.resize(P_MicroThreshold+1, 0);
		MicroBins.resize(RateBinCount, 0);
		IntegratedMicroRate = 0;
		return;
		}

	if (N == 1)
		Die("%s, cannot specify just one rate", Msg);
	
	MicroRates.resize(P_MicroThreshold+1, DBL_MAX);
	MicroBins.resize(RateBinCount, GPOS_T_MAX);

	gpos_t LastLength = 0;
	double LastRate = 0;
	for (unsigned i = 0; i < N; ++i)
		{
		const pair<gpos_t, double> LengthRate = Rates[i];
		gpos_t Length = LengthRate.first;
		double Rate = LengthRate.second;

		if (Length == 0)
			Die("ComputeMicroDist(%s): zero length illegal", Msg);

		if (Length <= LastLength)
			Die("%s, rates not specified in increasing length order", Msg);

		if (Length > P_MicroThreshold)
			break;

		MicroRates[Length] = Rate;

	// Interpolate any missing values
		double Gradient = (Rate - LastRate)/(double) (Length - LastLength);
		for (gpos_t L = LastLength + 1; L < Length; ++L)
			{
			if (L > P_MicroThreshold)
				break;

			MicroRates[L] = LastRate + (L - LastLength)*Gradient;
			}

		LastLength = Length;
		LastRate = Rate;
		}

	for (gpos_t Length = 1; Length <= P_MicroThreshold; ++Length)
		asserta(MicroRates[Length] != DBL_MAX);

	IntegratedMicroRate = 0;
	for (gpos_t Length = 1; Length <= P_MicroThreshold; ++Length)
		IntegratedMicroRate += MicroRates[Length];

	MicroRates[0] = 0;
	SetWeightVec(MicroRates, MicroBins, RateBinCount);
	}
