#include "utils.h"
#include "cmafile.h"
#include "revfile.h"
#include "seqfile.h"
#include "getopts.h"

static void Add(RevFile &Rev, const CoordVec &Coords, const StrVec &Rows)
	{
	if (Coords.size() != Rows.size())
		Die("Coords.size() != Rows.size()");

	AlnInfo AI;

	const unsigned GenomeCount = (unsigned) Coords.size();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		const CoordData &CD = Coords[GenomeIndex];

		GCSeg S;
		S.GenomeIndex = GenomeIndex;
		S.ChrId = CD.ChrIndex;
		S.lo = CD.StartPos;
		S.hi = CD.EndPos;
		S.Plus = !CD.Reversed;

		AI.Segs.push_back(S);
		}

	StdStrVec StdRows;
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		StdRows.push_back(Rows[GenomeIndex]);

	AI.Score = 0;
	Rev.AddAln(AI, StdRows);
	}

static void GIsFromSeqFiles(const CMAFile &CMA, std::vector<GenInfo> &GIs)
	{
	const unsigned GenomeCount = CMA.GetGenomeCount();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		char SeqFileName[64];
		sprintf(SeqFileName, "seq%u", GenomeIndex + 1);

		SeqFile Seq;
		Seq.Open(SeqFileName);

		std::vector<ChrInfo> CIs;
		const ChrNameIndex &CNI = CMA.GetChrNameIndex(GenomeIndex);
		unsigned CMAFileChrId = UINT_MAX;
		const unsigned ChrCount = CNI.GetChrCount();
		for (unsigned i = 0; i < ChrCount; ++i)
			{
			if (i == 0)
				CMAFileChrId = CNI.GetFirstChrIndex();
			else
				CMAFileChrId = CNI.GetNextChrIndex(CMAFileChrId);

			const std::string &ChrName = CNI.GetName(CMAFileChrId);

			unsigned SeqFileChrId = Seq.GetChrIndex(ChrName.c_str());
			coord_t ChrLength = Seq.GetChrLength(SeqFileChrId);

			ChrInfo CI;
			CI.Create(CMAFileChrId, ChrName, ChrLength);

			CIs.push_back(CI);
			}

		Seq.Close();

		std::string GenomeName = CMA.GetGenomeName(GenomeIndex);

		GenInfo GI;
		GI.Create(GenomeName, CIs);

		GIs.push_back(GI);
		}
	}

void CvtCMA2Rev()
	{
	const char *CMAFileName = RequiredValueOpt("fromcma");
	const char *RevFileName = RequiredValueOpt("torev");

	CMAFile CMA;
	RevFile Rev;

	CMA.Open(CMAFileName);

	std::vector<GenInfo> GIs;
	GIsFromSeqFiles(CMA, GIs);

	Rev.Create(RevFileName, GIs);
	const unsigned CMACount = CMA.GetCMACount();
	for (unsigned CMAIndex = 0; CMAIndex < CMACount; ++CMAIndex)
		{
		CoordVec Coords;
		StrVec Rows;
		CMA.GetCMA(CMAIndex, Coords, Rows);
		Add(Rev, Coords, Rows);
		}

	CMA.Close();
	Rev.Close();
	}
