#include "evo.h"

static unsigned InvalidCounts[256];
static unsigned g_TotalInvalidCount;

unsigned GetInvalidLetterCount()
	{
	return g_TotalInvalidCount;
	}

void ClearInvalidCounts()
	{
	memset(InvalidCounts, 0, sizeof(InvalidCounts));
	g_TotalInvalidCount = 0;
	}

void SkipInvalid(unsigned char c)
	{
	++(InvalidCounts[c]);
	++g_TotalInvalidCount;
	}

void WarnInvalids()
	{
//	unsigned Total = 0;
	for (int i = 0; i < 256; ++i)
		{
		unsigned Count = InvalidCounts[i];
		if (Count > 0)
			{
			char c = (char) i;
			if (isprint(c))
				Warning("Invalid letter '%c' found %u times, replaced by random letters",
				  c, Count);
			else
				Warning("Invalid letter 0x02x found %u times, replaced by random letters",
				  c, Count);
			}
		}
	}

void Evolver::FromFASTAFile(const char *FileName)
	{
	Clear();
	ClearInvalidCounts();

	FILE *f = OpenStdioFile(FileName);
	off64_t Size = GetStdioFileSize(f);
	off64_t BytesLeft = Size;
	const unsigned BUFF_SIZE = ONE_MILLION;
	char Buffer[BUFF_SIZE];
	unsigned BufferPos = 0;
	unsigned BufferBytes = 0;
	unsigned BufferCount = (unsigned) (Size/BUFF_SIZE);
	unsigned BufferIndex = 0;

	enum
		{
		Start,
		Annot,
		SeqData
		} State = Start;

	ProgressStart("Reading FASTA file %.16s", FileName);
	for (;;)
		{
		if (BufferPos == BufferBytes)
			{
			if (BytesLeft == 0)
				break;
			ProgressStep(BufferIndex++, BufferCount);
			off64_t BytesToRead64 = (off64_t) BytesLeft;
			if (BytesToRead64 > BUFF_SIZE)
				BytesToRead64 = BUFF_SIZE;
			unsigned BytesToRead = off64_t_to_unsigned(BytesToRead64);
			ReadStdioFile(f, Buffer, BytesToRead);
			BufferBytes = BytesToRead;
			BytesLeft -= BytesToRead;
			BufferPos = 0;
			}
		char c = Buffer[BufferPos++];
		switch (State)
			{
		case Start:
			{
			if (c != '>')
				Die("Expected '>' at start of FASTA file, got '%c'", c);
			State = Annot;
			break;
			}

		case Annot:
			{
			if (c == '\n' || c == '\r')
				{
				State = SeqData;
				continue;
				}
			break;
			}

		case SeqData:
			{
			if (c == '>')
				{
				State = Annot;
				continue;
				}
			if (isspace(c))
				break;
			if (!isacgt(c))
				{
				SkipInvalid(c);
				c = RandLetter();
				}
			AppendAncestralBase(c);
			break;
			}
		default:
			asserta(false);
			}
		}
	ProgressDone();
	WarnInvalids();

	CloseStdioFile(f);
	}
