#include "evo.h"

static gpos_t g_BaseCounts[UINT8_MAX+1];
static gpos_t g_MatchCounts[UINT8_MAX+1];
static gpos_t g_CatCounts[UINT8_MAX+1][BaseCategoryCount];

void Evolver::IncStats() const
	{
	FOR_CEVBASES(p)
		{
		const BaseInfo &Base = *p;
		BaseCategory Cat = Base.GetCategory();
		asserta((unsigned) Cat < BaseCategoryCount);

		unsigned CC = Base.CC;
		asserta(CC <= UINT8_MAX);

		++(g_CatCounts[CC][Cat]);

		if (Cat != BC_Ancestral)
			continue;

		char AncLetter = GetAncLetter(Base.Pos);
		if (AncLetter == '-')
			continue;
		if (!isacgt(AncLetter))
			Die("IncMatchCounts, ancestral letter=%c", AncLetter);

		char Letter = Base.Letter;
		if (Letter == '-')
			continue;
		if (!isacgt(Letter))
			Die("IncMatchCounts, letter=%c", Letter);

		++(g_BaseCounts[CC]);
		if (toupper(Letter) == toupper(AncLetter))
			++(g_MatchCounts[CC]);
		}
	}

void LogMatchCounts()
	{
#if	0	// TODO
	gpos_t TotalBaseCount = 0;
	for (unsigned uCC = 0; uCC <= UINT8_MAX; ++uCC)
		{
		uint8 CC = (uint8) uCC;
		unsigned BaseCount = g_BaseCounts[CC];
		TotalBaseCount += BaseCount;
		}

	Log(" CC       Description       Novel  Novel%%        Copy   Copy%%   Ancestral    Anc%%\n");
	Log("===  ================  ==========  ======  ==========  ======  ==========  ======\n");
	unsigned NovelTotal = 0;
	unsigned CopyTotal = 0;
	unsigned AncTotal = 0;
	TotalBaseCount = 0;
	for (unsigned uCC = 0; uCC <= UINT8_MAX; ++uCC)
		{
		uint8 CC = (uint8) uCC;
		const char *CCName = GetCCName(CC);
		unsigned NovelCount = g_CatCounts[uCC][BC_Novel];
		unsigned CopyCount = g_CatCounts[uCC][BC_Copy];
		unsigned AncCount = g_CatCounts[uCC][BC_Ancestral];
		unsigned TotalCount = NovelCount + CopyCount + AncCount;
		if (TotalCount == 0)
			continue;
		TotalBaseCount += TotalCount;
		NovelTotal += NovelCount;
		CopyTotal += CopyCount;
		AncTotal += AncCount;
		double NovelPct = (NovelCount*100.0)/TotalCount;
		double CopyPct = (CopyCount*100.0)/TotalCount;
		double AncPct = (AncCount*100.0)/TotalCount;

		Log("%3u", uCC);
		Log("  %16.16s", CCName);
		Log("  %10u", NovelCount);
		Log("  %5.1f%%", NovelPct);
		Log("  %10u", CopyCount);
		Log("  %5.1f%%", CopyPct);
		Log("  %10u", AncCount);
		Log("  %5.1f%%", AncPct);
		Log("\n");
		}
	if (TotalBaseCount > 0)
		{
		Log("                       ==========  ======  ==========  ======  ==========  ======\n");
		double NovelPct = (NovelTotal*100.0)/TotalBaseCount;
		double CopyPct = (CopyTotal*100.0)/TotalBaseCount;
		double AncPct = (AncTotal*100.0)/TotalBaseCount;
		Log("   ");
		Log("                  ");
		Log("  %10u", NovelTotal);
		Log("  %5.1f%%", NovelPct);
		Log("  %10u", CopyTotal);
		Log("  %5.1f%%", CopyPct);
		Log("  %10u", AncTotal);
		Log("  %5.1f%%", AncPct);
		Log("  Total %u %.1f%%\n", TotalBaseCount, NovelPct + CopyPct + AncPct);
		Log("\n");
		}

	Log("\n");
	Log("Matches (anc. only)\n");
	Log(" CC       Description       Bases  %%Total     Matches     %%id\n");
	Log("===  ================  ==========  ======  ==========  ======\n");

	for (unsigned uCC = 0; uCC <= UINT8_MAX; ++uCC)
		{
		uint8 CC = (uint8) uCC;
		unsigned BaseCount = g_BaseCounts[CC];
		unsigned MatchCount = g_MatchCounts[CC];
		asserta(MatchCount <= BaseCount);
		if (BaseCount == 0 &&
		  !(CC == CC_Unconstrained || CC == CC_DefaultCDS || CC == CC_DefaultNCCE))
			continue;

		const char *CCName = GetCCName(CC);
		double PctId = BaseCount == 0 ? 0 :
		  (double) MatchCount*100.0/(double) BaseCount;
		double PctOfTotal =
		  (TotalBaseCount == 0 ? 0 : (BaseCount*100.0)/TotalBaseCount);
		Log("%3u  %16.16s  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%\n",
		  uCC,
		  CCName,
		  BaseCount,
		  PctOfTotal,
		  MatchCount,
		  PctId);
		}
	Log("                       ==========\n");
	Log("                       %10"PF"\n",  TotalBaseCount);
	Log("\n");
#endif
	}
