#include "evo.h"
#include "annot.h"

static gpos_t g_CDSBaseCount;
static gpos_t g_NCCEBaseCount;
static gpos_t g_TandemBaseCount;

ANNOT_TYPE StrToAnnotType(const char *Str)
	{
#if	_MSC_VER
#define A(a)	if (_stricmp(Str, #a) == 0) return ANNOT_TYPE_##a;
#else
#define A(a)	if (strcasecmp(Str, #a) == 0) return ANNOT_TYPE_##a;
#endif
#include "annots.h"
	return ANNOT_TYPE_Invalid;
	}

uint16 GetGeneIndex(const char *Attrs, bool Required)
	{
	if (Required)
		{
		unsigned Index = GFFGetRequiredIntAttr(Attrs, "gene_index");
		if (Index >= UINT16_MAX)
			Die("gene_index > max (%u)", (unsigned) UINT16_MAX - 1);
		else
			return (uint16) Index;
		}
	else
		{
		unsigned Index = UINT_MAX;
		bool Found = GFFGetIntAttr(Attrs, "gene_index", Index);
		if (Found)
			{
			if (Index >= UINT16_MAX)
				Die("gene_index > max (%u)", (unsigned) UINT16_MAX - 1);
			else
				return (uint16) Index;
			}
		}
	return UINT16_MAX;
	}

uint8 GetCEC(const char *Attrs)
	{
	unsigned uCEC = UINT_MAX;
	bool Found = GFFGetIntAttr(Attrs, "cec", uCEC);
	if (!Found)
		return UINT8_MAX;
	if (uCEC > UINT8_MAX)
		Die("Invalid cec %u > UINT8_MAX", uCEC);
	return (uint8) uCEC;
	}

gpos_t GetMicrosatMotifLength(const char *Attrs)
	{
	return GFFGetRequiredIntAttr(Attrs, "replen");
	}

uint8 GetGeneArrayIndex(const char *Attrs)
	{
	unsigned uIx = UINT_MAX;
	bool Found = GFFGetIntAttr(Attrs, "gene_array", uIx);
	if (!Found)
		return UINT8_MAX;
	if (uIx >= UINT8_MAX)
		Die("Invalid gene_array %u >= UINT8_MAX", uIx);
	return (uint8) uIx;
	}

void Evolver::GetAnnots(const char *FileName)
	{
	FILE *f = OpenStdioFile(FileName);
	unsigned FileSize = (unsigned) GetStdioFileSize(f);
	GFFRecord Rec;
	unsigned RecordCount = 0;
	ProgressStart("Reading GFF records from %.16s", FileName);

	vector<gpos_t> GeneStartPositions;
	vector<gpos_t> GeneStopPositions;
	while (GetNextGFFRecord(f, Rec))
		{
		if ((RecordCount++)%1000 == 0)
			{
			unsigned Pos = (unsigned) GetStdioFilePos(f);
			ProgressStep(Pos, FileSize);
			}
		SetAnnot(Rec, GeneStartPositions, GeneStopPositions);
		}
	ProgressDone();

	gpos_t AncestralLength = GetAncestralLength();
	if (AncestralLength == 0)
		return;

	double CDSPct = (g_CDSBaseCount*100.0)/AncestralLength;
	double NCCEPct = (g_NCCEBaseCount*100.0)/AncestralLength;
	double TandemPct = (g_TandemBaseCount*100.0)/AncestralLength;

	Log("\n");
	ProgressNoRes("%10u annotations", RecordCount);
	ProgressNoRes("%10"PF" CDS bases,    %5.1f%% of genome",
	  g_CDSBaseCount, CDSPct);
	ProgressNoRes("%10"PF" NCCE bases,   %5.1f%% of genome",
	  g_NCCEBaseCount, NCCEPct);
	ProgressNoRes("%10"PF" tandem bases, %5.1f%% of genome",
	  g_TandemBaseCount, TandemPct);
	Log("\n");

	SetIntrons(GeneStartPositions, GeneStopPositions);

	CloseStdioFile(f);
	}
