#include "bndfile.h"
#include "seqfile.h"
#include "valpath.h"
#include "utils.h"
#include <ctype.h>

extern SeqFile *g_Seq1;
extern SeqFile *g_Seq2;

extern unsigned g_ChrIndex1;
extern unsigned g_ChrIndex2;

void RevComp(char *S, unsigned L);
unsigned GetSeq1ChrIndex();
unsigned GetSeq2ChrIndex();

void HitToAln(const SegPair &Hit, const char *Path,
  char **ptrRow1, char **ptrRow2)
	{
	ValidateHitPath(Hit, Path);

	unsigned ColCount = (unsigned) strlen(Path);
	char *Row1 = myalloc(char, ColCount + 1);
	char *Row2 = myalloc(char, ColCount + 1);

	Row1[ColCount] = 0;
	Row2[ColCount] = 0;

	unsigned Length1 = (unsigned) (Hit.Segs[0].Length());
	unsigned Length2 = (unsigned) (Hit.Segs[1].Length());

	char *SeqData1 = myalloc(char, Length1);
	char *SeqData2 = myalloc(char, Length2);

	coord_t ChrPos1 = Hit.Segs[0].lo;
	coord_t ChrPos2 = Hit.Segs[1].lo;

	unsigned ChrIndex1 = GetSeq1ChrIndex();
	unsigned ChrIndex2 = GetSeq2ChrIndex();

	g_Seq1->GetSeqData(ChrIndex1, ChrPos1, SeqData1, Length1);
	g_Seq2->GetSeqData(ChrIndex2, ChrPos2, SeqData2, Length2);
#if	TRACE
	Log("HitToAln: seq1 chr=%u pos=%u len=%u seq=%.*s\n", g_ChrIndex1,
	  (unsigned) ChrPos1, Length1, Length1, SeqData1);
	Log("HitToAln: seq2 chr=%u pos=%u len=%u seq=%.*s\n", g_ChrIndex2,
	  (unsigned) ChrPos2, Length2, Length2, SeqData2);
#endif
	if (Hit.Reverse)
		{
		RevComp(SeqData2, Length2);
#if	TRACE
		Log("RevComp: seq2 chr=%u pos=%u len=%u seq=%.*s\n", g_ChrIndex2,
		  (unsigned) ChrPos2, Length2, Length2, SeqData2);
#endif
		}

	unsigned Pos1 = 0;
	unsigned Pos2 = 0;
	for (unsigned ColIndex = 0; ColIndex < ColCount; ++ColIndex)
		{
		char c = Path[ColIndex];
		switch (c)
			{
		case 'M':
			Row1[ColIndex] = SeqData1[Pos1++];
			Row2[ColIndex] = SeqData2[Pos2++];
			break;

		case 'D':
			Row1[ColIndex] = SeqData1[Pos1++];
			Row2[ColIndex] = '-';
			break;

		case 'I':
			Row1[ColIndex] = '-';
			Row2[ColIndex] = SeqData2[Pos2++];
			break;
		default:
			Die("HitToAln, invalid char %c in path", c);
			}
		}

	if (Pos1 != Length1 || Pos2 != Length2)
		{
		Log("Hit=%u %u %u %u\n",
		  (unsigned) Hit.Segs[0].lo,
		  (unsigned) Hit.Segs[0].hi,
		  (unsigned) Hit.Segs[1].lo,
		  (unsigned) Hit.Segs[1].hi);
		Log("Path=%s\n", Path);
		Die("HitToAln Pos1=%u Length1=%u Pos2=%u Length2=%u",
		  Pos1, Length1, Pos2, Length2);
		}

	*ptrRow1 = Row1;
	*ptrRow2 = Row2;

#if	TRACE
	Log("Row1=%.*s\n", ColCount, Row1);
	Log("Row2=%.*s\n", ColCount, Row2);
#endif

	myfree(SeqData1);
	myfree(SeqData2);
	}
