#include "evo.h"

// TODO: max MUL size

void Macro()
	{
	const char *InFileName = RequiredValueOpt("macro");
	const char *OutFileName = RequiredValueOpt("out");
	const char *GFFFileName = RequiredValueOpt("ce_gff");
	const char *MEFileName = ValueOpt("me");
	const char *ModelFileName = ValueOpt("model");
	double Duration = RequiredFloatOpt("branchlength");
	bool LogEvents = FlagOpt("logevents");
	bool ValidateEvents = FlagOpt("valevents");
	bool Dump = FlagOpt("dump");
	g_NoEvents = FlagOpt("noevents");
	FloatOpt("durf", &g_DurationFactor);
	unsigned RandSeed = 1;
	IntOpt("seed", &RandSeed);

	SetModel(ModelFileName);

	bool ShowProgress = true;

	FILE *f = CreateStdioFile(OutFileName);

	BigEvolver Big;
	GenInfo Genome;
	Big.InitFromEvoFile(InFileName, Genome);
	Big.LogEvMULs();

	map<string, gpos_t> ChrNameToStartPos;
	map<gpos_t, string> ChrStartPosToName;
	MakeChrPosMap(Genome, ChrNameToStartPos, ChrStartPosToName);
	Big.AnnotsFromGFF(GFFFileName, ChrNameToStartPos, ChrStartPosToName, Genome);

	if (MEFileName != 0)
		{
		Big.GetMEsFromFile(MEFileName, ChrNameToStartPos);
		Big.AppendMEMULs();
		}

	Big.SetValidateEvents(ValidateEvents);
	Big.SetLogEvents(LogEvents);
	Big.LogSummary();

	Progress("Validate initial state");
	Big.Validate();

	Big.Evolve(Duration, RandSeed, ShowProgress);
	Big.SplitOversizeMULs();

	Progress("Validate final state");
	Big.Validate();

	Big.LogEventRates();
	Progress("Write MUL file");
	Big.ToFile(f);
	if (Dump)
		{
		Progress("Dumping");
		Big.LogMe();
		}
	Progress("Logging summary");
	Big.LogSummary();

	CloseStdioFile(f);
	f = 0;

	unsigned LeafCount = 0;
	unsigned InternalCount = 0;
	Big.GetEvolvedMULCounts(LeafCount, InternalCount);
	LogERCounts(false, LeafCount);
	LogERDetails(false);
	}
