#include "stack.h"

#define TRACE	0

/***
Use a dumb O(N^2) method to construct an overlap graph
of segments. This can be done faster if needed.
Creates a symmetrical adjacency matrix in Edges[][].

The representation is sparse:
If there are N nodes, then Edges[i][j] is the index
(0 .. N-1) of the jth neighbor of node i, i = 0 .. N-1.
***/

void MakeSegOverlapGraph(const GCSegVec &Segs, IntVecVec &Edges)
	{
	const unsigned SegCount = (unsigned) Segs.size();

#if	TRACE
	{
	Log("\n");
	Log("MakeSegOverlapGraph, segs:\n");
	for (unsigned i = 0; i < SegCount; ++i)
		{
		const GCSeg &Seg = Segs[i];
		Log("[%3u] ", i);
		LogGCSeg(Seg);
		Log("\n");
		}
	Log("\n");
	}
#endif

	Edges.clear();
	Edges.resize(SegCount);

	for (unsigned SegIndexA = 0; SegIndexA < SegCount; ++SegIndexA)
		{
		const GCSeg &SegA = Segs[SegIndexA];
		
		for (unsigned SegIndexB = SegIndexA + 1; SegIndexB < SegCount; ++SegIndexB)
			{
			const GCSeg &SegB = Segs[SegIndexB];
			
			if (GetOverlapLength(SegA, SegB) > 0)
				{
				Edges[SegIndexA].push_back(SegIndexB);
				Edges[SegIndexB].push_back(SegIndexA);
				}
			}
		}

#if	TRACE
	Log("\n");
	Log("Adjacency matrix:\n");
	LogIntVecVec(Edges);
#endif
	}
