#ifndef	SEQFILE_H
#define SEQFILE_H

#include <stdio.h>
#include <list>
#include <vector>
#include "chrnameindex.h"
#include "types.h"

const unsigned MaxGenName = 31;
const unsigned SeqFileMagic = 0xbd1235;

class SeqFile
	{
public:
	struct SeqFileHeader
		{
		unsigned Magic;
		unsigned ChrCount;
		char GenomeName[MaxGenName+1];
		uint64 FileSize;
		uint64 ChrNameIndexStartPos;
		uint64 ChrSeqDataIndexStartPos;
		};

private:
	SeqFile(SeqFile &rhs);
	SeqFile &operator=(SeqFile &rhs);

public:
	SeqFile();
	virtual ~SeqFile();

	void Close();

// Functions to read an existing file
	void Open(const char *FileName);
	const ChrNameIndex &GetChrNameIndex() const;
	unsigned GetChrCount() const;
	coord_t GetChrLength(unsigned ChrIndex) const;
	void GetSeqData(unsigned ChrIndex, coord_t Pos, char *Buffer, coord_t Length) const;
	const char *GetGenomeName() const;
	const char *GetChrName(unsigned ChrIndex) const;
	unsigned GetChrIndex(const char *ChrName) const;

// Functions to create a new file
	void Create(const char *FileName, const char *GenomeName);
	void AppendChr(const char *ChrName);

// AppendChrData can be called > 1 time for the same chromosome.
	void AppendChrData(const char *SeqData, coord_t Length);

private:
	const char *m_FileName;
	bool m_Creating;
	unsigned m_ChrCount;
	SeqFileHeader m_Header;
	FILE *m_f;
	ChrNameIndex m_ChrNameIndex;
	coord_t m_CurrChrLength;

// Chromosome positions and lengths
// Lists for creating (typical operation is append)
	std::list<uint64> m_ChrPosOut;
	std::list<coord_t> m_ChrLengthOut;

// Vectors for reading (typical operation is random access)
	std::vector<uint64> m_ChrPosIn;
	std::vector<coord_t> m_ChrLengthIn;
	};

#endif	// SEQFILE_H
