#ifndef STACK_H
#define STACK_H

#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "getopts.h"
#include "mkedgesai.h"
#include "stacker.h"

bool Overlap(const AlnInfoX &AIA, const AlnInfoX &AIB, unsigned GenomeIndex);
coord_t GetOverlapLength(const GCSeg &SegA,const GCSeg &SegB);
void GetBendpoints(const std::vector<AlnInfoX> &AIXs, unsigned GenomeIndex,
  unsigned ChrId, std::vector<AIBP> &BPs, unsigned Margin, bool Subscript);
void MakeEdges(const AIXVec &AIXs, ptrLinkFunction Link, unsigned GenomeIndex,
  unsigned MaxDist, IntVecVec &Edges, bool ShowProgress);
void GetConnectedComponents(const IntVecVec &AdjMx_, IntVecVec &CCs,
  bool ShowProgress);
void GetConnectedComponents(const IntVecVec &AdjMx_, BoolVecVec &SignsMx_,
  IntVecVec &CCs, BoolVec &NodeSigns, bool ShowProgress);
void StackStats();
void DumpStackStats(const RevFile &Rev, unsigned GenIndex, unsigned ChrId,
  const AIXVec &AXIs);
void DumpStackStatsTotal();
void TruncLabel(std::string &Label);
void StackMulti();
void StackCC(const RevFile &In, const AIXVec &AIXs,
  RevFile &Out);
void MakeOverlapGraph(const AIXVec &AIXs, unsigned GenIndex, unsigned ChrId,
  IntVecVec &Edges);
void MakeSegOverlapGraph(const GCSegVec &Segs, IntVecVec &Edges);
void LogAIRows(const AlnInfo &AI, const StdStrVec &Rows, const RevFile *Rev = 0);
void MakeColOffsetVecs(const StdStr &Row, IntVec &ColToOffset,
  IntVec &OffsetToCol, BoolVec &ColHasGap, bool Plus);
void StripGaps(const std::string &In, std::string &Out);
void ToLower(const std::string &In, std::string &Out);
void RevComp(const std::string &Seq, std::string &RevCompSeq);
void RevComp(std::string &Seq);
bool SeqIsAllGaps(const std::string &Seq);
void LogAIBPs(const AIBPVec &BPs);
void LogIntW(unsigned Value, unsigned SpacesBefore, unsigned Width);
void BaseStack(const RevFile &In, const AIXVec &AIXs, unsigned CCIndex,
  RevFile &Out);
char CompLetter(char c);
bool ColLT(const std::vector<BaseWithSign> &ColA,
  const std::vector<BaseWithSign> &ColB);
bool ColPtrLT(const std::vector<BaseWithSign> *ColA,
  const std::vector<BaseWithSign> *ColB);
void AssertColSorted(const std::vector<BaseWithSign> &Col);
void SetSortGenIndex(unsigned GenIndex);
bool ColLTGen(const std::vector<BaseWithSign> *ColA,
  const std::vector<BaseWithSign> *ColB);
unsigned GetOverlappingSegIndex(const AlnInfo &AI, unsigned GenomeIndex,
  unsigned ChrId, coord_t lo, coord_t hi);
void OutputBlockTop(const RevFile &In, unsigned BPIndex, const AIVec &AIs,
  const AIBPVec &BPs, coord_t lo, coord_t hi, RevFile &Out);
void StackTop();
void StackSplit();
void ValidateSegLength(const GCSeg &Seg, const StdStr &Row);
unsigned GetRowLength(const std::string &Row);
unsigned GetUngappedSeqLength(const std::string &Seq);

extern unsigned g_GenIndex;
extern unsigned g_MaxCCSize;
extern unsigned g_MaxAlnCount;
extern unsigned g_MaxColSize;
extern unsigned g_MinBlockCols;
extern unsigned g_MaxBlockRows;
extern unsigned g_InconsistentRatio;
extern RevFile *g_ptrDel;
extern bool g_LogInconsistentAlns;

#endif // STACK_H
