#ifndef STACKER_H
#define STACKER_H

char CompLetter(char c);

const unsigned SANE_BASE_COUNT = 100000;

enum CONSEC
	{
	CONSEC_ILLEGAL_ENUM_VALUE = 0,
	CONSEC_VALID,
	CONSEC_VALID_INVERT,
	CONSEC_INVALID,
	CONSEC_NEW_CHR,
	};

#ifdef _MSC_VER
extern int _cdecl ColQsortCmp(const void *, const void *);
#else
extern int ColQsortCmp(const void *, const void *);
#endif

struct GenChrPos
	{
	unsigned GenIndex;
	unsigned ChrId;
	coord_t Pos;

	bool operator<(const GenChrPos &rhs) const
		{
		if (GenIndex != rhs.GenIndex)
			return GenIndex < rhs.GenIndex;
		if (ChrId != rhs.ChrId)
			return ChrId < rhs.ChrId;
		return Pos < rhs.Pos;
		}
	};

struct BaseInfo
	{
	unsigned GenIndex;
	unsigned ChrId;
	coord_t Pos;
	char Letter;
	double ConfScore;

	void LogMe() const
		{
		Log(" Gen%u", GenIndex);
		Log(" Chr%u", ChrId);
		Log(" Pos%u", Pos);
		if (isprint(Letter))
			Log("  %c", Letter);
		else
			Log(" <UNPRINTABLE 0x%02x>", Letter);
		}

	bool operator<(const BaseInfo &rhs) const
		{
		if (GenIndex != rhs.GenIndex)
			return GenIndex < rhs.GenIndex;
		if (ChrId != rhs.ChrId)
			return ChrId < rhs.ChrId;
		if (Pos != rhs.Pos)
			return Pos < rhs.Pos;
		asserta(toupper(Letter) == toupper(rhs.Letter));
		return Letter < rhs.Letter;
		}

	bool operator>(const BaseInfo &rhs) const
		{
		if (GenIndex != rhs.GenIndex)
			return GenIndex > rhs.GenIndex;
		if (ChrId != rhs.ChrId)
			return ChrId > rhs.ChrId;
		if (Pos != rhs.Pos)
			return Pos > rhs.Pos;
		asserta(toupper(Letter) == toupper(rhs.Letter));
		return Letter > rhs.Letter;
		}

	bool operator==(const BaseInfo &rhs) const
		{
		return Pos == rhs.Pos && GenIndex == rhs.GenIndex &&
		  ChrId == rhs.ChrId && Letter == rhs.Letter;
		}

	bool operator!=(const BaseInfo &rhs) const
		{
		return !(*this == rhs);
		}
	};

struct BaseWithSign
	{
	BaseWithSign()
		{
		}
	BaseWithSign(unsigned GenIndex, unsigned ChrId,	coord_t Pos,
	  char Letter, double ConfScore, bool Sign)
		{
		Base.GenIndex = GenIndex;
		Base.ChrId = ChrId;
		Base.Pos = Pos;
		Base.Letter = Letter;
		Base.ConfScore = ConfScore;
		Plus = Sign;
		}
	BaseInfo Base;
	bool Plus;
	
	char GetLetter() const
		{
		return Plus ? Base.Letter : CompLetter(Base.Letter);
		}

	bool operator<(const BaseWithSign &rhs) const
		{
		if (Base != rhs.Base)
			return Base < rhs.Base;
		return Plus < rhs.Plus;
		}

	bool operator==(const BaseWithSign &rhs) const
		{
		return Base == rhs.Base && Plus == rhs.Plus;
		}

	bool operator!=(const BaseWithSign &rhs) const
		{
		return !(*this == rhs);
		}

	bool operator<=(const BaseWithSign &rhs) const
		{
		return *this < rhs || *this == rhs;
		}
	bool Adjacent(const BaseWithSign &b) const
		{
		if (b.Base.GenIndex != Base.GenIndex)
			return false;
		if (b.Base.ChrId != Base.ChrId)
			return false;
		coord_t Pos = Base.Pos;
		coord_t bPos = b.Base.Pos;
		if (Pos + 1 == bPos || bPos + 1 == Pos)
			return true;
		return false;
		}
	};

typedef std::vector<BaseWithSign> OutputCol_t;
typedef OutputCol_t *PtrToOutputCol;

class Stacker
	{
private:
// Disable default c'tor
	Stacker();

public:
	Stacker(const RevFile &In, const AIXVec &AIXs, unsigned CCIndex,
	  RevFile &Out) :
		ptrIn(&In), ptrAIXs(&AIXs), InputCCIndex(CCIndex), ptrOut(&Out)
		{
		Clear();
		Stack();
		}
private:
	void Clear();
	void Stack();

	unsigned AddBase(const BaseInfo &Base);
	void BuildBaseGraph();
	void AddToBaseGraph(const RevFile &In, const AlnInfoX &AIX);

	void BuildOutputColBases(const IntVecVec &CCs, const BoolVec &Signs);
	void CCToOutputCol(const IntVec &BaseIndexes, const BoolVec &Signs,
	  std::vector<BaseWithSign> &OutputCol);

	void BuildAndWriteOutputBlocks();
	void WriteOutputBlock();
	void BuildOutputRows(StdStrVec &Rows) const;
	void InitOutputBlock(const std::vector<BaseWithSign> &Col);
	void AddOutputCol(unsigned ColIndex, bool Invert, bool Next);
	void OutputInconsistentCol(const OutputCol_t &Col) const;

	CONSEC ValidConsecutiveSeg(const GCSeg &Seg, const BaseWithSign &BWS,
	  bool Next);
	void DeleteOutputCol(unsigned OutputColIndex);
	bool ValidConsecutive(const GCSegVec &Segs,
	  const std::vector<BaseWithSign> &Bases, bool Next, bool &Invert);
	bool ColIsInconsistent(const std::vector<BaseWithSign> &Col) const;

	unsigned GetAdjacentSegIndex(const BaseWithSign &BWS, const GCSegVec &Segs,
	  CONSEC &Consec, bool Next);
	unsigned GetAdjacentBaseIndex(const GCSeg &Seg, const std::vector<BaseWithSign> &Bases,
	  CONSEC &Consec, bool Next);
	unsigned GetAdjacentOutputCol(bool Next, bool &Invert);

	const BaseInfo &GetBase(unsigned BaseIndex) const;

	bool ValidateNoOutputOverlaps();
	bool ValidateNoOutputOverlapsGenome(unsigned GenIndex);

	void ComputeInputSegCount();

	void LogBaseInfo(const BaseInfo &Base) const;
	void LogBase(unsigned BaseIndex) const;
	void LogBWS(const BaseWithSign &BWS) const;
	void LogGraph() const;
	void LogCCs(const IntVecVec &CCs, const BoolVec &Signs) const;
	void LogOutputColBases() const;
	void LogCol(const std::vector<BaseWithSign> &Col) const;
	void LogOutputBlock() const;

private:
// Input
	const RevFile *ptrIn;
	const AIXVec *ptrAIXs;
	unsigned InputCCIndex;
	RevFile *ptrOut;

private:
// Bases[i] stores information about the base with BaseIndex i.
	std::vector<BaseInfo> Bases;

// BaseInfoToIndex is an index which returns the BaseIndex
// given a BaseInfo structure.
	std::map<BaseInfo, unsigned> BaseInfoToIndex;

// Edges in the base graph, represented as a sparse matrix.
// For the base with base index i, Edges[i] is a vector of
// integers. Each integer j in that vector indicates that
// there is an edge i-j.
	IntVecVec Edges;

// EdgePluses is a vector of BoolVecs, one for each base.
// EdgePluses[i][j] indicates whether the edge i-j has
// a plus or minus sign. True indicates the bases are
// aligned on the same strand, false on oppostite strands.
	std::vector<BoolVec> EdgePluses;

// OutputColBases[i] is the vector of BaseWithSign's for
// the output column with index i.
	std::vector<std::vector<BaseWithSign> > OutputColBases;

// GCPToColIndex[GCP] is the output column index, i.e.
// the index into OutputColBases, of the output column
// containing the base with the given coordinates.
// Note that there can only be one such column because
// the column was constructed as a connected component.
	std::map<GenChrPos, unsigned> GCPToColIndex;

// OutputAI is the AlnInfo for the growing output block.
	AlnInfo OutputAI;

// Columns in the growing output alignment.
// Each column is a string of letters.
// All columns have the same number of letters.
// Stored as a list<> rather than the usual vector<> to
// allow O(1) insertion at both the beginning and end
// of the alignment.
	std::list<StdStr> OutputCols;

// Accumulated sum of confidence scores of all bases
// in the growing output alignment. Used to compute
// the confidence score of the block when completed.
	double OutputConfScore;

// Vector of AIs for all output alignments.
// Used only to validate that there are no overlaps.
	AIVec OutputAIs;

// Total number of segments in input AIs.
// Used for reporting only.
	unsigned InputSegCount;

public:
// Totals for reporting / diagnostics only.
	unsigned OutputBlockCount;
	unsigned DeletedBigColCount;
	unsigned ShortColCount;
	unsigned InconsistentColCount;
	};

#endif // STACKER_H
