#include "stack.h"

// Convert one connected component of the base graph into a vector
// of bases which will become a column in an output block.
void Stacker::CCToOutputCol(const IntVec &BaseIndexes, const BoolVec &Signs,
  std::vector<BaseWithSign> &OutputCol)
	{
	OutputCol.clear();
	const unsigned BaseCount = SIZE(BaseIndexes);
	for (unsigned i = 0; i < BaseCount; ++i)
		{
		unsigned BaseIndex = BaseIndexes[i];
		bool Plus = Signs[BaseIndex];
		const BaseInfo &Base = GetBase(BaseIndex);
		BaseWithSign BWS;
		BWS.Base = Base;
		BWS.Plus = Plus;
		OutputCol.push_back(BWS);
		}
	std::sort(OutputCol.begin(), OutputCol.end());
	}

// Convert connected components of the base graph into vectors of bases.
// Each vector will become one column in an output block.
void Stacker::BuildOutputColBases(const IntVecVec &CCs, const BoolVec &Signs)
	{
	const unsigned CCCount = SIZE(CCs);
	OutputColBases.reserve(CCCount);

	unsigned OutputColIndex = 0;
	for (unsigned CCIndex = 0; CCIndex < CCCount; ++CCIndex)
		{
		const IntVec &CC = CCs[CCIndex];
		unsigned BaseCount = SIZE(CC);

		asserta(InputSegCount > 0);

		if (BaseCount > g_MaxBlockRows && g_MaxBlockRows != 0)
			{
			++DeletedBigColCount;
			continue;
			}

		std::vector<BaseWithSign> OutputCol;
		CCToOutputCol(CC, Signs, OutputCol);
		if (ColIsInconsistent(OutputCol))
			{
			ProgressNoRes("CC %6u  Col %6u  Alns %6u  Segs %6u  InconBases %6u",
			  InputCCIndex,
			  CCIndex,
			  SIZE(*ptrAIXs),
			  InputSegCount,
			  BaseCount);
			OutputInconsistentCol(OutputCol);
			++InconsistentColCount;
			continue;
			}

		const unsigned BaseCount2 = SIZE(OutputCol);
		asserta(BaseCount2 == BaseCount);
		asserta(BaseCount > 0 && BaseCount < SANE_BASE_COUNT);

		OutputColBases.push_back(OutputCol);

		for (unsigned BaseIndex = 0; BaseIndex < BaseCount; ++BaseIndex)
			{
			const BaseWithSign &BWS = OutputCol[BaseIndex];

			GenChrPos GCP;
			GCP.GenIndex = BWS.Base.GenIndex;
			GCP.ChrId = BWS.Base.ChrId;
			GCP.Pos = BWS.Base.Pos;

			GCPToColIndex[GCP] = OutputColIndex;
			}
		++OutputColIndex;
		}
	}

// Col MUST be sorted!
bool Stacker::ColIsInconsistent(const std::vector<BaseWithSign> &Col) const
	{
	const unsigned BaseCount = SIZE(Col);

// Note loop starts from 1 not 0
	for (unsigned BaseIndex = 1; BaseIndex < BaseCount; ++BaseIndex)
		{
		const BaseWithSign &PrevBWS = Col[BaseIndex-1];
		const BaseWithSign &BWS = Col[BaseIndex];
		assert(PrevBWS < BWS);
		if (BWS.Adjacent(PrevBWS))
			return true;
		}
	return false;
	}
