#include "mus.h"

#if	0
static vector<double> *g_ptrMinQs;
static vector<double> *g_ptrMaxQs;
static bool LTdQ(uns i, uns j)
	{
	double MinQi = (*g_ptrMinQs)[i];
	double MinQj = (*g_ptrMinQs)[j];

	double MaxQi = (*g_ptrMaxQs)[i];
	double MaxQj = (*g_ptrMaxQs)[j];

	double di = MaxQi - MinQi;
	double dj = MaxQj - MinQj;

	if (di != dj)
		return di > dj;
	return MaxQi < MaxQj;
	}

void SuperTrain()
	{
	uns SubsetSize = 100;
	uns SubsetSize2 = 500;
	uns InitIters = 10;
	uns InitIters2 = 10;
	uns LearnIters = 50;
	uns LearnIters2 = 50;
	double LearnRate = 0.001;
	double LearnRate2 = 0.001;
	uns k = 8;
	uns k2 = 8;
	double QPct = 2;
	double QPct2 = 2;

	const char *InFileName = RequiredValueOpt("strain");

	IntOpt("inititers", &InitIters);
	IntOpt("inititers2", &InitIters2);
	IntOpt("learniters", &LearnIters);
	IntOpt("learniters2", &LearnIters2);
	FloatOpt("rate", &LearnRate);
	FloatOpt("rate2", &LearnRate2);

	const char *Style = ValueOpt("dwstyle");
	bool DoTrace = FlagOpt("trace");
	IntOpt("subset", &SubsetSize);
	IntOpt("subset2", &SubsetSize2);
	IntOpt("k", &k);
	FloatOpt("qpct", &QPct);
	bool SaveSubsets = FlagOpt("savesubsets");
	unsigned Seed = UINT_MAX;
	IntOpt("seed", &Seed);

	if (Seed != UINT_MAX)
		InitRand(Seed);

	DeltaWeightStyle dwStyle = dWAlternate;
	if (Style != 0)
		{
		if (streq(Style, "aln"))
			dwStyle = dWPerAln;
		else if (streq(Style, "iter"))
			dwStyle = dWPerIter;
		else if (streq(Style, "alt"))
			dwStyle = dWAlternate;
		else
			Die("Invalid -dwstyle %s", Style);
		}

	SetLogProgress(false);

	GetSubstMxFromCmdLine();

	MSASet M;
	M.FromFile(InFileName);
	uns MSACount = M.GetCount();
	if (MSACount == 0)
		Die("Empty training set");

// Construct random triples (MSAIndex, SeqIndex1, SeqIndex2)
	vector<uns> RandomMSAIndexes;
	for (uns i = 0; i < MSACount; ++i)
		RandomMSAIndexes.push_back(i);
	RandomizeVec(RandomMSAIndexes);

	if (SubsetSize < MSACount)
		RandomMSAIndexes.resize(SubsetSize);
	else
		SubsetSize = MSACount;

	vector<pair<uns, uns> > RandomSeqIndexes;
	for (uns i = 0; i < SubsetSize; ++i)
		{
		uns MSAIndex = RandomMSAIndexes[i];
		uns k = M.GetSeqCount(MSAIndex);
		if (k == 1)
			Die("Alignment has 1 seq");
		uns SeqIndex1 = RandInt32()%k;
		uns SeqIndex2 = RandInt32()%(k-1);
		if (SeqIndex2 >= SeqIndex1)
			++SeqIndex2;
		RandomSeqIndexes.push_back(pair<uns, uns>(SeqIndex1, SeqIndex2));
		}

// Get pairwise subset defined by random triples
	PWSet RandomSubset;
	M.GetPWSet(RandomMSAIndexes, RandomSeqIndexes, RandomSubset);

	if (SaveSubsets)
		RandomSubset.ToFile("subset.fa");

	ParamVec BestParams;
	RandomSubset.Train(InitIters, LearnIters, DoTrace, LearnRate, dwStyle, BestParams);
	RandomSubset.ParamSpaceToFile("randomps.txt");//@@

// Get diverse k-best parameters based on this subset
	vector<ParamVec> Ws;
	vector<double> Qs;
	RandomSubset.ParamSpaceToVectors(Ws, Qs);

	vector<uns> Indexes;
	GetDiverseBest(Ws, Qs, k, QPct, Indexes);

// Get paramter sets
	vector<ParamVec> DivWs;
	vector<double> DivQs;
	const uns DivN = SIZE(Indexes);
	for (uns i = 0; i < DivN; ++i)
		{
		uns j = Indexes[i];
		DivWs.push_back(Ws[j]);
		DivQs.push_back(Qs[j]);
		}

	Log("\n");
	Log("Diverse params from subset:\n");
	LogWQs(DivWs, DivQs);

	PWSet AllPairs;
	M.GetAllPairs(AllPairs);
	const uns AllPairCount = AllPairs.GetPairCount();
	vector<double> MinQs(AllPairCount, -1);
	vector<double> MaxQs(AllPairCount, -1);
	for (uns i = 0; i < DivN; ++i)
		{
		const ParamVec &W = DivWs[i];
		vector<double> Qs;
		string Tmp;
		Progress("All Qs %u of %u %s", i+1, DivN, W.ToStr(Tmp).c_str());
		double Q = AllPairs.ComputeQs(W, Qs);
		Log("Q=%5.3f All pairs, %s\n", Q, W.ToStr(Tmp).c_str());
		asserta(SIZE(Qs) == AllPairCount);
		for (uns j = 0; j < AllPairCount; ++j)
			{
			double Q = Qs[j];
			if (i == 0)
				{
				MinQs[j] = Q;
				MaxQs[j] = Q;
				}
			else
				{
				MinQs[j] = min(Q, MinQs[j]);
				MaxQs[j] = max(Q, MaxQs[j]);
				}
			}
		}

// Sort in decreasing order of MaxQ-MinQ, i.e.
// decreasing order of sensitivity
	vector<uns> SortOrder;
	for (uns i = 0; i < AllPairCount; ++i)
		SortOrder.push_back(i);

	g_ptrMinQs = &MinQs;
	g_ptrMaxQs = &MaxQs;
	sort(SortOrder.begin(), SortOrder.end(), LTdQ);

// Construct maximum sensitivity training set
	vector<bool> Selected(AllPairCount, false);
	vector<unsigned> SeqsFromMSACount(MSACount, 0);
	unsigned SelectedCount = 0;
	for (uns Try = 0; ; ++Try)
		{
		if (SelectedCount == AllPairCount)
			break;
		for (uns i = 0; i < AllPairCount; ++i)
			{
			uns PairIndex = SortOrder[i];
			if (Selected[PairIndex])
				continue;

			uns MSAIndex;
			uns SeqIndex1;
			uns SeqIndex2;
			AllPairs.GetMSACoords(PairIndex, MSAIndex, SeqIndex1, SeqIndex2);
			asserta(MSAIndex < SIZE(SeqsFromMSACount));
			uns N = SeqsFromMSACount[MSAIndex];
			if (N > Try)
				continue;

			Selected[PairIndex] = true;
			++(SeqsFromMSACount[MSAIndex]);
			++SelectedCount;
			if (SelectedCount == SubsetSize2)
				goto Done;
			}
		}

Done:
	vector<uns> MaxSensitivityPairIndexes;
	for (uns PairIndex = 0; PairIndex < AllPairCount; ++PairIndex)
		if (Selected[PairIndex])
			MaxSensitivityPairIndexes.push_back(PairIndex);
	sort(MaxSensitivityPairIndexes.begin(), MaxSensitivityPairIndexes.end(), LTdQ);
	asserta(SIZE(MaxSensitivityPairIndexes) == SelectedCount);

	Log("\n");
	Log("Max sensivity subset (%u):\n", SelectedCount);
	Log("            Name    MSA    Seq1  Seq2  Weight   MinQ   MaxQ     dQ\n");
	Log("================  =====  =====  =====  ======  =====  =====  =====\n");
	double SumMinQ = 0;
	double SumMaxQ = 0;
	for (uns i = 0; i < SelectedCount; ++i)
		{
		uns PairIndex = MaxSensitivityPairIndexes[i];
		const string &Name = AllPairs.GetPairName(PairIndex);
		double Weight = AllPairs.GetWeight(PairIndex);
		double MinQ = MinQs[PairIndex];
		double MaxQ = MaxQs[PairIndex];
		SumMinQ += MinQ;
		SumMaxQ += MaxQ;
		double dQ = MaxQ - MinQ;
		uns MSAIndex;
		uns SeqIndex1;
		uns SeqIndex2;
		AllPairs.GetMSACoords(PairIndex, MSAIndex, SeqIndex1, SeqIndex2);
		Log("%16.16s  %5u  %5u  %5u  %6.3f  %5.3f  %5.3f  %5.3f\n",
		  Name.c_str(), MSAIndex, SeqIndex1, SeqIndex2, Weight, MinQ, MaxQ, dQ);
		}
	Log("\n");
	Log("AvgMinQ=%5.3f\n", SumMinQ/SelectedCount);
	Log("AvgMaxQ=%5.3f\n", SumMaxQ/SelectedCount);

	vector<uns> MS_MSAIndexes;
	vector<pair<uns, uns> > MS_MSASeqIndexes;
	for (uns i = 0; i < SelectedCount; ++i)
		{
		uns PairIndex = MaxSensitivityPairIndexes[i];

		uns MSAIndex;
		uns SeqIndex1;
		uns SeqIndex2;
		AllPairs.GetMSACoords(PairIndex, MSAIndex, SeqIndex1, SeqIndex2);
		MS_MSAIndexes.push_back(MSAIndex);
		MS_MSASeqIndexes.push_back(pair<uns, uns>(SeqIndex1, SeqIndex2));
		}

// Train on max sensitivity subset
	PWSet MaxSensitivitySubset;
	M.GetPWSet(MS_MSAIndexes, MS_MSASeqIndexes, MaxSensitivitySubset);
	MaxSensitivitySubset.Train(InitIters2, LearnIters2, DoTrace, LearnRate2,
	  dwStyle, BestParams);
	MaxSensitivitySubset.ParamSpaceToFile("msps.txt");//@@

// Get diverse k-best parameters based on MS subset
	vector<ParamVec> MS_Ws;
	vector<double> MS_Qs;
	MaxSensitivitySubset.ParamSpaceToVectors(MS_Ws, MS_Qs);

	vector<uns> MS_Indexes;
	GetDiverseBest(MS_Ws, MS_Qs, k2, QPct2, MS_Indexes);
	Log("\n");
	Log("k2=%u QPct2=%.1f, %u diverse params\n",
	  k2, QPct2, SIZE(MS_Indexes));

// Get paramter sets
	vector<ParamVec> MS_DivWs;
	vector<double> MS_DivQs;
	const uns MS_DivN = SIZE(MS_Indexes);
	for (uns i = 0; i < MS_DivN; ++i)
		{
		uns j = MS_Indexes[i];
		MS_DivWs.push_back(MS_Ws[j]);
		MS_DivQs.push_back(MS_Qs[j]);
		}

	Log("\n");
	Log("Diverse params from max-sensitivity subset:\n");
	LogWQs(MS_DivWs, MS_DivQs);

	for (uns i = 0; i < MS_DivN; ++i)
		{
		Progress("All pairs %u of %u", i+1, MS_DivN);
		const ParamVec &W = MS_DivWs[i];
		double Q = AllPairs.ComputeQ(W);
		string Tmp;
		Log("Q=%5.3f  %s\n", Q, W.ToStr(Tmp).c_str());
		}

// Hill climb from each starting point
	for (uns i = 0; i < MS_DivN; ++i)
		{
		const ParamVec &W = MS_DivWs[i];
		ParamVec G;
		G.Zero(0);
		AllPairs.HillClimb(W, G);
		}
	AllPairs.ParamSpaceToFile("allpairs_ps.txt");
	}
#endif // 0
