#define TRACE	0

#include "evo.h"

// TODO: merge exon fragments that span MUL boundaries?

void UnwindAnnots()
	{
	asserta(sizeof(coord_t) == sizeof(gpos_t));

// This is the evolved (not unwound) evo file.
	const char *EvoFileName = RequiredValueOpt("evo");
	const char *InGFFFileName = RequiredValueOpt("ingff");
	const char *OutGFFFileName = RequiredValueOpt("outgff");

	FILE *fEvo = OpenStdioFile(EvoFileName);
	FILE *fInGFF = OpenStdioFile(InGFFFileName);
	FILE *fOutGFF = CreateStdioFile(OutGFFFileName);

	EvoFileHeader Hdr;
	vector<MULIndexEntryData> IEs;
	GenInfo Genome;
	ReadEvoIndex(fEvo, Hdr, IEs, Genome);

	map<string, gpos_t> ChrNameToStartPos;
	map<gpos_t, string> ChrStartPosToName;
	MakeChrPosMap(Genome, ChrNameToStartPos, ChrStartPosToName);
	LogChrPosMap(ChrNameToStartPos, ChrStartPosToName);

	unsigned LeafMULCount = Hdr.LeafMULCount;
	unsigned TotalMULCount = Hdr.TotalMULCount;
	vector<gpos_t> MULEvPos;
	MULEvPos.resize(TotalMULCount, GPOS_T_MAX);
	unsigned MULIndex = Hdr.FirstMULIndex;
	unsigned SafetyCounter = 0;
	gpos_t EvPos = 0;
	for (;;)
		{
		if (MULIndex == UINT_MAX)
			break;
		asserta(MULIndex < TotalMULCount);

		++SafetyCounter;
		if (SafetyCounter > LeafMULCount)
			Die("Loop in MUL index");

		double Pct = SafetyCounter*100.0/Hdr.LeafMULCount;
		Progress("MUL %u of %u EvPos %"PF" %.1f%%",
		  SafetyCounter,
		  Hdr.LeafMULCount,
		  EvPos,
		  Pct);

		MULIndexEntryData IE;
		ReadIE(fEvo, MULIndex, IE, true);

		MULEvPos[MULIndex] = EvPos;
		EvPos += IE.MI.EvolvedLength;
		MULIndex = IE.Next;
		}

	GFFRecord Rec;
	unsigned FileSize = (unsigned) GetStdioFileSize(fInGFF);
	unsigned RecordCount = 0;
	ProgressStart("Processing");
	while (GetNextGFFRecord(fInGFF, Rec))
		{
TRC
		Rec.LogMe();
		Log("\n");
ENDTRC
		if ((RecordCount++)%1000 == 0)
			{
			unsigned Pos = (unsigned) GetStdioFilePos(fInGFF);
			ProgressStep(Pos, FileSize);
			}
		const char *Label = Rec.SeqName;
		unsigned MULIndex = UINT_MAX;
		int n = sscanf(Label, "MUL%u", &MULIndex);
		if (n != 1)
			Die("Expected MUL<n>, got %s", Label);
		if (MULIndex >= TotalMULCount)
			Die("MUL%u out of range, total=%u", MULIndex, TotalMULCount);

		gpos_t MULPos = MULEvPos[MULIndex];

	// Leaf?
		if (MULPos == GPOS_T_MAX)
			continue;

		string ChrNameStart;
		string ChrNameEnd;
		gpos_t StartPos = GPOS_T_MAX;
		gpos_t EndPos = GPOS_T_MAX;
		gpos_t MULStartPos = MULPos + Rec.Start - 1;
		gpos_t MULEndPos = MULPos + Rec.End - 1;

		PosToChrPos(MULStartPos, ChrStartPosToName, ChrNameStart, StartPos);
		PosToChrPos(MULEndPos, ChrStartPosToName, ChrNameEnd, EndPos);
TRC
		Log("MULStartPos=%"PF" Name=%s Pos=%"PF"\n",
		  MULStartPos, ChrNameStart.c_str(), StartPos);
		Log("MULEndPos=%"PF" Name=%s Pos=%"PF"\n",
		  MULEndPos, ChrNameEnd.c_str(), EndPos);
ENDTRC
		asserta(ChrNameStart == ChrNameEnd);

		Rec.SeqName = ChrNameStart.c_str();
		Rec.Start = StartPos + 1;
		Rec.End = EndPos + 1;

		WriteGFFRec(fOutGFF, Rec);
		}
	ProgressDone();

	CloseStdioFile(fEvo);
	CloseStdioFile(fInGFF);
	CloseStdioFile(fOutGFF);
	}
