#include "evolver.h"
#include "evomodel.h"
#include "myrand.h"

#define TRACE	0

double GetAASubstProb(unsigned AminoIndexBefore, unsigned AminoIndexAfter);

static double GetCodonSubstAcceptProb(double AcceptProb,
  unsigned AminoIndexBefore, unsigned AminoIndexAfter)
	{
// Silent substitution
	if (AminoIndexBefore == AminoIndexAfter)
		return 1.0;

// Cannot mutate to or from STOP
	if (AminoIndexBefore == CODON_STP || AminoIndexAfter == CODON_STP)
		return 0.0;

// Otherwise, product of accept prob and a.a. subst prob.
	double pAA = GetAASubstProb(AminoIndexBefore, AminoIndexAfter);
	return AcceptProb*pAA*P_AminoAcidProbMultiplier;
	}

bool Evolver::AcceptCodonSubst(double AcceptProb, const string &CodonBefore,
  const string &CodonAfter) const
	{
	if (AcceptProb == 0)
		return false;

	asserta(CodonBefore.size() == 3);
	asserta(CodonAfter.size() == 3);

	unsigned AminoBefore = CharsToAminoIndex(CodonBefore[0], CodonBefore[1],
	  CodonBefore[2]);

	unsigned AminoAfter = CharsToAminoIndex(CodonAfter[0], CodonAfter[1],
	  CodonAfter[2]);

	double Prob = GetCodonSubstAcceptProb(AcceptProb, AminoBefore, AminoAfter);
	bool Accept = RandTrue(Prob);
#if	TRACE
	Log("AcceptCodonSubstitution(%s, %s) %c -> %c Prob=%g Accept=%c\n",
	  CodonBefore.c_str(),
	  CodonAfter.c_str(),
	  AminoBefore,
	  AminoAfter,
	  Prob,
	  tof(Accept));
#endif
	return Accept;
	}
