#define TRACE	0

#include "evolver.h"
#include "evomodel.h"
#include "myrand.h"

char CompLetter(char c);
//bool Is4D(char a1, char a2, char b1, char b2);
//void LogRandState();

void Evolver::GetExonRange(gpos_t Pos, gpos_t &ExonStart,
  gpos_t &ExonLength) const
	{
TRC
	Log("GetExonRange(Pos=%"PF")\n", Pos);
ENDTRC
// Rewind to start of exon
	ExonStart = Pos;
	ExonLength = 1;
	for (;;)
		{
		gpos_t PrevPos = GetPrevPos(ExonStart);
		if (PrevPos == GPOS_T_MAX)
			break;
		const BaseInfo &Base = GetBase(PrevPos);
TRC
		Log("Pos %"PF" IsCoding=%c\n", PrevPos, tof(Base.IsCoding()));
ENDTRC
		if (!Base.IsCoding())
			break;

		ExonStart = PrevPos;
		++ExonLength;
		}

// Forward to end of exon
	gpos_t End = Pos;
	for (;;)
		{
		gpos_t NextPos = GetNextPos(End);
		if (NextPos == GPOS_T_MAX)
			break;
		const BaseInfo &Base = GetBase(NextPos);
TRC
		Log("Pos %"PF" IsCoding=%c\n", NextPos, tof(Base.IsCoding()));
ENDTRC
		if (!Base.IsCoding())
			break;

		End = NextPos;
		++ExonLength;
		}
	}

bool Evolver::AcceptExonSubst(gpos_t SubstPos, char NewLetter) const
	{
TRC
	Log("AcceptExonSubst(Pos=%"PF" NewLetter=%c\n", SubstPos, NewLetter);
ENDTRC
	
	CodonInfo Codon;
	bool Ok = GetCodon(SubstPos, Codon);
	if (!Ok)
		Die("AcceptExonSubst: GetCodon failed");

	const BaseInfo &Base = GetBase(SubstPos);
	string NewCodon(Codon.Letters);
	for (int i = 0; ; ++i)
		{
		if (i == 3)
			Die("SubstPos not found in CodonInfo");
		if (Codon.Pos[i] == SubstPos)
			{
			NewCodon[i] = (Codon.Strand == Base.Plus) ? NewLetter : CompLetter(NewLetter);
			break;
			}
		}
	asserta(NewCodon != Codon.Letters);

	double AcceptProb = Base.GetAcceptProb();
	bool Accept = AcceptCodonSubst(AcceptProb, Codon.Letters, NewCodon);

TRC
	Log("AcceptCodonSubst(CC=%u, OldCodon=%s, NewCodon=%s) = %c\n",
	  Base.ProbIx, Codon.Letters.c_str(), NewCodon.c_str(), tof(Accept));
ENDTRC
	if (!Accept)
		return false;

	if (PosInStartCodon(SubstPos))
		return false;

	extern FILE *g_fCodonSubs;
	if (g_fCodonSubs != 0)
		fprintf(g_fCodonSubs, "%s %s\n", Codon.Letters.c_str(), NewCodon.c_str());

	return true;
	}
