#ifndef base_h
#define base_h

#define	BITFIELDS	1

char CompLetter(char c);

enum BIT_LETTER
	{
	BIT_LETTER_A,
	BIT_LETTER_C,
	BIT_LETTER_G,
	BIT_LETTER_T,
	BIT_LETTER_GAP,
	BIT_LETTER_INVALID,
	};

enum BaseCategory
	{
	BC_Ancestral,
	BC_Copy,
	BC_Novel,
	};
const unsigned BaseCategoryCount = 3;

enum CE_TYPE
	{
#define C(c)	CE_##c,
#include "cetypes.h"
	};

const unsigned CETypeCount = 0
#define C(c)	+1
#include "cetypes.h"
	;

enum BaseType
	{
	BT_NonGene,
	BT_UntranscribedGene,
	BT_UTR,
	BT_CDS,
	BT_START,
	BT_STOP,
	BT_Splice,
	BT_Intron,
	};
const char *BaseTypeToStr(BaseType Type);

enum BaseType2
	{
	BT2_NGE,
	BT2_NXE,
	BT2_NGNeutral,
	BT2_UGNeutral,
	BT2_UTR,
	BT2_CDSPlus,
	BT2_CDSMinus,
	BT2_START,
	BT2_STOP,
	BT2_Splice,
	BT2_IntronNeutral,
	BT2_IntronCE,
	};
const char *BaseType2ToStr(BaseType2 Type);

byte CharToBitLetter(char c);
char BitLetterToChar(byte BitLetter);

#if	BITFIELDS
const int NovelLetter = BIT_LETTER_GAP;
const int DeletedLetter = BIT_LETTER_GAP;
#else
const char NovelLetter = '-';
const char DeletedLetter = '-';
#endif

const byte NEUTRAL_PROBIX = 0xff;
const byte MAX_NON_NEUTRAL_PROBIX = 0xfe;
const double MIN_ACCEPT_PROB = 1.0/255.0;

#ifndef __APPLE_CC__
#pragma pack(push,1)
#endif

/***
Evolved letter. Changes due to substitutions but NOT inversions.
Use GetOrientedLetter() to get complement iff Plus=false.
If base has been deleted, is '-'.
char Letter;
char AncLetter;

Plus is false if the base has been inverted relative to its
homologous base in the ancestral genome, true if it remains on
the same strand. For newly inserted bases, Plus is initially
true, but may change due to inversions.
bool Plus;

True if in CpG island, false otherwise.
bool Island;

Index into accept probability lookup table.
Zero means zero probability.
byte ProbIx;

uint8 Type; 

For exons only, Codex specifies strand for translation
and position within codon. Sign is strand, absolute value
is position within codon, 1 meaning first base within
the codon. Zero means untranslated, SCHAR_MAX means
not an exon. (The whimsical  name "Codex" is chosen to
sound like "codon index" but cannot be the same since
CodonIndex, per our naming convention, would be the index
of a codon within an exon).
	int8 Codex;
***/

struct BaseInfo
	{
#if	BITFIELDS
private:
	byte Letter : 3;
	byte AncLetter : 3;
public:
	bool Plus : 1;
	bool Island : 1;
	byte ProbIx : 8;
	uint8 Type : 3; 
	int8 Codex : 3;
#else
private:
	char Letter;
	char AncLetter;
public:
	bool Plus;
	bool Island;
	byte ProbIx;
	uint8 Type; 
	int8 Codex;
#endif

	BaseInfo()
		{
		Letter = BIT_LETTER_INVALID;
		AncLetter = BIT_LETTER_INVALID;
		Plus = true;
		Island = false;
		ProbIx = NEUTRAL_PROBIX;
		Codex = 0;
		Type = (uint8) BT_NonGene;
		};

	void SetLetter(char NewLetter)
		{
#if	BITFIELDS
		Letter = CharToBitLetter(NewLetter);
#else
		Letter = NewLetter;
#endif
		}

	void SetAncLetter(char NewLetter)
		{
#if	BITFIELDS
		AncLetter = CharToBitLetter(NewLetter);
#else
		AncLetter = NewLetter;
#endif
		}

	char GetLetter() const
		{
#if	BITFIELDS
		return BitLetterToChar(Letter);
#else
		return Letter;
#endif
		}

	char GetAncLetter() const
		{
#if	BITFIELDS
		return BitLetterToChar(AncLetter);
#else
		return AncLetter;
#endif
		}

	char GetOrientedLetter() const
		{
		return Plus ? GetLetter() : CompLetter(GetLetter());
		}

	char GetOrientedAncLetter() const
		{
		return Plus ? GetAncLetter() : CompLetter(GetAncLetter());
		}

	char GetCodonLetter() const
		{
		asserta(IsCoding());
		bool Strand = (Codex > 0);
		bool PlusLetter = (Strand == Plus);
		return PlusLetter ? GetLetter() : CompLetter(GetLetter());
		}

	void SetStrandLetter(char Letter_, bool Strand_)
		{
		bool PlusLetter = (Strand_ == Plus);
		if (PlusLetter)
			SetLetter(Letter_);
		else
			SetLetter(CompLetter(Letter_));
		}

	char GetStrandLetter(bool Strand) const
		{
		bool PlusLetter = (Strand == Plus);
		return PlusLetter ? GetLetter() : CompLetter(GetLetter());
		}

	double GetAcceptProb() const
		{
		extern double g_AcceptProbs[256];
		return g_AcceptProbs[ProbIx];
		}

	void Delete()
		{
		Letter = DeletedLetter;
		}

	bool IsDeleted() const
		{
		return Letter == DeletedLetter;
		}

	bool GetStrand() const
		{
		return Codex > 0;
		}

	bool IsGene() const
		{
		return Type != BT_NonGene;
		}

	bool IsIntron() const
		{
		return Type == BT_Intron || Type == BT_Splice;
		}

	bool IsStartCodon() const
		{
		return Type == BT_START;
		}
	
	bool IsStartStop() const
		{
		return IsStartCodon() || IsStopCodon();
		}

	bool IsStopCodon() const
		{
		return Type == BT_STOP;
		}

	bool IsUntranscribedGene() const
		{
		return Type == BT_UntranscribedGene;
		}

	bool IsCoding() const
		{
		return Type == BT_CDS || Type == BT_START || Type == BT_STOP;
		}

	bool IsCDS() const
		{
		return IsCoding();
		}

	bool IsUTR() const
		{
		return Type == BT_UTR;
		}

	bool IsExon() const
		{
		return IsUTR() || IsCDS();
		}

	bool IsNXE() const
		{
		return IsGene() && IsConstrained() && !IsExon() && !IsSplice();
		}

	bool IsNGE() const
		{
		return IsConstrained() && !IsGene();
		}

	bool IsSplice() const
		{
		return Type == BT_Splice;
		}

	bool IsUnconstrained() const
		{
		return ProbIx == NEUTRAL_PROBIX;
		}

	bool IsConstrained() const
		{
		return ProbIx != NEUTRAL_PROBIX;
		}

	bool IsFullyConstrained() const
		{
		return ProbIx == 0;
		}

	bool IsIsland() const
		{
		return Island;
		}

	void SetIsland(bool On)
		{
		Island = On;
		}

	bool IsOcean() const
		{
		return !Island && !IsExon();
		}

	bool PlusFrame() const
		{
		asserta(IsCoding());
		return Codex > 0;
		}

	void FullyConstrain()
		{
		ProbIx = 0;
		}

	void SetSplice()
		{
		Type = BT_Splice;
		FullyConstrain();
		}

	void SetUTR()
		{
		Type = BT_UTR;
		}

	void SetCDS()
		{
		Type = BT_CDS;
		}

	void SetStartCodon()
		{
		Type = BT_START;
		}

	void SetStopCodon()
		{
		Type = BT_STOP;
		}

	void SetIntron()
		{
		Type = BT_Intron;
		}

	void SetUntranscribedGene()
		{
		Type = BT_UntranscribedGene;
		}

	void SetNonGene()
		{
		Type = BT_NonGene;
		}

	void Invert()
		{
		Plus = !Plus;
		Codex = -Codex;
		}

	void Neutralize()
		{
		SetNonGene();
		Deconstrain();
		}

	void Deconstrain()
		{
		ProbIx = NEUTRAL_PROBIX;
		}

	CE_TYPE GetCEType() const
		{
		if (!IsConstrained())
			return CE_Neutral;
		if (IsUTR())
			return CE_UTR;
		if (IsCDS())
			return CE_CDS;
		if (IsNXE())
			return CE_NXE;
		if (IsNGE())
			return CE_NGE;
		if (IsSplice())
			return CE_Splice;
		asserta(false);
		UNREACHABLE_RETURN(CE_Invalid);
		}
	
	BaseType2 GetType2() const
		{
		switch ((BaseType) Type)
			{
		case BT_NonGene:
			if (IsConstrained())
				return BT2_NGE;
			else
				return BT2_NGNeutral;

		case BT_UntranscribedGene:
			if (IsConstrained())
				return BT2_NXE;
			else
				return BT2_UGNeutral;

		case BT_UTR:
			return BT2_UTR;

		case BT_CDS:
			if (Codex > 0)
				return BT2_CDSPlus;
			else
				return BT2_CDSMinus;

		case BT_START:
			return BT2_START;

		case BT_STOP:
			return BT2_STOP;

		case BT_Splice:
			return BT2_Splice;

		case BT_Intron:
			if (IsConstrained())
				return BT2_IntronCE;
			else
				return BT2_IntronNeutral;

		default:
			asserta(false);
			}
		UNREACHABLE_RETURN((BaseType2) 0)
		}

	void LogMe(bool NL = false) const
		{
		const char *BaseCategoryToStr(BaseCategory Cat);
		Log(" %c%c", GetLetter(), pom(Plus));
		if (IsCoding())
			Log("cx%+d", Codex);
		else
			Log(" %s", BaseTypeToStr((BaseType) Type));
		if (ProbIx == NEUTRAL_PROBIX)
			Log(" p=Neut", ProbIx);
		else
			Log(" p=%02x", ProbIx);
		if (NL)
			Log("\n");
		}
	};

#ifndef __APPLE_CC__
#pragma pack(pop)
#endif

int8 IncCodex(int8 Codex);
const char *CETypeToStr(CE_TYPE Type);

#endif // base_h
