#ifndef BMSOLVER_H
#define BMSOLVER_H
/* min-cost flow */
/* successive approximation algorithm */
/* Copyright C IG Systems, igsys@eclipse.com */
/* any use except for evaluation purposes requires a licence */

/************************************** constants  &  parameters ********/

#include <stdlib.h>
#include "bmtypes.h"
#include "bmgraph.h"

class BMSolver
{
	private:

	node dnd, *dnode;

	arc     *sa, *sb;
	int    d_cap;

	int n_push,
			 n_relabel,
			 n_discharge,
			 n_refine,
			 n_update,
			 n_scan,
			 n_prscan,
			 n_prscan1,
			 n_prscan2,
			 n_bad_pricein,
			 n_bad_relabel,
			 n_prefine;

	int   n,                    /* number of nodes */
				 m;                    /* number of arcs */

	int   *cap;                 /* array containig capacities */

	node   *nodes,               /* array of nodes */
				 *sentinel_node,       /* next after last */
				 *excq_first,          /* first node in push-queue */
				 *excq_last;           /* last node in push-queue */

	arc    *arcs,                /* array of arcs */
				 *sentinel_arc;        /* next after last */

	bucket *buckets,             /* array of buckets */
				 *l_bucket;            /* last bucket */
	int   linf;                 /* number of l_bucket + 1 */

	int time_for_price_in;

	price_t epsilon,              /* quality measure */
				 dn;                   /* cost multiplier - number of nodes  + 1 */
	price_t price_min,           /* lowest bound for prices */
				 mmc;                  /* multiplied maximal cost */
	double f_scale,              /* scale factor */
				 cut_off_factor,       /* multiplier to produce cut_on and cut_off
					from n and epsilon */
				 cut_on,               /* the bound for returning suspended arcs */
				 cut_off;              /* the bound for suspending arcs */

	excess_t total_excess;       /* total excess */

	int   n_rel,                /* number of relabels from last price update */
				 n_ref,                /* current number of refines */
				 n_src;                /* current number of nodes with excess */

	int   flag_price,        /* if = 1 - signal to start price-in ASAP - 
					maybe there is infeasibility because of
					susoended arcs */
				flag_updt;         /* if = 1 - update failed some sources are 
					unreachable: either the problem is
					unfeasible or you have to return 
																	suspended arcs */

	//int  empty_push_bound;      /* maximal possible number of zero pushes
	//                                during one discharge */

	int   snc_max;               /* maximal number of cycles cancelled
																	during price refine */

	arc   d_arc;                 /* dummy arc - for technical reasons */

	node  d_node,                /* dummy node - for technical reasons */
				*dummy_node;           /* the address of d_node */

#ifdef CHECK_SOLUTION
	long long int *node_balance;
#endif

	/* Functions */
	void err_end(int cc);
	void cs_init(int n_p, int m_p, node *nodes_p, arc *arcs_p, int f_sc, price_t max_c, int *cap_p);
	void up_node_scan(node *i);
	void price_update();
	int relabel(node *i);
	void discharge(node *i);
	int price_in();
	void refine();
	int price_refine();
	void compute_prices();
	void price_out();
	int update_epsilon();

#ifdef CHECK_SOLUTION
	int check_feas(node *nodes, arc *arp);
#endif

	int check_cs();
	int check_eps_opt();
	void finishup (double *obj_ad);
	void init_solution ();

	void cs2 (int n_p, int m_p, node *nodes_p, arc *arcs_p, int f_sc, price_t max_c, int *cap_p, double *obj_ad);

#ifdef COST_RESTART
	void cs_cost_reinit()
	void cs2_cost_restart(double *obj_ad);
#endif

#ifdef PRINT_ANS
	void print_solution(node *ndp, arc *arp, int nmin);
#endif

	void Solve(BMGraph &BMG);

	public:

	BMSolver(BMGraph &BMG)
	{
		n_push      =0,
		n_relabel   =0,
		n_discharge =0,
		n_refine    =0,
		n_update    =0,
		n_scan      =0,
		n_prscan    =0,
		n_prscan1   =0,
		n_prscan2   =0,
		n_bad_pricein = 0,
		n_bad_relabel = 0,
		n_prefine   =0;

		flag_price = 0;
		flag_updt = 0;
		buckets = NULL;

		Solve(BMG);
	}

	~BMSolver()
	{
		if (buckets != NULL)
			free(buckets);
	}

};

#endif
