#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "getopts.h"

// NOTE: ALMOST IDENTICAL TO MergeHits, SHOULD CONSOLIDATE

void BundleChrA(const RevFile &In, unsigned ChrIdA, const AIXVec &AXIs, RevFile &Out);

extern bool g_ShowProgress;

unsigned g_WindowLength = 4000;
unsigned g_GenomeIndexA = 0;
unsigned g_GenomeIndexB = 1;

bool LTByChrIdB(const AlnInfoX &AIX1, const AlnInfoX &AIX2)
	{
	const GCSeg &Seg1 = AIX1.AI.GetSeg(g_GenomeIndexB);
	const GCSeg &Seg2 = AIX2.AI.GetSeg(g_GenomeIndexB);

	return Seg1.ChrId < Seg2.ChrId;
	}

int main(int argc, char *argv[])
	{
	ProcessArgVect(argc, argv);

	const char *InFileName = RequiredValueOpt("in");
	const char *OutFileName = RequiredValueOpt("out");

	IntOpt("window", &g_WindowLength);

	g_ShowProgress = !FlagOpt("quiet");

	RevFile In;
	RevFile Out;

	In.Open(InFileName);
	const std::vector<GenInfo> &Genomes = In.GetGenomes();
	
	const GenInfo &GIA = In.GetGenomeInfo(g_GenomeIndexA);
//	const GenInfo &GIB = In.GetGenomeInfo(g_GenomeIndexB);

	const std::string &GenomeNameA = In.GetGenomeName(g_GenomeIndexA);
	const std::string &GenomeNameB = In.GetGenomeName(g_GenomeIndexB);

	unsigned ChrCountA = In.GetChrCount(g_GenomeIndexA);
	unsigned ChrCountB = In.GetChrCount(g_GenomeIndexB);

	Progress("Genome %.16s has %u chrs", GenomeNameA.c_str(), ChrCountA);
	Progress("Genome %.16s has %u chrs", GenomeNameB.c_str(), ChrCountB);

	Out.Create(OutFileName, Genomes);

	for (unsigned ChrIndexA = 0; ChrIndexA < ChrCountA; ++ChrIndexA)
		{
		const ChrInfo &CIA = GIA.GetChrInfoByIndex(ChrIndexA);
		unsigned ChrIdA = CIA.GetId();

		double Pct = ChrIndexA*100.0/ChrCountA;
		const std::string &ChrNameA = GIA.GetChrName(ChrIdA);

		Progress("Bundling chr %u of %u %.2f%% %.16s",
		  ChrIndexA, ChrCountA, Pct, ChrNameA.c_str());

		Progress("Get AIIndexes");
		IntVec AIIndexes;
		In.GetAppendAlnInfoIndexesChr(g_GenomeIndexA, ChrIdA, AIIndexes);

		Progress("Get AIIs");
		AIVec AIs;
		In.GetAlnInfos(AIIndexes, AIs);

		const unsigned AICount = (unsigned) AIIndexes.size();
		assert(AICount == (unsigned) AIs.size());

		AIXVec AIXs;
		AIXs.resize(AICount);
		for (unsigned i = 0; i < AICount; ++i)
			{
			AlnInfoX &AIX = AIXs[i];
			unsigned AIIndex = AIIndexes[i];
			AIX.AlnInfoIndex = AIIndex;
			AIX.AI = AIs[i];
			AIX.AlnIndex = In.GetAlnIndex(AIIndex);
			}
		AIs.clear();

		std::sort(AIXs.begin(), AIXs.end(), LTByChrIdB);
		BundleChrA(In, ChrIdA, AIXs, Out);
		}

	Progress("Closing files");
	In.Close();
	Out.Close();

	ProgressExit();
	return 0;
	}
