#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "mkedgesai.h"

#define TRACE	0

void GetConnectedComponents(const IntVecVec &AdjMx_, IntVecVec &CCs,
  bool ShowProgress);

bool LinkBundle(const AlnInfo &AI1, const AlnInfo &AI2);

extern unsigned g_WindowLength;
extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

static unsigned GetChrId(const AlnInfo &AI, unsigned GenomeIndex)
	{
	const GCSeg &Seg = AI.GetSeg(GenomeIndex);
	return Seg.ChrId;
	}

static void AssertChrPair(const AIXVec &AIXs, unsigned ChrIdA,
  unsigned ChrIdB)
	{
	const unsigned AICount = (unsigned) AIXs.size();
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIXs[AIIndex].AI;

		unsigned AIChrIdA = GetChrId(AI, g_GenomeIndexA);
		unsigned AIChrIdB = GetChrId(AI, g_GenomeIndexB);

		if (AIChrIdA != ChrIdA || AIChrIdB != ChrIdB)
			Die("ChrId failed %u,%u %u,%u",
			  AIChrIdA, ChrIdA, AIChrIdB, ChrIdB);
		}
	}

void BundleChrPair(const RevFile &In, unsigned ChrIdA, unsigned ChrIdB,
  const AIXVec &AIXs, RevFile &Out)
	{
	const unsigned AIXCount = (unsigned) AIXs.size();
	if (AIXCount == 0)
		return;

	AssertChrPair(AIXs, ChrIdA, ChrIdB);

	const std::string &ChrNameA = In.GetChrName(g_GenomeIndexA, ChrIdA);
	const std::string &ChrNameB = In.GetChrName(g_GenomeIndexB, ChrIdB);

	Progress("BundleChrPair %.16s[%u] %.16s[%u] %u AIXs",
	  ChrNameA.c_str(), ChrIdA, ChrNameB.c_str(), ChrIdB, AIXCount);

	IntVecVec Edges;
	MakeEdges(AIXs, LinkBundle, g_GenomeIndexA, g_WindowLength, Edges, true);

	IntVecVec Bundles;
	GetConnectedComponents(Edges, Bundles, true);

	const unsigned BundleCount = (unsigned) Bundles.size();
	Progress("%u bundles", BundleCount);

	for (unsigned BundleIndex = 0; BundleIndex < BundleCount; ++BundleIndex)
		{
		const IntVec &AIIndexes = Bundles[BundleIndex];
		for_CIntVec(AIIndexes, p)
			{
			unsigned AIXIndex = *p;
			assert(AIXIndex < AIXCount);
			AlnInfoX AIX = AIXs[AIXIndex];
			unsigned AlnIndex = AIX.AlnIndex;
			AIX.AI.SetBundleIndex(BundleIndex);
			if (AlnIndex == UINT_MAX)
				Out.AddAlnInfo(AIX.AI);
			else
				{
				StdStrVec Rows;
				In.GetAlnRows(AlnIndex, Rows);
#if	TRACE
				Log("Bundle=%u AIIndex=%u AlnIndex=%u\n", BundleIndex, AIIndex, AlnIndex);
#endif
				Out.AddAln(AIX.AI, Rows);
				}
			}
		}
	}
