#include "revfile.h"
#include "utils.h"
#include "filter4.h"

void GetBoundRect(const AIVec &AIs, unsigned GenomeIndexA, unsigned GenomeIndexB,
  AlnInfo &Rect);

bool AIsArePlus(const AIVec &TrueAIs, unsigned GenomeIndexA,
  unsigned GenomeIndexB);

extern unsigned g_EndHoleSize;


/***
    Plus							Minus
	 ----------------  C           	 ----------------  C     --E
	|            |E  | h           	|S  |            | h     --F
	|             ---| r           	|---             | r
	|    ........    | 1           	|    ........    | 1
	|    .      .    | |           	|    .      .    | |
	|    ........    | v           	|    ........    | v
	|--- Bnd Rect    |             	|    Bnd Rc  ----|
	|S  |            |             	|           |E   |       --G
	 ----------------              	 ----------------        --H
	Chr0->							Chr0->

	|  |          |  |              |  |         |   |
    |  |          |  |              |  |         |   |
	A  B          C  D              A  B         C   D

***/

void CalcStartEndHoles(const AIVec &AIs, AlnInfo &StartHole, AlnInfo &EndHole)
	{
	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	const ChrInfo &CIA = GetFilterChrInfoA();
	const ChrInfo &CIB = GetFilterChrInfoB();

	coord_t ChrLengthA = CIA.GetLength();
	coord_t ChrLengthB = CIB.GetLength();

	if (AIs.size() == 0)
		Die("CalcStartEndHoles, no AIs");
	if (ChrLengthA == 0 && ChrLengthB == 0)
		Die("CalcStartEndHoles, ChrLength zero");

	memset(&StartHole, 0, sizeof(StartHole));
	memset(&EndHole, 0, sizeof(StartHole));

	bool Plus = AIsArePlus(AIs, GenomeIndexA, GenomeIndexB);

	AlnInfo BoundRect;
	GetBoundRect(AIs, GenomeIndexA, GenomeIndexB, BoundRect);

	const GCSeg &BoundRectSegA = BoundRect.GetSeg(GenomeIndexA);
	const GCSeg &BoundRectSegB = BoundRect.GetSeg(GenomeIndexB);

	coord_t A = (BoundRectSegA.lo >= g_EndHoleSize ?
	  BoundRectSegA.lo - g_EndHoleSize : 0);

	coord_t B = (BoundRectSegA.lo >= 1 ? BoundRectSegA.lo - 1 : 0);

	coord_t C = BoundRectSegA.hi + 1;
	coord_t D = BoundRectSegA.hi + g_EndHoleSize;

	C = std::min(C, ChrLengthA - 1);
	D = std::min(D, ChrLengthA - 1);

	coord_t E = BoundRectSegB.hi + g_EndHoleSize;
	coord_t F = BoundRectSegB.hi + 1;

	E = std::min(E, ChrLengthB - 1);
	F = std::min(F, ChrLengthB - 1);

	coord_t G = (BoundRectSegB.lo >= 1 ? BoundRectSegB.lo - 1 : 0);

	coord_t H = (BoundRectSegB.lo >= g_EndHoleSize ?
	  BoundRectSegB.lo - g_EndHoleSize : 0);

	assert(A <= B);
	assert(B < C);
	assert(C <= D);
	assert(F <= E);
	assert(G < F);
	assert(H <= G);
	assert(B - A + 1 <= g_EndHoleSize);
	assert(D - C + 1 <= g_EndHoleSize);
	assert(E - F + 1 <= g_EndHoleSize);
	assert(G - H + 1 <= g_EndHoleSize);
	assert(D - A <= BoundRectSegA.GetLength() + 2*g_EndHoleSize);
	assert(E - H <= BoundRectSegB.GetLength() + 2*g_EndHoleSize);

	StartHole.Attrs.push_back(ALN_ATTR_TraceType);
	StartHole.Values.push_back(TraceType_StartHole);

	EndHole.Attrs.push_back(ALN_ATTR_TraceType);
	EndHole.Values.push_back(TraceType_EndHole);

	if (Plus)
		{
		FilterMakePairwiseAlnInfo(A, B, H, G, true, StartHole,
		  ALN_ATTR_TraceType, TraceType_StartHole);

		FilterMakePairwiseAlnInfo(C, D, F, E, true, EndHole,
		  ALN_ATTR_TraceType, TraceType_EndHole);
		}
	else
		{
		FilterMakePairwiseAlnInfo(A, B, F, E, true, StartHole,
		  ALN_ATTR_TraceType, TraceType_StartHole);

		FilterMakePairwiseAlnInfo(C, D, H, G, true, EndHole,
		  ALN_ATTR_TraceType, TraceType_EndHole);
		}
	}
