#include "chrinfo.h"
#include "utils.h"

/***
Structure:
	unsigned Id;
	coord_t Length;
	std::string Name;

Stored as:
	unsigned Id
	coord_t Length
	unsigned NameLength
	char char ... Name
***/

#define ReadData(v)		ReadStdioFile(f, &v, (unsigned) sizeof(v))
#define WriteData(v)	WriteStdioFile(f, &v, (unsigned) sizeof(v))

const unsigned SANE_NAME_LENGTH = 32*1024;

void ChrInfo::FromFile(FILE *f)
	{
	ReadData(m_Id);
	ReadData(m_Length);

	unsigned NameLength;
	ReadData(NameLength);
	if (NameLength > SANE_NAME_LENGTH)
		Die("ChrInfo::FromFile: id=%u, insane name length %u",
		  m_Id, NameLength);
	m_Name.reserve(NameLength);
	for (unsigned i = 0; i < NameLength; ++i)
		{
		char c;
		ReadData(c);
		m_Name.push_back(c);
		}
	}

void ChrInfo::ToFile(FILE *f) const
	{
	WriteData(m_Id);
	WriteData(m_Length);

	unsigned NameLength = (unsigned) m_Name.size();
	if (NameLength > SANE_NAME_LENGTH)
		Die("ChrInfo::FromFile: id=%u, insane name length %u",
		  m_Id, NameLength);
	WriteData(NameLength);

	for (unsigned i = 0; i < NameLength; ++i)
		{
		char c = m_Name[i];
		WriteData(c);
		}
	}

bool ChrInfo::operator==(const ChrInfo &rhs) const
	{
	return m_Id == rhs.m_Id && m_Name == rhs.m_Name && m_Length == rhs.m_Length;
	}
