#include "utils.h"
#include "codon.h"

/***
TTT Phe  TCT Ser  TAT Tyr  TGT Cys  
TTC Phe  TCC Ser  TAC Tyr  TGC Cys  
TTA Leu  TCA Ser  TAA STP  TGA STP  
TTG Leu  TCG Ser  TAG STP  TGG Trp  
CTT Leu  CCT Pro  CAT His  CGT Arg  
CTC Leu  CCC Pro  CAC His  CGC Arg  
CTA Leu  CCA Pro  CAA Gln  CGA Arg  
CTG Leu  CCG Pro  CAG Gln  CGG Arg  
ATT Ile  ACT Thr  AAT Asn  AGT Ser  
ATC Ile  ACC Thr  AAC Asn  AGC Ser  
ATA Ile  ACA Thr  AAA Lys  AGA Arg  
ATG Met  ACG Thr  AAG Lys  AGG Arg  
GTT Val  GCT Ala  GAT Asp  GGT Gly  
GTC Val  GCC Ala  GAC Asp  GGC Gly  
GTA Val  GCA Ala  GAA Glu  GGA Gly  
GTG Val  GCG Ala  GAG Glu  GGG Gly  
***/
CODON LettersToCodon(char c1, char c2, char c3)
	{
#define THREE_LETTERS(c1, c2, c3)	((c1) | ((c2) << 8) | ((c3) << 16))
	const int i = THREE_LETTERS(c1, c2, c3);
	switch (i)
		{
	case THREE_LETTERS('T', 'T', 'T'): return CODON_Phe;
	case THREE_LETTERS('T', 'T', 'C'): return CODON_Phe;
	case THREE_LETTERS('T', 'T', 'A'): return CODON_Leu;
	case THREE_LETTERS('T', 'T', 'G'): return CODON_Leu;
	case THREE_LETTERS('C', 'T', 'T'): return CODON_Leu;
	case THREE_LETTERS('C', 'T', 'C'): return CODON_Leu;
	case THREE_LETTERS('C', 'T', 'A'): return CODON_Leu;
	case THREE_LETTERS('C', 'T', 'G'): return CODON_Leu;
	case THREE_LETTERS('A', 'T', 'T'): return CODON_Ile;
	case THREE_LETTERS('A', 'T', 'C'): return CODON_Ile;
	case THREE_LETTERS('A', 'T', 'A'): return CODON_Ile;
	case THREE_LETTERS('A', 'T', 'G'): return CODON_Met;
	case THREE_LETTERS('G', 'T', 'T'): return CODON_Val;
	case THREE_LETTERS('G', 'T', 'C'): return CODON_Val;
	case THREE_LETTERS('G', 'T', 'A'): return CODON_Val;
	case THREE_LETTERS('G', 'T', 'G'): return CODON_Val;
	case THREE_LETTERS('T', 'C', 'T'): return CODON_Ser;
	case THREE_LETTERS('T', 'C', 'C'): return CODON_Ser;
	case THREE_LETTERS('T', 'C', 'A'): return CODON_Ser;
	case THREE_LETTERS('T', 'C', 'G'): return CODON_Ser;
	case THREE_LETTERS('C', 'C', 'T'): return CODON_Pro;
	case THREE_LETTERS('C', 'C', 'C'): return CODON_Pro;
	case THREE_LETTERS('C', 'C', 'A'): return CODON_Pro;
	case THREE_LETTERS('C', 'C', 'G'): return CODON_Pro;
	case THREE_LETTERS('A', 'C', 'T'): return CODON_Thr;
	case THREE_LETTERS('A', 'C', 'C'): return CODON_Thr;
	case THREE_LETTERS('A', 'C', 'A'): return CODON_Thr;
	case THREE_LETTERS('A', 'C', 'G'): return CODON_Thr;
	case THREE_LETTERS('G', 'C', 'T'): return CODON_Ala;
	case THREE_LETTERS('G', 'C', 'C'): return CODON_Ala;
	case THREE_LETTERS('G', 'C', 'A'): return CODON_Ala;
	case THREE_LETTERS('G', 'C', 'G'): return CODON_Ala;
	case THREE_LETTERS('T', 'A', 'T'): return CODON_Tyr;
	case THREE_LETTERS('T', 'A', 'C'): return CODON_Tyr;
	case THREE_LETTERS('T', 'A', 'A'): return CODON_STP;
	case THREE_LETTERS('T', 'A', 'G'): return CODON_STP;
	case THREE_LETTERS('C', 'A', 'T'): return CODON_His;
	case THREE_LETTERS('C', 'A', 'C'): return CODON_His;
	case THREE_LETTERS('C', 'A', 'A'): return CODON_Gln;
	case THREE_LETTERS('C', 'A', 'G'): return CODON_Gln;
	case THREE_LETTERS('A', 'A', 'T'): return CODON_Asn;
	case THREE_LETTERS('A', 'A', 'C'): return CODON_Asn;
	case THREE_LETTERS('A', 'A', 'A'): return CODON_Lys;
	case THREE_LETTERS('A', 'A', 'G'): return CODON_Lys;
	case THREE_LETTERS('G', 'A', 'T'): return CODON_Asp;
	case THREE_LETTERS('G', 'A', 'C'): return CODON_Asp;
	case THREE_LETTERS('G', 'A', 'A'): return CODON_Glu;
	case THREE_LETTERS('G', 'A', 'G'): return CODON_Glu;
	case THREE_LETTERS('T', 'G', 'T'): return CODON_Cys;  
	case THREE_LETTERS('T', 'G', 'C'): return CODON_Cys;  
	case THREE_LETTERS('T', 'G', 'A'): return CODON_STP;  
	case THREE_LETTERS('T', 'G', 'G'): return CODON_Trp;  
	case THREE_LETTERS('C', 'G', 'T'): return CODON_Arg;  
	case THREE_LETTERS('C', 'G', 'C'): return CODON_Arg;  
	case THREE_LETTERS('C', 'G', 'A'): return CODON_Arg;  
	case THREE_LETTERS('C', 'G', 'G'): return CODON_Arg;  
	case THREE_LETTERS('A', 'G', 'T'): return CODON_Ser;  
	case THREE_LETTERS('A', 'G', 'C'): return CODON_Ser;  
	case THREE_LETTERS('A', 'G', 'A'): return CODON_Arg;  
	case THREE_LETTERS('A', 'G', 'G'): return CODON_Arg;  
	case THREE_LETTERS('G', 'G', 'T'): return CODON_Gly;  
	case THREE_LETTERS('G', 'G', 'C'): return CODON_Gly;  
	case THREE_LETTERS('G', 'G', 'A'): return CODON_Gly;  
	case THREE_LETTERS('G', 'G', 'G'): return CODON_Gly;  
	default:
		Die("Invalid codon");
		}
#undef THREE_LETTERS
	UNREACHABLE_RETURN(CODON_Ala)
	}

char *CodonToStr(CODON c)
	{
	switch (c)
		{
	case CODON_Ala: return "Ala";
	case CODON_Cys: return "Cys";
	case CODON_Asp: return "Asp";
	case CODON_Glu: return "Glu";
	case CODON_Phe: return "Phe";
	case CODON_Gly: return "Gly";
	case CODON_His: return "His";
	case CODON_Ile: return "Ile";
	case CODON_Lys: return "Lys";
	case CODON_Leu: return "Leu";
	case CODON_Met: return "Met";
	case CODON_Asn: return "Asn";
	case CODON_Pro: return "Pro";
	case CODON_Gln: return "Gln";
	case CODON_Arg: return "Arg";
	case CODON_Ser: return "Ser";
	case CODON_Thr: return "Thr";
	case CODON_Val: return "Val";
	case CODON_Trp: return "Trp";
	case CODON_Tyr: return "Tyr";
	default:
		return "ERR";
		}
	}

char CodonToAminoChar(CODON c)
	{
	switch (c)
		{
	case CODON_Ala: return 'A';
	case CODON_Cys: return 'C';
	case CODON_Asp: return 'D';
	case CODON_Glu: return 'E';
	case CODON_Phe: return 'F';
	case CODON_Gly: return 'G';
	case CODON_His: return 'H';
	case CODON_Ile: return 'I';
	case CODON_Lys: return 'K';
	case CODON_Leu: return 'L';
	case CODON_Met: return 'M';
	case CODON_Asn: return 'N';
	case CODON_Pro: return 'P';
	case CODON_Gln: return 'Q';
	case CODON_Arg: return 'R';
	case CODON_Ser: return 'S';
	case CODON_Thr: return 'T';
	case CODON_Val: return 'V';
	case CODON_Trp: return 'W';
	case CODON_Tyr: return 'Y';
	case CODON_STP: return '*';
	default:
		break;
		}
	return '!';
	}

CODON AminoCharToCodon(char c)
	{
	switch (toupper(c))
		{
	case 'A': return CODON_Ala;
	case 'C': return CODON_Cys;
	case 'D': return CODON_Asp;
	case 'E': return CODON_Glu;
	case 'F': return CODON_Phe;
	case 'G': return CODON_Gly;
	case 'H': return CODON_His;
	case 'I': return CODON_Ile;
	case 'K': return CODON_Lys;
	case 'L': return CODON_Leu;
	case 'M': return CODON_Met;
	case 'N': return CODON_Asn;
	case 'P': return CODON_Pro;
	case 'Q': return CODON_Gln;
	case 'R': return CODON_Arg;
	case 'S': return CODON_Ser;
	case 'T': return CODON_Thr;
	case 'V': return CODON_Val;
	case 'W': return CODON_Trp;
	case 'Y': return CODON_Tyr;
	case '*': return CODON_STP;
	default:
		break;
		}
	return CODON_Invalid;
	}

char CharsToAmino(char c1, char c2, char c3)
	{
	CODON Codon = LettersToCodon(c1, c2, c3);
	return CodonToAminoChar(Codon);
	}

unsigned CharsToAminoIndex(char c1, char c2, char c3)
	{
	return (unsigned) LettersToCodon(c1, c2, c3);
	}

char LettersToCodonChar(char c1, char c2, char c3)
	{
	CODON Codon = LettersToCodon(c1, c2, c3);
	return CodonToAminoChar(Codon);
	}

/***
4D sites:
CTT Leu  GTT Val  TCT Ser  CCT Pro
CTC Leu  GTC Val  TCC Ser  CCC Pro
CTA Leu  GTA Val  TCA Ser  CCA Pro
CTG Leu  GTG Val  TCG Ser  CCG Pro

ACT Thr  GCT Ala  CGT Arg  GGT Gly
ACC Thr  GCC Ala  CGC Arg  GGC Gly
ACA Thr  GCA Ala  CGA Arg  GGA Gly
ACG Thr  GCG Ala  CGG Arg  GGG Gly
***/

bool Is4D(char a1, char a2, char b1, char b2)
	{
	a1 = (char) toupper(a1);
	a2 = (char) toupper(a2);
	b1 = (char) toupper(b1);
	b2 = (char) toupper(b2);

	if (a1 != b1 || a2 != b2)
		return false;

#define X(x, y)	if (a1 == x && a2 == y) return true;
	X('C', 'T')  // Leu
	X('G', 'T')  // Val
	X('T', 'C')  // Ser
	X('C', 'C')  // Pro
	X('A', 'C')  // Thr
	X('G', 'C')  // Ala
	X('C', 'G')  // Arg
	X('G', 'G')  // Gly
#undef X
	return false;
	}
