#include <math.h>
#include <vector>

using namespace std;

// Compute Pearson correlation coefficient.
// http://en.wikipedia.org/wiki/Correlation
double GetCorrel(const vector<pair<double, double> > &XYPairs)
	{
	if (XYPairs.empty())
		return 0;

	double Sum_x = 0;
	double Sum_y = 0;
	double Sum_x2 = 0;
	double Sum_y2 = 0;
	double Sum_xy = 0;

	for (vector<pair<double, double> >::const_iterator p = XYPairs.begin();
	  p != XYPairs.end(); ++p)
		{
		double x = p->first;
		double y = p->second;

		Sum_x += x;
		Sum_y += y;
		Sum_x2 += x*x;
		Sum_y2 += y*y;
		Sum_xy += x*y;
		}

	size_t N = XYPairs.size();
	double Top = N*Sum_xy - Sum_x*Sum_y;
	double Bottom1 = sqrt(N*Sum_x2 - Sum_x*Sum_x);
	double Bottom2 = sqrt(N*Sum_y2 - Sum_y*Sum_y);
	if (Bottom1 == 0 || Bottom2 == 0)
		return 0;
	return Top/(Bottom1*Bottom2);
	}

void TestCorrel()
	{
	vector<pair<double, double> > XYPairs;
#define xy(x, y)	XYPairs.push_back(pair<double, double>((x), (y)));
	
	xy(0.0, 0.0);
	xy(1.0, 1.0);
	xy(2.0, 3.0);

	for (vector<pair<double, double> >::const_iterator p = XYPairs.begin();
	  p != XYPairs.end(); ++p)
		printf("x=%.4g y=%.4g\n", p->first, p->second);

	printf("\n");
	printf("R=%7.4f\n", GetCorrel(XYPairs));
	}
