#include "utils.h"
#include "gff2.h"
#include "getopts.h"

static map<unsigned, gpos_t> g_GeneStarts;
static map<unsigned, bool> g_GeneStrands;
static map<gpos_t, unsigned> g_StartToGeneIndex;

static void Set(unsigned GeneIndex, gpos_t Pos, bool Plus)
	{
	g_GeneStrands[GeneIndex] = Plus;
	map<unsigned, gpos_t>::iterator p = g_GeneStarts.find(GeneIndex);
	if (p == g_GeneStarts.end())
		g_GeneStarts[GeneIndex] = Pos;
	else
		{
		gpos_t CurrPos = p->second;
		if (Plus)
			{
			if (Pos < CurrPos)
				g_GeneStarts[GeneIndex] = Pos;
			}
		else
			{
			if (Pos > CurrPos)
				g_GeneStarts[GeneIndex] = Pos;
			}
		}
	}

static void GetNearestGeneStart(gpos_t CpGEndPos, unsigned &GeneIndex,
  gpos_t &StartPos, int &Dist)
	{
	gpos_t Start1 = GPOS_T_MAX;
	gpos_t Start2 = GPOS_T_MAX;

	int d1 = INT_MAX;
	int d2 = INT_MAX;

	unsigned i1 = UINT_MAX;
	unsigned i2 = UINT_MAX;

	map<gpos_t, unsigned>::const_iterator p1 = 
	  g_StartToGeneIndex.lower_bound(CpGEndPos);
	if (p1 != g_StartToGeneIndex.begin())
		--p1;

	map<gpos_t, unsigned>::const_iterator p2 = 
	  g_StartToGeneIndex.upper_bound(CpGEndPos);

	if (p1 != g_StartToGeneIndex.end())
		{
		Start1 = p1->first;
		i1 = p1->second;
		d1 = (int) CpGEndPos - (int) Start1;
		}

	if (p2 != g_StartToGeneIndex.end())
		{
		Start2 = p2->first;
		i2 = p2->second;
		d2 = (int) CpGEndPos - (int) Start2;
		}

	if (abs(d1) < abs(d2))
		{
		GeneIndex = i1;
		StartPos = Start1;
		Dist = d1;
		}
	else
		{
		GeneIndex = i2;
		StartPos = Start2;
		Dist = d2;
		}
	}

void CpGDist()
	{
	const char *CpGFileName = RequiredValueOpt("cpgdist");
	const char *GeneFileName = RequiredValueOpt("genes");

	FILE *fCpG = OpenStdioFile(CpGFileName);
	FILE *fGenes = OpenStdioFile(GeneFileName);

	GFFRecord2 Rec;
	while (GetNextGFFRecord(fGenes, Rec))
		{
		unsigned GeneIndex = GFFGetRequiredIntAttr(Rec.Attrs, "gene_index");
		bool Plus = Rec.Strand == '+';
		gpos_t Pos = Plus ? Rec.Start - 1 : Rec.End - 1;
		Set(GeneIndex, Pos, Plus);
		}

	for (map<unsigned, gpos_t>::const_iterator p = g_GeneStarts.begin();
	  p != g_GeneStarts.end(); ++p)
		g_StartToGeneIndex[p->second] = p->first;

	Log(" CpGEndPos  GeneStartPos  Distance  GeneIndex  +\n");
	Log("==========  ============  ========  =========  =\n");
	while (GetNextGFFRecord(fCpG, Rec))
		{
		gpos_t CpGStartPos = Rec.Start - 1;
		gpos_t CpGEndPos = Rec.End - 1;

		unsigned GeneIndex1;
		unsigned GeneIndex2;
		gpos_t StartPos1;
		gpos_t StartPos2;
		int Dist1;
		int Dist2;
		GetNearestGeneStart(CpGEndPos, GeneIndex1, StartPos1, Dist1);
		GetNearestGeneStart(CpGStartPos, GeneIndex2, StartPos2, Dist2);
		if (abs(Dist1) < abs(Dist2))
			{
			bool Plus = g_GeneStrands[GeneIndex1];
			Log("%10u  %12u  %8d  %9u  %c\n",
			  CpGEndPos, StartPos1, Plus ? Dist1 : -Dist1, GeneIndex1, pom(Plus));
			}
		else
			{
			bool Plus = g_GeneStrands[GeneIndex2];
			Log("%10u  %12u  %8d  %9u  %c\n",
			  CpGStartPos, StartPos2, Plus ? Dist2 : -Dist2, GeneIndex2, pom(Plus));
			}
		}

	CloseStdioFile(fCpG);
	CloseStdioFile(fGenes);
	}
