#include "utils.h"
#include "revfile.h"
#include "getopts.h"

const char *TruncLabel(const char *Label);
void TruncLabel(std::string &Label);
void WriteFasta(FILE *f, const char *SeqData, unsigned Length);

void CvtRev2FASTA()
	{
	const char *RevFileName = RequiredValueOpt("fromrev");
	const char *FASTAFileName = RequiredValueOpt("tofasta");
	bool GenDotChr = FlagOpt("gendotchr");

	RevFile Rev;
	Rev.Open(RevFileName);

	unsigned SeqCount = Rev.GetSeqCount();
	if (SeqCount == 0)
		Die("No seqs in rev file");

	FILE *fFASTA = CreateStdioFile(FASTAFileName);

	for (unsigned SeqIndex = 0; SeqIndex < SeqCount; ++SeqIndex)
		{
		coord_t cLength = Rev.GetSeqLengthByIndex(SeqIndex);
		unsigned Length = (unsigned) cLength;
		if ((coord_t) Length != cLength)
			Die("Seq length overflows unsigned int");

		char *Seq = myalloc(char, Length);

		unsigned GenIndex;
		unsigned ChrId;
		Rev.GetSeqByIndex(SeqIndex, Seq, Length, &GenIndex, &ChrId);

		std::string ChrName = Rev.GetChrName(GenIndex, ChrId);
		TruncLabel(ChrName);

		if (GenDotChr)
			{
			std::string GenName = Rev.GetGenomeName(GenIndex);
			TruncLabel(GenName);
			fprintf(fFASTA, ">%s.%s\n", GenName.c_str(), ChrName.c_str());
			}
		else
			fprintf(fFASTA, ">%s\n", ChrName.c_str());

		WriteFasta(fFASTA, Seq, Length);

		myfree(Seq);
		}

	Rev.Close();
	CloseStdioFile(fFASTA);
	}
