#include "utils.h"
#include "revfile.h"
#include "getopts.h"

void DumpActiveChrPairs()
	{
	const char *FileName = RequiredValueOpt("dumpacp");

	RevFile Rev;
	Rev.Open(FileName);

	std::set<IntPair> ChrIdPairs;
	Rev.GetActiveChrPairs(0, 1, ChrIdPairs);

	Dump("ChrId0             Name0  ChrId1             Name1      AIs\n");
	Dump("======  ================  ======  ================  =======\n");

	for (std::set<IntPair>::const_iterator p = ChrIdPairs.begin();
	  p != ChrIdPairs.end(); ++p)
		{
		const IntPair &ChrIds = *p;

		unsigned ChrId0 = ChrIds.first;
		unsigned ChrId1 = ChrIds.second;

		const std::string &ChrName0 = Rev.GetChrName(0, ChrId0);
		const std::string &ChrName1 = Rev.GetChrName(1, ChrId1);

		IntVec AIIndexes;
		Rev.GetAppendAlnInfoIndexesChr(0, ChrId0, AIIndexes);

		AIVec AIs;
		Rev.GetAlnInfos(AIIndexes, AIs);

		assert(AIs.size() == AIIndexes.size());

		const unsigned AICount = (unsigned) AIs.size();
		unsigned Count = 0;
		for (unsigned i = 0; i < AICount; ++i)
			{
			const AlnInfo &AI = AIs[i];
			const unsigned SegCount = (unsigned) AI.Segs.size();
			for (unsigned j = 0; j < SegCount; ++j)
				{
				const GCSeg &S = AI.Segs[j];
				if (S.GenomeIndex == 1 && S.ChrId == ChrId1)
					{
					++Count;
					break;
					}
				}
			}
		Dump("%6u  %16s  %6u  %16s  %7u\n",
		  ChrId0,
		  ChrName0.c_str(),
		  ChrId1,
		  ChrName1.c_str(),
		  Count);
		}
	}
