#include "evo.h"
#include "myrand.h"

void DumpEvo()
	{
	const char *FileName = RequiredValueOpt("dumpevo");

	Evolver Evo;
	Evo.FromFile(FileName);
	Evo.DumpMe();
	Log("\n");

	Log("Rate managers:\n");
#define E(e) extern RateMgr *g_##e##RateMgr; g_##e##RateMgr->LogMe();
// Only subset of events have a RateMgr, can't #include "events.h" here.
	E(Delete)
	E(Invert)
	E(Move)
	E(Copy)
	E(Tandem)
	E(Insert)
	E(MEInsert)
	E(NCCE)
#undef E

	Log("\n");
	Log("Random number state:\n");
	LogRandState();
	}

void Evolver::DumpMe() const
	{
	Log("\n");
	Log("Ancestral length             %10"PF"\n", m_AncestralLength);
	Log("Evolved length               %10"PF"\n", m_EvolvedLength);
	Log("First evolved pos            %10"PF"\n", m_FirstEvolvedPos);
	Log("Last evolved pos             %10"PF"\n", m_LastEvolvedPos);
	Log("First deleted pos            %10"PF"\n", m_FirstDeletedPos);
	Log("Last deleted pos             %10"PF"\n", m_LastDeletedPos);

	Log("\n");
	Log("Branch length                %10.3g\n", m_BranchLength);
	Log("Event index                  %10u\n", m_EventIndex);
	Log("Event count                  %10u\n", m_EventCount);
	Log("Accepted events              %10u\n", m_LastAcceptedEventIndex);
	Log("Tandem recompute counter     %10u\n", m_TandemRateRecomputeCounter);
	Log("Avg wait time                %10.3g\n", m_AverageWaitingTime);

	Log("\n");
	Log("Do CpG sweeps                %10c\n", tof(m_DoCpGSweeps));
	Log("CpG sweep event counter      %10u\n", m_TandemRateRecomputeCounter);
	Log("CpG sweep interval           %10u\n", m_CpGSweepInterval);

	Log("\n");
	Log("Container sizes:\n");
	Log("  Bases                      %10u\n", SIZE(m_Bases));
	Log("  Fwds                       %10u\n", SIZE(m_BaseListFwds));
	Log("  Bwds                       %10u\n", SIZE(m_BaseListBwds));
	Log("  AncPositions               %10u\n", SIZE(m_AncPositions));
	Log("  Tandems                    %10u\n", SIZE(m_Tandems));
	Log("  GeneIndexes                %10u\n", SIZE(m_GeneIndexes));
	Log("  Paralogs                   %10u\n", SIZE(m_Paralogs));

	Log("\n");
	Log("MEs:\n");
	LogMEs();

	Log("\n");
	Log("           Event        Rate      RandHi\n");
	Log("----------------  ----------  ----------\n");
	for (unsigned i = 0; i < EventTypeCount; ++i)
		{
		Log("%16.16s  %10.3g  %10u\n",
		  EventTypeToStr((EVENT_TYPE) i),
		  m_IntegratedEventRates[i],
		  m_EventRandDistHis[i]);
		}

	Log("\n");
	Log("Ancestral genome info:\n");
	m_AncGenInfo.LogMe();
	}
