#define TRACE	0
#define VAL_RANGE	0

#include "evolver.h"
#include "evomodel.h"

void Evolver::Delete(gpos_t Pos, gpos_t Length)
	{
#if	VAL_RANGE
	if (!RangeExists(Pos, Length))
		Die("Evolver::Delete, range does not exist");
#endif

	gpos_t DeletePos = Pos;
	for (uint32 i = 0; i < Length; ++i)
		{
		gpos_t NextPos = GetNextPos(DeletePos);
		DeleteBase(DeletePos);
		DeletePos = NextPos;
		}
	}

void Evolver::Insert(gpos_t Pos, const string &Seq)
	{
	if (PosIsDeleted(Pos))
		Die("Evolver::Insert, target pos is deleted");

	gpos_t InsertPos = Pos;
	for (string::const_iterator p = Seq.begin(); p != Seq.end(); ++p)
		{
		char Letter = *p;
		InsertPos = InsertNovelBase(InsertPos, Letter);
		}
	SetInsertConstraints(Pos, SIZE(Seq));
	}

void Evolver::Invert(gpos_t Pos, gpos_t Length)
	{
TRC
	Log("\n");
	Log("Invert(Pos=%"PF", Length=%"PF")\n", Pos, Length);
ENDTRC
#if	VAL_RANGE
	if (!RangeExists(Pos, Length))
		Die("Evolver::Invert, range does not exist");
#endif
	gpos_t p = Pos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(p);
		Base.Invert();
		p = GetNextPos(p);
		}
	BaseList_Invert(Pos, Length);
	}

void Evolver::Move(gpos_t FromPos, gpos_t ToPos, gpos_t Length)
	{
TRC
	Log("\n");
	Log("Move(Pos=%"PF", Length=%"PF", ToPos=%"PF")\n",
	  FromPos, Length, ToPos);
ENDTRC

#if	VAL_RANGE
	if (!RangeExists(FromPos, Length))
		Die("Evolver::Move, range does not exist");
	if (Length > 1 && PosInRange(ToPos, FromPos, Length))
		Die("Evolver::Move, insert pos in source range");
#endif

	gpos_t SourcePos = FromPos;
	gpos_t TargetPos = ToPos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		gpos_t NextSourcePos = GetNextPos(SourcePos);
		MoveBase(SourcePos, TargetPos);
		SourcePos = NextSourcePos;
		TargetPos = GetNextPos(TargetPos);
		}
	}

void Evolver::Copy(gpos_t FromPos, gpos_t ToPos, gpos_t Length,
  vector<gpos_t> &GeneStarts, vector<gpos_t> &CopiedGeneStarts, bool DoInvert)
	{
	GeneStarts.clear();
	CopiedGeneStarts.clear();

#if	VAL_RANGE
	if (!RangeExists(FromPos, Length))
		Die("Evolver::Copy, range does not exist");

// Length-1 because last pos is ok (tandem dupe)
	if (Length > 1 && PosInRange(ToPos, FromPos, Length-1))
		Die("Evolver::Copy, insert pos in source range");
#endif

	gpos_t SourcePos = FromPos;
	gpos_t TargetPos = ToPos;
	gpos_t GeneStartPos = GPOS_T_MAX;
	gpos_t CopiedGeneStartPos = GPOS_T_MAX;
	bool Hom = (Length >= P_MinCopyAln);
	bool PrevBaseInGene = false;
	vector<gpos_t> CopiedGeneEnds;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo Base = GetBase(SourcePos);
		bool InGene = Base.IsGene();
		Base.Neutralize();
		if (Hom)
			TargetPos = BaseList_Insert(Base, TargetPos, SourcePos);
		else
			{
			Base.SetAncLetter(NovelLetter);
			TargetPos = BaseList_Insert(Base, TargetPos, UINT32_MAX);
			}

		if (!PrevBaseInGene && InGene && i > 0)
			{
			GeneStartPos = SourcePos;
			CopiedGeneStartPos = TargetPos;
			}
		else if (PrevBaseInGene && !InGene)
			{
			if (GeneStartPos != GPOS_T_MAX)
				{
				GeneStarts.push_back(GeneStartPos);
				CopiedGeneStarts.push_back(CopiedGeneStartPos);
				GeneStartPos = GPOS_T_MAX;
				CopiedGeneStartPos = GPOS_T_MAX;
				gpos_t PrevPos = GetPrevPos(TargetPos);
				CopiedGeneEnds.push_back(PrevPos);
				}
			}

		SourcePos = GetNextPos(SourcePos);
		PrevBaseInGene = InGene;
		}
	
	if (DoInvert)
		{
		gpos_t FirstInsertedPos = GetNextPos(ToPos);
		Invert(FirstInsertedPos, Length);
//		CopiedGeneStarts = CopiedGeneEnds;
		}

	SetInsertConstraints(ToPos, Length);
	}
