#include <math.h>
#include "evolver.h"
#include "eventresult.h"

unsigned g_ERTCounts[EventResultTypeCount];
extern FILE *g_fStats;

set<unsigned> g_AnGeneIndexes;
set<unsigned> g_EvGeneIndexes;
unsigned g_GeneDupeCount;
unsigned g_ActiveGeneDupeCount;
unsigned g_GeneInGeneInsertCount;

unsigned g_TotalMoveCount;
unsigned g_TotalCopyCount;
unsigned g_TotalTandemCount;

unsigned g_InvertedMoveCount;
unsigned g_InvertedCopyCount;
unsigned g_InvertedTandemCount;

unsigned g_InvertableMoveCount;
unsigned g_InvertableCopyCount;
unsigned g_InvertableTandemCount;

unsigned g_InsertRejectAccProb;
unsigned g_InsertRejectInCDSMod3Count;
unsigned g_InsertCreatesStopCodonCount;
unsigned g_InsertInStartOrStopCodonCount;
unsigned g_InsertAccConstrained;

unsigned g_MoveRejInsIntoCEOrGene;

unsigned g_InsAccConstrained;
unsigned g_InsRejConstrained;
unsigned g_InsAccNeutral;

unsigned g_DelAccConstrained;
unsigned g_DelRejConstrained;
unsigned g_DelAccNeutral;
unsigned g_DelAccCDS;

const char *ChgPct(double x, double y)
	{
	if (y == 0)
		{
		if (x == 0)
			return "0%";
		else
			return "";
		}

	double c = (x/y - 1)*100.0;
	if (fabs(c) < 0.1)
		return "0%";

	static char Str[16];
	sprintf(Str, "%+.1f%%", c);
	return Str;
	}

static const char *ERToStr(EVENT_RESULT ER)
	{
	switch (ER)
		{
	case ER_Fail:
		return "Fail";
	case ER_Reject:
		return "Reject";
	case ER_Accept:
		return "Accept";
		}
	static char Str[16];
	sprintf(Str, "ER_%d", ER);
	return Str;
	}

const char *ERTToStr(ER_TYPE ERT)
	{
	switch (ERT)
		{
#define DEF_FAIL(EventName, Operation)	case ERF_##EventName##_##Operation: return "FAIL_" #EventName "_" #Operation;
#define DEF_REJECT(EventName, Mutation)	case ERR_##EventName##_##Mutation: return "REJECT_" #EventName "_" #Mutation;
#define DEF_ACCEPT(EventName)			case ERA_##EventName: return "ACCEPT_" #EventName;
#include "eventresults.h"
		}
	static char Str[16];
	sprintf(Str, "ER_%u", ERT);
	return Str;
	}

static EVENT_TYPE ERT2ET(ER_TYPE Type)
	{
	switch (Type)
		{
#define DEF_FAIL(EventName, Operation)	case ERF_##EventName##_##Operation: return EVENT_TYPE_##EventName;
#define DEF_REJECT(EventName, Mutation)	case ERR_##EventName##_##Mutation: return EVENT_TYPE_##EventName;
#define DEF_ACCEPT(EventName)			case ERA_##EventName: return EVENT_TYPE_##EventName;
#include "eventresults.h"
		}
	asserta(false);
	UNREACHABLE_RETURN((EVENT_TYPE) INT_MAX);
	}

static EVENT_RESULT ERT2ER(ER_TYPE Type)
	{
	switch (Type)
		{
#define DEF_FAIL(EventName, Operation)	case ERF_##EventName##_##Operation: return ER_Fail;
#define DEF_REJECT(EventName, Mutation)	case ERR_##EventName##_##Mutation: return ER_Reject;
#define DEF_ACCEPT(EventName)			case ERA_##EventName: return ER_Accept;
#include "eventresults.h"
		}
	asserta(false);
	UNREACHABLE_RETURN((EVENT_RESULT) INT_MAX);
	}

static const char *ERT2SubtypeStr(ER_TYPE Type)
	{
	switch (Type)
		{
#define DEF_FAIL(EventName, Operation)	case ERF_##EventName##_##Operation: return #Operation;
#define DEF_REJECT(EventName, Mutation)	case ERR_##EventName##_##Mutation: return #Mutation;
#define DEF_ACCEPT(EventName)			case ERA_##EventName: return "";
#include "eventresults.h"
		}
	asserta(false);
	UNREACHABLE_RETURN("?");
	}

static bool LT(const pair<unsigned, double> &p1, const pair<unsigned, double> &p2)
	{
	return p1.second > p2.second;
	}

void LogEventStats(double BranchLength, const vector<double> &InitialRates,
  const vector<double> &FinalRates)
	{
	vector<pair<unsigned, double> > Pairs;
	unsigned TotalNonFailCount = 0;
	for (unsigned iERT = 0; iERT < EventResultTypeCount; ++iERT)
		{
		ER_TYPE ERT = (ER_TYPE) iERT;
		EVENT_RESULT ER = ERT2ER(ERT);
		if (ER != ER_Fail)
			TotalNonFailCount += g_ERTCounts[iERT];
		}

	for (unsigned iET = 0; iET < EventTypeCount; ++iET)
		{
		double Rate = FinalRates[iET];
		Pairs.push_back(pair<unsigned, double>(iET, Rate));
		}
	sort(Pairs.begin(), Pairs.end(), LT);

	vector<unsigned> AcceptCounts(EventTypeCount, 0);

	Log("\n");
	Log("               Event  Init Rate  FinalRate       Fails     Rejects     Accepts     Fail%%   Reject%%   Accept%%    Expected       Total   %%Events\n");
	Log("====================  =========  =========  ==========  ==========  ==========  ========  ========  ========  ==========  ==========  ========\n");
	double TotalPctEvents = 0;
	unsigned TotalNonFailCount2 = 0;
	unsigned TotalFailCount = 0;
	unsigned TotalAcceptCount = 0;
	unsigned TotalRejectCount = 0;
	unsigned TotalEventCount = 0;
	for (unsigned i = 0; i < EventTypeCount; ++i)
		{
		const pair<unsigned, double> &Pair = Pairs[i];
		unsigned iET = Pair.first;
		asserta(iET < EventTypeCount);
		unsigned FailCount = 0;
		unsigned RejectCount = 0;
		unsigned AcceptCount = 0;
		EVENT_TYPE ET = (EVENT_TYPE) iET;

		for (unsigned iERT = 0; iERT < EventResultTypeCount; ++iERT)
			{
			ER_TYPE ERT = (ER_TYPE) iERT;
			EVENT_TYPE ET2 = ERT2ET(ERT);
			if (ET2 != ET)
				continue;
			EVENT_RESULT ER = ERT2ER(ERT);
			switch (ER)
				{
			case ER_Fail:
				FailCount += g_ERTCounts[iERT];
				break;
			case ER_Reject:
				RejectCount += g_ERTCounts[iERT];
				break;
			case ER_Accept:
				AcceptCount += g_ERTCounts[iERT];
				break;
			default:
				asserta(false);
				}
			}

		double AvgRate = (InitialRates[iET] + FinalRates[iET])/2;
		unsigned ExpectedCount = (unsigned) (BranchLength*AvgRate + 0.5);
		unsigned Count = RejectCount + AcceptCount;

		const char *EvType = EventTypeToStr(ET);
		Log("%20.20s", EvType);
		Log("  %9.3g", InitialRates[iET]);
		Log("  %9.3g", FinalRates[iET]);
		Log("  %10u", FailCount);
		Log("  %10u", RejectCount);
		Log("  %10u", AcceptCount);

		if (g_fStats != 0)
			{
			if (FailCount > 0)
				fprintf(g_fStats, "EV_FAILS;%s;%u\n", EvType, FailCount);

			if (RejectCount > 0)
				fprintf(g_fStats, "EV_REJECTS;%s;%u\n", EvType, RejectCount);

			if (AcceptCount > 0)
				fprintf(g_fStats, "EV_ACCEPTS;%s;%u\n", EvType, AcceptCount);
			}

		if (Count > 0)
			{
			Log("   %6.1f%% ", Pct(FailCount, Count + FailCount));
			Log("  %6.1f%% ", Pct(RejectCount, Count));
			Log("  %6.1f%% ", Pct(AcceptCount, Count));
			}
		else
			{
			Log("           ");
			Log("          ");
			Log("          ");
			}

		Log(" %10u", ExpectedCount);
		Log("  %10u", Count);

		double PctEvents = TotalNonFailCount == 0 ? 0 : 
		  (double) (RejectCount + AcceptCount)*100.0/(double) TotalNonFailCount;
		TotalPctEvents += PctEvents;
		TotalEventCount += RejectCount + AcceptCount;
		TotalFailCount += FailCount;
		TotalAcceptCount += AcceptCount;
		TotalRejectCount += RejectCount;
		TotalNonFailCount2 += RejectCount + AcceptCount;
		AcceptCounts[iET] = AcceptCount;
		
		Log("  %7.1f%%", PctEvents);

		Log("\n");
		}
	asserta(TotalNonFailCount == TotalNonFailCount2);

	Log("                                            ");
	Log("==========  ==========  ==========");
	Log("                                            ==========  ========\n");

	Log("                                            ");
	Log("%10u  %10u  %10u",
	  TotalFailCount, TotalRejectCount, TotalAcceptCount);
	Log("                                            %10u  %7.1f%%",
	  TotalEventCount, TotalPctEvents);
	Log("\n");
	}

void LogERDetails()
	{
	Log("\n");
	Log("               Event       Count  Result  Reason\n");
	Log("====================  ==========  ======  ======\n");
	for (unsigned iET = 0; iET < EventTypeCount; ++iET)
		{
		unsigned FailCount = 0;
		unsigned RejectCount = 0;
		unsigned AcceptCount = 0;
		EVENT_TYPE ET = (EVENT_TYPE) iET;

		for (unsigned iERT = 0; iERT < EventResultTypeCount; ++iERT)
			{
			ER_TYPE ERT = (ER_TYPE) iERT;
			EVENT_TYPE ET2 = ERT2ET(ERT);
			if (ET2 != ET)
				continue;
			EVENT_RESULT ER = ERT2ER(ERT);
			switch (ER)
				{
			case ER_Fail:
				FailCount += g_ERTCounts[iERT];
				break;
			case ER_Reject:
				RejectCount += g_ERTCounts[iERT];
				break;
			case ER_Accept:
				AcceptCount += g_ERTCounts[iERT];
				break;
			default:
				asserta(false);
				}
			}

		bool Any = false;
		for (unsigned iERT = 0; iERT < EventResultTypeCount; ++iERT)
			{
			unsigned Count = g_ERTCounts[iERT];
			if (Count == 0)
				continue;
			ER_TYPE ERT = (ER_TYPE) iERT;
			EVENT_TYPE ET2 = ERT2ET(ERT);
			if (ET2 != ET)
				continue;
			Any = true;
			EVENT_RESULT ER = ERT2ER(ERT);
			const char *EvType = EventTypeToStr(ET);
			const char *ResultType = ERToStr(ER);
			const char *Subtype = ERT2SubtypeStr(ERT);
			Log("%20.20s  %10u  %6.6s  %s\n",
			  EvType,
			  Count,
			  ResultType,
			  Subtype);
			if (g_fStats != 0)
				{
				if (Count > 0)
					fprintf(g_fStats, "EV_DETAIL;%s;%s;%s;%u\n",
					  EvType, ResultType, Subtype, Count);
				}
			}
		if (Any)
			Log("\n");
		}

	void LogLengthCounts();
	LogLengthCounts();

	extern unsigned g_GeneDupeCount;
	extern unsigned g_ActiveGeneDupeCount;
	extern unsigned g_GeneInGeneInsertCount;

	unsigned NonIntronCopies = g_GeneDupeCount - g_GeneInGeneInsertCount;
	unsigned DisabledCount = NonIntronCopies - g_ActiveGeneDupeCount;

	double ParalogPct =
	  g_GeneDupeCount == 0 ? 0 : (g_ActiveGeneDupeCount*100.0)/g_GeneDupeCount;

	double IntronPct =
	  g_GeneDupeCount == 0 ? 0 : (g_GeneInGeneInsertCount*100.0)/g_GeneDupeCount;

	double NonIntronPct =
	  g_GeneDupeCount == 0 ? 0 : (NonIntronCopies*100.0)/g_GeneDupeCount;

	double DisabledPct =
	  g_GeneDupeCount == 0 ? 0 : (DisabledCount*100.0)/g_GeneDupeCount;

	double Pct(double x, double T);
	Log("\n");
	Log("Moves                 %10u\n", g_TotalMoveCount);
	Log("Invertable moves      %10u\n", g_InvertableMoveCount);
	Log("Inverted moves        %10u (%.1f%% of total, %.1f%% of invertable)\n",
	  g_InvertedMoveCount, Pct(g_InvertedMoveCount, g_TotalMoveCount),
	  Pct(g_InvertedMoveCount, g_InvertableMoveCount));
	Log("\n");
	Log("Non-tandem copies     %10u\n", g_TotalCopyCount);
	Log("Invertable n-t copies %10u\n", g_InvertableCopyCount);
	Log("Inverted n-t copies   %10u (%.1f%% of total, %.1f%% of invertable)\n",
	  g_InvertedCopyCount, Pct(g_InvertedCopyCount, g_TotalCopyCount),
  	  Pct(g_InvertedCopyCount, g_InvertableCopyCount));

	Log("\n");
	Log("Tandem copies         %10u\n", g_TotalTandemCount);
	Log("Invertable tandems    %10u\n", g_InvertableTandemCount);
	Log("Inverted tandems      %10u (%.1f%% of total, %.1f%% of invertable)\n",
	  g_InvertedTandemCount, Pct(g_InvertedTandemCount, g_TotalTandemCount),
	  Pct(g_InvertedTandemCount, g_InvertableTandemCount));

	Log("\n");
	Log("Ins rej constrained   %10u\n", g_InsRejConstrained);
	Log("Ins acc constrained   %10u\n", g_InsAccConstrained);
	Log("Ins acc neutral       %10u\n", g_InsAccNeutral);

	Log("\n");
	Log("Del rej constrained   %10u\n", g_DelRejConstrained);
	Log("Del acc constrained   %10u\n", g_DelAccConstrained);
	Log("Del acc neutral       %10u\n", g_DelAccNeutral);
	Log("Del acc CDS           %10u\n", g_DelAccCDS);

	Log("\n");
	Log("Move rej ins CE/Gene  %10u\n", g_MoveRejInsIntoCEOrGene);

	if (g_fStats != 0)
		{
		fprintf(g_fStats, "EV_DETAIL;sub_ins;Reject;Constraint;%u\n", g_InsRejConstrained);
		fprintf(g_fStats, "EV_DETAIL;sub_ins;Accept;AccProb;%u\n", g_InsAccConstrained);
		fprintf(g_fStats, "EV_DETAIL;sub_ins;Accept;Neutral;%u\n", g_InsAccNeutral);

		fprintf(g_fStats, "EV_DETAIL;sub_del;Reject;Constraint;%u\n", g_DelRejConstrained);
		fprintf(g_fStats, "EV_DETAIL;sub_del;Accept;AccProb;%u\n", g_DelAccConstrained);
		fprintf(g_fStats, "EV_DETAIL;sub_del;Accept;Neutral;%u\n", g_DelAccNeutral);
		fprintf(g_fStats, "EV_DETAIL;sub_del;Accept;CDS;%u\n", g_DelAccCDS);
		}

	Log("\n");
	Log("Ins. acc. in CEs      %10u\n", g_InsertAccConstrained);
	Log("Insert rejections     %10u\n", g_InsertRejectAccProb +
	  g_InsertRejectInCDSMod3Count + g_InsertCreatesStopCodonCount +
	  g_InsertInStartOrStopCodonCount);
	Log("  by acc prob         %10u\n", g_InsertRejectAccProb);
	Log("  CDS not mod 3       %10u\n", g_InsertRejectInCDSMod3Count);
	Log("  creates STOP codon  %10u\n", g_InsertCreatesStopCodonCount);
	Log("  into START or STOP  %10u\n", g_InsertInStartOrStopCodonCount);

	unsigned AnGeneCount = SIZE(g_AnGeneIndexes);
	unsigned EvGeneCount = SIZE(g_EvGeneIndexes);

	set<unsigned> DeletedGenes;
	set<unsigned> NewGenes;

	unsigned DeletedGeneCount = 0;
	for (set<unsigned>::const_iterator p = g_AnGeneIndexes.begin();
	  p != g_AnGeneIndexes.end(); ++p)
		{
		unsigned Index = *p;
		if (g_EvGeneIndexes.find(Index) == g_EvGeneIndexes.end())
			++DeletedGeneCount;
		}

	Log("\n");
	Log("Ancestral gene count       %5u\n", AnGeneCount);
	Log("Evolved gene count         %5u\n", EvGeneCount);
	Log("Gene count change          %+5d\n", (int) EvGeneCount - (int) AnGeneCount);
	Log("Gene duplications          %5u\n", g_GeneDupeCount);
	Log("Gene dupes into genes      %5u (%5.1f%%)\n", g_GeneInGeneInsertCount, IntronPct);
	Log("Non-intron dupes           %5u (%5.1f%%)\n", NonIntronCopies, NonIntronPct);
	Log("Disabled non-intron dupes  %5u (%5.1f%%)\n", DisabledCount, DisabledPct);
	Log("New paralogs               %5u (%5.1f%%)\n", g_ActiveGeneDupeCount, ParalogPct);
	Log("Deleted genes              %5u", DeletedGeneCount);
	if (DeletedGeneCount > g_ActiveGeneDupeCount)
		Log(" << WARNING!, more deleted than new paralogs\n");
	Log("\n");
	
	Log("\n");
	}

void LogCS(const CatStats &AnCS, const CatStats &EvCS)
	{
#define X(x)	AnCS.x##BaseCount, Pct(AnCS.x##BaseCount, AnCS.BaseCount),		\
				EvCS.x##BaseCount, Pct(EvCS.x##BaseCount, EvCS.BaseCount),		\
				(int) ((int64) EvCS.x##BaseCount - (int64) AnCS.x##BaseCount),	\
				ChgPct(EvCS.x##BaseCount, AnCS.x##BaseCount)

	Log("\n");
	Log("   Base Type   Ancestral    An.%%     Evolved    Ev.%%   Chg.Bases    Chg%%\n");
	Log("============  ==========  ======  ==========  ======  ==========  ======\n");

	Log("       Total  %10"PF"  100.0%%  %10"PF"  100.0%%  %10d  %6s\n",
	  AnCS.BaseCount, EvCS.BaseCount,
	  (int) ((int64) EvCS.BaseCount - (int64) AnCS.BaseCount),
	  ChgPct(EvCS.BaseCount, AnCS.BaseCount));
	Log("\n");
	Log(" Constrained  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Constrained));
	Log("     Neutral  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Neutral));
	Log("\n");
	Log("   Ancestral  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Ancestral));
	Log("      Copied  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Copy));
	Log("       Novel  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Novel));
	Log("\n");
	//Log("    Microsat  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Tandem));
	Log("\n");
	Log("         CpG  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(CpG));
	Log("\n");
	Log("        Gene  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Gene));
	Log("         UTR  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(UTR));
	Log("         CDS  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(CDS));
	Log("      Intron  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Intron));
	Log("      Splice  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(Splice));
	Log("         NXE  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(NXE));
	Log("         NGE  %10"PF"  %5.1f%%  %10"PF"  %5.1f%%  %10d  %6s\n", X(NGE));
	Log("\n");
	}

static char *rs(int i)
	{
	char *s = "?";
	switch (i)
		{
	case 0: s = "0"; break;
	case 1: s = "1-9"; break;
	case 2: s = "10-99"; break;
	case 3: s = "100-999"; break;
	case 4: s = "1000-9999"; break;
	case 5: s = "10000-99999"; break;
	case 6: s = "100000+"; break;
	case 7: s = "Min"; break;
	case 8: s = "Max"; break;
		}
	return s;
	}

void LogGeneModelLengthDist(const vector<gpos_t> &InitialIntronLengthDist,
  const vector<gpos_t> &InitialExonLengthDist,
  const vector<gpos_t> &FinalIntronLengthDist,
  const vector<gpos_t> &FinalExonLengthDist)
	{
	Log("\n");
	Log("Introns:\n");
	Log("Length range   Ancestral     Evolved\n");
	Log("============  ==========  ==========\n");

	for (int i = 0; i < 9; ++i)
		{
		if (i == 7)
			Log("\n");
		Log("%12.12s", rs(i));
		Log("  %10"PF, InitialIntronLengthDist[i]);
		Log("  %10"PF, FinalIntronLengthDist[i]);
		Log("\n");
		}

	Log("\n");
	Log("Exons:\n");
	Log("Length range   Ancestral     Evolved\n");
	Log("============  ==========  ==========\n");
	for (int i = 0; i < 9; ++i)
		{
		if (i == 7)
			Log("\n");
		Log("%12.12s", rs(i));
		Log("  %10"PF, InitialExonLengthDist[i]);
		Log("  %10"PF, FinalExonLengthDist[i]);
		Log("\n");
		}
	Log("\n");
	}
