#include <time.h>
#include "evolver.h"
#include "progress.h"
#include "myrand.h"
#include "evomodel.h"

bool DoCpGSweeps();
double GetCpGSweepDuration(double BranchLength);

// Init event count from branch length
void Evolver::InitBranchLength(double BranchLength)
	{
	asserta(m_AverageWaitingTime > 0);
	unsigned EventCount = (unsigned) (BranchLength/m_AverageWaitingTime + 0.5);
	InitEventCount(EventCount);
	}

// Init branch length & max event from event count
void Evolver::InitEventCount(unsigned EventCount)
	{
	asserta(m_AverageWaitingTime > 0);
	m_EventCount = EventCount;
	m_BranchLength = m_AverageWaitingTime*m_EventCount;
	}

void Evolver::Run()
	{
	Progress("T=%.5f, %u events", m_BranchLength, m_EventCount);
	ProgressStart("Evolving");

	for (; m_EventIndex <= m_EventCount; ++m_EventIndex)
		{
		if (m_EventIndex%10000 == 0)
			{
			double Tick = (m_BranchLength*m_EventIndex)/m_EventCount;
			double AncLen = GetAncestralLength();
			double Pct = (100.0*GetEvolvedLength())/AncLen;
			double ExtPct = (100.0*GetExtendedLength())/AncLen;
			SetProgressMsg("t=%.5f  L=%.1f%%  ExtL=%.1f%%",
			  Tick,
			  Pct,
			  ExtPct);
			ProgressStep(m_EventIndex, m_EventCount);
			}

		TryEvent();

		if (m_DoCpGSweeps)
			{
			++m_CpGSweepEventCounter;
			if (m_CpGSweepEventCounter >= m_CpGSweepInterval)
				{
				if (m_LogEvents)
					Log("@CPG SWEEP\n");
				//Progress("Start CpG sweep");
				CpG_Sweep(m_CpGSweepInterval*m_AverageWaitingTime);
				//Progress("CpG sweep done");
				m_CpGSweepEventCounter = 0;
				}
			}
		}
	CpG_Sweep(m_CpGSweepEventCounter*m_AverageWaitingTime);
	ProgressDone();

	m_LastAcceptedEvent = EVENT_TYPE_Invalid;
	m_LastAcceptedEventPos = GPOS_T_MAX;
	}
