#include "align4.h"

#define	TRACE	0

void AddOffsets(const AlnInfo &Region, AlnInfo &Hit)
	{
	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	const GCSeg &RegionSegA = Region.GetSeg(GenomeIndexA);
	const GCSeg &RegionSegB = Region.GetSeg(GenomeIndexB);

	GCSeg *ptrHitSegA = Hit.GetModifiableSegPtr(GenomeIndexA);
	GCSeg *ptrHitSegB = Hit.GetModifiableSegPtr(GenomeIndexB);

	if (ptrHitSegA == 0 || ptrHitSegB == 0)
		Die("AddOffsets, Seg=0");

	ptrHitSegA->lo += RegionSegA.lo;
	ptrHitSegA->hi += RegionSegA.lo;

	ptrHitSegB->lo += RegionSegB.lo;
	ptrHitSegB->hi += RegionSegB.lo;
	}

void AddOffsetsMinus(const AlnInfo &Region, coord_t LengthA,
  coord_t LengthB, AlnInfo &Hit)
	{
	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	const GCSeg &RegionSegA = Region.GetSeg(GenomeIndexA);
	const GCSeg &RegionSegB = Region.GetSeg(GenomeIndexB);

	GCSeg *ptrHitSegA = Hit.GetModifiableSegPtr(GenomeIndexA);
	GCSeg *ptrHitSegB = Hit.GetModifiableSegPtr(GenomeIndexB);

	if (ptrHitSegA == 0 || ptrHitSegB == 0)
		Die("AddOffsets, Seg=0");

	ptrHitSegA->Plus = true;
	ptrHitSegB->Plus = false;

	coord_t Alo = ptrHitSegA->lo;
	coord_t Ahi = ptrHitSegA->hi;

	coord_t Blo = ptrHitSegB->lo;
	coord_t Bhi = ptrHitSegB->hi;

	if (Alo >= LengthA || Ahi >= LengthA || Blo >= LengthB || Bhi >= LengthB ||
	  Alo > Ahi || Blo > Bhi)
		Die("A/B lo/hi Len");

	coord_t InvBlo = LengthB - Bhi - 1;
	coord_t InvBhi = LengthB - Blo - 1;

	assert(Alo <= Ahi);
	assert(InvBlo <= InvBhi);

	const ChrInfo &CIA = GetFilterChrInfoA();
	const ChrInfo &CIB = GetFilterChrInfoB();

	if (Ahi >= CIA.GetLength())
		Die("AddOffsetsMinus: overflows chr length");

	if (InvBhi >= CIB.GetLength())
		Die("AddOffsetsMinus: overflows chr length");

	ptrHitSegA->lo = Alo + RegionSegA.lo;
	ptrHitSegA->hi = Ahi + RegionSegA.lo;

	ptrHitSegB->lo = InvBlo + RegionSegB.lo;
	ptrHitSegB->hi = InvBhi + RegionSegB.lo;
	}

void AddOffsetsVec(const AlnInfo &Region, AIVec &AIs)
	{
	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		AlnInfo &AI = AIs[i];
		AddOffsets(Region, AI);
		}
	}

void AddOffsetsMinusVec(const AlnInfo &Region, coord_t LengthA, coord_t LengthB,
  AIVec &AIs)
	{
	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		AlnInfo &AI = AIs[i];
		AddOffsetsMinus(Region, LengthA, LengthB, AI);
		}
	}

void FilterAwareAppendAlignAILocal(const AlnInfo &UserAI, AIVec &UserHits, StdStrVec &Paths,
  AlignStyle Style)
	{
	assert(UserHits.size() == Paths.size());

	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	unsigned ChrIdA = GetFilterChrIdA();
	unsigned ChrIdB = GetFilterChrIdB();

	const GCSeg &UserSegA = UserAI.GetSeg(GenomeIndexA);
	const GCSeg &UserSegB = UserAI.GetSeg(GenomeIndexB);

	unsigned LengthA = UserSegA.GetLength();
	unsigned LengthB = UserSegB.GetLength();

	char *UserSeqA = myalloc(char, LengthA+1);
	char *UserSeqB = myalloc(char, LengthB+1);
	
	coord_t PosA = UserSegA.lo;
	coord_t PosB = UserSegB.lo;

	const RevFile &SeqRevFileA = GetSeqRevFileA();
	const RevFile &SeqRevFileB = GetSeqRevFileB();

	FilterAwareGetSeqData(SeqRevFileA, GenomeIndexA, ChrIdA, PosA, LengthA,
	  UserSeqA, true);

	FilterAwareGetSeqData(SeqRevFileB, GenomeIndexB, ChrIdB, PosB, LengthB,
	  UserSeqB, true);

	UserSeqA[LengthA] = 0;
	UserSeqB[LengthB] = 0;

	coord_t AlnLength = UserAI.GetAlnLength();
// Defense against absent or junk SegPair.AlnLength data in input
	if (AlnLength == UINT_MAX || AlnLength > 2*LengthA && AlnLength > 2*LengthB)
		AlnLength = 0;	// 0 means unknown

	unsigned Radius = CalcRadius((coord_t) LengthA, (coord_t) LengthB,
	  (coord_t) AlnLength);

#if	TRACE
	{
#define	MIN(a,b)	(((a) < (b)) ? (a) : (b))
	Log("\n");
	Log("FilterAwareAppendAlignAILocal UserAI=");
	LogAI(UserAI);
	Log("\n");
	Log("LA=%5u UserSeqA=%.*s", LengthA, MIN(LengthA, 64), UserSeqA);
	if (LengthA > 64)
		Log("...");
	Log("\n");
	Log("LB=%5u UserSeqB=%.*s", LengthB, MIN(LengthB, 64), UserSeqB);
	if (LengthB > 64)
		Log("...");
	Log("\n");
	Log("Radius=%u\n", Radius);
	}
#endif

	AIVec UserLocalHits;

	bool Strand = (UserSegA.Plus == UserSegB.Plus);

	bool Plus = false;
	bool Minus = false;
	if (Strand == true)
		{
		Plus = (Style == ALIGN_STYLE_PlusOnly || Style == ALIGN_STYLE_PlusAndMinus);
		Minus = (Style == ALIGN_STYLE_MinusOnly || Style == ALIGN_STYLE_PlusAndMinus);
		}
	else
		{
		Plus = (Style == ALIGN_STYLE_MinusOnly || Style == ALIGN_STYLE_PlusAndMinus);
		Minus = (Style == ALIGN_STYLE_PlusOnly || Style == ALIGN_STYLE_PlusAndMinus);
		}
	
	if (Plus)
		{
		AlignNWLocalAI(UserSeqA, LengthA, UserSeqB, LengthB, Radius, UserLocalHits, Paths);
		AddOffsetsVec(UserAI, UserLocalHits);
		}
	else
		{
		RevComp(UserSeqB, LengthB);
		AlignNWLocalAI(UserSeqA, LengthA, UserSeqB, LengthB, Radius, UserLocalHits, Paths);
		AddOffsetsMinusVec(UserAI, LengthA, LengthB, UserLocalHits);
		RevComp(UserSeqB, LengthB);
		}

	UserHits.insert(UserHits.end(), UserLocalHits.begin(), UserLocalHits.end());
	assert(UserHits.size() == Paths.size());

	myfree(UserSeqA);
	myfree(UserSeqB);

	assert(UserHits.size() == Paths.size());
	}

void FilterAwareAlignAIsLocal(const AIVec &UserAIs, AIVec &UserHits, StdStrVec &Paths,
  AlignStyle Style)
	{
	const unsigned AICount = (unsigned) UserAIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfo &UserAI = UserAIs[i];
		FilterAwareAppendAlignAILocal(UserAI, UserHits, Paths, Style);
		}
	}

void FilterAwareAppendAlignAIGlobal(const AlnInfo &UserAI, StdStr &Path)
	{
	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	unsigned ChrIdA = GetFilterChrIdA();
	unsigned ChrIdB = GetFilterChrIdB();

	const GCSeg &UserSegA = UserAI.GetSeg(GenomeIndexA);
	const GCSeg &UserSegB = UserAI.GetSeg(GenomeIndexB);

	unsigned LengthA = UserSegA.GetLength();
	unsigned LengthB = UserSegB.GetLength();

	char *UserSeqA = myalloc(char, LengthA);
	char *UserSeqB = myalloc(char, LengthB);
	
	coord_t PosA = UserSegA.lo;
	coord_t PosB = UserSegB.lo;

	const RevFile &SeqRevFileA = GetSeqRevFileA();
	const RevFile &SeqRevFileB = GetSeqRevFileB();

	bool Plus = (UserSegA.Plus == UserSegB.Plus);
	FilterAwareGetSeqData(SeqRevFileA, GenomeIndexA, ChrIdA, PosA, LengthA, UserSeqA, true);
	FilterAwareGetSeqData(SeqRevFileB, GenomeIndexB, ChrIdB, PosB, LengthB, UserSeqB, Plus);

	coord_t AlnLength = UserAI.GetAlnLength();

// Defense against absent or junk SegPair.AlnLength data in input
	if (AlnLength == UINT_MAX || AlnLength > 2*LengthA && AlnLength > 2*LengthB)
		AlnLength = 0;	// 0 means unknown

	unsigned Radius = CalcRadius((coord_t) LengthA, (coord_t) LengthB,
	  (coord_t) AlnLength);

#if	TRACE
	{
	Log("\n");
	Log("FilterAwareAppendAlignAIGlobal UserAI=");
	LogAI(UserAI);
	Log("\n");
	Log("LA=%5u UserSeqA=%.*s", LengthA, MIN(LengthA, 64), UserSeqA);
	if (LengthA > 64)
		Log("...");
	Log("\n");
	Log("LB=%5u UserSeqB=%.*s", LengthB, MIN(LengthB, 64), UserSeqB);
	if (LengthB > 64)
		Log("...");
	Log("\n");
	Log("Radius=%u\n", Radius);
	}
#endif

	NWBanded((unsigned char *) UserSeqA, LengthA, (unsigned char *) UserSeqB,
	  LengthB, Radius, Path);
	}

void FilterAwareAlignAIsGlobal(const AIVec &UserAIs, StdStrVec &Paths)
	{
	const unsigned AICount = (unsigned) UserAIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfo &UserAI = UserAIs[i];

		StdStr Path;
		FilterAwareAppendAlignAIGlobal(UserAI, Path);

		Paths.push_back(Path);
		}
	}
