#include "filter4.h"
#include "progress.h"

#define TRACE	0

static void AddToBundle(unsigned BundleId, const AlnInfo &AI, unsigned AIIndex,
  std::vector<AIVec> &Bundles, std::vector<IntVec> &BundleAIIndexes, IntVec &BundleIds,
  std::map<unsigned, unsigned> &BundleIdToIndex)
	{
#if	TRACE
	Log("AddToBundle Id=%u AIIndex=%u\n",
	  BundleId,
	  AIIndex);

	Log("Sizes: Bundles=%u BundlesAIIndexes=%u BundleIds=%u BundleIdToIndex=%u\n",
	  (unsigned) Bundles.size(),
	  (unsigned) BundleAIIndexes.size(),
	  (unsigned) BundleIds.size(),
	  (unsigned) BundleIdToIndex.size());
#endif
	std::map<unsigned, unsigned>::const_iterator p =
	  BundleIdToIndex.find(BundleId);

	if (p == BundleIdToIndex.end())
		{
#if	TRACE
		Log("Not found\n");
#endif
		assert(Bundles.size() == BundleIdToIndex.size());
		assert(Bundles.size() == BundleAIIndexes.size());
		assert(Bundles.size() == BundleIds.size());

		unsigned BundleIndex = (unsigned) BundleIdToIndex.size();
		std::map<unsigned, unsigned>::value_type v(BundleId, BundleIndex);
		BundleIdToIndex.insert(v);

		AIVec AV;
		AV.push_back(AI);
		Bundles.push_back(AV);

		IntVec IV;
		IV.push_back(AIIndex);
		BundleAIIndexes.push_back(IV);

		BundleIds.push_back(BundleId);

		assert(Bundles.size() == BundleIdToIndex.size());
		assert(Bundles.size() == BundleAIIndexes.size());
		assert(Bundles.size() == BundleIds.size());
#if	TRACE
		Log("After not found: Bundles=%u BundlesAIIndexes=%u BundleIds=%u BundleIdToIndex=%u\n",
	  (unsigned) Bundles.size(),
	  (unsigned) BundleAIIndexes.size(),
	  (unsigned) BundleIds.size(),
	  (unsigned) BundleIdToIndex.size());
#endif
		}
	else
		{
		unsigned BundleIndex = p->second;
		assert(BundleIndex < (unsigned) Bundles.size());
		assert(BundleIndex < (unsigned) BundleAIIndexes.size());

#if	TRACE
		Log("Found, BundleIndex=%u", BundleIndex);
		Log("Sizes: Bundles=%u BundlesAIIndexes=%u BundleIds=%u BundleIdToIndex=%u\n",
		  (unsigned) Bundles.size(),
		  (unsigned) BundleAIIndexes.size(),
		  (unsigned) BundleIds.size(),
		  (unsigned) BundleIdToIndex.size());
#endif
		AIVec &AIs = Bundles[BundleIndex];
#if	TRACE
		Log("AIs=%u\n", (unsigned) AIs.size());
#endif
		AIs.push_back(AI);

		IntVec &AIIndexes = BundleAIIndexes[BundleIndex];
#if	TRACE
		Log("AIIndexes=%u\n", (unsigned) AIIndexes.size());
#endif
		AIIndexes.push_back(AIIndex);
		}
	}

void FilterChrPair4(const RevFile &In)
	{
	BeforeChrPair_();

	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	unsigned ChrIdA = GetFilterChrIdA();
	unsigned ChrIdB = GetFilterChrIdB();

	Progress("Get AIIndexes");
	IntVec AIIndexes;
	In.GetAppendAlnInfoIndexesChr(GenomeIndexA, ChrIdA, AIIndexes);

	Progress("Get AIs");
	AIVec AIs;
	In.GetAlnInfos(AIIndexes, AIs);

	assert(AIs.size() == AIIndexes.size());

	Progress("Build bundles");

	std::vector<AIVec> Bundles;
	std::vector<IntVec> BundleAIIndexes;
	IntVec BundleIds;
	std::map<unsigned, unsigned> BundleIdToIndex;

	unsigned AICount = (unsigned) AIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfo &AI = AIs[i];

		assert(AI.GetSegPtr(GenomeIndexA) != 0);

		const GCSeg *SegB = AI.GetSegPtr(GenomeIndexB);
		if (SegB == 0)
			continue;
		if (SegB->ChrId != ChrIdB)
			continue;

		unsigned BundleId = AI.GetBundleId();
		AddToBundle(BundleId, AI, AIIndexes[i], Bundles, BundleAIIndexes, BundleIds,
		  BundleIdToIndex);
		}

	Progress("Filter");

	const unsigned BundleCount = (unsigned) Bundles.size();
	for (unsigned BundleIndex = 0; BundleIndex < BundleCount; ++BundleIndex)
		{
		unsigned BundleId = BundleIds[BundleIndex];
		SetFilterBundleId(BundleId);

		const AIVec &AIs = Bundles[BundleIndex];
		const IntVec &AIIndexes = BundleAIIndexes[BundleIndex];
		FilterWrapper4(AIs, AIIndexes);
		}

	AfterChrPair_();
	}
