#define TRACE	0

#include "evolver.h"

bool IsGeneFeature(const string &Feature);

const char *CETypeToStr(CE_TYPE Type)
	{
	switch (Type)
		{
#define C(c)	case CE_##c: return #c;
#include "cetypes.h"
		}
	static char Str[16];
	sprintf(Str, "CEType%u", Type);
	return Str;
	}

void Evolver::GenerateAnnots(FILE *f, const char *ChrName) const
	{
	map<unsigned, bool> GeneIndexToStrand;
	for (map<gpos_t, unsigned>::const_iterator p = m_GeneIndexes.begin();
	  p != m_GeneIndexes.end(); ++p)
		{
		gpos_t StartPos = p->first;
		unsigned GeneIndex = p->second;
		const BaseInfo &Base = GetBase(StartPos);
		bool Strand = Base.GetStrand();
		GeneIndexToStrand[GeneIndex] = Strand;
		}

	gpos_t EvStartPos = GPOS_T_MAX;
	gpos_t EvPos = 0;
	gpos_t StartPos = GPOS_T_MAX;
	gpos_t HomStartPos = GPOS_T_MAX;
	gpos_t IslandStartEvPos = GPOS_T_MAX;
	gpos_t IslandStartAncPos = GPOS_T_MAX;
	CE_TYPE PrevCEType = CE_Neutral;
TRC
	Log("     AcPos       EvPos  X  Cat  Pix    CE\n");
	Log("----------  ----------  -  ---  ---  ----\n");
ENDTRC

	IvTree<LoHi> EvGeneIx;
	GetGeneRangesEv(EvGeneIx);

	gpos_t Pos = m_FirstEvolvedPos;
	for (;;)
		{
		CE_TYPE CEType = CE_Neutral;
		BaseCategory Cat = BC_Ancestral;
		gpos_t HomPos = GPOS_T_MAX;
		bool IsIsland = false;
		byte ProbIx = NEUTRAL_PROBIX;
		char c = '?';
		if (Pos != UINT_MAX)
			{
			const BaseInfo &Base = m_Bases[Pos];
			IsIsland = Base.IsIsland();
			HomPos = GetHomologousPos(Pos);
			CEType = Base.GetCEType();
			Cat = GetCategory(Pos);
			ProbIx = Base.ProbIx;
			c = Base.GetLetter();
TRC
			if (Base.IsConstrained())
				{
				Log("%10"PF"  %10"PF"  %c  %3.3s  %3u  %4.4s",
				  Pos,
				  EvPos,
				  Base.GetLetter(),
				  BaseCategoryToStr(Cat),
				  Base.ProbIx,
				  CETypeToStr(CEType));
				Log("\n");
				}
ENDTRC
			}

		if (CEType != PrevCEType)
			{
			GenerateAnnot(f, ChrName, PrevCEType, StartPos, HomStartPos,
			  EvStartPos, EvPos - 1, EvGeneIx, GeneIndexToStrand);
			StartPos = Pos;
			HomStartPos = HomPos;
			EvStartPos = EvPos;
			}

		if (IsIsland)
			{
			if (IslandStartEvPos == GPOS_T_MAX)
				{
				IslandStartEvPos = EvPos;
				IslandStartAncPos = HomPos;
				}
			}
		else
			{
			if (IslandStartEvPos != GPOS_T_MAX)
				{
				GenerateIslandAnnot(f, ChrName, IslandStartEvPos, EvPos-1,
				  IslandStartAncPos);
				IslandStartEvPos = GPOS_T_MAX;
				IslandStartAncPos = GPOS_T_MAX;
				}
			}

		if (Pos == UINT_MAX)
			break;
		Pos = m_BaseListFwds[Pos];

		++EvPos;
		PrevCEType = CEType;
		}
	}

// GFF fields are:
// <seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
//     0         1         2        3      4      5        6       7         8           9

void Evolver::GenerateAnnot(FILE *f, const char *ChrName, CE_TYPE CEType,
  gpos_t Pos, gpos_t HomPos, gpos_t EvStartPos, gpos_t EvEndPos,
  const IvTree<LoHi> &EvGeneIx, const map<unsigned, bool> &GeneIndexToStrand) const
	{
	if (CEType == CE_Neutral || CEType == CE_Splice)
		return;
TRC
	Log("CE Pos=%"PF" EvStart=%"PF" EvEnd=%"PF"\n",
	  Pos, EvStartPos, EvEndPos);
ENDTRC
	const string &Feature = CETypeToStr(CEType);
	fprintf(f, "%s\tevo\t%s\t%"PF"\t%"PF"\t0",
	  ChrName,
	  Feature.c_str(),
	  EvStartPos + 1,
	  EvEndPos + 1);

	unsigned GeneIndex = UINT_MAX;
	bool Strand = true;
	if (IsGeneFeature(Feature))
		{
		vector<LoHi> LHs;
		pair<gpos_t, gpos_t> Iv(EvStartPos, EvEndPos);
		EvGeneIx.GetOverlaps(Iv, LHs);
		const unsigned GeneCount = SIZE(LHs);
		if (GeneCount != 1)
			{
			Log("\n");
			Log("Pos=%"PF" HomPos %"PF" EvStartPos %"PF" EvEndPos %"PF"\n",
			  Pos, HomPos, EvStartPos, EvEndPos);
			Log("Genes: ");
			for (unsigned i = 0; i < GeneCount; ++i)
				{
				const LoHi &LH = LHs[i];
				Log(" %"PF" - %"PF"(%u)", LH.Lo, LH.Hi, LH.Index);
				}
			Log("\n");
			Die("%s found in %u genes", CETypeToStr(CEType), GeneCount);
			}
		GeneIndex = LHs.front().Index;

		map<unsigned, bool>::const_iterator p = GeneIndexToStrand.find(GeneIndex);
		if (p == GeneIndexToStrand.end())
			Die("GeneIndexToStrand: index %u not found", GeneIndex);
		Strand = p->second;
		}

	if (CEType == CE_CDS)
		{
		const BaseInfo &Base = GetBase(Pos);
		asserta(Base.IsCDS());
		gpos_t Length = EvEndPos - EvStartPos + 1;
		bool Strand = Base.GetStrand();
		fprintf(f, "\t%c", pom(Strand));
		fprintf(f, "\t%d", GetCDSFrame(Pos, Length));
		}
	else if (CEType == CE_UTR)
		{
		fprintf(f, "\t%c", pom(Strand));
		fprintf(f, "\t.");
		}
	else
		fprintf(f, "\t.\t.");

	fprintf(f, "\t");
	fprintf(f, "anc_pos %"PF";", Pos);
	if (HomPos != Pos)
		fprintf(f, " hom_pos %"PF";", HomPos);

	if (GeneIndex != UINT_MAX)
		fprintf(f, " gene_index %u;", GeneIndex);

	gpos_t Length = EvEndPos - EvStartPos + 1;
	GenerateProbsAttr(f, Pos, Length);
	fprintf(f, "\n");
	}

void Evolver::GenerateProbsAttr(FILE *f, gpos_t Start, gpos_t Length) const
	{
	fprintf(f, " probs \"");
	gpos_t Pos = Start;
	for (gpos_t i = 0; i < Length; ++i)
		{
		const BaseInfo &Base = GetBase(Pos);
		fprintf(f, "%02x", Base.ProbIx);
		Pos = GetNextPos(Pos);
		}
	fprintf(f, "\";");
	}

void Evolver::GenerateIslandAnnot(FILE *f, const char *ChrName, gpos_t EvStartPos,
  gpos_t EvEndPos, gpos_t AncStartPos) const
	{
	fprintf(f, "%s\tevo\tisland\t%"PF"\t%"PF"\t0\t+\t.\t",
	  ChrName,
	  EvStartPos + 1,
	  EvEndPos + 1);
	fprintf(f, " anc_pos %"PF";", AncStartPos);
	fprintf(f, "\n");
	}
