#ifndef GENINFO_H
#define GENINFO_H

#include <vector>
#include <map>
#include <stdio.h>
#include "utils.h"
#include "chrinfo.h"
#include "iface.h"

const unsigned GenInfoStartMagic = 0x3E312F0;
const unsigned GenInfoEndMagic = 0x3E312F1;

class GenInfo
	{
public:
	GenInfo() {} ;
	GenInfo(const GenInfo &rhs);
	void Clear();
	void Create(const std::string &Name, const std::vector<ChrInfo> &Chrs);
	void Create(const std::string &Name);
	GenInfo &operator=(const GenInfo &rhs);

	void FromFile(FILE *f);
	void ToFile(FILE *f) const;

	unsigned AddChr(const std::string &Name, coord_t Length);
	void AddChr(unsigned Id, const std::string &Name, coord_t Length);
	void AddChr(const ChrInfo &CI, unsigned Id = UINT_MAX);

	const std::string &GetName() const;

	unsigned GetChrCount() const;

	const std::string &GetChrName(unsigned ChrId) const;
	const std::string *GetChrNamePtr(unsigned ChrId) const;
	unsigned GetChrId(const std::string &ChrName) const;
	unsigned GetChrId(unsigned ChrIndex) const;
	unsigned GetChrIndex(unsigned ChrId) const;

	void GetChrInfo(unsigned ChrId, ChrInfo &CI) const;
	const ChrInfo &GetChrInfo(const std::string &ChrName) const;
	const ChrInfo &GetChrInfo(unsigned ChrId) const;
	const ChrInfo *GetChrInfoPtr(unsigned ChrId) const;
	const ChrInfo *GetChrInfoPtr(const std::string &ChrName) const;

	const ChrInfo &GetChrInfoByIndex(unsigned ChrIndex) const;
	ChrInfo &GetModifiableChrInfoByIndex(unsigned ChrIndex);

	ChrInfo *GetModifiableChrInfoPtr(unsigned ChrId);
	ChrInfo *GetModifiableChrInfoPtr(const std::string &ChrName);

	bool GetChrId(const std::string &ChrName, unsigned *ptrChrId) const;

	unsigned GetUnusedChrId() const;
	bool operator==(const GenInfo &rhs) const;
	bool operator!=(const GenInfo &rhs) const { return !operator==(rhs); }
	coord_t GetLength() const;
	coord64_t GetLength64() const;
	void LogMe() const;

	void SetName(const std::string &Name);

private:
	std::string m_Name;
	std::vector<ChrInfo> m_Chrs;
	std::map<std::string, unsigned> m_NameToChrId;
	std::map<unsigned, unsigned> m_ChrIdToIndex;
	};

#endif	// GENINFO_H
