#include "utils.h"
#include "intlist.h"
#include "scoremx.h"
#include "align_params.h"

#define SUBST(cA, cB)		(DNAScoreMx[(unsigned char) (cA)][(unsigned char) (cB)])

#define TRACE	0

void LogRuns(const char *Row1, const char *Row2, unsigned /* Length */,
  const IntVec &RunStarts, const IntVec &RunEnds,
  const std::vector<score_t> &RunScores)
	{
	Log("Run  Start  End  RunScore  cc\n");
	Log("===  =====  ===  ========  ==\n");
	//   123  12345  123  12345678  12

	const unsigned RunCount = (unsigned) RunScores.size();
	for (unsigned RunIndex = 0; RunIndex < RunCount; ++RunIndex)
		{
		unsigned RunStart = RunStarts[RunIndex];
		unsigned RunEnd = RunEnds[RunIndex];

		Log("\n");
		Log("%3d  %5d  %3d  %8d  ", RunIndex, RunStart, RunEnd, RunScores[RunIndex]);
		for (unsigned Col = RunStart; Col <= RunEnd; ++Col)
			{
			if (Col > RunStart)
				Log("                           ");
			Log("%c%c\n", Row1[Col], Row2[Col]);
			}
		}
	}

score_t GetAlnRuns(const char *Row1, const char *Row2, unsigned Length,
  IntVec &RunStarts, IntVec &RunEnds, std::vector<score_t> &RunScores)
	{
	bool Gapping = false;
	bool NonNegative = false;
	score_t RunScore = 0;
	score_t TotalScore = 0;

#if TRACE
	Log("Col  cc  Gapping  NonNeg  ColScore  RunScore  RunCount\n");
	Log("===  ==  =======  ======  ========  ========  ========\n");
	//   123  12  1234567  123456  12345678  12345678  12345678
#endif

	for (unsigned Col = 0; Col < Length; ++Col)
		{
		char c1 = Row1[Col];
		char c2 = Row2[Col];

		score_t ColScore = 0;
		assert(c1 != '-' || c2 != '-');
		if (c1 == '-' || c2 == '-')
			{
			if (!Gapping)
				{
				ColScore = (Col == 0 ? TERMGAP : GAPOPEN);
				Gapping = true;
				}
			else
				ColScore = GAPEXTEND;
			}
		else
			{
			if (Gapping)
				Gapping = false;
			ColScore = SUBST(c1, c2);
			}

		if (Col == 0)
			{
			NonNegative = (ColScore >= 0);
			RunStarts.push_back(Col);
			}
		else
			{
			if (ColScore >= 0)
				{
				if (!NonNegative)
					{
					TotalScore += RunScore;
					NonNegative = true;
					RunEnds.push_back(Col-1);
					RunScores.push_back(RunScore);
					RunStarts.push_back(Col);
					RunScore = 0;
					}
				}
			else
				{
				if (NonNegative)
					{
					TotalScore += RunScore;
					NonNegative = false;
					RunEnds.push_back(Col-1);
					RunScores.push_back(RunScore);
					RunStarts.push_back(Col);
					RunScore = 0;
					}
				}
			}
		RunScore += ColScore;
#if	TRACE
		Log("%3d  %c%c  %7c  %6c  %8d  %8d  %8d\n",
			Col, c1, c2, Gapping ? 'T' : 'F', NonNegative ? 'T' : 'F', ColScore,
			RunScore, (int) RunScores.size());
#endif
		}

	if (Gapping)
		RunScore -= (GAPOPEN - TERMGAP);

	TotalScore += RunScore;
	RunScores.push_back(RunScore);
	RunEnds.push_back(Length-1);

	if (RunStarts.size() != RunEnds.size())
		Die("%u starts, %u ends", RunStarts.size(), RunEnds.size());
	if (RunStarts.size() != RunScores.size())
		Die("%u starts, %u scores", RunStarts.size(), RunScores.size());

#if	TRACE
	LogRuns(Row1, Row2, Length, RunStarts, RunEnds, RunScores);
#endif

	return TotalScore;
	}
