#ifndef GFF_H
#define GFF_H

#include "types.h"
#include "utils.h"

const size_t MAX_GFF_LINE = 64*1024 - 1;
const size_t MAX_GFF_FEATURE_LENGTH = 64*1024 - 1;

struct GFFRecord
	{
	const char *SeqName;
	const char *Source;
	const char *Feature;
	coord_t Start;
	coord_t End;
	float Score;
	char Strand;
	int Frame;
	const char *Attrs;

	coord_t GetLength() const
		{
		return End - Start + 1;
		}

	void LogMe() const
		{
		Log("%s %s %s Start=%"PF" End=%"PF" %c %s",
		  SeqName, Source, Feature, Start, End, Strand, Attrs);
		}
	};

bool GetNextGFFRecord(FILE *f, GFFRecord &Rec);
void ParseTargetAttrs(const char *Attrs, char SeqName[],
  int SeqNameBytes, coord_t *ptrStart, coord_t *ptrEnd);
extern bool g_KeepEdges;
extern unsigned g_Window;
extern unsigned g_MinusWindow;
bool ParseSPAttrs(const char *Attrs, unsigned *ptrSPIndex);
void ParseSPsAttrs(const char *Attrs, unsigned *ptrSPIndex1, unsigned *ptrSPIndex2);
bool ParseBundleAttrs(const char *Attrs, unsigned *ptrBundleIndex);
bool ParseAlnLenAttrs(const char *Attrs, unsigned *ptrAlnLen);
bool HasTargetAttrs(const char *Attrs);
void WriteGFFRec(FILE *f, const GFFRecord &Rec);
void SaveGFFStrings(GFFRecord &Rec);
void FreeGFFStrings(GFFRecord &Rec);
bool GFFGetIntAttr(const std::string &Attrs, const std::string &Name,
  unsigned &Value);
bool GFFGetAttr(const std::string &Attrs, const std::string &Name,
  std::string &Value);
unsigned GFFGetRequiredIntAttr(const std::string &Attrs,
  const std::string &Name);
void GFFGetRequiredAttr(const std::string &Attrs, const std::string &Name,
  std::string &Value);

#endif // GFF_H
