#ifndef GFF2_H
#define GFF2_H

#include "types.h"
#include "utils.h"

using namespace std;

struct GFFRecord2
	{
	string SeqName;
	string Source;
	string Feature;
	coord_t Start;
	coord_t End;
	float Score;
	char Strand;
	int Frame;
	string Attrs;

	bool operator<(const GFFRecord2 &rhs) const
		{
		if (SeqName != rhs.SeqName)
			return SeqName < rhs.SeqName;
		if (Start != rhs.Start)
			return Start < rhs.Start;
		return End < rhs.End;
		}

	GFFRecord2()
		{
		Clear();
		}

	coord_t GetLength() const
		{
		asserta(End >= Start);
		return End - Start + 1;
		}

	bool GetPlus() const
		{
		if (Strand == '+')
			return true;
		else if (Strand == '-')
			return false;
		Die("Invalid strand '%c' in GFF record", Strand);
		UNREACHABLE_RETURN(true);
		}

	void Clear()
		{
		SeqName.clear();
		Source.clear();
		Feature.clear();
		Start = COORD_T_MAX;
		End = COORD_T_MAX;
		Score = FLT_MAX;
		Strand = '?';
		Frame = INT_MAX;
		Attrs.clear();
		}

	void LogMe(bool NL = false) const
		{
		Log("%s %s %s Start=%"COORD_T_PRINTF" End=%"COORD_T_PRINTF,
		  SeqName.c_str(),
		  Source.c_str(),
		  Feature.c_str(),
		  Start,
		  End);
		if (NL)
			Log("\n");
		}
	};

bool GetNextGFFRecord(FILE *f, GFFRecord2 &Rec);
void WriteGFFRec(FILE *f, const GFFRecord2 &Rec);
bool GFFGetIntAttr(const string &Attrs, const string &Name, unsigned &Value);
bool GFFGetAttr(const string &Attrs, const string &Name, string &Value);
unsigned GFFGetRequiredIntAttr(const string &Attrs, const string &Name);
void GFFGetRequiredAttr(const string &Attrs, const string &Name, string &Value);
void GFFGetRequiredAttr(const string &Attrs, const string &Name, string &Value);
void GFFAppendIntAttr(string &Attrs, const string &Name, unsigned Value);
void GFFPrependIntAttr(string &Attrs, const string &Name, unsigned Value);
void GFFAppendStrAttr(string &Attrs, const string &Name, const string &Value);
void GFFPrependStrAttr(GFFRecord2 &Rec, const string &Name, const string &Value);
bool GFFDeleteAttr(string &Attrs, const string &Name);
void GFFAppendTargetAttrs(string &Attrs, const string &SeqName, coord_t Lo,
  coord_t Hi);

#endif // GFF2_H
