#ifndef hist_h
#define hist_h

#include <math.h>
#include <string>

using namespace std;

class HistData
	{
public:
	HistData()
		{
		Clear();
		};

	void Clear()
		{
		m_Name.clear();
		m_Counts.clear();
		m_TotalCount = 0;
		m_BinCount = 0;
		m_LoCount = 0;
		m_HiCount = 0;
		m_Min = 0;
		m_Max = 0;
		m_Sum = 0;
		m_Sum2 = 0;
		}

	void Init(const string &Name, unsigned BinCount = 10, double Min = 0,
	  double Max = 1)
		{
		asserta(BinCount > 0);
		asserta(Max > Min);

		Clear();

		m_Name = Name;
		m_BinCount = BinCount;
		m_Min = Min;
		m_Max = Max;
		m_LoCount = 0;
		m_HiCount = 0;
		m_Counts.resize(BinCount);
		}

	void Add(double x)
		{
		++m_TotalCount;

		m_Sum += x;
		m_Sum2 += x*x;

		if (x < m_Min)
			++m_LoCount;
		else if (x > m_Max)
			++m_HiCount;
		else
			{
		// r = 0 .. 1
			double r = (x - m_Min)/(m_Max - m_Min);
			unsigned BinIndex = (unsigned) (m_BinCount*r);
			if (BinIndex >= m_BinCount)
				BinIndex = m_BinCount - 1;
			++m_Counts[BinIndex];
			}
		}

	void LogHist() const
		{
		Log("\n");
		if (!m_Name.empty())
			Log("%s:\n", m_Name.c_str());
		Log("Mean %.3g, std.dev %.3g\n", GetMean(), GetStdDev());
		unsigned MaxCount = 0;
		for (unsigned i = 0; i < m_BinCount; ++i)
			{
			unsigned n = m_Counts[i];
			MaxCount = max(n, MaxCount);
			}

		if (m_LoCount > 0)
			{
			if (m_Min >= 0.0 && m_Max <= 99.0)
				Log("< %7.4f        ", m_Min);
			else
				Log("< %10.4g           ", m_Min);

			unsigned n = m_LoCount;
			Log("  %10u  ", n);
			unsigned Stars = (unsigned) (32*((double) n / (double) MaxCount));
			for (unsigned j = 0; j < Stars; ++j)
				Log("*");
			Log("\n");
			}

		double d = (m_Max - m_Min)/m_BinCount;
		for (unsigned i = 0; i < m_BinCount; ++i)
			{
			double xlo = m_Min + i*d;
			double xhi = m_Min + (i + 1)*d;
			if (m_Min >= 0.0 && m_Max <= 99.0)
				Log("%7.4f - %7.4f", xlo, xhi);
			else
				Log("%10.4g - %10.4g", xlo, xhi);

			unsigned n = m_Counts[i];
			Log("  %10u  ", n);
			unsigned Stars = (unsigned) (32*((double) n / (double) MaxCount));
			for (unsigned j = 0; j < Stars; ++j)
				Log("*");
			Log("\n");
			}

		if (m_HiCount > 0)
			{
			if (m_Min >= 0.0 && m_Max <= 99.0)
				Log("< %7.4f        ", m_Max);
			else
				Log("< %10.4g           ", m_Max);

			unsigned n = m_HiCount;
			Log("  %10u  ", n);
			unsigned Stars = (unsigned) (32*((double) n / (double) MaxCount));
			for (unsigned j = 0; j < Stars; ++j)
				Log("*");
			Log("\n");
			}
		}

	unsigned GetCount() const
		{
		return m_TotalCount;
		}

	double GetMean() const
		{
		if (m_TotalCount == 0)
			return 0;

		return m_Sum/m_TotalCount;
		}

	double GetStdDev() const
		{
		if (m_TotalCount == 0)
			return 0;
		double Mean = GetMean();
		return sqrt(m_Sum2/m_TotalCount - Mean*Mean);
		}

private:
	string m_Name;
	unsigned m_TotalCount;
	vector<unsigned> m_Counts;
	unsigned m_BinCount;
	unsigned m_LoCount;
	unsigned m_HiCount;
	double m_Min;
	double m_Max;
	double m_Sum;
	double m_Sum2;
	};

#endif // hist_h
