#ifndef IntList_H
#define IntList_H

#include <list>
#include <vector>
#include <set>
#include <map>

#include "lotypes.h"

typedef std::list<unsigned> IntList;
typedef IntList::iterator IntListIter;
typedef IntList::const_iterator CIntListIter;

typedef std::vector<unsigned> IntVec;
typedef std::vector<unsigned char> ByteVec;
typedef IntVec::iterator IntVecIter;
typedef IntVec::const_iterator CIntVecIter;
typedef ByteVec::const_iterator CByteVecIter;

typedef std::set<unsigned> IntSet;
typedef IntSet::iterator IntSetIter;
typedef IntSet::const_iterator CIntSetIter;

typedef std::map<unsigned, unsigned> IntMap;
typedef IntMap::iterator IntMapIter;
typedef IntMap::const_iterator CIntMapIter;

typedef std::vector<bool> BoolVec;
typedef std::pair<unsigned, unsigned> IntPair;
typedef std::pair<uint64, uint64> LongIntPair;

typedef std::vector<IntVec> IntVecVec;
typedef std::vector<BoolVec> BoolVecVec;
typedef std::list<bool> BoolList;

typedef std::vector<IntPair> IntPairVec;
typedef std::set<IntPair> IntPairSet;

// Warning: sorts A and B in place:
void IntersectIntLists2(IntList &A, IntList &B, IntList &Common);

// More expensive but safer (makes tmp copies of A and B):
void IntersectIntLists(const IntList &A, const IntList &B, IntList &Common);

void LogIntList(const IntList &List);
void LogIntVec(const IntVec &Vec);
void LogIntVecVec(const IntVecVec &IVV);
void LogByteVec(const ByteVec &Vec);

#define	for_IntList(L, p)		for (IntListIter p = L.begin(); p != L.end(); ++p)
#define	for_CIntList(L, p)		for (CIntListIter p = L.begin(); p != L.end(); ++p)

#define	for_IntVec(L, p)		for (IntVecIter p = L.begin(); p != L.end(); ++p)
#define	for_CIntVec(L, p)		for (CIntVecIter p = L.begin(); p != L.end(); ++p)

#define	for_CByteVec(L, p)		for (CByteVecIter p = L.begin(); p != L.end(); ++p)

#define	for_IntSet(L, p)		for (IntSetIter p = L.begin(); p != L.end(); ++p)
#define	for_CIntSet(L, p)		for (CIntSetIter p = L.begin(); p != L.end(); ++p)

#define	for_IntMap(L, p)		for (IntMapIter p = L.begin(); p != L.end(); ++p)
#define	for_CIntMap(L, p)		for (CIntMapIter p = L.begin(); p != L.end(); ++p)

#endif	// IntList_H
