#ifndef iovecs_h
#define iovecs_h

#include "utils.h"

using namespace std;

// These read/write functions are buffered for speed
// because otherwise were slow on both Win32 and Linux.
template<typename T>
void WriteVec(FILE *f, const vector<T> &v)
	{
	uint32 N = SIZE(v);
	WriteStdioFile(f, &N, sizeof(N));
	for (unsigned i = 0; i < N; ++i)
		{
		T u = v[i];
		WriteStdioFile(f, &u, sizeof(T));
		}
	}

template<typename T>
void ReadVec(FILE *f, vector<T> &v)
	{
	v.clear();
	uint32 N = UINT32_MAX;
	ReadStdioFile(f, &N, sizeof(N));
	v.reserve(N);
	for (unsigned i = 0; i < N; ++i)
		{
		T u;
		ReadStdioFile(f, &u, sizeof(T));
		v.push_back(u);
		}
	}

template<typename T>
void WriteIntMap(FILE *f, const T &m)
	{
	asserta(sizeof(uint32) >= sizeof(typename T::key_type));
	asserta(sizeof(uint32) >= sizeof(typename T::mapped_type));
	uint32 N = SIZE(m);
	WriteStdioFile(f, &N, sizeof(uint32));
	uint32 *Buffer = new uint32[2*N];
	size_t i = 0;
	for (typename T::const_iterator p = m.begin(); p != m.end(); ++p)
		{
		Buffer[i++] = p->first;
		Buffer[i++] = p->second;
		}
	asserta(i == 2*N);
	WriteStdioFile(f, Buffer, 2*N*sizeof(uint32));
	delete[] Buffer;
	}

template<typename T>
void ReadIntMap(FILE *f, T &m)
	{
	asserta(sizeof(uint32) >= sizeof(typename T::key_type));
	asserta(sizeof(uint32) >= sizeof(typename T::mapped_type));
	uint32 N = UINT32_MAX;
	ReadStdioFile(f, &N, sizeof(uint32));
	uint32 *Buffer = new uint32[2*N];
	ReadStdioFile(f, Buffer, 2*N*sizeof(uint32));
	size_t j = 0;
	for (unsigned i = 0; i < N; ++i)
		{
		uint32 Key = Buffer[j++];
		uint32 Value = Buffer[j++];
		asserta(m.find(Key) == m.end());
		m[Key] = Value;
		}
	delete[] Buffer;
	}

#endif // iovecs_h
