#include "revfile.h"

extern unsigned g_WindowLength;
extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

static inline unsigned Dist(coord_t a, coord_t b)
	{
	if (a >= b)
		return a - b;
	else
		return b - a;
	}

static bool CloseEnough(coord_t Alo, coord_t Ahi, coord_t Blo, coord_t Bhi)
	{
	if (Dist(Alo, Blo) <= g_WindowLength)
		return true;
	if (Dist(Alo, Bhi) <= g_WindowLength)
		return true;
	if (Dist(Ahi, Blo) <= g_WindowLength)
		return true;
	if (Dist(Ahi, Bhi) <= g_WindowLength)
		return true;
	return false;
	}

static bool CloseEnough(const GCSeg &Seg1, const GCSeg &Seg2)
	{
	return CloseEnough(Seg1.lo, Seg1.hi, Seg2.lo, Seg2.hi);
	}

// Hits are close enough to be in the same bundle if their closest
// bases in both genomes are <= window size apart.
bool LinkBundle(const AlnInfo &AI1, const AlnInfo &AI2)
	{
	const GCSeg &Seg1A = AI1.GetSeg(g_GenomeIndexA);
	const GCSeg &Seg1B = AI1.GetSeg(g_GenomeIndexB);
	const GCSeg &Seg2A = AI2.GetSeg(g_GenomeIndexA);
	const GCSeg &Seg2B = AI2.GetSeg(g_GenomeIndexB);

	bool Plus1 = (Seg1A.Plus == Seg1B.Plus);
	bool Plus2 = (Seg2A.Plus == Seg2B.Plus);

	if (Plus1 != Plus2)
		return false;

	return CloseEnough(Seg1A, Seg2A) && CloseEnough(Seg1B, Seg2B);
	}

bool LinkBundle(const AlnInfoX &AIX1, const AlnInfoX &AIX2)
	{
	const AlnInfo &AI1 = AIX1.AI;
	const AlnInfo &AI2 = AIX2.AI;
	return LinkBundle(AI1, AI2);
	}
