#include "utils.h"
#include "revfile.h"

void LogGCSeg(const GCSeg &Seg)
	{
	if (Seg.GenomeIndex == UINT_MAX)
		Log("*");
	else
		Log("%u", Seg.GenomeIndex);
	if (Seg.ChrId == UINT_MAX)
		Log(",*");
	else
		Log(",%u", Seg.ChrId);
	Log(" %u-%u(%u) %c",
	  (unsigned) Seg.lo,
	  (unsigned) Seg.hi,
	  Seg.GetLength(),
	  Seg.Plus ? '+' : '-');
	}

void LogAIN(const AlnInfo &AI)
	{
	LogAI(AI);
	Log("\n");
	}

void LogAIXN(const AlnInfoX &AIX)
	{
	LogAIX(AIX);
	Log("\n");
	}

void LogAI(const AlnInfo &AI)
	{
	const unsigned AttrCount = (unsigned) AI.Attrs.size();
	for (unsigned i = 0; i < AttrCount; ++i)
		{
		AlnAttr Attr = AI.Attrs[i];
		if (i > 0)
			Log(" ");
		Log("%s=", AlnAttrToStr(Attr));
		unsigned uAttr = (unsigned) Attr;
		if (uAttr < KnownAttrCount)
			{
			AlnAttrType AAT = AATypes[uAttr];
			switch (AAT)
				{
			case ALN_ATTR_TYPE_UInt32:
				{
				unsigned Value = AI.GetAttr(Attr);
				Log("%u", Value);
				break;
				}
			case ALN_ATTR_TYPE_Float32:
				{
				double Value = AI.GetFloatAttr(Attr);
				Log("%g", Value);
				break;
				}
			default:
				{
				unsigned Value = AI.Attrs[i];
				Log("0x%02x", Value);
				}
				}
			}
		else
			Log("%0x02x", AI.Values[i]);
		}
	const unsigned SegCount = (unsigned) AI.Segs.size();
	for (unsigned i = 0; i < SegCount; ++i)
		{
		const GCSeg &Seg = AI.Segs[i];
		Log(" ");
		LogGCSeg(Seg);
		}
	}

void LogAIX(const AlnInfoX &AIX)
	{
	Log("AIIx=%u", AIX.AlnInfoIndex);
	if (AIX.AlnIndex == UINT_MAX)
		Log(" AlnIx=* ");
	else
		Log(" AlnIx=%u ", AIX.AlnIndex);
	LogAI(AIX.AI);
	}

void LogAIs(const AIVec &AIs)
	{
	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfo &AI = AIs[i];
		Log("[%2u] ", i);
		LogAI(AI);
		Log("\n");
		}
	}

void LogAIXs(const AIXVec &AIXs)
	{
	const unsigned AICount = (unsigned) AIXs.size();
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfoX &AIX = AIXs[i];
		LogAIX(AIX);
		Log("\n");
		}
	}
