#include "utils.h"
#include "revfile.h"

const unsigned BLOCK_SIZE = 60;

unsigned GetRowLength(const std::string &Row)
	{
	const unsigned N = (unsigned) Row.size();
	unsigned Length = 0;
	for (unsigned i = 0; i < N; ++i)
		{
		char c = Row[i];
		if (c != '-')
			++Length;
		}
	return Length;
	}

void LogAIRows(const AlnInfo &AI, const StdStrVec &Rows, const RevFile *Rev)
	{
	const unsigned RowCount = (unsigned) Rows.size();
	const unsigned SegCount = (unsigned) AI.Segs.size();
	
	if (RowCount != SegCount)
		Die("LogAIRows: RowCount=%u SegCount=%u", RowCount, SegCount);

	if (RowCount == 0)
		{
		Log("Empty alignment\n");
		return;
		}

	const unsigned ColCount = (unsigned) Rows[0].size();
	const unsigned BlockCount = (ColCount + BLOCK_SIZE - 1)/BLOCK_SIZE;

	if (BlockCount == 0)
		{
		Log("%u segs but no cols", SegCount);
		return;
		}

	std::vector<coord_t> Coords;
	Coords.resize(RowCount);
	for (unsigned RowIndex = 0; RowIndex < RowCount; ++RowIndex)
		{
		const GCSeg &Seg = AI.Segs[RowIndex];
		Coords[RowIndex] = Seg.Plus ? Seg.lo : Seg.hi;
		}

	for (unsigned BlockIndex = 0; BlockIndex < BlockCount; ++BlockIndex)
		{
		for (unsigned RowIndex = 0; RowIndex < RowCount; ++RowIndex)
			{
			unsigned ColLo = BlockIndex*BLOCK_SIZE;
			unsigned ColHi = ColLo + BLOCK_SIZE - 1;
			if (ColHi >= ColCount)
				ColHi = ColCount - 1;

			const std::string &Row = Rows[RowIndex];
			bool AllBlank = true;

			for (unsigned ColIndex = ColLo; ColIndex <= ColHi; ++ColIndex)
				{
				char c = Row[ColIndex];
				if (c != '-')
					{
					AllBlank = false;
					break;
					}
				}

			coord_t StartCoord = Coords[RowIndex];
			coord_t EndCoord = Coords[RowIndex];

			if (BlockIndex > 0 && AllBlank)
				{
				--StartCoord;
				--EndCoord;
				}

			const GCSeg &Seg = AI.Segs[RowIndex];
			unsigned GenomeIndex = Seg.GenomeIndex;
			unsigned ChrId = Seg.ChrId;
			if (Rev != 0)
				{
				const std::string &GenName = Rev->GetGenomeName(GenomeIndex);
				const std::string &ChrName = Rev->GetChrName(GenomeIndex, ChrId);
				const std::string &Name = GenName + "." + ChrName;
				Log("%16.16s [%5u] %c %10u  ",
				  Name.c_str(),
				  ChrId,
				  Seg.Plus ? '+' : '-',
				  (unsigned) StartCoord);
				}
			else
				{
				char Name[64];
				sprintf(Name, "Gen[%u].Chr[%u]", Seg.GenomeIndex, Seg.ChrId);
				Log("%16.16s %c %10u  ",
				  Name,
				  Seg.Plus ? '+' : '-',
				  (unsigned) StartCoord);
				}

			for (unsigned ColIndex = ColLo; ColIndex <= ColHi; ++ColIndex)
				{
				char c = Row[ColIndex];
				Log("%c", c);
				if (c != '-')
					EndCoord = (Seg.Plus ? Coords[RowIndex]++ : Coords[RowIndex]--);
				}

			Log("  %-10u", (unsigned) EndCoord);
			if (BlockIndex == BlockCount - 1)
				{
				coord_t SegEndCoord = (Seg.Plus ? Seg.hi : Seg.lo);
				if (EndCoord != SegEndCoord)
					Log(" [ERROR: seglength=%u rowlength=%u]",
					  (unsigned) Seg.GetLength(),
					  SIZE(Row));
				unsigned Length = (Seg.hi - Seg.lo + 1);
				Log(" (%u)", Length);
				}
			Log("\n");
			}
		Log("\n");
		}
	}
