#include "align4.h"

#define TRACE	0

/***
Make a graph in which hits are nodes and holes are edges.
Connected components in this graph are monotonic chains,
i.e. in- and out-degree of each node is 1, except that
in-degree of first node and out-degree of last node is 0.
***/

void MakeChains4(unsigned HitCount, unsigned HoleCount, const IntVec &HoleToLoHit,
  const IntVec &HoleToHiHit, IntVecVec &Chains)
	{
#if	TRACE
	{
	Log("\n");
	Log("MakeChains()\n");
	Log("\n");
	Log("Hole  LoHit  HiHit\n");
	Log("====  =====  =====\n");
	for (size_t i = 0; i < HoleCount; ++i)
		Log("%4u  %5s  %5s\n",
		  (unsigned) i, UIntToStr(HoleToLoHit[i]), UIntToStr(HoleToHiHit[i]));
	}
#endif
	assert(HoleToLoHit.size() == HoleToHiHit.size());

	IntVecVec Edges;
	Edges.resize(HitCount);
	for (unsigned HoleIndex = 0; HoleIndex < HoleCount; ++HoleIndex)
		{
		unsigned LoHit = HoleToLoHit[HoleIndex];
		unsigned HiHit = HoleToHiHit[HoleIndex];

		assert(LoHit == UINT_MAX || LoHit < HitCount);
		assert(HiHit == UINT_MAX || HiHit < HitCount);

		if (LoHit != UINT_MAX && HiHit != UINT_MAX)
			{
			Edges[LoHit].push_back(HiHit);
#if	TRACE
			Log("Edge LoHit %3s -> HiHit %3s\n", UIntToStr(LoHit), UIntToStr(HiHit));
#endif
			}
		}

	GetConnectedComponents(Edges, Chains, false);

#if	TRACE
	{
	const unsigned ChainCount = Chains.GetVectorLength();
	Log("\n");
	Log("%u chains\n", ChainCount);
	Log("Chain  Hits\n");
	Log("=====  ====\n");
	for (unsigned ChainIndex = 0; ChainIndex < ChainCount; ++ChainIndex)
		{
		Log("\n");
		Log("%5u ", ChainIndex);
		const IntVec &HitIndexes = Chains[ChainIndex];
		for_CIntVec(HitIndexes, p)
			{
			unsigned HitIndex = *p;
			Log(" %u", HitIndex);
			}
		Log("\n");
		}
	}
#endif
	}
