#include "utils.h"
#include "getopts.h"
#include "fastastr.h"
#include "wordhist.h"
#include "progress.h"
#include "xlat.h"

void DoMakeHist(const char *SeqFileName, const char *HistFileName,
  ALPHA SeqAlpha, ALPHA HistAlpha)
	{
	if (SeqAlpha == ALPHA_Amino && HistAlpha == ALPHA_Nucleo)
		Die("Cannot make nucleotide histogram from amino acid sequence data");

	unsigned WordSize = RequiredIntOpt("wordsize");
	bool Append = FlagOpt("append");

	WordHist Hist;
	Hist.Init(HistAlpha, WordSize);

	if (Append)
		Hist.FromFile(HistFileName);

	bool Xlat = (SeqAlpha == ALPHA_Nucleo && HistAlpha == ALPHA_Amino);
	unsigned MerSize = (Xlat ? 3*WordSize : WordSize);

	FastaStream FS;
	FS.Open(SeqFileName, MerSize);

	ProgressStart("Counting %u-mers in %s", MerSize, SeqFileName);
	unsigned Steps = (unsigned) (FS.GetFileSize()/1000000 + 1);
	unsigned PrevStep = UINT_MAX;
	for (;;)
		{
		unsigned Step = (unsigned) (FS.GetFilePos()/1000000);
		if (Step != PrevStep)
			{
			ProgressStep(Step, Steps);
			PrevStep = Step;
			}

		uint64 Word;
		if (SeqAlpha == ALPHA_Amino)
			{
			uint64 Word;
			bool OK = FS.GetNextAminoWord(&Word);
			if (!OK)
				break;

			Hist.IncWordCount(Word);
			}
		else
			{
			bool OK = FS.GetNextNucWord(&Word);
			if (!OK)
				break;

			uint64 RCWord = RevCompNucWord(Word, WordSize);
			if (HistAlpha == ALPHA_Amino)
				{
				uint64 AminoWord;
				uint64 AminoRCWord;

				bool Ok = NucWordToAminoWord(Word, WordSize, &AminoWord);
				if (Ok)
					Hist.IncWordCount(AminoWord);
				Ok = NucWordToAminoWord(RCWord, WordSize, &AminoRCWord);
				if (Ok)
					Hist.IncWordCount(AminoRCWord);
				}
			else if (HistAlpha == ALPHA_Nucleo)
				{
				Hist.IncWordCount(Word);
				Hist.IncWordCount(RCWord);
				}
			else
				Die("Invalid HistAlpha=%d", HistAlpha);
			}
		}
	ProgressDone();

	Progress("Writing histogram file %s", HistFileName);

	uint64 GenomeSize = FS.GetTotalChars();
	Hist.ToFile(HistFileName, GenomeSize);
	}

void MakeHist()
	{
	const char *SeqFileName = RequiredValueOpt("makehist");
	const char *HistFileName = RequiredValueOpt("hist");
	bool Xlat = FlagOpt("xlat");
	bool Amino = FlagOpt("aminoseq");

	if (Xlat && Amino)
		Die("-xlat and -aminoseq are incompatible");

	ALPHA SeqAlpha = (Amino ? ALPHA_Amino : ALPHA_Nucleo);
	ALPHA HistAlpha = ((Amino || Xlat ) ? ALPHA_Amino : ALPHA_Nucleo);

	DoMakeHist(SeqFileName, HistFileName, SeqAlpha, HistAlpha);
	}
