#include "utils.h"
#include "wordhist.h"
#include "getopts.h"
#include "progress.h"
#include "masktab.h"
#include "xlat.h"
#include <math.h>

#define TRACE	0

unsigned CalcThreshold(unsigned WordSize, WordHist &WH1, WordHist &WH2);

extern double g_ThreshAlpha;

void DoMakeTab(const char *HistFileName1, const char *HistFileName2,
  const char *TabFileName)
	{
	WordHist WH1;
	WordHist WH2;

	Progress("Reading histogram file %s", HistFileName1);
	WH1.FromFile(HistFileName1);

	Progress("Reading histogram file %s", HistFileName2);
	WH2.FromFile(HistFileName2);

	ALPHA Alpha = WH1.GetAlpha();
	if (Alpha != WH2.GetAlpha())
		Die("Hist files have different alphabets");

	unsigned WordSize = WH1.GetWordSize();
	if (WordSize != WH2.GetWordSize())
		Die("Hist files have different word sizes %u, %u",
		  WordSize, WH2.GetWordSize());

	Progress("Computing threshold");

	unsigned Threshold = CalcThreshold(WordSize, WH1, WH2);
	double z = 0.0;
	double za = 0.0;
	double zb = 0.0;
	double GenomeLength1 = (double) WH1.GetGenomeSize();
	double GenomeLength2 = (double) WH2.GetGenomeSize();
	double k4 = (double) pow4(WordSize);
	if (g_ThreshAlpha == 0.0)
		{
		if (Threshold >= COUNT_TYPE_MAX)
			Die("Threshold %u too big, max is %u", Threshold, COUNT_TYPE_MAX);
		if (Threshold == 0)
			Warning("Threshold zero, all bases will be masked");
		}
	else
		{
		z = (double) ((2*GenomeLength1)/(GenomeLength1 + GenomeLength2));
		za = z/2;
		zb = (2 - z)/2;
		}

	MaskTable MT;
	MT.Init(Alpha, WordSize);

	uint64 SetCount = 0;
	uint64 FoundCount = 0;
	Progress("Building word table");
	double TotalPairs = 0;
	double PairsWithMasking = 0;
	uint64 WordCount = CalcWordCount(Alpha, WordSize);
	for (uint64 Word = 0; Word < WordCount; ++Word)
		{
		unsigned Count1 = WH1.GetCount(Word);
		unsigned Count2 = WH2.GetCount(Word);
		TotalPairs += Count1*Count2;
		if (Count1 > 0)
			++FoundCount;

		bool MaskThisWord = false;
		if (g_ThreshAlpha == 0.0)
			{
			double s = sqrt(Count1*Count2);
			MaskThisWord = (s >= Threshold);
			}
		else
			{
		// Mask if : 4^k  *  (X1/L1) ^ (z/2)  * (X2/L2) ^ ( (2-z)/2 ) > alpha
			double a = (double) Count1 / GenomeLength1;
			double b = (double) Count2 / GenomeLength2;
			double Value = k4*pow(a, za)*pow(b, zb);
			MaskThisWord = (Value > g_ThreshAlpha);
			}

		if (MaskThisWord)
			{
			++SetCount;
			MT.SetBit(Word);
			}
		else
			PairsWithMasking += Count1*Count2;
#if	TRACE
		{
		char TmpStr[64];
		Log("Word %s   Mask %c  Pairs %7g  Count1=%7u  Count2=%7u  s=%g\n",
		  NucWordToStr(Word, WordSize, TmpStr),
		  MaskThisWord ? 'T' : 'F',
		  PairsWithMasking,
		  Count1,
		  Count2,
		  s);
		}
#endif
		}

	double Pct = (SetCount*100.0)/FoundCount;
	double Ratio = (PairsWithMasking*100)/TotalPairs;
	ProgressNoRes("%s distinct words to be masked", uint64tostr(SetCount));
	ProgressNoRes(" = %.1f%% of distinct words in genome 1", Pct);
	ProgressNoRes("Seed pairs without masking %.3g", TotalPairs);
	ProgressNoRes("Seed pairs with masking %.3g pairs = %.1f %%", PairsWithMasking, Ratio);

	Progress("Writing masking table file %s", TabFileName);
	MT.ToFile(TabFileName);
	}

void MakeTab()
	{
	const char *HistFileName1 = RequiredValueOpt("hist1");
	const char *HistFileName2 = RequiredValueOpt("hist2");
	const char *TabFileName = RequiredValueOpt("tab");
	DoMakeTab(HistFileName1, HistFileName2, TabFileName);
	}
