#include "alninfo.h"
#include <math.h>

extern unsigned g_DiagFactor;
extern unsigned g_LenFactor;
extern unsigned g_OvFactor;

extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

// Distance function between two AIs for micro-chaining.
// Weighted combination of difference in diagonals,
// length of gaps between them and length of overlaps.
unsigned MicroChainDistance(const AlnInfo &AI1, const AlnInfo &AI2)
	{
	const GCSeg &Seg1A = AI1.GetSeg(g_GenomeIndexA);
	const GCSeg &Seg2A = AI2.GetSeg(g_GenomeIndexA);
	const GCSeg &Seg1B = AI1.GetSeg(g_GenomeIndexB);
	const GCSeg &Seg2B = AI2.GetSeg(g_GenomeIndexB);

	bool Plus_1 = (Seg1A.Plus == Seg1B.Plus);
	bool Plus_2 = (Seg2A.Plus == Seg2B.Plus);

	assert(Plus_1 == Plus_2);

//	coord_t Alo0 = Seg1A.lo;
	coord_t Ahi0 = Seg1A.hi;
	coord_t Blo0 = Seg2A.lo;
//	coord_t Bhi0 = Seg2A.hi;

	coord_t Alo1 = Seg1B.lo;
	coord_t Ahi1 = Seg1B.hi;
	coord_t Blo1 = Seg2B.lo;
	coord_t Bhi1 = Seg2B.hi;

// d0 is genome 0 distance between AIs, similarly for d1.
// So gap between AIs is itself a segment pair of size d0 x d1.
	long d0 = (long) Blo0 - (long) Ahi0;
	long d1 = 0;
	if (Plus_1)
		d1 = (long) Blo1 - (long) Ahi1;
	else
		d1 = (long) Alo1 - (long) Bhi1;

	long negd0 = (d0 < 0) ? labs(d0) : 0;
	long negd1 = (d1 < 0) ? labs(d1) : 0;

// dDiag = difference in diagonal
// (Note this works OK for reversed & not reversed).
	long dDiag = labs(d0 - d1);

// Len is length of gap (averaged over the two genomes)
	long dLen = labs((d0 + d1)/2);
	long Ov = negd0 + negd1;

	return g_DiagFactor*dDiag + g_LenFactor*dLen + g_OvFactor*Ov;
	}
