#include "revfile.h"
#include "filter4.h"

#define TRACE	0

bool LinkBundle(const AlnInfo &AI1, const AlnInfo &AI2);
coord_t GetOverlapLength(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex);
coord_t GetAILength(const AlnInfo &AI);

double g_MaxOverlapFract = 0.2;

static unsigned g_GenomeIndexA;
static unsigned g_GenomeIndexB;

// Return ratio of larger overlap of the AIs (considering
// the two genomes) to the length of the shorter AI.
static double MaxOverlapFract(const AlnInfo &AIA, const AlnInfo &AIB)
	{
	double Ov0 = (double) GetOverlapLength(AIA, AIB, 0);
	double Ov1 = (double) GetOverlapLength(AIA, AIB, 1);
	double MaxOv = std::max(Ov0, Ov1);

	double LengthA = (double) GetAILength(AIA);
	double LengthB = (double) GetAILength(AIB);
	double MinLength = std::min(LengthA, LengthB);

	if (MinLength == 0)
		MinLength = 1;
	
	return MaxOv/MinLength;
	}

/***
Successor is true if AIB follows AIA w.r.t. given genome.
Allows for some overlap, like this:

		____________ AIB
	_____ AIA

***/
static bool Successor(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex)
	{
	assert(GenomeIndex == 0 || GenomeIndex == 1);
	if (AIA.GetSeg(g_GenomeIndexA).Plus != AIB.GetSeg(g_GenomeIndexB).Plus)
		return false;

	const GCSeg &SegA = AIA.GetSeg(GenomeIndex);
	const GCSeg &SegB = AIB.GetSeg(GenomeIndex);
	double LenA = SegA.hi - SegA.lo + 1;

	if (GenomeIndex == 0 || AIA.GetSeg(g_GenomeIndexA).Plus)
		{
		if (SegB.lo > SegA.hi)
			return true;

		double d = SegA.hi - SegB.lo;
		if (d/LenA > g_MaxOverlapFract)
			return false;

		return true;
		}
	else
		{
		assert(GenomeIndex == 1 && !AIA.GetSeg(g_GenomeIndexA).Plus);

		if (SegB.hi < SegA.lo)
			return true;

		double d = SegA.lo - SegB.hi;
		if (d/LenA > g_MaxOverlapFract)
			return false;

		return true;
		}
	}

static bool Successor(const AlnInfo &AI1, const AlnInfo &AI2)
	{
	bool b = Successor(AI1, AI2, 0) && Successor(AI1, AI2, 1);
	return b;
	}

bool MicroChainLinkAIs(const AlnInfo &AIA, const AlnInfo &AIB)
	{
#if	TRACE
	Log("MicroChainLinkAIs(");
	LogAI(AIA);
	Log(", ");
	LogAI(AIB);
	Log("\n");
#endif
	g_GenomeIndexA = GetFilterGenomeIndexA();
	g_GenomeIndexB = GetFilterGenomeIndexB();

	if (!LinkBundle(AIA, AIB))
		{
#if	TRACE
		Log("  FALSE: !LinkBundle\n");
#endif
		return false;
		}

	if (MaxOverlapFract(AIA, AIB) > g_MaxOverlapFract)
		{
#if	TRACE
		Log("  FALSE: MaxOverlapFract=%g > g_MaxOverlapFract=%g\n",
		  MaxOverlapFract(AIA, AIB), g_MaxOverlapFract);
#endif
		return false;
		}

	if (!Successor(AIA, AIB))
		{
#if	TRACE
		Log("  FALSE: !Successor\n");
#endif
		return false;
		}

#if	TRACE
		Log("  TRUE\n");
#endif
	return true;
	}
