#include "utils.h"
#include "revfile.h"
#include "getopts.h"
#include "progress.h"

// NOTE: ALMOST IDENTICAL TO Bundle4, SHOULD CONSOLIDATE
// ONLY DIFFERENCE IS (a) Link FUNCTION FOR MakeEdges
// AND (b) MERGE HITS INSTEAD OF ANNOTATING WITH
// BUNDLE ID. 98% OF CODE IS THE SAME.

void MergeHitsChrA(const RevFile &In, unsigned ChrIdA, const AIVec &AIs,
  RevFile &Out);

extern bool g_ShowProgress;

unsigned g_WindowLength = 4000;
unsigned g_MaxDiagDiff = 100;
unsigned g_DiagDiffAlwaysMerge = 24;
unsigned g_Margin = 0;
double g_MaxDiagFract = 0.25;

unsigned g_GenomeIndexA = 0;
unsigned g_GenomeIndexB = 1;

bool LTByChrIdB(const AlnInfo &AI1, const AlnInfo &AI2)
	{
	const GCSeg &Seg1 = AI1.GetSeg(g_GenomeIndexB);
	const GCSeg &Seg2 = AI2.GetSeg(g_GenomeIndexB);

	return Seg1.ChrId < Seg2.ChrId;
	}

int main(int argc, char *argv[])
	{
	ProcessArgVect(argc, argv);

	const char *InFileName = RequiredValueOpt("in");
	const char *OutFileName = RequiredValueOpt("out");

	IntOpt("window", &g_WindowLength);
	IntOpt("margin", &g_Margin);
	IntOpt("maxdiagdiff", &g_MaxDiagDiff);
	IntOpt("diagdiffalwaysmerge", &g_DiagDiffAlwaysMerge);
	FloatOpt("maxdiagfract", &g_MaxDiagFract);

	g_ShowProgress = !FlagOpt("quiet");

	RevFile In;
	RevFile Out;

	In.Open(InFileName);

	const std::vector<GenInfo> &Genomes = In.GetGenomes();

	const GenInfo &GIA = In.GetGenomeInfo(g_GenomeIndexA);
//	const GenInfo &GIB = In.GetGenomeInfo(g_GenomeIndexB);

	const std::string &GenomeNameA = In.GetGenomeName(g_GenomeIndexA);
	const std::string &GenomeNameB = In.GetGenomeName(g_GenomeIndexB);

	unsigned ChrCountA = In.GetChrCount(g_GenomeIndexA);
	unsigned ChrCountB = In.GetChrCount(g_GenomeIndexB);

	Progress("Genome %.16s has %u chrs", GenomeNameA.c_str(), ChrCountA);
	Progress("Genome %.16s has %u chrs", GenomeNameB.c_str(), ChrCountB);

	Out.Create(OutFileName, Genomes);

	for (unsigned ChrIndexA = 0; ChrIndexA < ChrCountA; ++ChrIndexA)
		{
		const ChrInfo &CIA = GIA.GetChrInfoByIndex(ChrIndexA);
		unsigned ChrIdA = CIA.GetId();

		double Pct = ChrIndexA*100.0/ChrCountA;
		const std::string &ChrNameA = GIA.GetChrName(ChrIdA);

		Progress("Bundling chr %u of %u %.2f%% %.16s",
		  ChrIndexA, ChrCountA, Pct, ChrNameA.c_str());

		Progress("Get AIIndexes");
		IntVec AIIndexes;
		In.GetAppendAlnInfoIndexesChr(g_GenomeIndexA, ChrIdA, AIIndexes);

		Progress("Get AIIs");
		AIVec AIs;
		In.GetAlnInfos(AIIndexes, AIs);

		std::sort(AIs.begin(), AIs.end(), LTByChrIdB);
		MergeHitsChrA(In, ChrIdA, AIs, Out);
		}

	Progress("Closing files");
	In.Close();
	Out.Close();

	ProgressExit();
	return 0;
	}
