#include "utils.h"
#include "revfile.h"
#include "progress.h"

// NOTE: THIS IS IDENTICAL TO BundleChrA.cpp EXCEPT
// BundleChrPair <-> MergeHitsChrPair
// SHOULD CONSOLIDATE.

void MergeHitsChrPair(const RevFile &In, unsigned ChrIdA, unsigned ChrIdB,
  const AIVec &ChrPairAIs, RevFile &Out);

extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

static unsigned GetChrIdB(const AlnInfo &AI)
	{
	const GCSeg &Seg = AI.GetSeg(g_GenomeIndexB);
	return Seg.ChrId;
	}

static void AssertSorted(const AIVec &AIs)
	{
	unsigned AICount = (unsigned) AIs.size();
	for (unsigned AIIndex = 1; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &PrevAI = AIs[AIIndex - 1];
		const AlnInfo &ThisAI = AIs[AIIndex];

		unsigned PrevChrIdB = GetChrIdB(PrevAI);
		unsigned ThisChrIdB = GetChrIdB(ThisAI);

		if (PrevChrIdB > ThisChrIdB)
			Die("Not sorted");
		}
	}

void MergeHitsChrA(const RevFile &In, unsigned ChrIdA, const AIVec &AIs, RevFile &Out)
	{
	unsigned AICount = (unsigned) AIs.size();
	if (AICount == 0)
		return;

	AssertSorted(AIs);

	unsigned BundledCount = 0;
	unsigned PrevChrIdB = UINT_MAX;
	unsigned StartIndex = UINT_MAX;
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];
		unsigned ChrIdB = GetChrIdB(AI);
		if (ChrIdB != PrevChrIdB)
			{
			if (StartIndex != UINT_MAX)
				{
				AIVec::const_iterator ptrStart = AIs.begin() + StartIndex;
				AIVec::const_iterator ptrEnd = AIs.begin() + AIIndex;

				AIVec ChrPairAIs;
				ChrPairAIs.insert(ChrPairAIs.end(), ptrStart, ptrEnd);

				MergeHitsChrPair(In, ChrIdA, PrevChrIdB, ChrPairAIs, Out);

				BundledCount += (unsigned) ChrPairAIs.size();
				}
			StartIndex = AIIndex;
			}
		PrevChrIdB = ChrIdB;
		}

	AIVec::const_iterator ptrStart = AIs.begin() + StartIndex;
	AIVec::const_iterator ptrEnd = AIs.end();

	AIVec ChrPairAIs;
	ChrPairAIs.insert(ChrPairAIs.end(), ptrStart, ptrEnd);

	MergeHitsChrPair(In, ChrIdA, PrevChrIdB, ChrPairAIs, Out);

	BundledCount += (unsigned) ChrPairAIs.size();

	if (BundledCount != AICount)
		Die("Bundled count = %u, != AICount = %u", BundledCount, AICount);
	}
