#include "revfile.h"

#include "utils.h"
#include "getopts.h"
#include "revfile.h"
#include "progress.h"

void ParseStrVec(const std::string &In, std::vector<std::string> &Out, char c = ',');
const std::vector<GenInfo> &CombineGenomes(
  const std::vector<const RevFile *> &RevFiles);
void ConvertGenIndexesToCombined(AlnInfo &AI, unsigned FileIndex);

// Append one input file to output file
static void AppendFile(const RevFile &InFile, unsigned FileIndex, RevFile &OutFile)
	{
	const unsigned AlnInfoCount = InFile.GetAlnInfoCount();
	ProgressStart("Copying %u AIs", AlnInfoCount);
	for (unsigned AlnInfoIndex = 0; AlnInfoIndex < AlnInfoCount; ++AlnInfoIndex)
		{
		if (AlnInfoIndex%1000 == 0)
			ProgressStep(AlnInfoIndex, AlnInfoCount);
		AlnInfo AI;
		unsigned AlnIndex = InFile.GetAlnInfo(AlnInfoIndex, AI);
		ConvertGenIndexesToCombined(AI, FileIndex);
		if (AlnIndex == UINT_MAX)
			OutFile.AddAlnInfo(AI);
		else
			{
			std::vector<std::string> Rows;
			unsigned AlnIndex2 = InFile.GetAlnRows(AlnIndex, Rows);
			if (AlnIndex2 != AlnIndex)
				Die("AlnIndex2 != AlnIndex");
			OutFile.AddAln(AI, Rows);
			}
		}
	ProgressDone();
	}

void MergeRev2()
	{
	const char *cstrInputFileNames = RequiredValueOpt("mergerev2");
	const char *OutputFileName = RequiredValueOpt("out");

	StdStrVec InputFileNames;
	ParseStrVec(cstrInputFileNames, InputFileNames);
	const unsigned InFileCount = SIZE(InputFileNames);

// Open input files
	std::vector<RevFile> InFiles;
	std::vector<const RevFile *> InFilePtrs;

	InFiles.resize(InFileCount);
	InFilePtrs.resize(InFileCount);

	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		{
		RevFile &InFile = InFiles[InFileIndex];
		InFilePtrs[InFileIndex] = &InFile;
		const std::string &FileName = InputFileNames[InFileIndex];
		InFile.Open(FileName);
		}

	const std::vector<GenInfo> &Genomes = CombineGenomes(InFilePtrs);

// Create output file
	RevFile OutFile;
	OutFile.Create(OutputFileName, Genomes);

// Copy data
	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		{
		RevFile &InFile = InFiles[InFileIndex];
		Progress("File %u of %u (%s)",
		  InFileIndex + 1,
		  InFileCount,
		  InFile.GetFileName().c_str());
		AppendFile(InFile, InFileIndex, OutFile);
		}

// Clean up
	Progress("Closing files and copying data");
	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		InFiles[InFileIndex].Close();
	OutFile.Close();
	}
