#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "getopts.h"
#include <string>

using namespace std;

void ParseStrVec(const string &In, vector<string> &Out, char c = ',');

static vector<RevFile> g_RevFiles;

// Merge sequences from a set of rev files.
void MergeRevSeqs()
	{
	const char *cstrRevFileNames = RequiredValueOpt("mergerevseqs");
	const char *OutFileName = RequiredValueOpt("out");
	unsigned GenomeIndex = 0;
	IntOpt("genid", &GenomeIndex);

	vector<string> RevFileNames;
	ParseStrVec(cstrRevFileNames, RevFileNames);

	if (RevFileNames.empty())
		Die("No rev files specified");

	const unsigned RevFileCount = (unsigned) RevFileNames.size();
	if (RevFileCount == 0)
		Die("No rev files specified");

	GenInfo GI;

	string GenomeName = "";
	g_RevFiles.resize(RevFileCount);
	unsigned ChrCount = 0;
	for (unsigned RevFileIndex = 0; RevFileIndex < RevFileCount; ++RevFileIndex)
		{
		RevFile &RevFile = g_RevFiles[RevFileIndex];
		const string &FileName = RevFileNames[RevFileIndex];
		Progress("Opening %.32s", FileName.c_str());
		RevFile.Open(FileName);

		if (RevFileIndex == 0)
			GenomeName = RevFile.GetGenomeName(GenomeIndex);
		else
			{
			if (RevFile.GetGenomeName(GenomeIndex) != GenomeName)
				Die("Rev files have different genomes with id %u: %s %s",
				  GenomeIndex,
				  GenomeName.c_str(),
				  RevFile.GetGenomeName(GenomeIndex).c_str());
			}

		unsigned SeqCount = RevFile.GetSeqCount();
		for (unsigned SeqIndex = 0; SeqIndex < SeqCount; ++SeqIndex)
			{
			unsigned GenomeIndex2;
			unsigned ChrId;
			RevFile.GetSeqGenChrByIndex(SeqIndex, &GenomeIndex2, &ChrId);
			if (GenomeIndex2 != GenomeIndex)
				continue;

			const string &ChrName = RevFile.GetChrName(0, ChrId);

			unsigned Id2;
			if (GI.GetChrId(ChrName, &Id2))
				Die("Chr %s found twice", ChrName.c_str());
			unsigned ChrLength = RevFile.GetSeqLengthByIndex(SeqIndex);
			GI.AddChr(ChrName, ChrLength);

			++ChrCount;
			}
		}

	//vector<GenInfo> Genomes;
	//Genomes.push_back(GI);

	RevFile OutRev;
	OutRev.Create(OutFileName);
	ProgressStart("Copying sequences");
	unsigned Counter = 0;
	for (unsigned RevFileIndex = 0; RevFileIndex < RevFileCount; ++RevFileIndex)
		{
		const RevFile &RevFile = g_RevFiles[RevFileIndex];

		unsigned SeqCount = RevFile.GetSeqCount();
		for (unsigned SeqIndex = 0; SeqIndex < SeqCount; ++SeqIndex)
			{
			ProgressStep(Counter++, ChrCount);
			unsigned GenomeIndex2;
			unsigned ChrId;
			RevFile.GetSeqGenChrByIndex(SeqIndex, &GenomeIndex2, &ChrId);
			if (GenomeIndex2 != GenomeIndex)
				continue;

			const string &ChrName = RevFile.GetChrName(0, ChrId);
			unsigned ChrLength = RevFile.GetSeqLengthByIndex(SeqIndex);
			char *Seq = new char[ChrLength];
			if (Seq == 0)
				Die("Out of memory");

			RevFile.GetSeq(GenomeIndex, ChrId, Seq, ChrLength);

			OutRev.AddSeq(GenomeName, ChrName, Seq, ChrLength);

			delete[] Seq;
			}
		}
	ProgressDone();

	Progress("Finalizing output file");
	OutRev.Close();
	Progress("Done.");
	}
