#include "utils.h"
#include "getopts.h"
#include "progress.h"
#include "revfile.h"

static unsigned g_GenomeIndex;

static bool LT(const AlnInfoX &AIX1, const AlnInfoX &AIX2)
	{
	const GCSeg *Seg1 = AIX1.AI.GetSegPtr(g_GenomeIndex);
	const GCSeg *Seg2 = AIX2.AI.GetSegPtr(g_GenomeIndex);

	if (Seg1 != 0 && Seg2 != 0)
		return Seg1->lo < Seg2->lo;
	return false;
	}

void MStats()
	{
	const char *FileName = RequiredValueOpt("mstats");
	const char *GenomeName = ValueOpt("genome");

	g_GenomeIndex = 0;

	RevFile In;
	In.Open(FileName);

	if (GenomeName != 0)
		g_GenomeIndex = In.GetGenomeIndex(GenomeName);

	const char *RefGenomeName = In.GetGenomeName(g_GenomeIndex).c_str();

	const unsigned GenomeCount = In.GetGenomeCount();
	vector<unsigned> Los(GenomeCount, UINT_MAX);
	vector<unsigned> His(GenomeCount, UINT_MAX);
	vector<unsigned> BaseCountPlus(GenomeCount, 0);
	vector<unsigned> BaseCountMinus(GenomeCount, 0);
	vector<unsigned> MoveCount(GenomeCount, 0);
	vector<vector<unsigned> > Coords(GenomeCount);

	vector<AlnInfoX> AIXs;
	In.GetAllAlnInfoXs(AIXs);

	sort(AIXs.begin(), AIXs.end(), LT);

	Log("\n");
	Log("     AI          lo          hi  +  Genome\n");
	Log("-------  ----------  ----------  -  ------\n");

	const unsigned AICount = SIZE(AIXs);
	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfoX &AIX = AIXs[i];
		const AlnInfo &AI = AIX.AI;
		if (i > 0)
			Log("\n");
		const unsigned SegCount = AI.GetSegCount();
		const GCSeg *Seg = AI.GetSegPtr(g_GenomeIndex);
		if (Seg == 0)
			{
			Log("%7u  %10.10s  %10.10s  .  %s\n", AIX.AlnInfoIndex, "-", "-", RefGenomeName);
			continue;
			}
		bool RefPlus = Seg->Plus;
		Log("%7u  %10u  %10u  +  %s\n", AIX.AlnInfoIndex, Seg->lo, Seg->hi, RefGenomeName);
		for (unsigned SegIndex = 0; SegIndex < SegCount; ++SegIndex)
			{
			const GCSeg *Seg = &AI.Segs[SegIndex];
			unsigned GenomeIndex = Seg->GenomeIndex;
			asserta(GenomeIndex < GenomeCount);
			Coords[GenomeIndex].push_back(Seg->lo);
			bool Plus = (Seg->Plus == RefPlus);
			if (Los[GenomeIndex] == UINT_MAX || Seg->lo < Los[GenomeIndex])
				Los[GenomeIndex] = Seg->lo;
			if (His[GenomeIndex] == UINT_MAX || Seg->hi > His[GenomeIndex])
				His[GenomeIndex] = Seg->hi;
			unsigned L = Seg->GetLength();
			if (Plus)
				BaseCountPlus[GenomeIndex] += L;
			else
				BaseCountMinus[GenomeIndex] += L;
			if (Seg->GenomeIndex == g_GenomeIndex)
				continue;
			const char *GenomeName = In.GetGenomeName(Seg->GenomeIndex).c_str();
			Log("%7u  %10u  %10u  %c  %s\n", AIX.AlnInfoIndex, Seg->lo, Seg->hi, pom(Plus), GenomeName);
			}
		}

	unsigned TotalInv = 0;
	unsigned TotalBases = 0;
	unsigned TotalRelocs = 0;
	Log("\n");
	Log("Genome        Plus       Minus  +  Relocs     Inv%%  Name\n");
	Log("------  ----------  ----------  -  ------  -------  ----\n");
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		unsigned Plus = BaseCountPlus[GenomeIndex];
		unsigned Minus = BaseCountMinus[GenomeIndex];
		const char *Name = In.GetGenomeName(GenomeIndex).c_str();
		unsigned N = Plus + Minus;
		if (N == 0)
			continue;
		TotalBases += N;

		bool Strand = Plus >= Minus;
		const vector<unsigned> &c = Coords[GenomeIndex];
		unsigned Relocs = 0;
		if (Strand)
			for (unsigned i = 1; i < SIZE(c); ++i)
				{
				if (c[i-1] >= c[i])
					++Relocs;
				}
		else
			for (unsigned i = 1; i < SIZE(c); ++i)
				{
				if (c[i-1] <= c[i])
					++Relocs;
				}

		unsigned Inv = min(Plus, Minus);
		TotalInv += Inv;
		double Pct = Inv*100.0/double(N);
		TotalRelocs += Relocs;
		Log("%6u  %10u  %10u  %c  %6u  %6.1f%%  %s",
		  GenomeIndex, Plus, Minus, pom(Strand), Relocs, Pct, Name);

		Log("\n");
		}
	Log("\n");
	if (TotalBases > 0)
		Log("Total bases %u, inv %u, inv pct %.1f%%, relocs %u\n",
		  TotalBases, TotalInv, 100.0*double(TotalInv)/double(TotalBases), TotalRelocs);
	}
