#include <stdlib.h>
#include <time.h>
#ifdef _MSC_VER
#include <process.h>
#else
#include <unistd.h>
#endif
#include "utils.h"

/***
A multiply-with-carry random number generator, see:
http://en.wikipedia.org/wiki/Multiply-with-carry

The particular multipliers used here were found on
the web where they are attributed to George Marsaglia.
***/

static uint32 g_X[5];

void WriteRandState(FILE *f)
	{
	WriteStdioFile(f, g_X, sizeof(g_X));
	}

void ReadRandState(FILE *f)
	{
	ReadStdioFile(f, g_X, sizeof(g_X));
	}

void LogRandState()
	{
	Log("\n");
	Log("Rand state: ");
	for (int i = 0; i < 5; ++i)
		Log(" %08X", g_X[i]);
	Log("\n");
	}

uint32 RandInt32()
	{
	uint64 Sum = 2111111111*(uint64) g_X[3] + 1492*(uint64) g_X[2] +
	  1776*(uint64) g_X[1] + 5115*(uint64) g_X[0] + g_X[4];
	g_X[3] = g_X[2];
	g_X[2] = g_X[1];
	g_X[1] = g_X[0];
	g_X[4] = (uint32) (Sum >> 32);
	g_X[0] = (uint32) Sum;
	return g_X[0];
	}

static uint32 g_SLCG_state = 1;

// Numerical values used by Microsoft C, according to wikipedia:
// http://en.wikipedia.org/wiki/Linear_congruential_generator
static uint32 g_SLCG_a = 214013;
static uint32 g_SLCG_c = 2531011;

// Simple Linear Congruential Generator
// Bad properties; used just to initialize the better generator.
uint32 SLCG_rand()
	{
	g_SLCG_state = g_SLCG_state*g_SLCG_a + g_SLCG_c;
	return g_SLCG_state;
	}

void SLCG_srand(uint32 Seed)
	{
	g_SLCG_state = Seed;
	for (int i = 0; i < 10; ++i)
		SLCG_rand();
	}

void InitRand(uint32 Seed)
	{
	SLCG_srand(Seed);

	for (unsigned i = 0; i < 5; i++)
		g_X[i] = SLCG_rand();

	for (unsigned i = 0; i < 100; i++)
		RandInt32();
	}

static bool Init()
	{
	unsigned Seed = (unsigned) (time(0)*getpid());
	Log("Seed=%u\n", Seed);
	InitRand(Seed);
	return true;
	}
static bool InitDone = Init();
