#include "revfile.h"
#include "utils.h"

coord_t GetOverlapLength(coord_t loA, coord_t hiA, coord_t loB, coord_t hiB)
	{
	assert(loA <= hiA);
	assert(loB <= hiB);

	coord_t maxlo = std::max(loA, loB);
	coord_t minhi = std::min(hiA, hiB);

	if (maxlo > minhi)
		return 0;

	return minhi - maxlo + 1;
	}

coord_t GetOverlapLength(const GCSeg &SegA,const GCSeg &SegB)
	{
	if (SegA.GenomeIndex != SegB.GenomeIndex)
		return 0;
	if (SegA.ChrId != SegB.ChrId)
		return 0;
	return GetOverlapLength(SegA.lo, SegA.hi, SegB.lo, SegB.hi);
	}

coord_t GetOverlapLength(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex)
	{
	const GCSeg *SegA = AIA.GetSegPtr(GenomeIndex);
	const GCSeg *SegB = AIB.GetSegPtr(GenomeIndex);
	
	if (SegA == 0 || SegB == 0)
		return 0;

	return GetOverlapLength(*SegA, *SegB);
	}

static void ShrinkSeg(GCSeg &Seg, unsigned Margin)
	{
	Seg.hi += Margin;

	if (Seg.lo >= Margin)
		Seg.lo -= Margin;
	else
		Seg.lo = 0;
	}

bool OverlapWithMargin(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex,
  unsigned Margin)
	{
	const GCSeg *ptrSegA = AIA.GetSegPtr(GenomeIndex);
	const GCSeg *ptrSegB = AIB.GetSegPtr(GenomeIndex);

	if (ptrSegA == 0 || ptrSegB == 0)
		return false;

	if (ptrSegA->Plus != ptrSegB->Plus)
		return false;

	GCSeg SegA = *ptrSegA;
	GCSeg SegB = *ptrSegB;

	ShrinkSeg(SegA, Margin);
	ShrinkSeg(SegB, Margin);

	return GetOverlapLength(SegA, SegB) > 0;
	}

bool Overlap(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex)
	{
	const GCSeg *SegA = AIA.GetSegPtr(GenomeIndex);
	const GCSeg *SegB = AIB.GetSegPtr(GenomeIndex);

	if (SegA == 0 || SegB == 0)
		return false;

	if (SegA->Plus != SegB->Plus)
		return false;

	const coord_t loA = SegA->lo;
	const coord_t hiA = SegA->hi;
	const coord_t loB = SegB->lo;
	const coord_t hiB = SegB->hi;

	if (hiA >= loB && hiA <= hiB)
		return true;
	if (hiB >= loA && hiB <= hiA)
		return true;
	return false;
	}

bool Overlap(const AlnInfo &AIA, const AlnInfo &AIB)
	{
	return Overlap(AIA, AIB, 0) || Overlap(AIA, AIB, 1);
	}

bool OverlapMinusOK(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex)
	{
	const GCSeg &SegA = AIA.GetSeg(GenomeIndex);
	const GCSeg &SegB = AIB.GetSeg(GenomeIndex);

	const coord_t loA = SegA.lo;
	const coord_t hiA = SegA.hi;
	const coord_t loB = SegB.lo;
	const coord_t hiB = SegB.hi;

	if (hiA >= loB && hiA <= hiB)
		return true;
	if (hiB >= loA && hiB <= hiA)
		return true;
	return false;
	}

bool OverlapMinusOK(const AlnInfo &AIA, const AlnInfo &AIB)
	{
	return OverlapMinusOK(AIA, AIB, 0) || Overlap(AIA, AIB, 1);
	}
