#ifndef pdf_h
#define pdf_h

#include <vector>
#include "myrand.h"

double Normal(double x, double Mean, double StdDev);

using namespace std;

struct ProbDensFunc
	{
	virtual ~ProbDensFunc() {} // just to suppresses gcc warning
	virtual double Func(double x) const = 0;
	};

// Generate random numbers from a probability density function
// provided via object member function.
class RandDist
	{
public:
	RandDist();
	void Init(const ProbDensFunc &PDF, double Min_x, double Max_x,
	  unsigned BinCount = 1000);
	double Get() const;
	void Test(unsigned SampleCount, unsigned SampleBinCount) const;
	double ComputeMean(unsigned SampleCount) const;
	void LogBins(unsigned SampleCount = 10000, unsigned BinCount = 10) const;

private:
	double m_Min_x;
	double m_Max_x;
	vector<unsigned> m_ThreshVec;
	};

class NormalPDF : public ProbDensFunc
	{
public:
	virtual ~NormalPDF() {} // just to suppresses gcc warning
	NormalPDF(double Mean, double StdDev) : m_Mean(Mean), m_StdDev(StdDev)
		{
		}
private:
// Disable default c'tor
	NormalPDF();

public:
	virtual double Func(double x) const
		{
		return Normal(x, m_Mean, m_StdDev);
		}

private:
	double m_Mean;
	double m_StdDev;
	};

void GetNormRD(double Mean, double StdDev, RandDist &RD);
void GetTruncNormRD(double Mean, RandDist &RD);
void GetTruncNormRD(double Mean, RandDist &RD);
void GetTruncNormRDNewton(double Mean, double StdDev, RandDist &RD);

#endif // pdf_h
