#include "align4.h"

#define TRACE	0

static score_t AlnScore(const char *Row1, const char *Row2, unsigned AlnLength)
	{
	IntVec RunStarts;
	IntVec RunEnds;
	std::vector<score_t> RunScores;
	return GetAlnRuns(Row1, Row2, AlnLength, RunStarts, RunEnds, RunScores);
	}

void PostProcSlide(const char *Seq1, unsigned Length1, const char *Seq2,
  unsigned Length2, const std::string &Path, AIVec &Hits, StdStrVec &Paths,
  IntVec &ColStarts, IntVec &ColEnds)
	{
	if (Length1 == 0 || Length2 == 0)
		return;

	std::string ssRow1;
	std::string ssRow2;
	MakeAln(Seq1, Length1, Seq2, Length2, Path, ssRow1, ssRow2);
	assert(ssRow1.size() == ssRow2.size());
	unsigned AlnLength = (unsigned) ssRow1.size();

	const char * const Row1 = ssRow1.c_str();
	const char * const Row2 = ssRow2.c_str();

#if TRACE
	{
	Log("\n");
	Log("PostProcSlide AlnLength=%u\n", AlnLength);
	Log("Seq1=");
	if (Length1 < 100)
		Log("%.*s\n", Length1, Seq1);
	else
		Log("%100.100s...\n", Seq1);

	Log("Seq2=");
	if (Length2 < 100)
		Log("%.*s\n", Length2, Seq2);
	else
		Log("%100.100s...\n", Seq2);

	if (Path.size() < 100)
		Log("Path=%s\n", Path.c_str());
	else
		Log("Path=%100.100s...\n", Path.c_str());

	Log("\n");
	Log("Row1=");
	if (AlnLength < 100)
		Log("%s\n", Row1);
	else
		Log("%100.100s...\n", Row1);

	Log("Row2=");
	if (AlnLength < 100)
		Log("%s\n", Row2);
	else
		Log("%100.100s...\n", Row2);
	}
#endif

	Slide(Row1, Row2, AlnLength, SLIDE, SLIDEOFF, MINBADLENGTH, ColStarts, ColEnds);
	assert(ColStarts.size() == ColEnds.size());

#if	TRACE
	{
	Log("\n");
	Log("After Slide HitCount=%u\n", (unsigned) ColStarts.size());
	Log("ColStart  ColEnd\n");
	Log("========  ======\n");
	//   12345678  123456
	for (size_t i = 0; i < ColStarts.size(); ++i)
		Log("%8u  %6u\n", ColStarts[i], ColEnds[i]);
	}
#endif

	IntVec SeqStarts1;
	IntVec SeqStarts2;
	IntVec SeqEnds1;
	IntVec SeqEnds2;
	ColToSeq(ColStarts, Path, SeqStarts1, SeqStarts2);
	ColToSeq(ColEnds, Path, SeqEnds1, SeqEnds2);

	assert(ColStarts.size() == SeqStarts1.size());
	assert(SeqStarts1.size() == SeqStarts2.size());
	assert(SeqEnds1.size() == SeqEnds2.size());

#if	TRACE
	{
	Log("\n");
	Log("After ColToSeq HitCount=%u\n", (unsigned) ColStarts.size());
	Log("ColStart  ColEnd  SeqStart1  SeqStart2  SeqEnd1  SeqEnd2\n");
	Log("========  ======  =========  =========  =======  =======\n");
	//   12345678  123456  123456789  123456789  1234567  1234567
	for (size_t i = 0; i < ColStarts.size(); ++i)
		Log("%8u  %6u  %9u  %9u  %7u  %7u\n",
		  ColStarts[i],
		  ColEnds[i],
		  SeqStarts1[i],
		  SeqStarts2[i],
		  SeqEnds1[i],
		  SeqEnds2[i]);
	}
#endif

	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	assert(ColStarts.size() == ColEnds.size());
	const unsigned HitCount = (unsigned) ColStarts.size();
	for (unsigned HitIndex = 0; HitIndex < HitCount; ++HitIndex)
		{
		unsigned ColStart = ColStarts[HitIndex];
		unsigned ColEnd = ColEnds[HitIndex];
		assert(ColEnd >= ColStart);

		unsigned AlnLength = ColEnd - ColStart + 1;

		const char *ThisRow1 = Row1 + ColStart;
		const char *ThisRow2 = Row2 + ColStart;

		coord_t Alo = SeqStarts1[HitIndex];
		coord_t Ahi = SeqEnds1[HitIndex];
		coord_t Blo = SeqStarts2[HitIndex];
		coord_t Bhi = SeqEnds2[HitIndex];

		AlnInfo Hit;
		FilterMakePairwiseAlnInfo(Alo, Ahi, Blo, Bhi, true, Hit);

		Hit.Score = (ai_score_t) AlnScore(ThisRow1, ThisRow2, AlnLength);
		Hit.SetAlnLength(AlnLength);

		Hits.push_back(Hit);

		std::string ThisPath = Path.substr(ColStart, AlnLength);
		if (ThisPath[0] != 'M')
			Die("ppslide: path begins with %c", ThisPath[0]);
		if (ThisPath[AlnLength-1] != 'M')
			Die("ppslide: path ends with %c", ThisPath[AlnLength-1]);

		Paths.push_back(ThisPath);
#if	TRACE
		{
		Log("\n");
		Log("Hit %u\n", HitIndex);
		Log("ThisRow1=%.100s\n", ThisRow1);
		Log("ThisRow2=%.100s\n", ThisRow2);
		Log("Path    =%.100s\n", Path.c_str());
		}
#endif
		ValidateHitPath(Hit, GenomeIndexA, GenomeIndexB, ThisPath);
		}
	}
