#include <string>
#include <vector>
#include "utils.h"

using namespace std;

bool ReadEvoParamsLine(FILE *f, string &Line);
void ParseStrVec(const string &Str, vector<string> &Fields, char Sep);

/***
An event rate distribution is specified as set of (length,rate)
pairs like this:

DeleteRates=
1,0.001;2,0.002;3,0.0025
//

Each pair is of the form <L>,<r> where <L> is the length and
<r> is the rate.

Pairs are separated by semi-colons. 

A final semi-colon is optional.

The list is terminated by a line containing just "//".

White space is ignored. The list may extend over several lines.

The rates for missing lengths are computed by linear
interpolation from neighboring values.

The rate is zero for lengths smaller than the smallest
specified and larger than the largest specified.
***/

void DeleteWhiteSpace(const string &In, string &Out)
	{
	Out.clear();
	for (string::const_iterator p = In.begin(); p != In.end(); ++p)
		{
		char c = *p;
		if (!isspace(c))
			Out.push_back(c);
		}
	}

void ReadDist(FILE *f, vector<pair<gpos_t, double> > &Rates)
	{
	Rates.clear();
	for (;;)
		{
		string Line;
		bool EndOfFile = ReadEvoParamsLine(f, Line);
		if (EndOfFile)
			Die("Premature EOF in distribution, missing '//'");

		if (Line.empty())
			continue;

		if (Line == "//")
			return;

		string sLine;
		DeleteWhiteSpace(Line, sLine);

		vector<string> Pairs;
		ParseStrVec(sLine, Pairs, ';');

		for (vector<string>::const_iterator p = Pairs.begin();
		  p != Pairs.end(); ++p)
			{
			const string &Pair = *p;

			vector<string> v;
			ParseStrVec(Pair, v, ',');
			if (v.size() != 2)
				Die("Invalid specification for distribution '%s'", Line.c_str());
			char *NotUsed;
			gpos_t Length = strtoul(v[0].c_str(), &NotUsed, 10);
			if (Length == 0)
				Die("Zero length invalid in distribution '%s'", Line.c_str());

			double Rate = atof(v[1].c_str());
			Rates.push_back(pair<gpos_t, double>(Length, Rate));
			}
		}
	}
