#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include <vector>
#include <string>

using namespace std;

bool ReadEvoParamsLine(FILE *f, string &Line);
void ParseStrVecWS(const std::string &In, std::vector<std::string> &Out);

static char *Outcomes[4] =
	{
	"Slower",
	"Same",
	"Faster",
	"Lost"
	};

const char *GeneDupeResultToStr(GENE_DUPE_RESULT Result)
	{
	switch (Result)
		{
	case GDR_Slower:
		return "Slower";
	case GDR_Same:
		return "Same";
	case GDR_Faster:
		return "Faster";
	case GDR_Lost:
		return "Lost";
		}
	asserta(false);
	UNREACHABLE_RETURN("?");
	}

// Arend's values per email of 1/20/2008
unsigned g_GeneDupeWeights[4][4] =
	{
//		OldSlower    OldSame    OldFaster   OldLost
	{	        0,         0,           0,        0 },	// NewSlower
	{	        0,         0,           1,       10 },	// NewSame
	{	        0,        10,          10,        0 },	// NewFaster
	{	        0,       100,           0,        0 },	// NewLost
	};

void LogGeneDupeParams()
	{
	Log("\n");
	Log("GeneDupeWeights=\n");
	Log("           ");
	for (unsigned i = 0; i < 4; ++i)
		Log("%10.10s", (string("Old") + Outcomes[i]).c_str());
	Log("\n");

	for (unsigned i = 0; i < 4; ++i)
		{
		Log("%10.10s ", (string("New") + Outcomes[i]).c_str());
		for (unsigned j = 0; j < 4; ++j)
			Log("%10u", g_GeneDupeWeights[i][j]);
		Log("\n");
		}
	}

void ReadGeneDupeWeights(FILE *f)
	{
	string Line;

	bool EndOfFile = ReadEvoParamsLine(f, Line);
	if (EndOfFile)
		Die("GeneDupeWeights=, premature end-of-file");
	vector<string> Fields;
	ParseStrVecWS(Line, Fields);

	if (Fields.size() != 4 || Fields[0] != "OldSlower" || Fields[1] != "OldSame" ||
	  Fields[2] != "OldFaster" || Fields[3] != "OldLost")
		Die("GeneDupeWeights=, expected heading line OldSlower OldSame OldSFaster OldLost, got '%s'",
		  Line.c_str());

	for (unsigned i = 0; i < 4; ++i)
		{
		bool EndOfFile = ReadEvoParamsLine(f, Line);
		if (EndOfFile)
			Die("GeneDupeWeights=, premature end-of-file");

		vector<string> Fields;
		ParseStrVecWS(Line, Fields);

		if (Fields.size() != 5 || Fields[0] != string("New") + Outcomes[i])
			Die("GeneDupeWeights=, expected row for New%s, got '%s'",
			  Outcomes[i], Line.c_str());

		for (unsigned j = 0; j < 4; ++j)
			{
			const string &f = Fields[j+1];
			unsigned u = (unsigned) atoi(f.c_str());
			g_GeneDupeWeights[i][j] = u;
			}
		}
	}
