#include "filter4.h"
#include "getopts.h"
#include "revfile.h"
#include "intlist.h"
#include "progress.h"

#define TRACE	0

typedef bool (*ptrLinkFunction)(const AlnInfo &AI1, const AlnInfo &AI2);
typedef unsigned (*ptrDistanceFunction)(const AlnInfo &AI1, const AlnInfo &AI2);

void RepfixFilter4Simple(const AIVec &AIs, const IntVec &AIIndexes);
void RepfixFilter4MicroChain(const AIVec &AIs, const IntVec &AIIndexes);

double g_MinOverlapFract = 0.8;
RevFile *g_ptrOut;
RevFile *g_ptrDel;
unsigned g_ChrPairDeletedCount;
unsigned g_TotalDeletedCount;
unsigned g_GenomeIndexA;
unsigned g_GenomeIndexB;

static bool g_Simple;

static void AfterChrPair()
	{
	if (g_ChrPairDeletedCount == 0)
		return;

	unsigned GenomeIndexA = GetFilterGenomeIndexA();
	unsigned GenomeIndexB = GetFilterGenomeIndexB();

	g_GenomeIndexA = GenomeIndexA;
	g_GenomeIndexB = GenomeIndexB;

	unsigned ChrIdA = GetFilterChrIdA();
	unsigned ChrIdB = GetFilterChrIdB();

	const RevFile &In = GetFilterInputRevFile();

	const std::string &ChrNameA = In.GetChrName(GenomeIndexA, ChrIdA);
	const std::string &ChrNameB = In.GetChrName(GenomeIndexB, ChrIdB);

	Progress("%.16s[%u], %.16s[%u] %u hits deleted",
	  ChrNameA.c_str(),
	  ChrIdA,
	  ChrNameB.c_str(),
	  ChrIdB,
	  g_ChrPairDeletedCount);
	g_TotalDeletedCount += g_ChrPairDeletedCount;
	g_ChrPairDeletedCount = 0;
	}

static void AfterAllChrPairs()
	{
	Progress("Total %u hits deleted", g_TotalDeletedCount);

	g_ptrOut->Close();

	if (g_ptrDel != 0)
		{
		g_ptrDel->Close();
		g_ptrDel = 0;
		}
	}

void InitFilter4()
	{
	const char *OutFileName = RequiredValueOpt("out");
	const char *DelFileName = ValueOpt("del");

	g_Simple = FlagOpt("simple");

	const RevFile &In = GetFilterInputRevFile();
	const std::vector<GenInfo> &Genomes = In.GetGenomes();

	g_ptrOut = new RevFile;
	g_ptrOut->Create(OutFileName, Genomes);

	if (DelFileName != 0)
		{
		g_ptrDel = new RevFile;
		g_ptrDel->Create(DelFileName, Genomes);
		}

	FloatOpt("minovfract", &g_MinOverlapFract);

	SetAfterChrPairCallback(AfterChrPair);
	SetAfterAllChrPairsCallback(AfterAllChrPairs);
	}

void Filter4(const AIVec &AIs, const IntVec &AIIndexes)
	{
	if (g_Simple)
		RepfixFilter4Simple(AIs, AIIndexes);
	else
		RepfixFilter4MicroChain(AIs, AIIndexes);
	}
