#include "filter4.h"
#include "getopts.h"
#include "revfile.h"
#include "intlist.h"
#include "progress.h"

#define TRACE	0

coord_t GetOverlapLength(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex);

extern double g_MinOverlapFract;
extern RevFile *g_ptrOut;
extern RevFile *g_ptrDel;
extern unsigned g_ChrPairDeletedCount;
extern unsigned g_TotalDeletedCount;

// AIA covers AIB w.r.t. given genome?
// Some margin for slop is allowed.
static bool Covers(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex)
	{
	coord_t OverlapLength = GetOverlapLength(AIA, AIB, GenomeIndex);
	if (0 == OverlapLength)
		return false;

	const GCSeg &SegB = AIB.GetSeg(GenomeIndex);
	const double LengthB = (double) SegB.GetLength();

	if (0.0 == LengthB || OverlapLength/LengthB < g_MinOverlapFract)
		return false;

	return true;
	}

// AIA covers AIB w.r.t. either genomes?
bool Covers(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndexA,
  unsigned GenomeIndexB)
	{
	return Covers(AIA, AIB, GenomeIndexA) || Covers(AIA, AIB, GenomeIndexB);
	}

void RepfixFilter4Simple(const AIVec &AIs, const IntVec &AIIndexes)
	{
	Die("RepfixFilter4Simple: Not tested");

	const unsigned GenomeIndexA = GetFilterGenomeIndexA();
	const unsigned GenomeIndexB = GetFilterGenomeIndexB();

// Dumb N^2 method, could be N log N with sort.
	const unsigned AICount = (unsigned) AIs.size();

	BoolVec Del;
	Del.resize(AICount);
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		Del[AIIndex] = false;

#if	TRACE
	Log("\n");
	Log("\n");
	Log("ix1         loA         hiA         loA        hiA   ix2        loB2        hiB2        loB2        hiB2   OvA   OvB  C\n");
	Log("===  ==========  ==========  ==========  ==========  ===  ==========  ==========  ==========  ==========  ====  ====  =\n");
#endif

	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];
		for (unsigned AIIndex2 = 0; AIIndex2 < AICount; ++AIIndex2)
			{
			if (AIIndex == AIIndex2)
				continue;

			const AlnInfo &AI2 = AIs[AIIndex2];
#if	TRACE
			{
			const GCSeg &SegA = AI.GetSeg(GenomeIndexA);
			const GCSeg &SegB = AI.GetSeg(GenomeIndexB);

			const GCSeg &SegA2 = AI2.GetSeg(GenomeIndexA);
			const GCSeg &SegB2 = AI2.GetSeg(GenomeIndexB);

			unsigned loA = (unsigned) SegA.lo;
			unsigned hiA = (unsigned) SegA.hi;
			unsigned loB = (unsigned) SegB.lo;
			unsigned hiB = (unsigned) SegB.hi;

			unsigned loA2 = (unsigned) SegA2.lo;
			unsigned hiA2 = (unsigned) SegA2.hi;
			unsigned loB2 = (unsigned) SegB2.lo;
			unsigned hiB2 = (unsigned) SegB2.hi;

			unsigned OvA = (unsigned) GetOverlapLength(AI, AI2, GenomeIndexA);
			unsigned OvB = (unsigned) GetOverlapLength(AI, AI2, GenomeIndexB);
			char c = (Covers(AI, AI2, GenomeIndexA, GenomeIndexB) ? 'Y' : ' ');

			Log("%3u  %10u  %10u  %10u  %10u  %3u  %10u  %10u  %10u  %10u  %4u  %4u  %c\n",
			  AIIndex,
			  loA,
			  hiA,
			  loB,
			  hiB,
			  AIIndex2,
			  loA2,
			  hiA2,
			  loB2,
			  hiB2,
			  OvA,
			  OvB,
			  c);
			}
#endif
			if (Covers(AI, AI2, GenomeIndexA, GenomeIndexB))
				{
				Del[AIIndex2] = true;
				++g_ChrPairDeletedCount;
				continue;
				}
			}
		}

	const RevFile &In = GetFilterInputRevFile();

	for (unsigned i = 0; i < AICount; ++i)
		{
		unsigned AIIndex = AIIndexes[i];

		if (Del[i])
			{
			if (g_ptrDel != 0)
				g_ptrDel->CopyAln(In, AIIndex);
			}
		else
			g_ptrOut->CopyAln(In, AIIndex);
		}
	}
