#include "revfile.h"
#include "utils.h"
#include "progress.h"

char CompLetter(char c);

void LogGenChrToAIIxRanges(const map<IntPair, set<IxLoHi> > &m)
	{
	Log("Map:\n");
	for (map<IntPair, set<IxLoHi> >::const_iterator p = m.begin(); p != m.end(); ++p)
		{
		const IntPair &gc = p->first;
		const set<IxLoHi> &s = p->second;
		Log("gen %2u chr %2u lohis: ", gc.first, gc.second);
		for (set<IxLoHi>::const_iterator q = s.begin(); q != s.end(); ++q)
			Log(" %u-%u", (*q).Lo, (*q).Hi);
		Log("\n");
		}
	}

static void InsertAIIndexIntoLoHiSet(unsigned AIIndex, set<IxLoHi> &Set)
	{
	IxLoHi LH;
	LH.Lo = AIIndex;
	LH.Hi = AIIndex;
	set<IxLoHi>::iterator p = Set.upper_bound(LH);
	if (p != Set.end())
		{
		IxLoHi LH2 = *p;
		if (AIIndex + 1 == LH2.Lo)
			{
			Set.erase(p);
			LH2.Lo = AIIndex;
			Set.insert(LH2);
			return;
			}
		}
	if (p == Set.begin())
		{
		Set.insert(LH);
		return;
		}
	--p;
	IxLoHi LH2 = *p;
	if (LH2.Hi + 1 == AIIndex)
		{
		Set.erase(p);
		LH2.Hi = AIIndex;
		Set.insert(LH2);
		return;
		}
	Set.insert(LH);
	}

static FILE *CreateTmpFile(const char *Prefix, std::string &sFileName)
	{
// In case > 1 rev file being created at one time,
// file name must be unique within the process.
	static int TmpFileCounter = 0;
	int pid = getpid();
	char FileName[128];
	sprintf(FileName, "%s.%d.%d.tmp", Prefix, pid, ++TmpFileCounter);
	sFileName = FileName;
	FILE *f = CreateStdioFile(FileName);
	return f;
	}

// Hash function used in gawk.
// Simple and fast, not especially good
// behavior but good enough for us.
static uint32 Hash(const byte *Key, unsigned Bytes)
	{
	uint32 h = 0;
	for (unsigned i = 0; i < Bytes; ++i)
		{
		byte c = Key[i];
		h = c + (h << 6) + (h << 16) - h;
		}
	return h;
    }

void Trunc64(byte *Target, uint64 i64, unsigned Bytes)
	{
	uint64 v = i64;
	for (unsigned i = 0; i < Bytes; ++i)
		{
		*Target++ = (byte) (v%0x100);
		v /= 0x100;
		}
	if (v != 0)
		Die("Trunc64(%"UINT64_PRINTFX", %u) overflows", i64, Bytes);
	}

void Untrunc64(const byte *Buffer, unsigned Bytes, uint64 &i64)
	{
	i64 = 0;
	uint64 Factor = 1;
	for (unsigned i = 0; i < Bytes; ++i)
		{
		byte b = Buffer[i];
		i64 += b*Factor;
		Factor *= 0x100;
		}
	}

bool EncodeRowsInPlace(const StdStrVec &Rows,
  byte BitRows[DATE_ALN_DATA_BYTES])
	{
	if (SIZE(Rows) != 2)
		return false;
	const unsigned ColCount = SIZE(Rows[0]);
	if (ColCount > 12)
		return false;
	memset(BitRows, 0, DATE_ALN_DATA_BYTES);

	int BytePos = 0;
	int Shift = 0;
	for (unsigned RowIndex = 0; RowIndex < 2; ++RowIndex)
		{
		const StdStr &Row = Rows[RowIndex];
		for (unsigned ColIndex = 0; ColIndex < ColCount; ++ColIndex)
			{
			char c = Row[ColIndex];
			byte i = 0;
			switch (c)
				{
			case 'A':
				i = 0;
				break;
			case 'C':
				i = 1;
				break;
			case 'G':
				i = 2;
				break;
			case 'T':
				i = 3;
				break;
			default:
				return false;
				}
			BitRows[BytePos] |= (i << Shift);
			Shift += 2;
			if (Shift == 8)
				{
				Shift = 0;
				++BytePos;
				}
			}
		}
	return true;
	}

void DecodeRowsInPlace(const byte BitRows[DATE_ALN_DATA_BYTES],
  unsigned ColCount, StdStrVec &Rows)
	{
	Rows.clear();
	Rows.resize(2);

	int BytePos = 0;
	int Shift = 0;
	for (unsigned RowIndex = 0; RowIndex < 2; ++RowIndex)
		{
		StdStr &Row = Rows[RowIndex];
		for (unsigned ColIndex = 0; ColIndex < ColCount; ++ColIndex)
			{
			char c = 0;
			byte i = (byte) ((BitRows[BytePos] >> Shift) & 0x3);
			Shift += 2;
			if (Shift == 8)
				{
				Shift = 0;
				++BytePos;
				}
			switch (i)
				{
			case 0:
				c = 'A';
				break;
			case 1:
				c = 'C';
				break;
			case 2:
				c = 'G';
				break;
			case 3:
				c = 'T';
				break;
			default:
				asserta(false);
				}
			Row.push_back(c);
			}
		}
	}

static byte CharTo4Bit(char c)
	{
	switch (c)
		{
#define x(c, f)	case c:	return FB_##f;
		x('A', A)
		x('C', C)
		x('G', G)
		x('T', T)
		x('N', N)
		x('a', a)
		x('c', c)
		x('g', g)
		x('t', t)
		x('n', n)
		x('-', Gap)
#undef x
	default:
		break;
		}
	if (isprint(c))
		Die("CharTo4Bit(%c), not supported in compressed alignment", c);
	else
		Die("CharTo4Bit(0x%02x), not supported in compressed alignment", (byte) c);
	UNREACHABLE_RETURN(0);
	}

static char CharFrom4Bit(byte b)
	{
	static char Letters[] = "ACGTNacgtn-?????";
	asserta(b <= 0x0f);
	return Letters[b];
	}

void EncodeRow4Bit(const StdStr &Row, byte *Buffer)
	{
	unsigned ColCount = SIZE(Row);
	for (unsigned Col = 0; Col < ColCount; ++Col)
		{
		byte b = CharTo4Bit(Row[Col]);
		if (Col%2 == 0)
			Buffer[Col/2] = b;
		else
			Buffer[Col/2] |= (b << 4);
		}
	}

void DecodeRow4Bit(const byte *Buffer, unsigned ColCount, StdStr &Row)
	{
	for (unsigned Col = 0; Col < ColCount; ++Col)
		{
		byte b = Buffer[Col/2];
		if (Col%2 == 0)
			Row.push_back(CharFrom4Bit(b & 0x0f));
		else
			Row.push_back(CharFrom4Bit(b >> 4));
		}
	}

RevFile::RevFile()
	{
	m_FileName = "";
	m_State = Closed;
	m_AlnInfoCount = 0;
	m_AlnCount = 0;
	m_fAlnInfoTOC = 0;
	m_fAlnInfoSegs = 0;
	m_fAlnTOC = 0;
	m_fAlns = 0;
	m_fSeqs = 0;
	m_f = 0;
	m_Hash = 0;
	m_CompressedAICount = 0;
	m_InPlaceRowsCount = 0;
	m_DATECacheFirstEntryIndex = 0;
	m_DATECacheSize = 0;
	m_DATECacheEntryCount = 0;
	m_DATECache = 0;
	m_AICacheHitCount = 0;
	m_AICacheRefreshCount = 0;
	}

RevFile::~RevFile()
	{
	if (m_State != Closed)
		Close();
	}

void RevFile::Create(const std::string &FileName)
	{
	std::vector<GenInfo> Genomes; // empty vector
	Create(FileName, Genomes);
	}

void RevFile::Create(const std::string &FileName,
  const std::vector<GenInfo> &Genomes)
	{
	STATE(Closed, Writing)

	m_f = CreateStdioFile(FileName.c_str());
	m_FileName = FileName;

	m_Genomes = Genomes;
	m_AlnInfoCount = 0;
	m_AlnCount = 0;

	m_fAlnInfoTOC = CreateTmpFile("alninfotoc", m_AlnInfoTOCFileName);
	m_fAlnInfoSegs = CreateTmpFile("alninfosegs", m_AlnInfoSegsFileName);
	m_fAlnTOC = CreateTmpFile("alntoc", m_AlnTOCFileName);
	m_fAlns = CreateTmpFile("aln", m_AlnsFileName);
	}

void RevFile::Open(const std::string &FileName)
	{
	STATE(Closed, Reading)

	m_f = OpenStdioFile(FileName.c_str());
	m_FileName = FileName;

// Header:
	ReadData_(m_Hdr);
	SeekStdioFile(m_f, (off64_t) m_Hdr.GenomesPos);

	unsigned Magic = GetMagic();
	if (m_Hdr.Magic != Magic || m_Hdr.EndHdrMagic != RevFileEndHdrMagic)
		Die("Invalid rev file %s, bad magic [%x,%x] [%x,%x]",
		  m_FileName.c_str(), m_Hdr.Magic, Magic,
		  m_Hdr.EndHdrMagic, RevFileEndHdrMagic);

	if (m_Hdr.FileFormatVersion != RevFileFormatVersion)
		Die("Rev file %s is file format v%u, this code requires v%u",
		  (unsigned) m_Hdr.FileFormatVersion, RevFileFormatVersion);

	m_AlnInfoCount = (unsigned) m_Hdr.AlnInfoCount;
	m_AlnCount = (unsigned) m_Hdr.AlnCount;

// Genomes:
	const unsigned GenomeCount = m_Hdr.GenomeCount;
	//if (GenomeCount == 0)
	//	Die("Reading .rev file: zero genomes");

	if (GenomeCount > MaxSaneGenomeCount)
		Die("Reading .rev file, header claims and insane number of genomes %u",
		  GenomeCount);

	m_Genomes.resize(GenomeCount);
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		m_Genomes[GenomeIndex].FromFile(m_f);

// Hash table:
	if (m_Hdr.HashTableBytes != m_Hdr.HashTableEntryCount*sizeof(HashTableEntry))
		Die("Hash table size conflict");

	m_Hash = myalloc(HashTableEntry, m_Hdr.HashTableEntryCount);
	ReadStdioFile(m_f, (off64_t) m_Hdr.HashTablePos, m_Hash, m_Hdr.HashTableBytes);

	m_HashTableEntryCount = m_Hdr.HashTableEntryCount;

// Seqs:
	const unsigned SeqCount = m_Hdr.SeqCount;
	m_SeqPosLength.reserve(SeqCount);
	off64_t SeqTOCPos = (off64_t) m_Hdr.SeqTOCPos;
	SeekStdioFile(m_f, SeqTOCPos);

	for (unsigned SeqIndex = 0; SeqIndex < SeqCount; ++SeqIndex)
		{
		DiskSeqTOCEntry TE;
		ReadStdioFile(m_f, &TE, sizeof(TE));

		IntPair GenChr(TE.GenIndex, TE.ChrId);
		LongIntPair PosLength((uint64) TE.FilePos, (uint64) TE.Length);

		std::map<IntPair, unsigned>::value_type v(GenChr, SeqIndex);

		std::pair<std::map<IntPair, unsigned>::iterator, bool> Result = 
		  m_SeqTOC.insert(v);

		if (!Result.second)
			Die("RevFile::Open, dupe Gen,Chr seq TOC entry %u,%u",
			  TE.GenIndex, TE.ChrId);

		m_SeqPosLength.push_back(PosLength);
		}
	}

void RevFile::Close()
	{
	switch (m_State)
		{
	case Closed:
		return;
	case Reading:
		CloseReading();
		return;
	case Writing:
		CloseWriting();
		return;
	default:
		Die("RevFile::Close, invalid state");
		}
	}

void RevFile::CloseReading()
	{
	STATE(Reading, Closed)

	CloseStdioFile(m_f);
	myfree(m_Hash);
	m_Hash = 0;
	}

void RevFile::CloseWriting()
	{
	STATE(Writing, Closed)

// Write invalid header.
// Write correct values when all other data written.
// This is designed to cause Open to fail unless all writes
// were completed when file was createed.
	memset(&m_Hdr, 0, sizeof(m_Hdr));
	WriteStdioFile(m_f, 0, &m_Hdr, sizeof(m_Hdr));

	WriteGenomes();
	WriteAlnInfoTOC();
	WriteAlnInfoSegs();
	WriteAlnTOC();

	m_Hdr.AlnsPos = GetStdioFilePos(m_f);
	WriteAlns();

	WriteHashTable();
	WriteSeqs();

// Update header with final values
	m_Hdr.GenomeCount = (uint32) m_Genomes.size();
	m_Hdr.AlnCount = (uint32) m_AlnCount;
	m_Hdr.AlnInfoCount = (uint32) m_AlnInfoCount;
	m_Hdr.GenomesPos = (uint64) sizeof(m_Hdr);

// Write updated header
	m_Hdr.Magic = GetMagic();
	m_Hdr.EndHdrMagic = RevFileEndHdrMagic;
	m_Hdr.FileFormatVersion = RevFileFormatVersion;

	WriteStdioFile(m_f, 0, &m_Hdr, sizeof(m_Hdr));

	m_Genomes.clear();

	m_AlnInfoCount = 0;
	m_AlnCount = 0;

	CloseStdioFile(m_f);
	m_f = 0;

	myfree(m_Hash);
	m_Hash = 0;
	m_HashTableEntryCount = 0;
	}

HashTableEntry *RevFile::FindHashEntry(unsigned GenomeIndex, unsigned ChrId) const
	{
	unsigned AKey[2];
	AKey[0] = GenomeIndex;
	AKey[1] = ChrId;

	byte *Key = (byte *) AKey;

	unsigned TableIndex = Hash(Key, sizeof(AKey))%m_HashTableEntryCount;
	unsigned SafetyCounter = 0;
	HashTableEntry *H = 0;
	for (;;)
		{
		H = &m_Hash[TableIndex];
		if (H->GenomeIndex == NullGenomeIndex)
			return H;

		if (H->GenomeIndex == GenomeIndex && H->ChrId == ChrId)
			return H;

		++TableIndex;
		TableIndex %= m_HashTableEntryCount;

		++SafetyCounter;
		if (SafetyCounter >= m_HashTableEntryCount)
			Die("hash table full");
		}
	}

void RevFile::AddHashEntry(unsigned GenomeIndex, unsigned ChrId, uint64 Pos,
  unsigned Count)
	{
	HashTableEntry *H = FindHashEntry(GenomeIndex, ChrId);
	if (H->GenomeIndex != NullGenomeIndex)
		Die("RevFile::AddHashEntry, already present");

	H->GenomeIndex = (uint16) GenomeIndex;
	H->ChrId = ChrId;
	H->FileOffset = Pos;
	H->Count = Count;
	}

unsigned RevFile::GetAlnInfoCount() const
	{
	return m_AlnInfoCount;
	}

unsigned RevFile::GetGenomeCount() const
	{
	return (unsigned) m_Genomes.size();
	}

const std::string &RevFile::GetGenomeName(unsigned GenomeIndex) const
	{
	if (GenomeIndex >= (unsigned) m_Genomes.size())
		Die("GetGenomeName(%u), invalid genome", GenomeIndex);

	return m_Genomes[GenomeIndex].GetName();
	}

const std::string &RevFile::GetChrName(unsigned GenomeIndex, unsigned ChrId) const
	{
	if (GenomeIndex >= (unsigned) m_Genomes.size())
		Die("RevFile::GetChrName, genome=%u, count=%u",
		  GenomeIndex, (unsigned) m_Genomes.size());

	const GenInfo &GI = m_Genomes[GenomeIndex];
	const ChrInfo &CI = GI.GetChrInfo(ChrId);
	return CI.GetName();
	}

void RevFile::WriteGenomes()
	{
	const unsigned GenomeCount = (unsigned) m_Genomes.size();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		m_Genomes[GenomeIndex].ToFile(m_f);
	}

// Append AlnInfo segs tmp to Rev file.
// No relocation needed, can copy data as-is.
void RevFile::WriteAlnInfoSegs()
	{
	if (m_AlnInfoCount > 0)
		{
		unsigned AlnInfoSegsBytes = (unsigned) GetStdioFileSize(m_fAlnInfoSegs);
		byte *AlnInfoSegsBuffer = myalloc(byte, AlnInfoSegsBytes);
		ReadStdioFile(m_fAlnInfoSegs, 0, AlnInfoSegsBuffer, AlnInfoSegsBytes);
		WriteStdioFile(m_f, AlnInfoSegsBuffer, AlnInfoSegsBytes);
		myfree(AlnInfoSegsBuffer);
		}

	CloseStdioFile(m_fAlnInfoSegs);
	m_fAlnInfoSegs = 0;

	DeleteStdioFile(m_AlnInfoSegsFileName.c_str());
	}

void RevFile::WriteAlnTOC()
	{
	m_Hdr.AlnTOCPos = (uint64) GetStdioFilePos(m_f);
	if (m_AlnCount > 0)
		{
		unsigned AlnTOCBytes = m_AlnCount*sizeof(DiskAlnTOCEntry);
		uint64 AlnBasePos = (uint64) (m_Hdr.AlnTOCPos + AlnTOCBytes);

	// Read entire Aln TOC tmp file into memory
		DiskAlnTOCEntry *TOCBuffer = myalloc(DiskAlnTOCEntry, m_AlnInfoCount);

		ReadStdioFile(m_fAlnTOC, 0, TOCBuffer, AlnTOCBytes);

		DiskAlnTOCEntry *TOC = TOCBuffer;

		for (unsigned AlnIndex = 0; AlnIndex < m_AlnInfoCount; ++AlnIndex)
			{
			TOC->FilePos += AlnBasePos;
			++TOC;
			}

	// Append TOC to Rev file
		WriteStdioFile(m_f, TOCBuffer, AlnTOCBytes);

	// Done with AlnTOC
		myfree(TOCBuffer);
		TOCBuffer = 0;
		}

	CloseStdioFile(m_fAlnTOC);
	m_fAlnInfoTOC = 0;

	DeleteStdioFile(m_AlnTOCFileName.c_str());
	}

// Append Alns tmp to Rev file.
// No relocation needed, can copy data as-is.
void RevFile::WriteAlns()
	{
	CopyStdioFile(m_fAlns, m_f);
	CloseStdioFile(m_fAlns);
	DeleteStdioFile(m_AlnsFileName.c_str());
	m_fAlns = 0;
	}

void RevFile::WriteSeqs()
	{
	const unsigned SeqCount = (unsigned) m_SeqTOC.size();
	off64_t SeqTOCPos = GetStdioFilePos(m_f);

	m_Hdr.SeqTOCPos = (uint64) SeqTOCPos;
	m_Hdr.SeqCount = SeqCount;

	if (SeqCount == 0)
		return;

// Position of first entry in seq TOC
	DiskSeqTOCEntry ZeroTE;
	memset(&ZeroTE, 0, sizeof(ZeroTE));
	for (unsigned i = 0; i < SeqCount; ++i)
		WriteStdioFile(m_f, &ZeroTE, sizeof(ZeroTE));

// Compute base position for sequence data
	off64_t RevFileSeqPos = SeqTOCPos + SeqCount*sizeof(DiskSeqTOCEntry);
	if (RevFileSeqPos != GetStdioFilePos(m_f))
		Die("SeqBasePos failed");

	for (std::map<IntPair, unsigned>::const_iterator p = m_SeqTOC.begin();
	  p != m_SeqTOC.end(); ++p)
		{
		const std::map<IntPair, unsigned>::value_type &v = *p;

		const IntPair &GenChr = v.first;
		const unsigned &SeqIndex = v.second;

		if (SeqIndex >= SeqCount)
			Die("WriteSeqs, SeqIndex >= SeqCount");

		const LongIntPair &PosLength = m_SeqPosLength[SeqIndex];
		uint64 u64TmpFilePos = PosLength.first;

		off64_t TmpFilePos = (off64_t) u64TmpFilePos;

		if ((uint64) TmpFilePos != u64TmpFilePos)
			Die("RevFile::WriteSeqs, TmpFilePos overflows off64_t");

		uint64 u64Length = PosLength.second;
		unsigned Length = (unsigned) u64Length;
		if ((uint64) Length != u64Length)
			Die("RevFile::WriteSeqs, chr length overflows unsigned int");

	// Read seq from tmp file
		char *Seq = (char *) alloc(Length);
		ReadStdioFile(m_fSeqs, TmpFilePos, Seq, Length);

	// Write seq to rev file
		WriteStdioFile(m_f, RevFileSeqPos, Seq, Length);
		myfree(Seq);

        DiskSeqTOCEntry E;
		E.GenIndex = GenChr.first;
		E.ChrId = GenChr.second;
		E.FilePos = RevFileSeqPos;
		E.Length = PosLength.second;

		off64_t TOCEntryPos = SeqTOCPos + SeqIndex*sizeof(DiskSeqTOCEntry);
		WriteStdioFile(m_f, TOCEntryPos, &E, sizeof(E));

		RevFileSeqPos += Length;
		}

	CloseStdioFile(m_fSeqs);
	m_fSeqs = 0;
	DeleteStdioFile(m_SeqsFileName.c_str());
	}

unsigned RevFile::GetChrId(unsigned GenomeIndex, const std::string &ChrName) const
	{
	const GenInfo &GI = GetGenomeInfo(GenomeIndex);
	return GI.GetChrId(ChrName);
	}

unsigned RevFile::GetChrId(unsigned GenomeIndex, unsigned ChrIndex) const
	{
	const GenInfo &GI = GetGenomeInfo(GenomeIndex);
	return GI.GetChrId(ChrIndex);
	}

unsigned RevFile::GetChrIndex(unsigned GenomeIndex, unsigned ChrId) const
	{
	const GenInfo &GI = GetGenomeInfo(GenomeIndex);
	return GI.GetChrIndex(ChrId);
	}

const GenInfo &RevFile::GetGenomeInfo(unsigned GenomeIndex) const
	{
	unsigned GenomeCount = (unsigned) m_Genomes.size();
	if (GenomeIndex >= GenomeCount)
		Die("RevFile::GetGenomeInfo(%u), count=%u", GenomeIndex, GenomeCount);
	return m_Genomes[GenomeIndex];
	}

GenInfo &RevFile::GetModifiableGenomeInfo(unsigned GenomeIndex)
	{
	if (GenomeIndex >= SIZE(m_Genomes))
		Die("RevFile::GetModifiableGenomeInfo(%u), out of range", GenomeIndex);
	return m_Genomes[GenomeIndex];
	}

GenInfo *RevFile::GetModifiableGenomeInfoPtr(const std::string &GenomeName,
  unsigned *ptrGenomeIndex)
	{
	unsigned GenomeCount = (unsigned) m_Genomes.size();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		GenInfo &GI = m_Genomes[GenomeIndex];
		if (GenomeName == GI.GetName())
			{
			*ptrGenomeIndex = GenomeIndex;
			return &GI;
			}
		}
	return 0;
	}

const std::vector<GenInfo> &RevFile::GetGenomes() const
	{
	return m_Genomes;
	}

const ChrInfo &RevFile::GetChrInfo(unsigned GenomeIndex, unsigned ChrId) const
	{
	unsigned GenomeCount = (unsigned) m_Genomes.size();
	if (GenomeIndex >= GenomeCount)
		Die("RevFile::GetChrInfo(%u), count=%u", GenomeIndex, GenomeCount);

	const GenInfo &GI = m_Genomes[GenomeIndex];
	return GI.GetChrInfo(ChrId);
	}

// Determine active chr pairs by reading all segs in all AIs (slow!)
void RevFile::GetActiveChrPairs(unsigned GenomeIndexA, unsigned GenomeIndexB,
  std::set<IntPair> &ChrIdPairs) const
	{
	ChrIdPairs.clear();

	const unsigned AICount = GetAlnInfoCount();
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		AlnInfo AI;
		GetAlnInfo(AIIndex, AI);
		if (AI.Segs.size() != 2)
			continue;

		const GCSeg *SegA = AI.GetSegPtr(GenomeIndexA);
		const GCSeg *SegB = AI.GetSegPtr(GenomeIndexB);

		if (SegA != 0 && SegB != 0)
			{
			IntPair ChrIdPair(SegA->ChrId, SegB->ChrId);
			ChrIdPairs.insert(ChrIdPair);
			}
		}
	}

void RevFile::GetAlnInfos(const IntVec &AlnInfoIndexes,
  std::vector<AlnInfo> &AlnInfos) const
	{
	const unsigned AICount = (unsigned) AlnInfoIndexes.size();
	AlnInfos.reserve(AICount);

	for (unsigned i = 0; i < AICount; ++i)
		{
		unsigned AIIndex = AlnInfoIndexes[i];

		AlnInfo AI;
		GetAlnInfo(AIIndex, AI);

		AlnInfos.push_back(AI);
		}
	}

void RevFile::GetAlnInfoXs(const IntVec &AlnInfoIndexes,
  std::vector<AlnInfoX> &AlnInfoXs) const
	{
	const unsigned AIXCount = (unsigned) AlnInfoIndexes.size();
	AlnInfoXs.reserve(AIXCount);

	for (unsigned i = 0; i < AIXCount; ++i)
		{
		unsigned AIIndex = AlnInfoIndexes[i];

		AlnInfoX AIX;
		GetAlnInfo(AIIndex, AIX.AI);

		AIX.AlnInfoIndex = AIIndex;
		AIX.AlnIndex = GetAlnIndex(AIIndex);

		AlnInfoXs.push_back(AIX);
		}
	}

void RevFile::GetAlnInfos(const IntSet &AIIndexes, AIVec &AIs) const
	{
	const unsigned AICount = (unsigned) AIIndexes.size();
	AIs.reserve(AICount);

	for (IntSet::const_iterator p = AIIndexes.begin(); p != AIIndexes.end(); ++p)
		{
		unsigned AIIndex = *p;

		AlnInfo AI;
		GetAlnInfo(AIIndex, AI);

		AIs.push_back(AI);
		}
	}

unsigned RevFile::GetGenomeIndex(const std::string &GenomeName) const
	{
	const unsigned GenomeCount = GetGenomeCount();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		const std::string &Name = GetGenomeName(GenomeIndex);
		if (GenomeName == Name)
			return GenomeIndex;
		}
	Die("RevFile::GetGenomeIndex(%s), not found", GenomeName.c_str());
	return UINT_MAX;
	}

unsigned RevFile::AddSeq(const std::string &GenomeName,
  const std::string &ChrName, const char *Seq, coord_t Length)
	{
	STATEC(Writing)

	unsigned GenIndex;
	GenInfo *GI = GetModifiableGenomeInfoPtr(GenomeName, &GenIndex);
	if (GI == 0)
		{
		GenIndex = AddGenome(GenomeName);
		GI = &(m_Genomes[GenIndex]);
		}

	unsigned ChrId = UINT_MAX;
	ChrInfo *CI = GI->GetModifiableChrInfoPtr(ChrName);
	if (CI == 0)
		ChrId = GI->AddChr(ChrName, Length);
	else
		{
		ChrId = CI->GetId();
		CI->SetLength(Length);
		}

	if (m_fSeqs == 0)
		m_fSeqs = CreateTmpFile("seq", m_SeqsFileName);

	off64_t FilePos = GetStdioFilePos(m_fSeqs);

	IntPair GenChr(GenIndex, ChrId);
	LongIntPair PosLength((uint64) FilePos, (uint64) Length);

	unsigned SeqIndex = (unsigned) m_SeqPosLength.size();
	m_SeqPosLength.push_back(PosLength);

	std::map<IntPair, unsigned>::value_type v(GenChr, SeqIndex);

	std::pair<std::map<IntPair, unsigned>::iterator, bool> Result =
	  m_SeqTOC.insert(v);

	if (!Result.second)
		Die("RevFile::AddSeq(%u,%u), already added", GenIndex, ChrId);

	size_t stLength = (size_t) Length;
	if ((coord_t) stLength != Length)
		Die("Sequence length overflows size_t");

	WriteStdioFile(m_fSeqs, Seq, stLength);

	return SeqIndex;
	}

unsigned RevFile::AddGenome(const std::string &Name)
	{
	unsigned GenIndex = (unsigned) m_Genomes.size();

	GenInfo GI;
	GI.Create(Name);

	m_Genomes.push_back(GI);

	return GenIndex;
	}

const ChrInfo *RevFile::GetChrInfo(unsigned GenIndex,
  const std::string &ChrName) const
	{
	const GenInfo &GI = GetGenomeInfo(GenIndex);
	return GI.GetChrInfoPtr(ChrName);
	}

coord_t RevFile::GetSeqLengthByIndex(unsigned SeqIndex) const
	{
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeqLengthByIndex(%u), SeqCount=%u", SeqIndex, SeqCount);

	const LongIntPair &PosLength = m_SeqPosLength[SeqIndex];
	return (coord_t) PosLength.second;
	}

bool RevFile::IsSeqPresent(unsigned GenomeIndex, unsigned ChrId) const
	{
	IntPair GenChr(GenomeIndex, ChrId);
	std::map<IntPair, unsigned>::const_iterator p = m_SeqTOC.find(GenChr);
	return p != m_SeqTOC.end();
	}

coord_t RevFile::GetSeqLength(unsigned GenomeIndex, unsigned ChrId) const
	{
	IntPair GenChr(GenomeIndex, ChrId);
	std::map<IntPair, unsigned>::const_iterator p = m_SeqTOC.find(GenChr);
	if (p == m_SeqTOC.end())
		Die("RevFile::GetSeqLength(%u,%u), not found", GenomeIndex, ChrId);

	unsigned SeqIndex = p->second;
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeqLength(%u,%u), SeqIndex >= SeqCount",
		  GenomeIndex, ChrId);

	const LongIntPair &PosLength = m_SeqPosLength[SeqIndex];
	return (coord_t) PosLength.second;
	}

void RevFile::GetSeq(unsigned GenomeIndex, unsigned ChrId, char *Seq, unsigned Bytes) const
	{
	STATEC(Reading)

	IntPair GenChr(GenomeIndex, ChrId);
	std::map<IntPair, unsigned>::const_iterator p = m_SeqTOC.find(GenChr);
	if (p == m_SeqTOC.end())
		Die("RevFile::GetSeq, not found");

	unsigned SeqIndex = p->second;
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeq, SeqIndex >= SeqCount");

	const LongIntPair &PosLength = m_SeqPosLength[SeqIndex];
	off64_t Pos = (off64_t) PosLength.first;
	unsigned Length = (unsigned) PosLength.second;

	if (Bytes < Length)
		Die("RevFile::GetSeq, buffer bytes=%u shorter than seq length=%u",
		  Bytes, Length);

	ReadStdioFile(m_f, Pos, Seq, Length);
	}

void RevFile::GetSubSeq3(const GCSeg &Seg, StdStr &Seq) const
	{
	GetSubSeq2(Seg.GenomeIndex, Seg.ChrId, Seq, Seg.lo, Seg.GetLength());
	if (!Seg.Plus)
		RevComp(Seq);
	}

void RevFile::GetSubSeq2(unsigned GenomeIndex, unsigned ChrId, StdStr &ssSeq,
  unsigned From, unsigned Length, bool DoRevComp) const
	{
	char *Seq = (char *) malloc(Length+1);
	if (Seq == 0)
		Die("RevFile::GetSubSeq, Out of memory (requested %u bytes)", Length);
	GetSubSeq(GenomeIndex, ChrId, Seq, From, Length, DoRevComp);
	Seq[Length] = 0;
	ssSeq = Seq;
	delete[] Seq;
	}

void RevFile::GetSubSeq(unsigned GenomeIndex, unsigned ChrId, char *Seq,
  unsigned From, unsigned Length, bool DoRevComp) const
	{
	STATEC(Reading)

	IntPair GenChr(GenomeIndex, ChrId);
	std::map<IntPair, unsigned>::const_iterator p = m_SeqTOC.find(GenChr);
	if (p == m_SeqTOC.end())
		Die("RevFile::GetSubSeq, not found");

	unsigned SeqIndex = p->second;
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeq, SeqIndex >= SeqCount");

	const LongIntPair &PosLength = m_SeqPosLength[SeqIndex];
	off64_t Pos = (off64_t) PosLength.first;
	unsigned SeqLength = (unsigned) PosLength.second;

	if (From + Length - 1 >= SeqLength)
		Die("RevFile::GetSubSeq, subseq past end of seq");

	ReadStdioFile(m_f, Pos + From, Seq, Length);

	if (DoRevComp)
		RevComp(Seq, Length);
	}

unsigned RevFile::GetSeqCount() const
	{
	return (unsigned) m_SeqPosLength.size();
	}

void RevFile::GetSeqByIndex(unsigned SeqIndex, char *Seq, unsigned Bytes,
  unsigned *ptrGenomeIndex, unsigned *ptrChrId) const
	{
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeqByIndex(%u), out of range", SeqIndex);

	DiskSeqTOCEntry TE;
	off64_t TEPos = (off64_t) m_Hdr.SeqTOCPos + SeqIndex*sizeof(TE);
	ReadStdioFile(m_f, TEPos, &TE, sizeof(TE));

	*ptrGenomeIndex = TE.GenIndex;
	*ptrChrId = TE.ChrId;
	unsigned Length = (unsigned) TE.Length;
	off64_t SeqPos = (off64_t) TE.FilePos;

	if (Length > Bytes)
		Die("RevFile::GetSeqByIndex, buff size=%u < seq length=%u",
		  Bytes, Length);
	ReadStdioFile(m_f, SeqPos, Seq, Length);
	}

unsigned RevFile::GetChrCount(unsigned GenomeIndex) const
	{
	const GenInfo &GI = GetGenomeInfo(GenomeIndex);
	return GI.GetChrCount();
	}

void RevFile::GetAllAlnInfos(AIVec &AIs, bool ShowProgress) const
	{
	AIs.clear();
	unsigned AICount = GetAlnInfoCount();
	AIs.resize(AICount);
	if (ShowProgress)
		ProgressStart("Reading %u AIs", AICount);
	
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (ShowProgress && AIIndex%100 == 0)
			ProgressStep(AIIndex, AICount);
		GetAlnInfo(AIIndex, AIs[AIIndex]);
		}
	if (ShowProgress)
		ProgressDone();
	}

void RevFile::GetAllAlnInfoXs(AIXVec &AIXs, bool ShowProgress) const
	{
	AIXs.clear();
	unsigned AICount = GetAlnInfoCount();
	AIXs.resize(AICount);
	if (ShowProgress)
		ProgressStart("Reading %u AIXs", AICount);
	
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (ShowProgress && AIIndex%100 == 0)
			ProgressStep(AIIndex, AICount);
		AlnInfoX &AIX =  AIXs[AIIndex];
		unsigned AlnIndex = GetAlnInfo(AIIndex, AIX.AI);
		AIX.AlnInfoIndex = AIIndex;
		AIX.AlnIndex = AlnIndex;
		}
	if (ShowProgress)
		ProgressDone();
	}

coord_t RevFile::GetChrLength(unsigned GenomeIndex, unsigned ChrId) const
	{
	const ChrInfo &CI = GetChrInfo(GenomeIndex, ChrId);
	return CI.GetLength();
	}

bool RevFile::ValidateSegLengths(const AlnInfo &AI, const StdStrVec &Rows,
  bool DieOnError)
	{
	const unsigned RowCount = (unsigned) Rows.size();
	const unsigned SegCount = (unsigned) AI.Segs.size();
	if (RowCount != SegCount)
		{
		LogAI(AI);
		if (DieOnError)
			Die("RevFile::ValidateSegLengths, RowCount=%u != SegCount=%u",
			  RowCount, SegCount);
		else
			Warning("RevFile::ValidateSegLengths, RowCount=%u != SegCount=%u",
			  RowCount, SegCount);
		return false;
		}
	if (RowCount == 0)
		return true;

	const unsigned ColCount = (unsigned) Rows[0].size();
	for (unsigned RowIndex = 0; RowIndex < RowCount; ++RowIndex)
		{
		const std::string &Row = Rows[RowIndex];
		const GCSeg &Seg = AI.Segs[RowIndex];
		if (Seg.lo > Seg.hi)
			{
			LogAI(AI);
			if (DieOnError)
				Die("RevFile::ValidateSegLengths, Invalid Seg.lo=%u > Seg.hi=%u",
				  (unsigned) Seg.lo, (unsigned) Seg.hi);
			else
				Warning("RevFile::ValidateSegLengths, Invalid Seg.lo=%u > Seg.hi=%u",
				  (unsigned) Seg.lo, (unsigned) Seg.hi);
			return false;
			}

		unsigned SegLength = (Seg.hi - Seg.lo + 1);
		const unsigned N = (unsigned) Row.size();
		if (N != ColCount)
			{
			LogAI(AI);
			if (DieOnError)
				Die("RevFile::ValidateSegLengths, row %u size=%u, row 0 size=%u",
				  RowIndex, N, ColCount);
			else
				Warning("RevFile::ValidateSegLengths, row %u size=%u, row 0 size=%u",
				  RowIndex, N, ColCount);
			return false;
			}

		unsigned RowLength = 0;
		for (unsigned i = 0; i < ColCount; ++i)
			if (Row[i] != '-')
				++RowLength;
			
		if (SegLength != RowLength)
			{
			LogAIN(AI);
			Log("Seq=%.64s\n", Row.c_str());
			if (DieOnError)
				Die("RevFile::ValidateSegLengths, Row=%u SegLength %u != RowLength %u",
				  RowIndex, SegLength, RowLength);
			else
				Warning("RevFile::ValidateSegLengths, Row=%u SegLength %u != RowLength %u",
				  RowIndex, SegLength, RowLength);
			return false;
			}
		}
	return true;
	}

void RevFile::GetSeqGenChrByIndex(unsigned SeqIndex, unsigned *ptrGenomeIndex,
  unsigned *ptrChrId) const
	{
	unsigned SeqCount = GetSeqCount();
	if (SeqIndex >= SeqCount)
		Die("RevFile::GetSeqGenChrByIndex(%u), out of range", SeqIndex);

	DiskSeqTOCEntry TE;
	off64_t TEPos = (off64_t) m_Hdr.SeqTOCPos + SeqIndex*sizeof(TE);
	ReadStdioFile(m_f, TEPos, &TE, sizeof(TE));

	*ptrGenomeIndex = TE.GenIndex;
	*ptrChrId = TE.ChrId;
	}

const std::string &RevFile::GetFileName() const
	{
	return m_FileName;
	}

uint64 RevFile::StoreBitRows(const StdStrVec &Rows)
	{
	asserta(SIZE(Rows) == 2 && SIZE(Rows[0]) == SIZE(Rows[1]));

	uint32 ColCount = (uint32) SIZE(Rows[0]);

// Make sure second row starts on byte boundary
	uint32 ColCount2 = ColCount + ColCount%2;
	uint64 Pos = GetStdioFilePos(m_fAlns);
	WriteStdioFile(m_fAlns, &ColCount, sizeof(ColCount));

	byte *Buffer = myalloc(byte, ColCount2);
	EncodeRow4Bit(Rows[0], Buffer);
	EncodeRow4Bit(Rows[1], Buffer + ColCount2/2);

	WriteStdioFile(m_fAlns, Buffer, ColCount2);

	myfree(Buffer);
	return Pos;
	}

void RevFile::GetAlnRowsInPlace(const DiskAlnInfoTOCEntryC &DATEC, StdStrVec &Rows) const
	{
	unsigned ColCount = DATEC.Length0;
	DecodeRowsInPlace(DATEC.AlnData, ColCount, Rows);
	}

void RevFile::GetDATE(unsigned AlnInfoIndex, DiskAlnInfoTOCEntry &DATE) const
	{
// Already in cache?
	if (AlnInfoIndex >= m_DATECacheFirstEntryIndex &&
	  AlnInfoIndex < m_DATECacheFirstEntryIndex + m_DATECacheEntryCount)
		{
		DiskAlnInfoTOCEntry *ptrDATE =
		  m_DATECache + (AlnInfoIndex - m_DATECacheFirstEntryIndex);
		memcpy(&DATE, ptrDATE, sizeof(DATE));
		++m_AICacheHitCount;
		return;
		}

	++m_AICacheRefreshCount;

// Refresh cache so that requested entry is first in cache (optimal
// assuming sequential reads in increasing order of AlnInfoIndex).
	if (m_DATECache == 0)
		{
		m_DATECache = new DiskAlnInfoTOCEntry[DATECacheSize];
		asserta(m_DATECache != 0);
		}

	unsigned Sz = sizeof(DiskAlnInfoTOCEntry);
	off64_t TOCPos = (off64_t) m_Hdr.AlnInfoTOCPos + AlnInfoIndex*Sz;

	m_DATECacheEntryCount = DATECacheSize;
	if (AlnInfoIndex + m_DATECacheEntryCount > m_AlnInfoCount)
		m_DATECacheEntryCount = m_AlnInfoCount - AlnInfoIndex;

	ReadStdioFile(m_f, TOCPos, m_DATECache,
	  m_DATECacheEntryCount*sizeof(DiskAlnInfoTOCEntry));
	m_DATECacheFirstEntryIndex = AlnInfoIndex;

	DiskAlnInfoTOCEntry *ptrDATE =
	  m_DATECache + (AlnInfoIndex - m_DATECacheFirstEntryIndex);
	memcpy(&DATE, ptrDATE, sizeof(DATE));
	}

unsigned RevFile::GetAlnInfoDATE(const DiskAlnInfoTOCEntry &DATE,
  unsigned AlnInfoIndex, AlnInfo &AI) const
	{
	const DiskAlnInfoTOCEntryC &DATEC = (DiskAlnInfoTOCEntryC &) DATE;
	if (DATE.AlnIndex != UINT_MAX && (DATEC.BitFields & BIT_AI_IS_COMPRESSED))
		{
		GetAlnInfoDATECompressed(DATEC, AI);
		return AlnInfoIndex | UINT32_HIBIT;
		}

	AI.Segs.clear();
	AI.Values.clear();
	AI.Attrs.clear();

	AI.Score = (ai_score_t) DATE.Score;
	AI.AlnIndex = DATE.AlnIndex;

	unsigned Bytes = DATE.SegCount*sizeof(DiskGCSeg) +
	  DATE.AttrCount*sizeof(DiskAlnAttr);

	byte *Buffer = myalloc(byte, Bytes);
	ReadStdioFile(m_f, (off64_t) DATE.FilePos, Buffer, Bytes);

	DiskGCSeg *BuffSeg = (DiskGCSeg *) Buffer;
	for (unsigned SegIndex = 0; SegIndex < DATE.SegCount; ++SegIndex)
		{
		GCSeg S;
		S.GenomeIndex = (unsigned) BuffSeg->GenomeIndex;
		S.ChrId = (unsigned) BuffSeg->ChrId;
		S.hi = (coord_t) BuffSeg->hi;
		S.lo = (coord_t) BuffSeg->lo;
		S.Plus = static_cast<bool>(BuffSeg->Plus);

		AI.Segs.push_back(S);

		++BuffSeg;
		}

	DiskAlnAttr *BuffAA = (DiskAlnAttr *) (Buffer + DATE.SegCount*sizeof(DiskGCSeg));
	for (unsigned AttrIndex = 0; AttrIndex < DATE.AttrCount; ++AttrIndex)
		{
		AlnAttr Attr = (AlnAttr) BuffAA->Attr;
		unsigned Value = (unsigned) BuffAA->Value;

		AI.Attrs.push_back(Attr);
		AI.Values.push_back(Value);

		++BuffAA;
		}

	myfree(Buffer);

	return DATE.AlnIndex;
	}

// Build TOC (Table Of Contents) for AlnInfo's.
// Do this by appending AlnInfoTOC tmp file to Rev file.
// This requires updating the DiskAlnInfoTOCEntry.FilePos
// field by the offset of the TOC in the Rev file.
void RevFile::WriteAlnInfoTOC()
	{
	off64_t Pos = GetStdioFilePos(m_f);
	m_Hdr.AlnInfoTOCPos = (uint64) Pos;

	if (m_AlnInfoCount > 0)
		{
		unsigned AlnInfoTOCBytes = m_AlnInfoCount*sizeof(DiskAlnInfoTOCEntry);
		unsigned AlnInfoSegsBytes = (unsigned) GetStdioFileSize(m_fAlnInfoSegs);
		unsigned AlnTOCFileSize = (unsigned) GetStdioFileSize(m_fAlnInfoTOC);
		asserta(AlnInfoTOCBytes == AlnTOCFileSize);

		uint64 AlnInfoSegsBasePos = (uint64) (m_Hdr.AlnInfoTOCPos + AlnInfoTOCBytes);

	// Read entire AlnInfo TOC tmp file into memory
		DiskAlnInfoTOCEntry *TOCBuffer = myalloc(DiskAlnInfoTOCEntry, m_AlnInfoCount);

		ReadStdioFile(m_fAlnInfoTOC, 0, TOCBuffer, AlnInfoTOCBytes);

		DiskAlnInfoTOCEntry *TOC = TOCBuffer;
		for (unsigned AIIndex = 0; AIIndex < m_AlnInfoCount; ++AIIndex)
			{
			uint32 AlnIndex = TOC->AlnIndex;
			if (AlnIndex == UINT_MAX || (AlnIndex & UINT32_HIBIT) == 0)
				{
				uint64 TmpFilePos = TOC->FilePos;
				asserta(TmpFilePos < AlnInfoSegsBytes);
				TOC->FilePos = TmpFilePos + AlnInfoSegsBasePos;
				}
			TOC++;
			}

	// Append TOC to Rev file
		WriteStdioFile(m_f, TOCBuffer, AlnInfoTOCBytes);

	// Done with AlnInfoTOC
		myfree(TOCBuffer);
		TOCBuffer = 0;
		}

	CloseStdioFile(m_fAlnInfoTOC);
	m_fAlnInfoTOC = 0;

	DeleteStdioFile(m_AlnInfoTOCFileName.c_str());
	}

unsigned RevFile::GetAlnIndex(unsigned AlnInfoIndex) const
	{
	DiskAlnInfoTOCEntry DATE;
	GetDATE(AlnInfoIndex, DATE);
	if (DATE.AlnIndex == UINT_MAX)
		return UINT_MAX;

	const DiskAlnInfoTOCEntryC &DATEC = (DiskAlnInfoTOCEntryC &) DATE;
	if ((DATEC.BitFields & BIT_AI_IS_COMPRESSED) != 0)
		return AlnInfoIndex | UINT32_HIBIT;

	return DATE.AlnIndex;
	}

unsigned RevFile::AddAlnInfo(const AlnInfo &AI)
	{
	return AddAlnInfo(AI, UINT_MAX);
	}

unsigned RevFile::AddAlnInfo(const AlnInfo &AI, unsigned AlnIndex)
	{
	STATEC(Writing)

	if (AlnIndex != UINT_MAX && CanCompressAI(AI))
		{
		AddAlnInfoCompress(AI);
		return m_AlnInfoCount++;
		}

	unsigned SegCount = (unsigned) AI.Segs.size();
	unsigned AttrCount = (unsigned) AI.Attrs.size();

	DiskAlnInfoTOCEntry AITE;
	AITE.AlnIndex = AlnIndex;
	AITE.SegCount = (uint16) SegCount;
	AITE.AttrCount = (uint16) AttrCount;
	AITE.Score = AI.Score;
	AITE.FilePos = (uint64) GetStdioFilePos(m_fAlnInfoSegs);

	WriteStdioFile(m_fAlnInfoTOC, &AITE, sizeof(AITE));

	unsigned SegAttrBytes = AITE.SegCount*sizeof(DiskGCSeg) +
	  AITE.AttrCount*sizeof(DiskAlnAttr);

	byte *Buffer = myalloc(byte, SegAttrBytes);

	byte *BuffPos = Buffer;
	for (unsigned SegIndex = 0; SegIndex < SegCount; ++SegIndex)
		{
		const GCSeg &S = AI.Segs[SegIndex];

		const unsigned GenomeCount = (unsigned) m_Genomes.size();
		const unsigned GenomeIndex = S.GenomeIndex;
		const unsigned ChrId = S.ChrId;
		if (GenomeIndex >= GenomeCount)
			Die("AddAlnInfo, GenomeIndex[%u]=%u, count=%u",
			  SegIndex, GenomeIndex, GenomeCount);

		DiskGCSeg DS;
		DS.GenomeIndex = (uint16) GenomeIndex;
		DS.ChrId = ChrId;
		DS.lo = (uint32) S.lo;
		DS.hi = (uint32) S.hi;
		DS.Plus = (uint16) S.Plus;

		if ((coord_t) DS.lo != S.lo || (coord_t) DS.hi != S.hi)
			Die("coord_t overflowed uint32");

		memcpy(BuffPos, &DS, sizeof(DS));
		BuffPos += sizeof(DS);

		AddToIndexList((unsigned) S.GenomeIndex, (unsigned) S.ChrId, m_AlnInfoCount);
		}

	for (unsigned AttrIndex = 0; AttrIndex < AttrCount; ++AttrIndex)
		{
		DiskAlnAttr DA;
		unsigned Attr = AI.Attrs[AttrIndex];
		DA.Attr = (uint16) Attr;
		DA.Value = AI.Values[AttrIndex];

		if ((unsigned) DA.Attr != Attr)
			Die("Aln attr overflows uint16");

		memcpy(BuffPos, &DA, sizeof(DA));
		BuffPos += sizeof(DA);
		}

	assert((unsigned) (BuffPos - Buffer) == SegAttrBytes);
	WriteStdioFile(m_fAlnInfoSegs, Buffer, SegAttrBytes);

	myfree(Buffer);
	return m_AlnInfoCount++;
	}

// Copy AlnInfo plus rows, if present, from another file
void RevFile::CopyAln(const RevFile &rhs, unsigned rhsAlnInfoIndex)
	{
	STATEC(Writing)

	AlnInfo AI;
	rhs.GetAlnInfo(rhsAlnInfoIndex, AI);

	unsigned rhsAlnIndex = rhs.GetAlnIndex(rhsAlnInfoIndex);
	if (rhsAlnIndex == UINT_MAX)
		AddAlnInfo(AI);
	else
		{
		StdStrVec Rows;
		rhs.GetAlnRows(rhsAlnIndex, Rows);
		AddAln(AI, Rows);
		}
	}

void RevFile::GetAlnInfoDATECompressed(const DiskAlnInfoTOCEntryC &DATEC,
  AlnInfo &AI) const
	{
	AI.Clear();
	AI.Segs.resize(2);
	GCSeg &Seg0 = AI.Segs[0];
	GCSeg &Seg1 = AI.Segs[1];

	Seg0.GenomeIndex = 0;
	Seg1.GenomeIndex = 1;

	Seg0.ChrId = (DATEC.ChrId0 & 0x7f);
	Seg1.ChrId = (DATEC.ChrId1 & 0x7f);

	Seg0.Plus = ((DATEC.ChrId0 & 0x80) == 0 ? false : true);
	Seg1.Plus = ((DATEC.ChrId1 & 0x80) == 0 ? false : true);

	Seg0.lo = DATEC.lo0;
	Seg1.lo = DATEC.lo1;

	unsigned Length0 = DATEC.Length0;
	int DiffLength1 = DATEC.DiffLength1;
	unsigned Length1 = (unsigned) ((int) Length0 + DiffLength1);

	Seg0.hi = Seg0.lo + Length0 - 1;
	Seg1.hi = Seg1.lo + Length1 - 1;

	if (DATEC.BitFields & BIT_AI_IS_PARALOG)
		{
		AI.Attrs.push_back(ALN_ATTR_Paralog);
		AI.Values.push_back(1);
		}
	}

unsigned RevFile::GetAlnInfo(unsigned AlnInfoIndex, AlnInfo &AI) const
	{
	if (AlnInfoIndex >= m_AlnInfoCount)
		Die("GetAlnInfo(%u) count=%u", AlnInfoIndex, m_AlnInfoCount);

	DiskAlnInfoTOCEntry DATE;
	GetDATE(AlnInfoIndex, DATE);
	return GetAlnInfoDATE(DATE, AlnInfoIndex, AI);
	}

unsigned RevFile::GetAlnInfoX(unsigned AlnInfoIndex, AlnInfoX &AIX) const
	{
	unsigned AlnIndex = GetAlnInfo(AlnInfoIndex, AIX.AI);
	AIX.AlnInfoIndex = AlnInfoIndex;
	AIX.AlnIndex = AlnIndex;
	return AlnIndex;
	}

unsigned RevFile::AddAln(const AlnInfo &AI, const StdStrVec &Rows)
	{
	STATEC(Writing)

	if (Rows.size() == 0)
		return UINT_MAX;

	if (AI.Segs.size() != Rows.size())
		Die("RevFile::AddAln, %u segs, %u rows",
		  (unsigned) AI.Segs.size(), (unsigned) Rows.size());

	ValidateSegLengths(AI, Rows);

	bool Comp = CanCompressAI(AI);
	if (Comp)
		{
		byte BitRows[DATE_ALN_DATA_BYTES];
		bool RowsInPlace = EncodeRowsInPlace(Rows, BitRows);
		if (RowsInPlace)
			++m_InPlaceRowsCount;
		uint64 FilePosRows = UINT64_MAX;
		if (!RowsInPlace)
			FilePosRows = StoreBitRows(Rows);
		byte *ptrBitRows = (RowsInPlace ? BitRows : 0);
		AddAlnInfoCompress(AI, ptrBitRows, FilePosRows);
		return m_AlnInfoCount++;
		}

	unsigned AlnIndex = m_AlnCount++;
	unsigned AlnInfoIndex = AddAlnInfo(AI, AlnIndex);

	unsigned Length0 = (unsigned) Rows[0].size();

	DiskAlnTOCEntry DH;
	DH.RowCount = (uint32) Rows.size();
	DH.ColCount = (uint32) Length0;
	DH.AlnInfoIndex = AlnInfoIndex;
	DH.FilePos = (uint64) GetStdioFilePos(m_fAlns);

	WriteStdioFile(m_fAlnTOC, &DH, sizeof(DH));

	unsigned Bytes = (unsigned) (DH.RowCount*DH.ColCount);
	char *Buffer = myalloc(char, Bytes);
	char *BuffPos = Buffer;
	for (unsigned RowIndex = 0; RowIndex < DH.RowCount; ++RowIndex)
		{
		const std::string &Row = Rows[RowIndex];
		unsigned Length = (unsigned) Row.size();
		if (Length != Length0)
			Die("AddAln, rows different lengths");

		memcpy(BuffPos, Row.c_str(), Length0);

		BuffPos += DH.ColCount;
		}
	assert(BuffPos == Buffer + Bytes);

	WriteStdioFile(m_fAlns, Buffer, Bytes);

	myfree(Buffer);

	return AlnInfoIndex;
	}

/***
When possible, the AlnInfo structure is stored in its TOC entry.
If short enough, rows will be stored in place also.
	
	Bytes		Description
	-----		-----------
	1			Bits:
					0	1=AI stored in-place, 0=not in place.
					1	1=Has rows, 0=no rows.
					2	1=Rows stored in-place, 0=not in place
					3	1=Paralog, 0=not paralog.
					4-7	RFU

	1			ChrId0
	2-5			lo0 (signed 4-byte int, sign is strand)
	6-7			Length0

	8			ChrId1
	9-12		lo1 (signed 4-byte int, sign is strand)
	13			Length1-Length as signed 1-byte int.

	If rows stored in place:
	14-16		Row0 stored in 2-bit code.
	17-19		Row1 stored in 2-bit code.

	If rows not stored in place:
	14-19		FilePos of rows as 6-byte unsigned int.
***/

void RevFile::AddAlnInfoCompress(const AlnInfo &AI, const byte *Rows,
  uint64 FilePosRows)
	{
	asserta(CanCompressAI(AI));

	bool RowsInPlace = (Rows != 0);
	bool RowsSeparate = (FilePosRows != UINT64_MAX);
	asserta(!(RowsInPlace && RowsSeparate));

	bool HasRows = (RowsInPlace || RowsSeparate);

	const GCSeg &Seg0 = AI.Segs[0];
	const GCSeg &Seg1 = AI.Segs[1];
	int Length0 = Seg0.GetLength();
	int Length1 = Seg1.GetLength();
	int DiffLength1 = Length1 - Length0;

	asserta(Length0 <= 0x7ffff);
	asserta(std::abs(DiffLength1) <= 0x7f);

	bool IsParalog = (AI.GetAttr(ALN_ATTR_Paralog) == 1);

	DiskAlnInfoTOCEntryC DATE;

// Bit fields
	DATE.BitFields = 0;
	DATE.BitFields |= BIT_AI_IS_COMPRESSED;

	if (HasRows)
		DATE.BitFields |= BIT_AI_HAS_ROWS;

	if (RowsInPlace)
		DATE.BitFields |= BIT_ROWS_IN_PLACE;

	if (IsParalog)
		DATE.BitFields |= BIT_AI_IS_PARALOG;

// Chr ids
	DATE.ChrId0 = (byte) Seg0.ChrId;
	DATE.ChrId1 = (byte) Seg1.ChrId;

// Strands
	if (Seg0.Plus)
		DATE.ChrId0 |= 0x80;
	if (Seg1.Plus)
		DATE.ChrId1 |= 0x80;

	DATE.lo0 = Seg0.lo;
	DATE.lo1 = Seg1.lo;

// Lengths
	DATE.Length0 = (uint16) Seg0.GetLength();
	DATE.DiffLength1 = (int8) DiffLength1;

// Rows
// Avoid uninitialized data
	memset(DATE.AlnData, 0xff, sizeof(DATE.AlnData));
	if (RowsInPlace)
		memcpy(DATE.AlnData, Rows, sizeof(DATE.AlnData));
	else if (RowsSeparate)
		Trunc64(DATE.AlnData, FilePosRows, DATE_ALN_DATA_BYTES);

	DiskAlnInfoTOCEntry &DATE_NC = (DiskAlnInfoTOCEntry &) DATE;
	asserta((DATE_NC.AlnIndex & UINT32_HIBIT) != 0);

	WriteStdioFile(m_fAlnInfoTOC, &DATE, sizeof(DATE));

	AddToIndexList(Seg0.GenomeIndex, Seg0.ChrId, m_AlnInfoCount);
	AddToIndexList(Seg1.GenomeIndex, Seg1.ChrId, m_AlnInfoCount);

	++m_CompressedAICount;
	}

void RevFile::GetAlnRowsCompressed(unsigned AlnIndex, StdStrVec &Rows) const
	{
	asserta((AlnIndex & UINT32_HIBIT) != 0);
	unsigned AlnInfoIndex = (AlnIndex & ~UINT32_HIBIT);
	
	DiskAlnInfoTOCEntryC DATEC;
	GetDATEC(AlnInfoIndex, DATEC);

	bool HasRows = ((DATEC.BitFields & BIT_AI_HAS_ROWS) != 0);
	if (!HasRows)
		Die("RevFile::GetAlnRowsCompressed, !BIT_AI_HAS_ROWS");

	bool RowsInPlace = ((DATEC.BitFields & BIT_ROWS_IN_PLACE) != 0);
	if (RowsInPlace)
		return GetAlnRowsInPlace(DATEC, Rows);

	uint64 FilePosRows;
	Untrunc64(DATEC.AlnData, DATE_ALN_DATA_BYTES, FilePosRows);

	uint64 AlnsPos = m_Hdr.AlnsPos;
	if (AlnsPos == 0)
		Die("GetAlnRowsCompressed, AlnsPos = 0 in file header");

	FilePosRows += AlnsPos;

	uint32 ColCount;
	ReadStdioFile(m_f, FilePosRows, &ColCount, sizeof(ColCount));
	uint32 ColCount2 = ColCount + ColCount%2;

	byte *Buffer = myalloc(byte, ColCount2);
	ReadStdioFile(m_f, Buffer, ColCount2);

	Rows.clear();
	Rows.resize(2);
	DecodeRow4Bit(Buffer, ColCount, Rows[0]);
	DecodeRow4Bit(Buffer + ColCount2/2, ColCount, Rows[1]);
	myfree(Buffer);
	}

void RevFile::GetDATEC(unsigned AlnInfoIndex, DiskAlnInfoTOCEntryC &DATEC) const
	{
	GetDATE(AlnInfoIndex, (DiskAlnInfoTOCEntry &) DATEC);
	}

unsigned RevFile::GetAlnRows(unsigned AlnIndex, StdStrVec &Rows) const
	{
	STATEC(Reading)

	if ((AlnIndex & UINT32_HIBIT) != 0)
		{
		GetAlnRowsCompressed(AlnIndex, Rows);
		return AlnIndex & ~UINT32_HIBIT;
		}

	if (AlnIndex >= m_AlnCount)
		Die("RevFile::GetAlnRows(%u) count=%u", AlnIndex, m_AlnCount);

	Rows.clear();

	off64_t TOCPos = (off64_t) m_Hdr.AlnTOCPos + AlnIndex*sizeof(DiskAlnTOCEntry);

	DiskAlnTOCEntry DATE;
	ReadStdioFile(m_f, TOCPos, &DATE, sizeof(DATE));

	unsigned Bytes = (unsigned) (DATE.RowCount*DATE.ColCount);
	char *Buffer = myalloc(char, Bytes);

	ReadStdioFile(m_f, (off64_t) DATE.FilePos, Buffer, Bytes);

	for (unsigned RowIndex = 0; RowIndex < DATE.RowCount; ++RowIndex)
		{
		char *ptrRow = Buffer + RowIndex*DATE.ColCount;
		std::string Row(ptrRow, DATE.ColCount);
		Rows.push_back(Row);
		}

	myfree(Buffer);
	return DATE.AlnInfoIndex;
	}

void RevFile::AddToIndexList(unsigned GenomeIndex, unsigned ChrId,
  unsigned AlnInfoIndex)
	{
	IntPair GC(GenomeIndex, ChrId);
	map<IntPair, set<IxLoHi> >::iterator p = m_GenChrToAIIxRanges.find(GC);
	if (p == m_GenChrToAIIxRanges.end())
		{
		IxLoHi LoHi;
		LoHi.Lo = AlnInfoIndex;
		LoHi.Hi = AlnInfoIndex;

		set<IxLoHi> Set;
		Set.insert(LoHi);
		//Log("\n\nLoHi new %u\n", AlnInfoIndex);
		m_GenChrToAIIxRanges[GC] = Set;
		//LogGenChrToAIIxRanges(m_GenChrToAIIxRanges);
		}
	else
		{
		set<IxLoHi> &Set = p->second;
		InsertAIIndexIntoLoHiSet(AlnInfoIndex, Set);
		//LogGenChrToAIIxRanges(m_GenChrToAIIxRanges);
		}
	}

void RevFile::GetAppendAIIxRangesChr(unsigned GenIndex, unsigned ChrId,
  IntPairVec &AlnInfoIndexLoHis) const
	{
	IntPairSet Set;
	GetAppendAIIxRangesChr(GenIndex, ChrId, Set);
	
	for (IntPairSet::const_iterator p = Set.begin(); p != Set.end(); ++p)
		AlnInfoIndexLoHis.push_back(*p);
	}

void RevFile::GetAppendAIIxRangesChr(unsigned GenIndex, unsigned ChrId,
  IntPairSet &AlnInfoIndexLoHis) const
	{
	STATEC(Reading)

	HashTableEntry *H = FindHashEntry(GenIndex, ChrId);
	if (H->GenomeIndex == NullGenomeIndex)
		return;
	
	off64_t Pos = (off64_t) H->FileOffset;
	SeekStdioFile(m_f, Pos);

	unsigned LoHiCount = (unsigned) H->Count;
	uint32 *LoHis = myalloc(uint32, 2*LoHiCount);
	ReadStdioFile(m_f, LoHis, 2*LoHiCount*sizeof(uint32));

	for (unsigned i = 0; i < LoHiCount; ++i)
		{
		unsigned Lo = (unsigned) LoHis[2*i];
		unsigned Hi = (unsigned) LoHis[2*i+1];
		std::pair<unsigned,unsigned> LoHi(Lo, Hi);
		AlnInfoIndexLoHis.insert(LoHi);
		}

	myfree(LoHis);
	}

// Build index lists and hash table, append to file.
void RevFile::WriteHashTable()
	{
	unsigned GenChrPairCount = (unsigned) m_GenChrToAIIxRanges.size();
	m_HashTableEntryCount = (unsigned) ((GenChrPairCount + 1)*HashExpandFactor);
	m_Hash = myalloc(HashTableEntry, m_HashTableEntryCount);
	memset(m_Hash, 0xff, m_HashTableEntryCount*sizeof(HashTableEntry));

	for (map<IntPair, set<IxLoHi> >::const_iterator p = m_GenChrToAIIxRanges.begin();
	  p != m_GenChrToAIIxRanges.end(); ++p)
		{
		const IntPair &GC = p->first;
		const set<IxLoHi> &LoHis = p->second;

		uint32 LoHiCount = (uint32) LoHis.size();

		uint64 Pos = GetStdioFilePos(m_f);
		AddHashEntry(GC.first, GC.second, Pos, LoHiCount);

		uint32 *Buffer = myalloc(uint32, 2*LoHiCount);
		uint32 *BuffPtr = Buffer;
		for (set<IxLoHi>::const_iterator q = LoHis.begin(); q != LoHis.end(); ++q)
			{
			*BuffPtr++ = q->Lo;
			*BuffPtr++ = q->Hi;
			}
		WriteStdioFile(m_f, Buffer, 2*LoHiCount*sizeof(uint32));
		myfree(Buffer);
		}

	unsigned HashTableBytes = m_HashTableEntryCount*sizeof(HashTableEntry);

	m_Hdr.HashTablePos = GetStdioFilePos(m_f);
	m_Hdr.HashTableBytes = HashTableBytes;
	m_Hdr.HashTableEntryCount = m_HashTableEntryCount;

// Write hash table
	WriteStdioFile(m_f, m_Hash, HashTableBytes);
	}

void RevFile::GetAppendAlnInfoIndexesChr(unsigned GenIndex, unsigned ChrId,
  IntSet &AlnInfoIndexes) const
	{
	IntPairSet LoHis;
	GetAppendAIIxRangesChr(GenIndex, ChrId, LoHis);
	for (IntPairSet::const_iterator p = LoHis.begin(); p != LoHis.end(); ++p)
		{
		const IntPair &LoHi = *p;
		unsigned Lo = LoHi.first;
		unsigned Hi = LoHi.second;
		for (unsigned AIIndex = Lo; AIIndex <= Hi; ++AIIndex)
			AlnInfoIndexes.insert(AIIndex);
		}
	}

void RevFile::GetAppendAlnInfoIndexesChr(unsigned GenIndex, unsigned ChrId,
  IntVec &AlnInfoIndexes) const
	{
	IntPairSet LoHis;
	GetAppendAIIxRangesChr(GenIndex, ChrId, LoHis);
	for (IntPairSet::const_iterator p = LoHis.begin(); p != LoHis.end(); ++p)
		{
		const IntPair &LoHi = *p;
		unsigned Lo = LoHi.first;
		unsigned Hi = LoHi.second;
		for (unsigned AIIndex = Lo; AIIndex <= Hi; ++AIIndex)
			AlnInfoIndexes.push_back(AIIndex);
		}
	}

unsigned RevFile::GetMagic() const
	{
	return RevFileHdrMagic;
	}

bool RevFile::CanCompressAI(const AlnInfo &AI) const
	{
	if (AI.Score != 0)
		return false;

	if (AI.GetSegCount() != 2)
		return false;

	const GCSeg &Seg0 = AI.Segs[0];
	const GCSeg &Seg1 = AI.Segs[1];

	if (Seg0.GenomeIndex != 0)
		return false;
	if (Seg1.GenomeIndex != 1)
		return false;

	if (Seg0.ChrId > 0x7f)
		return false;
	if (Seg1.ChrId > 0x7f)
		return false;

	int Length0 = (int) Seg0.GetLength();
	if (Length0 < 0 || Length0 > USHRT_MAX)
		return false;

	int Length1 = (int) Seg1.GetLength();
	if (Length1 < 0)
		return false;

	int DiffLength1 = Length1 - Length0;

	if (std::abs(DiffLength1) > 127)
		return false;

	unsigned AttrCount = SIZE(AI.Attrs);
	if (AttrCount > 1)
		return false;
	else if (AttrCount == 1)
		{
		if (AI.Attrs[0] != ALN_ATTR_Paralog)
			return false;
		}
	return true;
	}

void GetPairwiseCols(unsigned GenomeIndex1, unsigned ChrId1,
  unsigned GenomeIndex2, unsigned ChrId2, 
  const AlnInfo &AI, const vector<string> &Rows,
  vector<PairwiseCol> &Cols)
	{
	if (GenomeIndex1 == UINT_MAX || GenomeIndex2 == UINT_MAX)
		Die("GetPairwiseCols, GenomeIndex cannot be wildcard");

	Cols.clear();
	const unsigned SegCount = AI.GetSegCount();
	unsigned SegIndex1 = UINT_MAX;
	unsigned SegIndex2 = UINT_MAX;
	for (unsigned SegIx1 = 0; SegIx1 < SegCount; ++SegIx1)
		{
		const GCSeg &Seg1 = AI.Segs[SegIx1];
		if (GenomeIndex1 != UINT_MAX && Seg1.GenomeIndex != GenomeIndex1)
			continue;
		if (ChrId1 != UINT_MAX && Seg1.ChrId != ChrId1)
			continue;
		SegIndex1 = SegIx1;
		break;
		}
	if (SegIndex1 == UINT_MAX)
		return;

	for (unsigned SegIx2 = 0; SegIx2 < SegCount; ++SegIx2)
		{
		if (SegIx2 == SegIndex1)
			continue;
		const GCSeg &Seg2 = AI.Segs[SegIx2];
		if (Seg2.GenomeIndex == GenomeIndex1)
			continue;
		if (GenomeIndex2 != UINT_MAX && Seg2.GenomeIndex != GenomeIndex2)
			continue;
		if (ChrId2 != UINT_MAX && Seg2.ChrId != ChrId2)
			continue;
		SegIndex2 = SegIx2;
		break;
		}
	if (SegIndex2 == UINT_MAX)
		return;

	const GCSeg &Seg1 = AI.Segs[SegIndex1];
	const GCSeg &Seg2 = AI.Segs[SegIndex2];

	const string &Row1 = Rows[SegIndex1];
	const string &Row2 = Rows[SegIndex2];
	const unsigned ColCount = SIZE(Row1);
	asserta(SIZE(Row2) == ColCount);
	Cols.reserve(ColCount);

	PairwiseCol Col;
	Col.GenomeIndex1 = Seg1.GenomeIndex;
	Col.GenomeIndex2 = Seg2.GenomeIndex;
	Col.ChrId1 = Seg1.ChrId;
	Col.ChrId2 = Seg2.ChrId;
	Col.Pos1 = (Seg1.Plus ? Seg1.lo : Seg1.hi);
	Col.Pos2 = (Seg2.Plus ? Seg2.lo : Seg2.hi);
	Col.Plus = (Seg1.Plus == Seg2.Plus);
	for (unsigned ColIndex = 0; ColIndex < ColCount; ++ColIndex)
		{
		Col.Letter1 = Row1[ColIndex];
		Col.Letter2 = Row2[ColIndex];

		if (!Seg1.Plus)
			Col.Letter1 = CompLetter(Col.Letter1);
		if (!Seg2.Plus)
			Col.Letter2 = CompLetter(Col.Letter2);

		Cols.push_back(Col);

		if (Col.Letter1 != '-')
			{
			if (Seg1.Plus)
				++Col.Pos1;
			else
				--Col.Pos1;
			}
		if (Col.Letter2 != '-')
			{
			if (Seg2.Plus)
				++Col.Pos2;
			else
				--Col.Pos2;
			}
		}
	}

void RevFile::EnumPairwiseCols(unsigned GenomeIndex1, unsigned ChrId1,
  unsigned GenomeIndex2, unsigned ChrId2, ON_PAIRWISE_COL OnCol, unsigned User,
  bool ShowProgress) const
	{
	const unsigned AICount = GetAlnInfoCount();
	if (ShowProgress)
		ProgressStart("Reading %u AIs", AICount);
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (ShowProgress && AIIndex%1000 == 0)
			ProgressStep(AIIndex, AICount);
		unsigned AlnIndex = GetAlnIndex(AIIndex);
		if (AlnIndex == UINT_MAX)
			continue;

		AlnInfo AI;
		GetAlnInfo(AIIndex, AI);

		StdStrVec Rows;
		GetAlnRows(AlnIndex, Rows);

		const unsigned SegCount = AI.GetSegCount();
		for (unsigned SegIndex1 = 0; SegIndex1 < SegCount; ++SegIndex1)
			{
			const GCSeg &Seg1 = AI.Segs[SegIndex1];
			if (GenomeIndex1 != UINT_MAX && Seg1.GenomeIndex != GenomeIndex1)
				continue;
			if (ChrId1 != UINT_MAX && Seg1.ChrId != ChrId1)
				continue;

			for (unsigned SegIndex2 = 0; SegIndex2 < SegCount; ++SegIndex2)
				{
				if (SegIndex2 == SegIndex1)
					continue;
				const GCSeg &Seg2 = AI.Segs[SegIndex2];
				if (Seg2.GenomeIndex == GenomeIndex1)
					continue;
				if (GenomeIndex2 != UINT_MAX && Seg2.GenomeIndex != GenomeIndex2)
					continue;
				if (ChrId2 != UINT_MAX && Seg2.ChrId != ChrId2)
					continue;

				PairwiseCol Col;
				Col.Pos1 = (Seg1.Plus ? Seg1.lo : Seg1.hi);
				Col.Pos2 = (Seg2.Plus ? Seg2.lo : Seg2.hi);

				const string &Row1 = Rows[SegIndex1];
				const string &Row2 = Rows[SegIndex2];
				const unsigned ColCount = SIZE(Row1);
				asserta(SIZE(Row2) == ColCount);

				Col.GenomeIndex1 = Seg1.GenomeIndex;
				Col.GenomeIndex2 = Seg2.GenomeIndex;
				Col.ChrId1 = Seg1.ChrId;
				Col.ChrId2 = Seg2.ChrId;
				Col.Plus = (Seg1.Plus == Seg2.Plus);
				for (unsigned ColIndex = 0; ColIndex < ColCount; ++ColIndex)
					{
					Col.Letter1 = Row1[ColIndex];
					Col.Letter2 = Row2[ColIndex];

					if (!Seg1.Plus)
						Col.Letter1 = CompLetter(Col.Letter1);
					if (!Seg2.Plus)
						Col.Letter2 = CompLetter(Col.Letter2);

					if (Col.Letter1 != '-' || Col.Letter2 != '-')
						OnCol(User, Col);

					if (Col.Letter1 != '-')
						{
						if (Seg1.Plus)
							++Col.Pos1;
						else
							--Col.Pos1;
						}
					if (Col.Letter2 != '-')
						{
						if (Seg2.Plus)
							++Col.Pos2;
						else
							--Col.Pos2;
						}
					}
				}
			}
		}
	if (ShowProgress)
		ProgressDone();
	}

static set<PairwiseCol> *g_Cols;
static void OnCol(unsigned /*User*/, const PairwiseCol &Col)
	{
	g_Cols->insert(Col);
	}

void RevFile::GetPairwiseCols(unsigned GenomeIndex1, unsigned ChrId1,
  unsigned GenomeIndex2, unsigned ChrId2,
  set<PairwiseCol> &Cols) const
	{
	g_Cols = &Cols;
	Cols.clear();
	EnumPairwiseCols(GenomeIndex1, ChrId1, GenomeIndex2, ChrId2, OnCol, 0);
	}
