#ifndef REVFILE_DISK
#define REVFILE_DISK

// These structures are stored on
// disk and therefore fully packed,
// i.e. at one-byte boundaries.

#ifndef __APPLE_CC__
#pragma	pack(push, 1)
#endif

const unsigned RevFileFormatVersion = 0;

const unsigned RevFileUnclosedHdrMagic = 0x8E4F22D;
const unsigned RevFileHdrMagic = 0x9E4F22E;
const unsigned RevFileEndHdrMagic = 0x8E4F22F;
const unsigned OldRevFileHdrMagic = 0x8E4F22E;

const uint16 NullGenomeIndex = (uint16) (~0);

// Purely for sanity checking, not a compiled limit.
const unsigned MaxSaneGenomeCount = 4096;

const unsigned CopyBufferSize = 1024*1024;
const double HashExpandFactor = 2.0;
const unsigned DATECacheSize = 1024;	// entries, not bytes

using namespace std;

const uint32 UINT32_HIBIT = 0x80000000;

const byte BIT_AI_IS_COMPRESSED = 0x80;
const byte BIT_AI_HAS_ROWS = 0x02;
const byte BIT_ROWS_IN_PLACE = 0x04;
const byte BIT_AI_IS_PARALOG = 0x08;

const unsigned DATE_ALN_DATA_BYTES = 6;

enum FOUR_BIT
	{
	FB_A,	// 0x00
	FB_C,	// 0x01
	FB_G,	// 0x02
	FB_T,	// 0x03
	FB_N,	// 0x04
	FB_a,	// 0x05
	FB_c,	// 0x06
	FB_g,	// 0x07
	FB_t,	// 0x08
	FB_n,	// 0x09
	FB_Gap,	// 0x0a
	FB_RFU1,// 0x0b
	FB_RFU2,// 0x0c
	FB_RFU3,// 0x0d
	FB_RFU4,// 0x0e
	FB_RFU5,// 0x0f
	};

struct RevFileHdr
	{
	uint32 Magic;
	uint32 GenomeCount;
	uint32 AlnInfoCount;
	uint32 AlnCount;
	uint64 GenomesPos;
	uint64 AlnInfoTOCPos;		// TOC=TableOfContents=vector of file offsets
	uint64 AlnTOCPos;			// Ditto.
	uint64 HashTablePos;
	uint32 HashTableBytes;
	uint32 HashTableEntryCount;
	uint32 SeqCount;
	uint64 SeqTOCPos;
	uint64 FileFormatVersion;
	uint64 AlnsPos;
	uint64 ForFutureUse[30];	// Hopefully won't have to break compatibility
	uint32 EndHdrMagic;			// Verifies write packing=read packing
	};

struct HashTableEntry
	{
	uint16 GenomeIndex;
	uint32 ChrId;
	uint64 FileOffset;
	uint32 Count; // of AIIndexes (old) or lo-hi pairs (new).
	};

struct OldDiskAlnInfoTOCEntry
	{
	uint32 AlnIndex;
	float64 Score;
	uint16 SegCount;
	uint16 AttrCount;
	uint64 FilePos;
	};

struct DiskAlnInfoTOCEntry
	{
	uint32 AlnIndex;
	float32 Score;
	uint16 SegCount;
	uint16 AttrCount;
	uint64 FilePos;
	};

// Compressed index entry,
// contains AlnInfo + possibly alignment rows also.
struct DiskAlnInfoTOCEntryC
	{
// Arrange things so that the 4'th byte is BitFields.
// This is so that the hi bit of BitFields is the hi
// bit of DiskAlnInfoTOCEntry.
// WARNING -- This is endian-dependent.
	byte ChrId0;
	byte ChrId1;
	int8 DiffLength1;
	byte BitFields;
	uint32 lo0;
	uint16 Length0;
	uint32 lo1;
	byte AlnData[DATE_ALN_DATA_BYTES];

	void LogMe(bool NL = true) const
		{
		Log("chrid0=%u lo0=%u Length0=%u chrid1=%u lo1=%u DiffLength1=%u BitFields=%x",
		  ChrId0,
		  lo0,
		  Length0,
		  ChrId1,
		  lo1,
		  DiffLength1,
		  BitFields);

		if (BitFields & BIT_AI_IS_COMPRESSED)
			Log(" Compressed");
		if (BitFields & BIT_AI_HAS_ROWS)
			Log(" AIHasRows");
		if (BitFields & BIT_AI_HAS_ROWS)
			Log(" Paralog");
		if (NL)
			Log("\n");
		}
	};

struct DiskAlnAttr
	{
	uint16 Attr;
	uint32 Value;
	};

struct DiskGCSeg
	{
	uint16 GenomeIndex;
	uint32 ChrId;
	uint32 lo;
	uint32 hi;
	uint16 Plus;
	};

struct DiskAlnTOCEntry
	{
	uint32 AlnInfoIndex;
	uint32 RowCount;
	uint32 ColCount;
	uint64 FilePos;
	uint32 Padding;
	};

struct DiskSeqTOCHeader
	{
	uint32 Magic;
	uint32 SeqCount;
	uint64 ReservedForFutureUse[8];
	uint32 EndMagic;
	};

struct DiskSeqTOCEntry
	{
	uint32 GenIndex;
	uint32 ChrId;
	uint64 Length;
	uint64 FilePos;
	};

#ifndef __APPLE_CC__
#pragma	pack(pop)
#endif

#endif // REVFILE_DISK
