#ifndef revfiletpl_h
#define revfiletpl_h

#include "revfilebase.h"

bool EncodeRowsInPlace(const StdStrVec &Rows,
  byte BitRows[DATE_ALN_DATA_BYTES]);
void DecodeRowsInPlace(const byte BitRows[DATE_ALN_DATA_BYTES],
  unsigned ColCount, StdStrVec &Rows);
void EncodeRow4Bit(const StdStr &Row, byte *Buffer);
void DecodeRow4Bit(const byte *Buffer, unsigned ColCount, StdStr &Row);

template<typename DiskAlnInfoTOCEntry_t>
  class RevFileTpl : public RevFileBase
{
protected:
	RevFileTpl()
		{
		m_DATECacheFirstEntryIndex = 0;
		m_DATECacheSize = 0;
		m_DATECacheEntryCount = 0;
		m_DATECache = 0;
		m_AICacheHitCount = 0;
		m_AICacheRefreshCount = 0;
		}

virtual bool IsOld() const
	{
	return false;
	}

// Valid when reading only
	mutable unsigned m_DATECacheFirstEntryIndex;
	mutable unsigned m_DATECacheSize;		// in entries, not bytes
	mutable unsigned m_DATECacheEntryCount;
	mutable DiskAlnInfoTOCEntry_t *m_DATECache;

void GetDATE(unsigned AlnInfoIndex, DiskAlnInfoTOCEntry_t &DATE) const
	{
// Already in cache?
	if (AlnInfoIndex >= m_DATECacheFirstEntryIndex &&
	  AlnInfoIndex < m_DATECacheFirstEntryIndex + m_DATECacheEntryCount)
		{
		DiskAlnInfoTOCEntry_t *ptrDATE =
		  m_DATECache + (AlnInfoIndex - m_DATECacheFirstEntryIndex);
		memcpy(&DATE, ptrDATE, sizeof(DATE));
		++m_AICacheHitCount;
		return;
		}

	++m_AICacheRefreshCount;

// Refresh cache so that requested entry is first in cache (optimal
// assuming sequential reads in increasing order of AlnInfoIndex).
	if (m_DATECache == 0)
		{
		m_DATECache = new DiskAlnInfoTOCEntry_t[DATECacheSize];
		asserta(m_DATECache != 0);
		}

	unsigned Sz = sizeof(DiskAlnInfoTOCEntry_t);
	off64_t TOCPos = (off64_t) m_Hdr.AlnInfoTOCPos + AlnInfoIndex*Sz;

	m_DATECacheEntryCount = DATECacheSize;
	if (AlnInfoIndex + m_DATECacheEntryCount > m_AlnInfoCount)
		m_DATECacheEntryCount = m_AlnInfoCount - AlnInfoIndex;

	ReadStdioFile(m_f, TOCPos, m_DATECache,
	  m_DATECacheEntryCount*sizeof(DiskAlnInfoTOCEntry_t));
	m_DATECacheFirstEntryIndex = AlnInfoIndex;

	DiskAlnInfoTOCEntry_t *ptrDATE =
	  m_DATECache + (AlnInfoIndex - m_DATECacheFirstEntryIndex);
	memcpy(&DATE, ptrDATE, sizeof(DATE));
	}

unsigned GetAlnInfoDATE(const DiskAlnInfoTOCEntry_t &DATE,
  unsigned AlnInfoIndex, AlnInfo &AI) const
	{
	const DiskAlnInfoTOCEntryC &DATEC = (DiskAlnInfoTOCEntryC &) DATE;
	if (DATE.AlnIndex != UINT_MAX && (DATEC.BitFields & BIT_AI_IS_COMPRESSED))
		{
		GetAlnInfoDATECompressed(DATEC, AI);
		return AlnInfoIndex | UINT32_HIBIT;
		}

	AI.Segs.clear();
	AI.Values.clear();
	AI.Attrs.clear();

	AI.Score = (ai_score_t) DATE.Score;
	AI.AlnIndex = DATE.AlnIndex;

	unsigned Bytes = DATE.SegCount*sizeof(DiskGCSeg) +
	  DATE.AttrCount*sizeof(DiskAlnAttr);

	byte *Buffer = myalloc(byte, Bytes);
	ReadStdioFile(m_f, (off64_t) DATE.FilePos, Buffer, Bytes);

	DiskGCSeg *BuffSeg = (DiskGCSeg *) Buffer;
	for (unsigned SegIndex = 0; SegIndex < DATE.SegCount; ++SegIndex)
		{
		GCSeg S;
		S.GenomeIndex = (unsigned) BuffSeg->GenomeIndex;
		S.ChrId = (unsigned) BuffSeg->ChrId;
		S.hi = (coord_t) BuffSeg->hi;
		S.lo = (coord_t) BuffSeg->lo;
		S.Plus = static_cast<bool>(BuffSeg->Plus);

		AI.Segs.push_back(S);

		++BuffSeg;
		}

	DiskAlnAttr *BuffAA = (DiskAlnAttr *) (Buffer + DATE.SegCount*sizeof(DiskGCSeg));
	for (unsigned AttrIndex = 0; AttrIndex < DATE.AttrCount; ++AttrIndex)
		{
		AlnAttr Attr = (AlnAttr) BuffAA->Attr;
		unsigned Value = (unsigned) BuffAA->Value;

		AI.Attrs.push_back(Attr);
		AI.Values.push_back(Value);

		++BuffAA;
		}

	myfree(Buffer);

	return DATE.AlnIndex;
	}

// Build TOC (Table Of Contents) for AlnInfo's.
// Do this by appending AlnInfoTOC tmp file to Rev file.
// This requires updating the DiskAlnInfoTOCEntry_t.FilePos
// field by the offset of the TOC in the Rev file.
void WriteAlnInfoTOC()
	{
	off64_t Pos = GetStdioFilePos(m_f);
	m_Hdr.AlnInfoTOCPos = (uint64) Pos;

	if (m_AlnInfoCount > 0)
		{
		unsigned AlnInfoTOCBytes = m_AlnInfoCount*sizeof(DiskAlnInfoTOCEntry_t);
		unsigned AlnInfoSegsBytes = (unsigned) GetStdioFileSize(m_fAlnInfoSegs);
		unsigned AlnTOCFileSize = (unsigned) GetStdioFileSize(m_fAlnInfoTOC);
		asserta(AlnInfoTOCBytes == AlnTOCFileSize);

		uint64 AlnInfoSegsBasePos = (uint64) (m_Hdr.AlnInfoTOCPos + AlnInfoTOCBytes);

	// Read entire AlnInfo TOC tmp file into memory
		DiskAlnInfoTOCEntry_t *TOCBuffer = myalloc(DiskAlnInfoTOCEntry_t, m_AlnInfoCount);

		ReadStdioFile(m_fAlnInfoTOC, 0, TOCBuffer, AlnInfoTOCBytes);

		DiskAlnInfoTOCEntry_t *TOC = TOCBuffer;
		for (unsigned AIIndex = 0; AIIndex < m_AlnInfoCount; ++AIIndex)
			{
			uint32 AlnIndex = TOC->AlnIndex;
			if (AlnIndex == UINT_MAX || (AlnIndex & UINT32_HIBIT) == 0)
				{
				uint64 TmpFilePos = TOC->FilePos;
				asserta(TmpFilePos < AlnInfoSegsBytes);
				TOC->FilePos = TmpFilePos + AlnInfoSegsBasePos;
				}
			TOC++;
			}

	// Append TOC to Rev file
		WriteStdioFile(m_f, TOCBuffer, AlnInfoTOCBytes);

	// Done with AlnInfoTOC
		myfree(TOCBuffer);
		TOCBuffer = 0;
		}

	CloseStdioFile(m_fAlnInfoTOC);
	m_fAlnInfoTOC = 0;

	DeleteStdioFile(m_AlnInfoTOCFileName.c_str());
	}

public:
unsigned GetAlnIndex(unsigned AlnInfoIndex) const
	{
	DiskAlnInfoTOCEntry_t DATE;
	GetDATE(AlnInfoIndex, DATE);
	if (DATE.AlnIndex == UINT_MAX)
		return UINT_MAX;

	const DiskAlnInfoTOCEntryC &DATEC = (DiskAlnInfoTOCEntryC &) DATE;
	if ((DATEC.BitFields & BIT_AI_IS_COMPRESSED) != 0)
		return AlnInfoIndex | UINT32_HIBIT;

	return DATE.AlnIndex;
	}

unsigned AddAlnInfo(const AlnInfo &AI)
	{
	return AddAlnInfo(AI, UINT_MAX);
	}

unsigned AddAlnInfo(const AlnInfo &AI, unsigned AlnIndex)
	{
	STATEC(Writing)

	if (AlnIndex != UINT_MAX && CanCompressAI(AI))
		{
		AddAlnInfoCompress(AI);
		return m_AlnInfoCount++;
		}

	unsigned SegCount = (unsigned) AI.Segs.size();
	unsigned AttrCount = (unsigned) AI.Attrs.size();

	DiskAlnInfoTOCEntry_t AITE;
	AITE.AlnIndex = AlnIndex;
	AITE.SegCount = (uint16) SegCount;
	AITE.AttrCount = (uint16) AttrCount;
	AITE.Score = AI.Score;
	AITE.FilePos = (uint64) GetStdioFilePos(m_fAlnInfoSegs);

	WriteStdioFile(m_fAlnInfoTOC, &AITE, sizeof(AITE));

	unsigned SegAttrBytes = AITE.SegCount*sizeof(DiskGCSeg) +
	  AITE.AttrCount*sizeof(DiskAlnAttr);

	byte *Buffer = myalloc(byte, SegAttrBytes);

	byte *BuffPos = Buffer;
	for (unsigned SegIndex = 0; SegIndex < SegCount; ++SegIndex)
		{
		const GCSeg &S = AI.Segs[SegIndex];

		const unsigned GenomeCount = (unsigned) m_Genomes.size();
		const unsigned GenomeIndex = S.GenomeIndex;
		const unsigned ChrId = S.ChrId;
		if (GenomeIndex >= GenomeCount)
			Die("AddAlnInfo, GenomeIndex[%u]=%u, count=%u",
			  SegIndex, GenomeIndex, GenomeCount);

		DiskGCSeg DS;
		DS.GenomeIndex = (uint16) GenomeIndex;
		DS.ChrId = ChrId;
		DS.lo = (uint32) S.lo;
		DS.hi = (uint32) S.hi;
		DS.Plus = (uint16) S.Plus;

		if ((coord_t) DS.lo != S.lo || (coord_t) DS.hi != S.hi)
			Die("coord_t overflowed uint32");

		memcpy(BuffPos, &DS, sizeof(DS));
		BuffPos += sizeof(DS);

		AddToIndexList((unsigned) S.GenomeIndex, (unsigned) S.ChrId, m_AlnInfoCount);
		}

	for (unsigned AttrIndex = 0; AttrIndex < AttrCount; ++AttrIndex)
		{
		DiskAlnAttr DA;
		unsigned Attr = AI.Attrs[AttrIndex];
		DA.Attr = (uint16) Attr;
		DA.Value = AI.Values[AttrIndex];

		if ((unsigned) DA.Attr != Attr)
			Die("Aln attr overflows uint16");

		memcpy(BuffPos, &DA, sizeof(DA));
		BuffPos += sizeof(DA);
		}

	assert((unsigned) (BuffPos - Buffer) == SegAttrBytes);
	WriteStdioFile(m_fAlnInfoSegs, Buffer, SegAttrBytes);

	myfree(Buffer);
	return m_AlnInfoCount++;
	}

// Copy AlnInfo plus rows, if present, from another file
void CopyAln(const RevFileTpl<DiskAlnInfoTOCEntry_t> &rhs, unsigned rhsAlnInfoIndex)
	{
	STATEC(Writing)

	AlnInfo AI;
	rhs.GetAlnInfo(rhsAlnInfoIndex, AI);

	unsigned rhsAlnIndex = rhs.GetAlnIndex(rhsAlnInfoIndex);
	if (rhsAlnIndex == UINT_MAX)
		AddAlnInfo(AI);
	else
		{
		StdStrVec Rows;
		rhs.GetAlnRows(rhsAlnIndex, Rows);
		AddAln(AI, Rows);
		}
	}

void GetAlnInfoDATECompressed(const DiskAlnInfoTOCEntryC &DATEC,
  AlnInfo &AI) const
	{
	AI.Clear();
	AI.Segs.resize(2);
	GCSeg &Seg0 = AI.Segs[0];
	GCSeg &Seg1 = AI.Segs[1];

	Seg0.GenomeIndex = 0;
	Seg1.GenomeIndex = 1;

	Seg0.ChrId = (DATEC.ChrId0 & 0x7f);
	Seg1.ChrId = (DATEC.ChrId1 & 0x7f);

	Seg0.Plus = ((DATEC.ChrId0 & 0x80) == 0 ? false : true);
	Seg1.Plus = ((DATEC.ChrId1 & 0x80) == 0 ? false : true);

	Seg0.lo = DATEC.lo0;
	Seg1.lo = DATEC.lo1;

	unsigned Length0 = DATEC.Length0;
	int DiffLength1 = DATEC.DiffLength1;
	unsigned Length1 = (unsigned) ((int) Length0 + DiffLength1);

	Seg0.hi = Seg0.lo + Length0 - 1;
	Seg1.hi = Seg1.lo + Length1 - 1;

	if (DATEC.BitFields & BIT_AI_IS_PARALOG)
		{
		AI.Attrs.push_back(ALN_ATTR_Paralog);
		AI.Values.push_back(1);
		}
	}

unsigned GetAlnInfo(unsigned AlnInfoIndex, AlnInfo &AI) const
	{
	if (AlnInfoIndex >= m_AlnInfoCount)
		Die("GetAlnInfo(%u) count=%u", AlnInfoIndex, m_AlnInfoCount);

	DiskAlnInfoTOCEntry_t DATE;
	GetDATE(AlnInfoIndex, DATE);
	return GetAlnInfoDATE(DATE, AlnInfoIndex, AI);
	}

unsigned GetAlnInfoX(unsigned AlnInfoIndex, AlnInfoX &AIX) const
	{
	unsigned AlnIndex = GetAlnInfo(AlnInfoIndex, AIX.AI);
	AIX.AlnInfoIndex = AlnInfoIndex;
	AIX.AlnIndex = AlnIndex;
	return AlnIndex;
	}

unsigned AddAln(const AlnInfo &AI, const StdStrVec &Rows)
	{
	STATEC(Writing)

	if (Rows.size() == 0)
		return UINT_MAX;

	if (AI.Segs.size() != Rows.size())
		Die("RevFileBase::AddAln, %u segs, %u rows",
		  (unsigned) AI.Segs.size(), (unsigned) Rows.size());

	ValidateSegLengths(AI, Rows);

	bool Comp = CanCompressAI(AI);
	if (Comp)
		{
		byte BitRows[DATE_ALN_DATA_BYTES];
		bool RowsInPlace = EncodeRowsInPlace(Rows, BitRows);
		if (RowsInPlace)
			++m_InPlaceRowsCount;
		uint64 FilePosRows = UINT64_MAX;
		if (!RowsInPlace)
			FilePosRows = StoreBitRows(Rows);
		byte *ptrBitRows = (RowsInPlace ? BitRows : 0);
		AddAlnInfoCompress(AI, ptrBitRows, FilePosRows);
		return m_AlnInfoCount++;
		}

	unsigned AlnIndex = m_AlnCount++;
	unsigned AlnInfoIndex = AddAlnInfo(AI, AlnIndex);

	unsigned Length0 = (unsigned) Rows[0].size();

	DiskAlnTOCEntry DH;
	DH.RowCount = (uint32) Rows.size();
	DH.ColCount = (uint32) Length0;
	DH.AlnInfoIndex = AlnInfoIndex;
	DH.FilePos = (uint64) GetStdioFilePos(m_fAlns);

	WriteStdioFile(m_fAlnTOC, &DH, sizeof(DH));

	unsigned Bytes = (unsigned) (DH.RowCount*DH.ColCount);
	char *Buffer = myalloc(char, Bytes);
	char *BuffPos = Buffer;
	for (unsigned RowIndex = 0; RowIndex < DH.RowCount; ++RowIndex)
		{
		const std::string &Row = Rows[RowIndex];
		unsigned Length = (unsigned) Row.size();
		if (Length != Length0)
			Die("AddAln, rows different lengths");

		memcpy(BuffPos, Row.c_str(), Length0);

		BuffPos += DH.ColCount;
		}
	assert(BuffPos == Buffer + Bytes);

	WriteStdioFile(m_fAlns, Buffer, Bytes);

	myfree(Buffer);

	return AlnInfoIndex;
	}

/***
When possible, the AlnInfo structure is stored in its TOC entry.
If short enough, rows will be stored in place also.
	
	Bytes		Description
	-----		-----------
	1			Bits:
					0	1=AI stored in-place, 0=not in place.
					1	1=Has rows, 0=no rows.
					2	1=Rows stored in-place, 0=not in place
					3	1=Paralog, 0=not paralog.
					4-7	RFU

	1			ChrId0
	2-5			lo0 (signed 4-byte int, sign is strand)
	6-7			Length0

	8			ChrId1
	9-12		lo1 (signed 4-byte int, sign is strand)
	13			Length1-Length as signed 1-byte int.

	If rows stored in place:
	14-16		Row0 stored in 2-bit code.
	17-19		Row1 stored in 2-bit code.

	If rows not stored in place:
	14-19		FilePos of rows as 6-byte unsigned int.
***/

void AddAlnInfoCompress(const AlnInfo &AI, const byte *Rows = 0,
  uint64 FilePosRows = UINT64_MAX)
	{
	asserta(CanCompressAI(AI));

	bool RowsInPlace = (Rows != 0);
	bool RowsSeparate = (FilePosRows != UINT64_MAX);
	asserta(!(RowsInPlace && RowsSeparate));

	bool HasRows = (RowsInPlace || RowsSeparate);

	const GCSeg &Seg0 = AI.Segs[0];
	const GCSeg &Seg1 = AI.Segs[1];
	int Length0 = Seg0.GetLength();
	int Length1 = Seg1.GetLength();
	int DiffLength1 = Length1 - Length0;

	asserta(Length0 <= 0x7ffff);
	asserta(std::abs(DiffLength1) <= 0x7f);

	bool IsParalog = 0;
	//unsigned AttrCount = SIZE(AI.Attrs);

	DiskAlnInfoTOCEntryC DATE;

// Bit fields
	DATE.BitFields = 0;
	DATE.BitFields |= BIT_AI_IS_COMPRESSED;

	if (HasRows)
		DATE.BitFields |= BIT_AI_HAS_ROWS;

	if (RowsInPlace)
		DATE.BitFields |= BIT_ROWS_IN_PLACE;

	if (IsParalog)
		DATE.BitFields |= BIT_AI_IS_PARALOG;

// Chr ids
	DATE.ChrId0 = (byte) Seg0.ChrId;
	DATE.ChrId1 = (byte) Seg1.ChrId;

// Strands
	if (Seg0.Plus)
		DATE.ChrId0 |= 0x80;
	if (Seg1.Plus)
		DATE.ChrId1 |= 0x80;

	DATE.lo0 = Seg0.lo;
	DATE.lo1 = Seg1.lo;

// Lengths
	DATE.Length0 = (uint16) Seg0.GetLength();
	DATE.DiffLength1 = (int8) DiffLength1;

// Rows
// Avoid uninitialized data
	memset(DATE.AlnData, 0xff, sizeof(DATE.AlnData));
	if (RowsInPlace)
		memcpy(DATE.AlnData, Rows, sizeof(DATE.AlnData));
	else if (RowsSeparate)
		Trunc64(DATE.AlnData, FilePosRows, DATE_ALN_DATA_BYTES);

	DiskAlnInfoTOCEntry &DATE_NC = (DiskAlnInfoTOCEntry &) DATE;
	asserta((DATE_NC.AlnIndex & UINT32_HIBIT) != 0);

	WriteStdioFile(m_fAlnInfoTOC, &DATE, sizeof(DATE));

	AddToIndexList(Seg0.GenomeIndex, Seg0.ChrId, m_AlnInfoCount);
	AddToIndexList(Seg1.GenomeIndex, Seg1.ChrId, m_AlnInfoCount);

	++m_CompressedAICount;
	}

void GetAlnRowsCompressed(unsigned AlnIndex, StdStrVec &Rows) const
	{
	asserta((AlnIndex & UINT32_HIBIT) != 0);
	unsigned AlnInfoIndex = (AlnIndex & ~UINT32_HIBIT);
	
	DiskAlnInfoTOCEntryC DATEC;
	GetDATEC(AlnInfoIndex, DATEC);

	bool HasRows = ((DATEC.BitFields & BIT_AI_HAS_ROWS) != 0);
	if (!HasRows)
		Die("RevFile::GetAlnRowsCompressed, !BIT_AI_HAS_ROWS");

	bool RowsInPlace = ((DATEC.BitFields & BIT_ROWS_IN_PLACE) != 0);
	if (RowsInPlace)
		return GetAlnRowsInPlace(DATEC, Rows);

	uint64 FilePosRows;
	Untrunc64(DATEC.AlnData, DATE_ALN_DATA_BYTES, FilePosRows);

	uint64 AlnsPos = m_Hdr.AlnsPos;
	if (AlnsPos == 0)
		Die("GetAlnRowsCompressed, AlnsPos = 0 in file header");

	FilePosRows += AlnsPos;

	uint32 ColCount;
	ReadStdioFile(m_f, FilePosRows, &ColCount, sizeof(ColCount));
	uint32 ColCount2 = ColCount + ColCount%2;

	byte *Buffer = myalloc(byte, ColCount2);
	ReadStdioFile(m_f, Buffer, ColCount2);

	Rows.clear();
	Rows.resize(2);
	DecodeRow4Bit(Buffer, ColCount, Rows[0]);
	DecodeRow4Bit(Buffer + ColCount2/2, ColCount, Rows[1]);
	myfree(Buffer);
	}

void GetDATEC(unsigned AlnInfoIndex, DiskAlnInfoTOCEntryC &DATEC) const
	{
	GetDATE(AlnInfoIndex, (DiskAlnInfoTOCEntry_t &) DATEC);
	}

unsigned GetAlnRows(unsigned AlnIndex, StdStrVec &Rows) const
	{
	STATEC(Reading)

	if ((AlnIndex & UINT32_HIBIT) != 0)
		{
		GetAlnRowsCompressed(AlnIndex, Rows);
		return AlnIndex & ~UINT32_HIBIT;
		}

	if (AlnIndex >= m_AlnCount)
		Die("RevFileBase::GetAlnRows(%u) count=%u", AlnIndex, m_AlnCount);

	Rows.clear();

	off64_t TOCPos = (off64_t) m_Hdr.AlnTOCPos + AlnIndex*sizeof(DiskAlnTOCEntry);

	DiskAlnTOCEntry DATE;
	ReadStdioFile(m_f, TOCPos, &DATE, sizeof(DATE));

	unsigned Bytes = (unsigned) (DATE.RowCount*DATE.ColCount);
	char *Buffer = myalloc(char, Bytes);

	ReadStdioFile(m_f, (off64_t) DATE.FilePos, Buffer, Bytes);

	for (unsigned RowIndex = 0; RowIndex < DATE.RowCount; ++RowIndex)
		{
		char *ptrRow = Buffer + RowIndex*DATE.ColCount;
		std::string Row(ptrRow, DATE.ColCount);
		Rows.push_back(Row);
		}

	myfree(Buffer);
	return DATE.AlnInfoIndex;
	}
};

#endif // revfiletpl_h
