#include "evolver.h"
#include "revfile.h"
#include "getopts.h"
#include "progress.h"
#include "myrand.h"

void LogERDetails();
unsigned GetSeed();

extern PTR_TRY_EVENT_FUNC TryEventFuncs[];

void TestGeneDupe()
	{
	const char *InSeqFileName = RequiredValueOpt("seq");
	const char *InAnnotFileName = RequiredValueOpt("annots");
	unsigned GenomeIndex = 0;
	IntOpt("genix", &GenomeIndex);
	unsigned ChrId = 0;
	IntOpt("chrid", &ChrId);
	const char *InChrName = ValueOpt("chrname");

	InitRand();

	RevFile In;
	In.Open(InSeqFileName);

	if (InChrName != 0)
		ChrId = In.GetChrId(GenomeIndex, InChrName);
	else
		InChrName = In.GetChrName(GenomeIndex, ChrId).c_str();

	Evolver Evo;

	Evo.FromRevFile(In, GenomeIndex, ChrId, true);
	Evo.GetAnnots(InAnnotFileName, InChrName);
	Evo.Validate();

	gpos_t L = Evo.GetEvolvedLength();
	vector<gpos_t> GeneStarts;
	vector<gpos_t> CopiedGeneStarts;

/// (1)
	Progress("Copy (1)");
	Evo.Copy(0, L-2, L/2, GeneStarts, CopiedGeneStarts);

	Progress("%u genes duplicated", SIZE(GeneStarts));

	Progress("DupeGenes");
	Evo.DupeGenes(GeneStarts, CopiedGeneStarts, false);

	Progress("Validate (1)");
	Evo.Validate();

/// (2)
	Progress("Copy (2)");
	Evo.Copy(0, L-2, L/2, GeneStarts, CopiedGeneStarts);

	Progress("%u genes duplicated", SIZE(GeneStarts));

	Progress("DupeGenes");
	Evo.DupeGenes(GeneStarts, CopiedGeneStarts, false);

	Progress("Validate (2)");
	Evo.Validate();

/// (3)
	Progress("Copy (3)");
	Evo.Copy(0, L-2, L/2, GeneStarts, CopiedGeneStarts, true);

	Progress("%u genes duplicated", SIZE(GeneStarts));

	Progress("DupeGenes");
	Evo.DupeGenes(GeneStarts, CopiedGeneStarts, true);

	Progress("Validate (3)");
	Evo.Validate();

	In.Close();
	}
