#include "ice.h"
#include "getopts.h"
#include "revfile.h"
#include "progress.h"
#include "myrand.h"

void SetModel(const char *FileName);
void XferAnnotsVec(const vector<GFFRecord2> &InRecs, const RevFile &AlnRev,
  unsigned FromGenomeIndex, unsigned ToGenomeIndex, FILE *f);

void TestInterChrGeneDupe()
	{
	const char *AnSeqFileName = RequiredValueOpt("testicgenedupe");
	const char *CEFileName = RequiredValueOpt("annots");
	const char *OutGFFFileName = ValueOpt("outannots");
	const char *OutSeqFileName = ValueOpt("outseq");
	const char *AlnFileName = ValueOpt("aln");
	const char *ModelFile = ValueOpt("model");
	bool LogEvents = FlagOpt("logevents");
	bool Dump = FlagOpt("dump");
	unsigned GenIndex = 0;
	IntOpt("genix", &GenIndex);
	const char *OutGenomeName = ValueOpt("outgenome");
	if (OutGenomeName == 0)
		OutGenomeName = "evolved";
	const char *OutContigsFileName = ValueOpt("outcontigs");
	bool ValInit = !FlagOpt("novalinit");
	bool ValFinal = !FlagOpt("novalfinal");

	InitRand();

	SetModel(ModelFile);

	vector<double> InitialRates;

	RevFile AnSeqRev;
	AnSeqRev.Open(AnSeqFileName);

	InterChrEvolver ICE;
	ICE.FromRev(AnSeqRev, GenIndex);

	vector<GFFRecord2> IslandRecs;
	ICE.ReadAnnots(CEFileName, IslandRecs);
	ICE.InitNeutrals();
	ICE.SetLogEvents(LogEvents);
	if (ValInit)
		ICE.Validate("Initial state");

	if (Dump)
		{
		Log("Initial state:\n");
		ICE.LogMe();
		}

	Progress("Copy");
	// gpos_t L = ICE.GetChrLength(0);

	unsigned FromChrIndex = 0;
	gpos_t FromPos = 8300;
	gpos_t CopyLength = 20000;
	unsigned ToChrIndex = 1;
	gpos_t ToPos = 1;
	bool DoInvert = true;

//unsigned g_GeneDupeWeights[4][4] =
//	{
////		OldSlower    OldSame    OldFaster   OldLost
//	{	        0,         0,           0,        0 },	// NewSlower
//	{	        0,         0,           1,       10 },	// NewSame
//	{	        0,        10,          10,        0 },	// NewFaster
//	{	        0,       100,           0,        0 },	// NewLost
//	};
	extern unsigned g_GeneDupeWeights[4][4];
	for (int i = 0; i < 4; ++i)
		for (int j = 0; j < 4; ++j)
			g_GeneDupeWeights[i][j] = 0;
	g_GeneDupeWeights[1][1] = 1; // OldSame, NewSame

	//ContigSet SourceContigs;
	//ICE.GetContigs(FromChrIndex, FromPos, CopyLength, SourceContigs);
	//Log("Source contigs:\n");
	//ICE.LogContigs(SourceContigs);

	ICE.Copy(FromChrIndex, FromPos, ToChrIndex, ToPos, CopyLength, DoInvert, UINT_MAX, true);

	//ContigSet CopiedContigs;
	//ICE.GetContigs(ToChrIndex, ToPos+1, CopyLength, CopiedContigs);
	//Log("Copied contigs:\n");
	//ICE.LogContigs(CopiedContigs);

	Progress("Gene dupe");
	ICE.GeneDupe(FromChrIndex, FromPos, ToChrIndex, ToPos, CopyLength, DoInvert);

	Progress("Merge neutrals");
	ICE.MergeContiguousNeutrals();

	if (ValFinal)
		ICE.Validate("Final state");

	if (OutGFFFileName != 0)
		{
		Progress("Write annotations");
		ICE.AnnotsToGFF(OutGFFFileName);
		}

	if (OutContigsFileName != 0)
		ICE.ToGFF(OutContigsFileName);

	if (Dump)
		ICE.LogMe();

	if (OutSeqFileName != 0)
		{
		Progress("Creating evolved sequence");
		ICE.SeqToRev(OutSeqFileName, AnSeqRev, GenIndex, OutGenomeName);
		}

	if (!IslandRecs.empty() && AlnFileName != 0)
		{
		FILE *fOut = fopen(OutGFFFileName, "a");
		if (fOut == 0)
			Die("Cannot open %s for appending", OutGFFFileName);

		RevFile AlnRev;
		AlnRev.Open(AlnFileName);
		XferAnnotsVec(IslandRecs, AlnRev, 0, 1, fOut);
		AlnRev.Close();
		fclose(fOut);
		}

	if (AlnFileName != 0)
		ICE.AlnToRev(AlnFileName, AnSeqRev, GenIndex, OutGenomeName, Dump);

	AnSeqRev.Close();
	}
