#define TRACE	0

#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

uint8 GetRandProbIndex(double MAC);

EVENT_RESULT Evolver::Try_CreateNGE()
	{
	vector<gpos_t> NGEStarts;
	vector<gpos_t> NGELengths;
	vector<double> NGEMACs;
	GetNGEs(NGEStarts, NGELengths, NGEMACs);
	unsigned NNGE = SIZE(NGEStarts);
	asserta(SIZE(NGELengths) == NNGE);
	asserta(SIZE(NGEMACs) == NNGE);
	if (NNGE == 0)
		EV_FAILN(CreateNGE, NoNGEs);

	vector<gpos_t> NeutralStarts;
	vector<gpos_t> NeutralLengths;
	GetNeutralRanges(m_FirstEvolvedPos, UINT32_MAX, false,
	  NeutralStarts, NeutralLengths);

	unsigned NNeutrals = SIZE(NeutralStarts); 
	asserta(SIZE(NeutralLengths) == NNeutrals);

	gpos_t NGEStart = GPOS_T_MAX;
	gpos_t NGELength = GPOS_T_MAX;
	gpos_t NeutralStart = GPOS_T_MAX;
	gpos_t NeutralLength = GPOS_T_MAX;
	for (unsigned Try = 0; ; ++Try)
		{
		if (Try >= 100)
			EV_FAILN(CreateNGE, MaxTries)

		unsigned rNGE = RandInt(SIZE(NGEStarts));
		NGEStart = NGEStarts[rNGE];
		NGELength = NGELengths[rNGE];

		vector<gpos_t> CandStarts;
		vector<gpos_t> CandLengths;
		vector<double> CandWeights;
		gpos_t MinCandLength = NGELength;
		for (unsigned i = 0; i < NNeutrals; ++i)
			{
			if (NeutralLengths[i] > MinCandLength)
				{
				CandStarts.push_back(NeutralStarts[i]);
				CandLengths.push_back(NeutralLengths[i]);
				CandWeights.push_back(NeutralLengths[i]);
				}
			}
		if (!CandStarts.empty())
			{
			unsigned rNeutral = RandWeightDist(CandWeights);
			NeutralStart = CandStarts[rNeutral];
			NeutralLength = CandLengths[rNeutral];
			break;
			}
		}

	vector<uint8> ProbIndexes;
	gpos_t Pos = NGEStart;
	m_CurrentEventPos = Pos;
	for (gpos_t i = 0; i < NGELength; ++i)
		{
		const BaseInfo &Base = GetBase(Pos);
		asserta(!Base.IsGene() && Base.IsConstrained());
		ProbIndexes.push_back(Base.ProbIx);
		}

	Pos = NeutralStart;
	for (gpos_t i = 0; i < NeutralLength; ++i)
		{
		const BaseInfo &Base = GetBase(Pos);
		asserta(!Base.IsGene() && Base.IsUnconstrained());
		Pos = GetNextPos(Pos);
		}

	asserta(NeutralLength > NGELength);
	gpos_t Offset = RandInt(NeutralLength - NGELength);

	Pos = NeutralStart;
	for (gpos_t i = 0; i < Offset; ++i)
		{
		const BaseInfo &Base = GetBase(Pos);
		asserta(!Base.IsGene() && Base.IsUnconstrained());
		Pos = GetNextPos(Pos);
		}

	gpos_t NewNGEStartPos = Pos;
	for (gpos_t i = 0; i < NGELength; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
		asserta(!Base.IsGene() && Base.IsUnconstrained());
		Base.ProbIx = ProbIndexes[i];
		Pos = GetNextPos(Pos);
		}

	EV_ACCEPTN(CreateNGE, NewNGEStartPos);
	}
