#define TRACE	0

#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

EVENT_RESULT Evolver::Try_CreateNXE()
	{
	vector<gpos_t> NXEStarts;
	vector<gpos_t> NXELengths;
	vector<double> NXEMACs;

	GetNXEs(m_FirstEvolvedPos, UINT32_MAX, NXEStarts, NXELengths, NXEMACs);
	unsigned NNXE = SIZE(NXEStarts);
	asserta(SIZE(NXELengths) == NNXE);
	asserta(SIZE(NXEMACs) == NNXE);
	if (NNXE == 0)
		EV_FAILN(CreateNXE, NoNXEs);

	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	gpos_t NXEStart = GPOS_T_MAX;
	gpos_t NXELength = GPOS_T_MAX;
	gpos_t LocusStart = GPOS_T_MAX;
	gpos_t LocusOffset = GPOS_T_MAX;
	unsigned Try = 1;
	for (; ; ++Try)
		{
#if	TRACE
		Log("\n");
		Log("Try %u\n", Try);
#endif
		if (Try >= 100)
			EV_FAILN(CreateNGE, MaxTries)

		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(CreateNXE, NoCandidates)

		unsigned rNXE = RandInt(SIZE(NXEStarts));
		NXEStart = NXEStarts[rNXE];
		NXELength = NXELengths[rNXE];
#if	TRACE
		Log("NXEStart %"PF" Length %"PF"\n", NXEStart, NXELength);
#endif
		vector<gpos_t> NeutralStarts;
		vector<gpos_t> NeutralLengths;
		GetNeutralRanges(Gene.Pos, Gene.Length, true, NeutralStarts,
		  NeutralLengths);

	// Locus is range of candidate start positions,
	// so margin and length of element are excluded.
		vector<gpos_t> LocusStarts;
		vector<gpos_t> LocusLengths;
		vector<double> LocusWeights;

		const unsigned N = SIZE(NeutralStarts);
		for (unsigned i = 0; i < N; ++i)
			{
			gpos_t NeutralStart = NeutralStarts[i];
			gpos_t NeutralLength = NeutralLengths[i];
#if	TRACE
			Log("NeutStart %"PF" NeutLen %"PF" MinLen %"PF"\n",
			  NeutralStart,
			  NeutralLength,
			  NXELength);
#endif
			if (NeutralLength > NXELength)
				{
				gpos_t LocusLength = NeutralLength - NXELength;
				LocusStarts.push_back(NeutralStart);
				LocusLengths.push_back(LocusLength);
				LocusWeights.push_back(LocusLength);
				}
			}
#if	TRACE
		Log("Gene %u neutral ranges %u loci %u\n",
		  Gene.Index, SIZE(NeutralStarts), SIZE(LocusStarts));
#endif
		if (!LocusStarts.empty())
			{
			unsigned r = RandWeightDist(LocusWeights);
			LocusStart = LocusStarts[r];
			gpos_t LocusLength = LocusLengths[r];
			LocusOffset = RandInt(LocusLength);
			break;
			}
		}

	gpos_t FromPos = NXEStart;
	gpos_t ToPos = LocusStart;
	m_CurrentEventPos = ToPos;

	for (gpos_t i = 0; i < LocusOffset; ++i)
		{
		const BaseInfo &Base = GetBase(ToPos);
		asserta(Base.IsGene());
		asserta(Base.IsUnconstrained());

		ToPos = GetNextPos(ToPos);
		}

	for (gpos_t i = 0; i < NXELength; ++i)
		{
		const BaseInfo &FromBase = GetBase(FromPos);
		BaseInfo &ToBase = GetBase(ToPos);

		asserta(FromBase.IsNXE());

		asserta(ToBase.IsGene());
		asserta(ToBase.IsUnconstrained());
	
		ToBase.ProbIx = FromBase.ProbIx;

		FromPos = GetNextPos(FromPos);
		ToPos = GetNextPos(ToPos);
		}

	EV_ACCEPTN(CreateNXE, NXEStart);
	}
