#include "ice.h"
#include "ratemgr.h"
#include "evomodel.h"
#include "myrand.h"

#define	TRACE	0

extern RateMgr *g_InterChrCopyRateMgr;
extern unsigned g_InvertableCopyCount;
extern unsigned g_InvertedCopyCount;
extern unsigned g_TotalCopyCount;

EVENT_RESULT InterChrEvolver::Try_InterChrCopy()
	{
	if (GetChrCount() == 1)
		EV_FAILN(InterChrCopy, ChrCount);

	gpos_t FromPos;
	unsigned FromChrIndex;
	GetRandomPos(FromChrIndex, FromPos);
	m_CurrentEventPos = FromPos;

	gpos_t CopyLength = g_InterChrCopyRateMgr->GetLength();
	if (!RangeExists(FromChrIndex, FromPos, CopyLength))
		EV_FAILN(InterChrCopy, RangeExists)

	gpos_t ToPos;
	unsigned ToChrIndex;
	GetRandomPosDifferentChr(FromChrIndex, ToChrIndex, ToPos);

	bool Accept = AcceptInsert(ToChrIndex, ToPos, CopyLength);
	if (!Accept)
		EV_REJECT(InterChrCopy, CopyLength, Insert);

	bool DoInvert = false;
	if (CopyLength >= P_MinCopyInvertLength)
		{
		++g_InvertableCopyCount;
		DoInvert = RandTrue(P_CopyInvertProb);
		if (DoInvert)
			++g_InvertedCopyCount;
		}
	++g_TotalCopyCount;

	bool Strand;
	unsigned GeneIndex = GetGeneIndex(ToChrIndex, ToPos, Strand);

#if	TRACE
	{
	Log("ICCOPY From %u,%"PF" To %u,%"PF" Length %"PF" Invert %c\n",
	  FromChrIndex, FromPos, ToChrIndex, ToPos, CopyLength, tof(DoInvert));
	ContigSet Contigs;
	GetOverlappingContigs(FromChrIndex, FromPos, CopyLength, Contigs);
	Log("Contigs to copy (overlapping):\n");
	LogContigs(Contigs);
	Log("Insert point (overlapping):\n");
	GetOverlappingContigs(ToChrIndex, ToPos, 2, Contigs);
	LogContigs(Contigs);
	}
#endif
	
	Copy(FromChrIndex, FromPos, ToChrIndex, ToPos, CopyLength, DoInvert,
	  GeneIndex, Strand);

#if	TRACE
	Log("Copy OK, starting gene dupe\n");
#endif

	GeneDupe(FromChrIndex, FromPos, ToChrIndex, ToPos, CopyLength, DoInvert);

	EV_ACCEPT(InterChrCopy, ToPos, CopyLength);
	}
