#include "evolver.h"
#include "myrand.h"
#include "evomodel.h"
#include "ratemgr.h"

#define TRACE	0

extern unsigned g_TotalMoveCount;
extern unsigned g_InvertableMoveCount;
extern unsigned g_InvertedMoveCount;

/***
	Source Segment	Insert point	Reject
	--------------	------------	------
	Any				CE				Always

	NG neutral		NG neutral		Never
	NG neutral		Gene neutral	Never

	Gene neutral	Any				Iff source is intron & intron length reduced < min

	Complete CE		NG neutral		Never
	Complete CE		Gene neutral	Always

	Complete gene	NG neutral		Never
	Complete gene	Gene neutral	Always

	Partial CE		Any				Always
	Partial gene	Any				Always
***/

EVENT_RESULT Evolver::Try_Move()
	{
	gpos_t FromPos = GetRandomPos();
	if (FromPos == GPOS_T_MAX)
		EV_FAIL(Move, 0, GetRandomPos)

	EV_GETLENGTH(Move, MoveLength)
	if (MoveLength == GPOS_T_MAX)
		EV_FAIL(Move, MoveLength, GetMoveLength)

	if (!RangeExists(FromPos, MoveLength))
		EV_FAIL(Move, MoveLength, RangeExists)

	gpos_t ToPos = GetRandomPos();
	if (ToPos == GPOS_T_MAX)
		EV_FAIL(Move, MoveLength, GetRandomPos)

	gpos_t ToPos2 = GetNextPos(ToPos);
	if (ToPos2 == GPOS_T_MAX)
		EV_FAIL(Move, MoveLength, RangeExists)

	if (PosInRange(ToPos, FromPos, MoveLength))
		EV_FAIL(Move, MoveLength, PosInRange)

	if (PosInCE(ToPos) || PosInCE(ToPos2))
		EV_REJECT(Move, MoveLength, InsIntoCE)//(1)

	if (PosInGene(FromPos))
		EV_REJECT(Move, MoveLength, PartialGene)//(2)

	if (PosInCE(FromPos))
		EV_REJECT(Move, MoveLength, PartialCE)//(3)

	gpos_t EndPos = GetRangeEndPos(FromPos, MoveLength);

	if (PosInGene(EndPos))
		EV_REJECT(Move, MoveLength, PartialGene)//(4)

	if (PosInCE(EndPos))
		EV_REJECT(Move, MoveLength, PartialCE)//(5)

	bool SourceHasGene = RangeHasGene(FromPos, MoveLength);
	bool InsertIntoGene = PosInGene(ToPos) || PosInGene(ToPos2);
	bool SourceHasCE = RangeHasCE(FromPos, MoveLength);

	if (SourceHasGene && InsertIntoGene)
		EV_REJECT(Move, MoveLength, GeneIntoGene)//(6)

	if (SourceHasCE && InsertIntoGene)
		EV_REJECT(Move, MoveLength, CEIntoGene)//(7)

	m_CurrentEventPos = FromPos;

	bool DoInvert = false;
	if (MoveLength > P_MinMoveInvertLength)
		{
		++g_InvertableMoveCount;
		DoInvert = RandTrue(P_MoveInvertProb);
		}

#if	TRACE
	Log("\n");
	Log("Move from %u to %u length %u inv %c\n", FromPos, ToPos, MoveLength, tof(DoInvert));
	Log("From before:\n");
	LogRegions(FromPos, MoveLength);
	Log("Insert pos before:\n");
	LogNeighborRegions(ToPos);
#endif

	Move(FromPos, ToPos, MoveLength);

	if (DoInvert)
		{
		gpos_t FirstInsertedPos = GetNextPos(ToPos);
		Invert(FirstInsertedPos, MoveLength);
		++g_InvertedMoveCount;
		}
	++g_TotalMoveCount;

	if (InsertIntoGene)
		SetInsertConstraints(ToPos, MoveLength);

#if	TRACE
	Log("Insert pos after:\n");
	LogNeighborRegions(ToPos);
#endif

	EV_ACCEPT(Move, ToPos, MoveLength)
	}
