#define TRACE	0

#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

uint8 GetRandProbIndex(double MAC);

EVENT_RESULT Evolver::Try_MoveNGE()
	{
	vector<gpos_t> Starts;
	vector<gpos_t> Lengths;
	vector<double> MACs;
	GetNGEs(Starts, Lengths, MACs);
	unsigned N = SIZE(Starts);
	asserta(SIZE(Lengths) == N);
	asserta(SIZE(MACs) == N);
	if (N == 0)
		EV_FAILN(MoveNGE, NoNGEs);

	gpos_t Length = GPOS_T_MAX;
	gpos_t StartPos = GPOS_T_MAX;
	gpos_t Dist = GPOS_T_MAX;
	double MAC = FLT_MAX;
	bool Plus = true;
	for (unsigned Try = 0; ; ++Try)
		{
		if (Try >= 50)
			EV_FAILN(MoveNGE, MaxTries)

		unsigned r = RandWeightDist(MACs);
		Length = Lengths[r];
		StartPos = Starts[r];
		Dist = (gpos_t) RandNorm(P_MoveNGEMeanDist, P_MoveNGEStdDevDist);
		if (Dist == 0)
			Dist = 1;
		Plus = RandTrue(0.5);
		gpos_t Pos = Plus ? GetRangeEndPos(StartPos, Length) : StartPos;
#if	TRACE
		Log("\n");
		Log("MoveBGGE try %u StartPos %"PF" Length %"PF" Dist %"PF" %c\n",
		  Try, StartPos, Length, Dist, pom(Plus));
#endif
		m_CurrentEventPos = Pos;
		for (gpos_t i = 0; i < Dist; ++i)
			{
			Pos = GetAdjPos(Pos, Plus);
			if (Pos == GPOS_T_MAX)
				goto NextTry;
			const BaseInfo &Base = GetBase(Pos);
#if	TRACE
			Log("Margin base ");
			Base.LogMe(true);
#endif
			if (Base.IsGene() || Base.IsConstrained())
				goto NextTry;
			}
		MAC = MACs[r];
		break;
	NextTry:;
		}

	vector<uint8> ProbIndexes;
#if	TRACE
	Log("\n");
	Log("Convert bases\n");
#endif
	gpos_t Pos = StartPos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
#if	TRACE
		Log("Old NGE ");
		Base.LogMe();
#endif
		asserta(!Base.IsGene() && Base.IsConstrained());
		ProbIndexes.push_back(Base.ProbIx);
		Base.Deconstrain();
#if	TRACE
		Log(" => ");
		Base.LogMe(true);
#endif
		Pos = GetNextPos(Pos);
		}
	asserta(SIZE(ProbIndexes) == Length);

	gpos_t NewStartPos = GPOS_T_MAX;
	if (Plus)
		NewStartPos = GetRangeEndPos(StartPos, Dist);
	else
		{
		NewStartPos = StartPos;
		for (gpos_t i = 0; i < Dist; ++i)
			NewStartPos = GetPrevPos(NewStartPos);
		}

#if	TRACE
	Log("NewStartPos %"PF"\n", NewStartPos);
#endif
	Pos = NewStartPos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
#if	TRACE
		Log("NewNGE ");
		Base.LogMe();
#endif
		asserta(!Base.IsGene() && !Base.IsConstrained());
		Base.ProbIx = ProbIndexes[i];
#if	TRACE
		Log(" => ");
		Base.LogMe(true);
#endif
		Pos = GetNextPos(Pos);
		}

	EV_ACCEPTN(MoveNGE, StartPos)
	}
