#define TRACE	2

#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

uint8 GetRandProbIndex(double MAC);

EVENT_RESULT Evolver::Try_MoveNXE()
	{
	vector<GeneInfo> Genes;
	vector<double> GeneMACs;
	InitTryGenes(Genes, GeneMACs);

	gpos_t Length = GPOS_T_MAX;
	gpos_t StartPos = GPOS_T_MAX;
	gpos_t Dist = GPOS_T_MAX;
	bool Plus = true;
	GeneInfo Gene;
	for (unsigned Try = 0; ; ++Try)
		{
		if (Try >= 50)
			EV_FAILN(MoveNXE, MaxTries)

		bool Any = NextTryGene(Genes, GeneMACs, Gene);
		if (!Any)
			EV_FAILN(MoveNXE, NoCandidates)

		vector<gpos_t> Starts;
		vector<gpos_t> Lengths;
		vector<double> NXEMACs;
		GetNXEs(Gene.Pos, Gene.Length, Starts, Lengths, NXEMACs);
		unsigned N = SIZE(Starts);
		asserta(SIZE(Lengths) == N);
		asserta(SIZE(NXEMACs) == N);
		if (N == 0)
			continue;

		unsigned r = RandWeightDist(NXEMACs);
		Length = Lengths[r];
		StartPos = Starts[r];
		Dist = (gpos_t) RandNorm(P_MoveNXEMeanDist, P_MoveNXEStdDevDist);
		if (Dist == 0)
			Dist = 1;
		Plus = RandTrue(0.5);
		gpos_t Pos = Plus ? GetRangeEndPos(StartPos, Length) : StartPos;
		m_CurrentEventPos = Pos;
TRC
		Log("\n");
		Log("MoveNXE try %u StartPos %"PF" Length %"PF" Dist %"PF" %c\n",
		  Try, StartPos, Length, Dist, pom(Plus));
ENDTRC
		for (gpos_t i = 0; i < Dist; ++i)
			{
			Pos = GetAdjPos(Pos, Plus);
			if (Pos == GPOS_T_MAX)
				goto NextTry;
			const BaseInfo &Base = GetBase(Pos);
TRC
			Log("Margin base ");
			Base.LogMe(true);
ENDTRC
			if (!Base.IsGene() || Base.IsConstrained())
				goto NextTry;
			}
		break;
	NextTry:;
		}

	vector<uint8> ProbIndexes;
TRC
	Log("\n");
	Log("Convert bases\n");
ENDTRC
	gpos_t Pos = StartPos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
TRC
		Log("Old NXE ");
		Base.LogMe();
ENDTRC
		asserta(Base.IsGene() && Base.IsNXE());
		ProbIndexes.push_back(Base.ProbIx);
		Base.Deconstrain();
TRC
		Log(" => ");
		Base.LogMe(true);
ENDTRC
		Pos = GetNextPos(Pos);
		}
	asserta(SIZE(ProbIndexes) == Length);

	gpos_t NewStartPos = GPOS_T_MAX;
	if (Plus)
		NewStartPos = GetRangeEndPos(StartPos, Dist);
	else
		{
		NewStartPos = StartPos;
		for (gpos_t i = 0; i < Dist; ++i)
			NewStartPos = GetPrevPos(NewStartPos);
		}

TRC
	Log("NewStartPos %"PF"\n", NewStartPos);
ENDTRC
	Pos = NewStartPos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
TRC
		Log("NewNXE ");
		Base.LogMe();
ENDTRC
		asserta(Base.IsGene() && !Base.IsConstrained());
		Base.ProbIx = ProbIndexes[i];
TRC
		Log(" => ");
		Base.LogMe(true);
ENDTRC
		Pos = GetNextPos(Pos);
		}

	GeneInfo NewGene;
	GetGene(StartPos, NewGene);
	unsigned ErrCount = ValidateGene(NewGene);
	if (ErrCount > 0)
		Die("MoveNXE %"PF", validate gene failed", StartPos);

	EV_ACCEPTN(MoveNXE, StartPos)
	}
