#define TRACE	0

#include "evolver.h"
#include "ratemgr.h"
#include "myrand.h"
#include "evomodel.h"

int8 IncCodexStrand(int8 Codex, bool Strand);
bool IsStopCodon(const string &Codon);
uint8 GetRandProbIndex(double MAC);

EVENT_RESULT Evolver::Try_MoveStopCodonIntoUTR()
	{
	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	gpos_t NewStopPos = GPOS_T_MAX;
	for (;;)
		{
		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(MoveStopCodonIntoUTR, NoCandidates)

		NewStopPos = GetMoveStopCodonIntoUTRCand(Gene);
		if (NewStopPos != GPOS_T_MAX)
			break;
		}
	asserta(NewStopPos != GPOS_T_MAX);
	m_CurrentEventPos = NewStopPos;

#if	TRACE
	Log("Select: StopPos %"PF"\n", NewStopPos);
	Log("Gene: ");
	Gene.LogMe();
#endif
	bool Strand = Gene.Plus;
	gpos_t Pos = Gene.StopCodonFirstBasePos;

// First base in current stop codon must have codex +/-1.
	int8 Codex = Gene.Plus ? 1 : -1;

	BaseInfo &Base = GetBase(Pos);
	asserta(Base.Codex == Codex);

// Mutate current stop codon to non-stop.
// Changing T to C or G is lazy but safe way to ensure not a stop,
// better would be to try random substitutions at random positions
// in the codon, but not worth the extra implementation effort.
	asserta(Base.GetStrandLetter(Gene.Plus) == 'T');
	if (RandTrue(0.5))
		Base.SetLetter('C');
	else
		Base.SetLetter('G');

#if	TRACE
	Log("NewStopPos %"PF" %"PF" %"PF"\n",
	  NewStopPos, SecondNewStopPos, LastNewStopPos);
#endif

	Pos = Gene.StopCodonFirstBasePos;
	unsigned BaseCount = 0;
	for (;;)
		{
		if (Pos == NewStopPos)
			break;
		++BaseCount;
		BaseInfo &Base = GetBase(Pos);
#if	TRACE
		Log("Convert base %u, before: ", BaseCount);
		Base.LogMe();
#endif
		asserta(Base.IsExon());
		Base.SetCDS();
		Base.Codex = Codex;
		Base.ProbIx = GetRandProbIndex(Gene.MeanAccProb);
#if	TRACE
		Log(" .. after: ");
		Base.LogMe(true);
#endif
		Codex = IncCodexStrand(Codex, Strand);
		Pos = GetAdjExonPos(Pos, Strand);
		}
	asserta(BaseCount > 0);
	asserta(BaseCount%3 == 0);

	SetStopCodon(NewStopPos, Strand);

	GeneInfo NewGene;
	GetGene(NewStopPos, NewGene);
	unsigned ErrCount = ValidateGene(NewGene);
	if (ErrCount > 0)
		Die("Try_MoveStopCodonIntoUTR: Validate gene failed");

	EV_ACCEPTN(MoveStopCodonIntoUTR, NewStopPos)
	}

// Returns first pos in translation order, so is always pos of T in
// of Txy whether or not gene is on minus strand.
gpos_t Evolver::GetMoveStopCodonIntoUTRCand(const GeneInfo &Gene) const
	{
#if	TRACE
	Log("\n");
	Log("GetMoveStopCand\n");
	Log("Gene: ");
	Gene.LogMe();
#endif

	bool Strand = Gene.Plus;

	gpos_t FirstUTRBaseCount = 0;
	gpos_t LastUTRBaseCount = 0;
	GetUTRBaseCounts(Gene, FirstUTRBaseCount, LastUTRBaseCount);
	gpos_t UTRBaseCount = Strand ? LastUTRBaseCount : FirstUTRBaseCount;

	gpos_t Pos = Gene.StopCodonLastBasePos;

// Position on first exon base following current STOP codon.
	Pos = GetAdjExonPos(Pos, Strand);
	for (unsigned CodonsMovedCount = 0; ; ++CodonsMovedCount)
		{
		if (Pos == GPOS_T_MAX)
			return GPOS_T_MAX;

		string Codon;
		gpos_t NextCodonPos = GetNextExonTriplet(Pos, Codon, Strand);
		if (NextCodonPos == GPOS_T_MAX)
			return GPOS_T_MAX;

#if	TRACE
		Log("Pos=%"PF" Codon=%s\n", Pos, Codon.c_str());
#endif
		if (IsStopCodon(Codon))
			{
			if (UTRBaseCount < CodonsMovedCount*3 + P_MinUTRBases)
				return GPOS_T_MAX;
#if	TRACE
			Log("Cand %s\n", Codon.c_str());
#endif
			return Pos;
			}
		Pos = NextCodonPos;
		}
	}
