#ifndef TYPES_H
#define TYPES_H

// Derived types.

#include "lotypes.h"
#include <limits.h>
#include <float.h>
#include <list>
#include <map>
#include <algorithm>
#include <vector>
#include <string>

#define	INTEGER_DP		0

// Genome coordinate
typedef uint32 coord_t;
typedef uint64 coord64_t;
typedef uint32 gpos_t;

#ifdef __APPLE_CC__
typedef off_t off64_t;
#endif

const uint32 COORD_T_MAX = UINT_MAX;
const coord64_t COORD64_T_MAX	= UINT64_MAX;
const gpos_t GPOS_T_MAX	= UINT_MAX;

#ifndef UINT32_MAX
#define UINT32_MAX	UINT_MAX
#endif

#ifdef _MSC_VER

#define COORD_T_PRINTF		"u"
#define COORD64_T_PRINTF	"llu"
#define COORD_T_PRINTFX		"x"
#define COORD64_T_PRINTFX	"llx"
#define UINT32_PRINTF		"u"

#elif defined(__x86_64__)

#define COORD_T_PRINTF		"u"
#define COORD64_T_PRINTF	"lu"
#define COORD_T_PRINTFX		"x"
#define COORD64_T_PRINTFX	"lx"
#define UINT32_PRINTF		"u"

#else

#define COORD_T_PRINTF		"u"
#define COORD64_T_PRINTF	"llu"
#define COORD_T_PRINTFX		"x"
#define COORD64_T_PRINTFX	"llx"
#define UINT32_PRINTF		"u"

#endif

#define d64		INT64_PRINTF
#define	PF		UINT32_PRINTF
#define	x64		UINT64_PRINTFX

// Number of times a word appears in a genome
typedef unsigned COUNT_TYPE;
const COUNT_TYPE COUNT_TYPE_MAX = UINT_MAX;
// Scores stored in BndFile's and SegPair objects.
typedef int32 sp_score_t;

// Scores stored in AlnInfo objects
typedef float ai_score_t;

#if	INTEGER_DP

typedef int score_t;
const score_t SCORE_UNINIT = 911911911;
const score_t MINUS_INF = -5000000;

#define EQ(a, b)	((a) == (b))

#else

typedef float score_t;
const score_t SCORE_UNINIT = (score_t) 911911911;
const score_t MINUS_INF = (score_t) (-1e20);

#define EQ(a, b)	(fabs((a) - (b)) < 0.01)

#endif

// Segment of a genome.
struct Seg
	{
	coord_t lo;
	coord_t hi;

	coord_t Midpoint() const
		{
		return (hi + lo)/2;
		}

	coord_t Length() const
		{
		return hi - lo + 1;
		}

	coord_t Overlap(const Seg &s) const
		{
	// For some unknown reason, std::max / min gives
	// MSC compiler error here.
		coord_t maxlo = lo >= s.lo ? lo : s.lo;
		coord_t minhi = hi <= s.hi ? hi : s.hi;

		if (maxlo > minhi)
			return 0;

		return minhi - maxlo + 1;
		}
	};
class SeqFile;
class ChrNameIndex;

typedef std::vector<const char *> StrVec;
typedef std::string StdStr;
typedef std::vector<std::string> StdStrVec;
typedef std::vector<StdStrVec> StdStrVecVec;
typedef std::vector<ChrNameIndex *> ChrNameIndexVec;
typedef std::vector<SeqFile *> SeqFileVec;
typedef std::vector<uint64> Uint64Vec;
typedef std::vector<char *> STRVEC;
typedef std::vector<char *> STRLIST;
typedef std::vector<coord_t> PosVec;
typedef std::map<coord64_t, unsigned> CoordMap;
#endif	//TYPES_H
