#include "revfile.h"
#include "utils.h"

static void PathLengths(const StdStr &Path, coord_t *ptrLength0, coord_t *ptrLength1)
	{
	coord_t Length0 = 0;
	coord_t Length1 = 0;
	const unsigned PathLength = (unsigned) Path.size();
	for (unsigned i = 0; i < PathLength; ++i)
		{
		char c = Path[i];
		switch (c)
			{
		case 'M':
			++Length0;
			++Length1;
			break;
		case 'D':
			++Length0;
			break;
		case 'I':
			++Length1;
			break;
		default:
			Die("Path lengths invalid char %c in path", c);
			}
		}
	*ptrLength0 = Length0;
	*ptrLength1 = Length1;
	}

void ValidateHitPath(const AlnInfo &Hit, unsigned GenomeIndexA,
  unsigned GenomeIndexB, const StdStr &Path)
	{
	const GCSeg &SegA = Hit.GetSeg(GenomeIndexA);
	const GCSeg &SegB = Hit.GetSeg(GenomeIndexB);

	unsigned LengthA_Hit = SegA.GetLength();
	unsigned LengthB_Hit = SegB.GetLength();

	coord_t LengthA_Path;
	coord_t LengthB_Path;
	PathLengths(Path, &LengthA_Path, &LengthB_Path);

	if (LengthA_Hit != LengthA_Path || LengthB_Hit != LengthB_Path)
		{
		Log("Path=%s\n", Path.c_str());
		Log("Hit=");
		LogAI(Hit);
		Log("\n");
		Die("ValidateHitPath: LenA_Hit=%u LenA_Path=%u, LenB_Hit=%u LenB_Path=%u",
		  LengthA_Hit, LengthA_Path, LengthB_Hit, LengthB_Path);
		}
	}
