#include "utils.h"
#include "revfile.h"

unsigned GetRowLength(const std::string &Row);

void ValidateSegLength(const GCSeg &Seg, const StdStr &Row)
	{
	unsigned SegLength = (Seg.hi - Seg.lo + 1);
	unsigned RowLength = GetRowLength(Row);

	if (Seg.lo > Seg.hi)
		{
		Log("\n");
		Log("Row=%.128s", Row.c_str());
		if (RowLength > 128)
			Log("...(%u)", RowLength);
		Log("\n");
		LogGCSeg(Seg);
		Log("\n");
		Die("ValidateSegLength: lo=%u > hi=%u",
		  (unsigned) Seg.lo,
		  (unsigned) Seg.hi);
		}
	if (SegLength != RowLength)
		{
		Log("\n");
		Log("ValidateSegLength: SegLength=%u RowLength=%u\n",
		  SegLength, RowLength);
		Log("Row=%.64s\n", Row.c_str());
		Log("hi=%u - lo=%u + 1 = SegLength=%u\n",
		  Seg.hi, Seg.lo, SegLength);
		Die("SegLength %u != RowLength %u", SegLength, RowLength);
		}
	}

// Given an alignment (AI+Rows), check that the total number
// of letters in each row matches the segment length in the
// corresponding AlnInfo segment.
void ValidateSegLengths(const AlnInfo &AI, const StdStrVec &Rows)
	{
	const unsigned RowCount = (unsigned) Rows.size();
	const unsigned SegCount = (unsigned) AI.Segs.size();
	if (RowCount != SegCount)
		Die("RowCount!=SegCount");

	for (unsigned RowIndex = 0; RowIndex < RowCount; ++RowIndex)
		{
		const std::string &Row = Rows[RowIndex];
		const GCSeg &Seg = AI.Segs[RowIndex];
		ValidateSegLength(Seg, Row);
		}
	}
