#include "utils.h"
#include "revfile.h"
#include "getopts.h"
#include "progress.h"

#define	TRACE	0

unsigned GetUngappedSeqLength(const std::string &Seq);

static unsigned g_MaxErrors = 10;

static void LogSeq(const std::string &Seq)
	{
	unsigned SeqLength = (unsigned) SIZE(Seq);
	unsigned SeqPos = 0;
	unsigned ColCount = 0;
	unsigned GapCount = 0;
	for (;;)
		{
		if (SeqPos >= SeqLength)
			break;
		char c = Seq[SeqPos++];
		if (c == '-')
			++GapCount;
		else
			{
			if (GapCount > 0)
				{
				if (GapCount > 8)
					{
					Log("-(%u)-",  GapCount);
					ColCount += 6;
					}
				else
					{
					for (unsigned i = 0; i < GapCount; ++i)
						Log("-");
					ColCount += GapCount;
					}
				GapCount = 0;
				}
			Log("%c", c);
			++ColCount;
			if (ColCount == 60)
				{
				Log("...");
				break;
				}
			}
		}
	if (GapCount > 0)
		{
		if (GapCount > 8)
			{
			Log("-(%u)-",  GapCount);
			ColCount += 6;
			}
		else
			{
			for (unsigned i = 0; i < GapCount; ++i)
				Log("-");
			ColCount += GapCount;
			}
		}
	}

void ValidateSegLengths()
	{
	const char *InFileName = RequiredValueOpt("valseglens");
	IntOpt("maxerrs", &g_MaxErrors);
	bool List = FlagOpt("list");

	RevFile In;
	In.Open(InFileName);

	AIVec AIs;
	In.GetAllAlnInfos(AIs);

	unsigned ErrCount = 0;
	const unsigned AICount = (unsigned) AIs.size();
	Progress("%u AIs", AICount);
	ProgressStart("Validating");
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (AIIndex%100 == 0)
			ProgressStep(AIIndex, AICount);

		unsigned AlnIndex = In.GetAlnIndex(AIIndex);
		if (AlnIndex == UINT_MAX)
			continue;

		const AlnInfo &AI = AIs[AIIndex];
		StdStrVec Rows;
		In.GetAlnRows(AlnIndex, Rows);

		const unsigned SegCount = SIZE(AI.Segs);
		const unsigned RowCount = SIZE(Rows);
		if (SegCount != RowCount)
			{
			++ErrCount;
			Log("\n");
			Log("AI=%u has %u segs but %u rows", SegCount, RowCount);
			}
		for (unsigned SegIndex = 0; SegIndex < SegCount; ++SegIndex)
			{
			const GCSeg &Seg = AI.Segs[SegIndex];
			const StdStr &Row = Rows[SegIndex];

			coord_t SegLength = Seg.GetLength();
			coord_t UngappedRowLength = GetUngappedSeqLength(Row);

			bool Err = (UngappedRowLength != SegLength);
			if (Err)
				++ErrCount;

			if (Err || List)
				{
				Log("\n");
				Log("Seg length=%u, row length=%u\n",
				  (unsigned) SegLength, (unsigned) UngappedRowLength);
				Log("AI=%u ", AIIndex);
				LogGCSeg(Seg);
				Log("  Row=");
				LogSeq(Row);
				Log("\n");
				}
			}
		if (ErrCount > g_MaxErrors && g_MaxErrors > 0)
			Die("Max errors (%u) exceeded", g_MaxErrors);
		}

	ProgressNoRes("%10u  AIs", AICount);
	ProgressNoRes("%10u  errors", ErrCount);
	ProgressDone();
	}
