#include <vector>
#include <map>
#include <string>
#include <iterator>
#include <algorithm>
#include "revfile.h"
#include "intlist.h"
#include "types.h"
#include "utils.h"
#include "getopts.h"
#include "progress.h"

/* The input files */
RevFile InFiles[2];
RevFile ValFile;

unsigned MaxErrs = 0;
const char *GFFFileName = NULL;

/* RefGenome[i] is the index of the 'reference genome' in the i-th revfile */
int RefGenome[2];
/* OtherGenome[i] is the index of the 'other genome' in the i-th revfile */
int OtherGenome[2];
/* ValGenome[i] is the index of OtherGenome[i] in ValFile */
int ValGenome[2];

/* End of important globals */

extern void ValAlign();
extern void ValAlignDisk();

VALUE_OPT ValueOpts[] =
{
	{ "in1",				0 },
	{ "in2",				0 },
	{ "val",				0 },
	{ "maxerrs",    0 },
	{ "gff",        0 },
	{ "log",				0 }
};
int ValueOptCount = sizeof(ValueOpts)/sizeof(ValueOpts[0]);

FLAG_OPT FlagOpts[] =
{
	{ "mem",				0 },
	{ "dummy",				0 }
};
int FlagOptCount = sizeof(FlagOpts)/sizeof(FlagOpts[0]);

void Usage()
{
	fprintf(stderr,
"Usage:\n"
"    valta -in1 <revfile> -in2 <revfile> -val <revfile>\n"
"\n"
	);
}

/* Choose the reference genome */
void FindRefGenome()
{
	if ((InFiles[0].GetGenomeCount() != 2) || (InFiles[1].GetGenomeCount() != 2) || (ValFile.GetGenomeCount() != 2))
		Die("This program can only handle two genomes at a time.");

	for (int i = 0; i != 2; ++i)
		for (int j = 0; j != 2; ++j)
			if (InFiles[0].GetGenomeInfo(i) == InFiles[1].GetGenomeInfo(j))
			{
				RefGenome[0] = i;
				RefGenome[1] = j;
				OtherGenome[0] = 1 - i;
				OtherGenome[1] = 1 - j;
				if (InFiles[0].GetGenomeName(1-i) == InFiles[1].GetGenomeName(1-j))
					Warning("Genome name collision! Using '%s' as reference...", InFiles[0].GetGenomeName(i).c_str());

				if ((ValFile.GetGenomeInfo(0) == InFiles[0].GetGenomeInfo(OtherGenome[0])) && ((ValFile.GetGenomeInfo(1) == InFiles[1].GetGenomeInfo(OtherGenome[1]))))
				{
					ValGenome[0] = 0;
					ValGenome[1] = 1;
				}
				else if ((ValFile.GetGenomeInfo(1) == InFiles[0].GetGenomeInfo(OtherGenome[0])) && ((ValFile.GetGenomeInfo(0) == InFiles[1].GetGenomeInfo(OtherGenome[1]))))
				{
					ValGenome[0] = 1;
					ValGenome[1] = 0;
				}
				else
					Die("Genome names do not match between in1,in2 and val revfiles.");
				return;
			}

	Die("No common genome between the files.");
}

int main(int argc, char *argv[])
{
	ProcessArgVect(argc, argv);
	const char *InFileName[2];
	const char *ValFileName;

	InFileName[0] = RequiredValueOpt("in1");
	InFileName[1] = RequiredValueOpt("in2");
	ValFileName = RequiredValueOpt("val");
	IntOpt("maxerrs", &MaxErrs);
	GFFFileName = ValueOpt("gff");

	/* Open input Rev Files */
	for (int i = 0; i != 2; ++i)
		InFiles[i].Open(InFileName[i]);
	ValFile.Open(ValFileName);

	FindRefGenome();
	if (FlagOpt("mem"))
		ValAlign();
	else
		ValAlignDisk();

	for (int i = 0; i != 2; ++i)
		InFiles[i].Close();
	ValFile.Close();

	return 0;
}
