#ifndef WRAPSTDIO_H
#define WRAPSTDIO_H

#include <stdio.h>
#include <string>
#include "types.h"

/* wrapstdio.h
 *
 * These functions are wrappers around the regular stdio functions, such that:
 *
 *     - They call Die() upon failure
 *     - They work with 64-bit pointers, even in Windows.
 */

FILE *OpenStdioFile(const char *FileName, bool ForWriting = false);
FILE *CreateStdioFile(const char *FileName);
FILE *CreateStdioTmpFile(const char *Prefix = "StdioTmpFile");
void SeekStdioFile(FILE *f, off64_t Pos);
void SetStdioFilePos(FILE *f, off64_t Pos);

void ReadStdioFile(FILE *f, off64_t Pos, void *Buffer, size_t Bytes);
void ReadStdioFile(FILE *f, void *Buffer, size_t Bytes);
bool ReadLineStdioFile(FILE *f, char *Line, size_t Bytes);
bool ReadLineStdioFile(FILE *f, std::string &Line);
void ReadLineStdioFile2(FILE *f, char *Line, size_t Bytes);
#define READV(v)		ReadStdioFile(f, &(v), sizeof(v))
//#define READV(v)		{ Log("%s:%d %u READV(%s)\n", __FILE__, __LINE__,  (unsigned) GetStdioFilePos(f), #v); ReadStdioFile(f, &(v), sizeof(v)); }

void WriteStdioFile(FILE *f, off64_t Pos, const void *Buffer, size_t Bytes);
void WriteStdioFile(FILE *f, const void *Buffer, size_t Bytes);
void WriteStdioFileStr(FILE *f, off64_t Pos, const char *Buffer);
void WriteStdioFileStr(FILE *f, const char *Buffer);
#define WRITEV(v)		WriteStdioFile(f, &(v), sizeof(v))
//#define WRITEV(v)		{ Log("%s:%d %u WRITEV(%s)\n", __FILE__, __LINE__,  (unsigned) GetStdioFilePos(f), #v); WriteStdioFile(f, &(v), sizeof(v)); }

void FlushStdioFile(FILE *f);
void CloseStdioFile(FILE *f);
off64_t GetStdioFilePos(FILE *f);
off64_t GetStdioFileSize(FILE *f);
void CopyStdioFile(FILE *fFrom, FILE *fTo);
void RenameStdioFile(const char *FileNameFrom, const char *FileNameTo);

void GetStdioFileName(FILE *f, std::string &FileName);
void LogStdioFileState(FILE *f);

void DeleteStdioFile(const char *FileName);

#endif	// WRAPSTDIO_H
