#include "revfile.h"
#include "getopts.h"
#include "utils.h"
#include "progress.h"

void ParseStrVec(const std::string &In, std::vector<std::string> &Out, char c = ',');

void CopyAln(const RevFile &In, unsigned AIIndex, RevFile &Out)
	{
	AlnInfo AI;
	In.GetAlnInfo(AIIndex, AI);

	unsigned AlnIndex = In.GetAlnIndex(AIIndex);
	if (AlnIndex == UINT_MAX)
		Out.AddAlnInfo(AI);
	else
		{
		StdStrVec Rows;
		In.GetAlnRows(AIIndex, Rows);
		Out.AddAln(AI, Rows);
		}
	}

void XAIs()
	{
	const char *InFileName = RequiredValueOpt("xais");
	const char *OutFileName = RequiredValueOpt("out");
	const char *cstrAIIndexes = RequiredValueOpt("indexes");

	StdStrVec strvecAIIndexes;
	ParseStrVec(cstrAIIndexes, strvecAIIndexes);

	IntSet setAIIndexes;
	const unsigned InCount = (unsigned) strvecAIIndexes.size();
	for (unsigned i = 0; i < InCount; ++i)
		{
		const std::string &strAIIndex = strvecAIIndexes[i];
		const char *cstrAIIndex = strAIIndex.c_str();
		unsigned AIIndex = (unsigned) atoi(cstrAIIndex);
		setAIIndexes.insert(AIIndex);
		}

	IntVec AIIndexes;
	for (IntSet::const_iterator p = setAIIndexes.begin();
	  p != setAIIndexes.end(); ++p)
		{
		unsigned AIIndex = *p;
		AIIndexes.push_back(AIIndex);
		}

	RevFile In;
	RevFile Out;

	In.Open(InFileName);
	const std::vector<GenInfo> &Genomes = In.GetGenomes();

	Out.Create(OutFileName, Genomes);

	const unsigned AICount = (unsigned) AIIndexes.size();
	Progress("%u alignments", AICount);
	ProgressStart("Extracting");
	for (unsigned i = 0; i < AICount; ++i)
		{
		if (i%100 == 0)
			ProgressStep(i, AICount);
		unsigned AIIndex = AIIndexes[i];
		CopyAln(In, AIIndex, Out);
		}
	ProgressDone();

	Progress("Closing files");
	In.Close();
	Out.Close();
	}
