#include "utils.h"
#include "fastastr.h"
#include "getopts.h"
#include "progress.h"
#include "xlat.h"

unsigned g_MinFrameLength = 99;
unsigned g_PlusORFCount = 0;
unsigned g_MinusORFCount = 0;
uint64 g_PlusORFBaseCount = 0;
uint64 g_MinusORFBaseCount = 0;
const char *g_SepStr = 0;

bool g_Stops = false;

void ExtractORFsSeq(FILE *f, const std::vector<char> &Label,
  const std::vector<unsigned char> &Seq, ALPHA OutAlpha);

void DoExtractORFs(const char *InFileName, const char *OutFileName,
  ALPHA OutAlpha)
	{
	g_Stops = FlagOpt("stops");

	IntOpt("minframe", &g_MinFrameLength);
	g_SepStr = ValueOpt("sep");
	if (g_SepStr == 0)
		g_SepStr = ";";

	FastaStream FS;
	FS.Open(InFileName);

	FILE *fOut = CreateStdioFile(OutFileName);

	std::vector<char> Label;
	std::vector<unsigned char> Seq;

	uint64 GenomeBaseCount = 0;
	ProgressStart("Extracting ORFs >= %u nucleotides", g_MinFrameLength);
	unsigned Steps = (unsigned) (FS.GetFileSize()/1000000 + 1);
	unsigned PrevStep = UINT_MAX;
	for (;;)
		{
		bool Ok = FS.GetNextSeq(Label, Seq);
		if (!Ok)
			break;

		unsigned Step = (unsigned) (FS.GetFilePos()/1000000);
		if (Step != PrevStep)
			{
			ProgressStep(Step, Steps);
			PrevStep = Step;
			}

		ExtractORFsSeq(fOut, Label, Seq, OutAlpha);

		GenomeBaseCount += (uint64) Seq.size();
		}
	ProgressDone();

	uint64 TotalORFBaseCount = g_PlusORFBaseCount + g_MinusORFBaseCount;

	ProgressNoRes("Genome nucleotides %s", atoh(uint64tostr(GenomeBaseCount)));
	ProgressNoRes("ORF amino acids %s", atoh(uint64tostr(TotalORFBaseCount/3)));
	}

void ExtractORFs()
	{
	const char *InFileName = RequiredValueOpt("xorfs");
	const char *OutFileName = RequiredValueOpt("out");
	bool Nuc = FlagOpt("nuc");
	ALPHA OutAlpha = (Nuc ? ALPHA_Nucleo : ALPHA_Amino);

	DoExtractORFs(InFileName, OutFileName, OutAlpha);
	}
